unit T_saver;
{
***********************************************************************
**       [STREAM]                                                    **
**                                                                   **
**                      ORBIS ... Saver                              **
**                                                                   **
**                       Topol Pro 1996,1997                        **
**                                                                   **
***********************************************************************
}

interface

uses
   Classes,
   Elements,
   e_Class,
   //f_Gauge,
   t_Recurs;

type

TResetAndListClasses = class(TRecursor)
 protected
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
 public
    procedure   DoIt; override;
end;


TOrbisSaver = class(TRecursor)
 private
    FIndex: longInt;
    FFileName: string;
    FClasses: TOrbisClasses;
    FActualElement: TElement;
    FActualPropertyIndex: integer;
    FInstance: TOrbisInstance;
    FPrepareClassesBefore: boolean;
    FComment: string;
    //F_gaugeForm: TF_gaugeForm;
    procedure SetClasses(aValue: TOrbisClasses);
    procedure SetFileName(aValue: string);
 protected
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessAfter(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
    property  Index: longInt read FIndex write FIndex default 0;
 public
    constructor CreateAll(aFile: string);
    procedure   DoIt; override;
    function    NumberToPropertyName(aNumber: longInt): string; override;
    procedure   StartGauge;
    procedure   EndGauge;
    procedure   Prepare; virtual;  { pprava vol bu ResetIDs(default) nebo PrepareClasses}
    procedure   ResetModelIDs; virtual;
    procedure   PrepareClasses; virtual;
    procedure   SaveClasses; virtual;
    procedure   SaveInfo; virtual; abstract;
    function    CheckClass: boolean; virtual;
    procedure   SaveClass(aClass: TOrbisClass); virtual; abstract;
    procedure   NewElement; virtual; abstract;
    procedure   SaveProperties; virtual;
    procedure   SaveElements(realy: boolean); virtual; abstract;
    procedure   SaveInteger(aValue: integer); virtual; abstract;
    procedure   SaveChar(aValue: char); virtual; abstract;
    procedure   SaveFloat(const aValue: extended); virtual; abstract;
    procedure   SaveString(const aValue: string); virtual; abstract;
    procedure   SaveElement(aValue: TElement); virtual; abstract;
    property    OrbisClasses: TOrbisCLasses read FClasses write SetClasses;
    property    ActualElement: TElement read FActualElement;
    property    ActualPropertyIndex: integer read FActualPropertyIndex write FActualPropertyIndex;
    property    Instance: TOrbisInstance read FInstance;
    property    PrepareClassesBefore: boolean read FPrepareClassesBefore write FPrepareClassesBefore;
    property    Comment: string read FComment write FComment;
published
    property    FileName: string read FFileName write SetFileName;
end;



TStreamSaver = class(TOrbisSaver)
 private
    FWriter: TWriter;
    FStream: TStream;
 public
    procedure   DoIt; override;
    procedure   SaveInfo; override;
    procedure   SaveClass(aClass: TOrbisClass); override;
    procedure   NewElement; override;
    procedure   SaveProperties; override;
    procedure   SaveElements(realy: boolean); override;
    procedure   SaveInteger(aValue: integer); override;
    procedure   SaveChar(aValue: char); override;
    procedure   SaveFloat(const aValue: extended); override;
    procedure   SaveString(const aValue: string); override;
    procedure   SaveElement(aValue: TElement); override;
    property    Writer: TWriter read FWriter write FWriter;
    property    Stream: TStream read FStream write FStream;
end;


TSave = class(TStreamSaver);
{

Saver
   DoIt:
     ResetIDs
     PrepareClasses
     *
     SaveClasses

                          Process:
                            SaveElement
                               CheckClass
                               SaveProperties
                               SaveElements

}

TSaver = class(TRecursor)
 private
   FIndex: longInt;
   FWriter: TWriter;
   FStream: TStream;
 protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessAfter(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   property  Index: longInt read FIndex write FIndex default 0;
   property  Writer: TWriter read FWriter write FWriter;
   property  Stream: TStream read FStream write FStream;
 public
   constructor Create; override;
   constructor CreateAll(aFile: string);
   procedure   DoIt; override;
end;

implementation


uses
   {!Dialogs,}
   {!T_Main,}
   Controls,
   Forms,
   SysUtils,
   m_Orbis,
   TypInfo,
   Props,
   Worlds;

{ TResetAndListClasses }


function  TResetAndListClasses.QProcessBefore(Element: TElement): boolean;
begin
   result := true;
end;

procedure TResetAndListClasses.Process(Element: TElement);
var
   C: TElementClass;
   S: string;
   OC, WC: TOrbisClasses;
   OCC: TOrbisClass;
begin
   Element.ID := BaseID;
   C := TElementClass(Element.ClassType);
   S := Element.ClassName;
   OC := TOrbisClasses(Output);
   if (OC.ActualClass = nil) or (not OC.ActualClass.IsClassName(S)) then
   if OC.GetClass(S) = nil then
   begin
      WC  := TOrbisClasses(World.ObjectClasses);
      OCC := WC.GetOrbisClassByName(S);
      if OCC = nil then
      begin
         WC.AddClass(C);
         OCC := WC.ActualClass;
      end;
      OC.AddOrbisClass(OCC);
   end;
end;

procedure TResetAndListClasses.DoIt;
begin
   Output := TOrbisClasses.Create;
   ResetBaseID;
   inherited DoIt;
end;



{ TOrbisSaver }


constructor TOrbisSaver.CreateAll(aFile: string);
begin
   inherited Create;
   FileName := aFile;
end;

function    TOrbisSaver.NumberToPropertyName(aNumber: longInt): string;
begin
   result := 'FileName';
end;





procedure TOrbisSaver.StartGauge;
var
   i: longInt;
begin
   Screen.Cursor:= crHourGlass;
   //F_GaugeForm:= TF_GaugeForm.Create(Application);
   //F_GaugeForm.Caption:= 'Ukldn modelu do souboru ' + FileName;
   if Input is TEntity then i := TEntity(Input).WholeCount
                       else i := 100;
   if i > 100 then
      StartProgress('Ukldn modelu do souboru ' + FileName +' -celkem '+IntToStr(i)+ ' Element' ,0,i);
   //F_GaugeForm.MaxValue:= i; {!!!!!!!!!!!!!!!!!!!!!!!!}
   //F_GaugeForm.Show
end;



procedure TOrbisSaver.EndGauge;
begin
   Screen.Cursor:= crDefault;
   EndProgress;
   //Application.ProcessMessages;
   (*
   if F_GaugeForm <> nil then
   begin
      F_GaugeForm.Free;
      F_GaugeForm:= nil
   end;
   *)
end;



procedure TOrbisSaver.SetClasses(aValue: TOrbisClasses);
begin
   if FClasses <> nil then
   begin
      FreeElement(FCLasses);
      FClasses := nil;
   end;
   FClasses := aValue;
   AddElement(FClasses);
end;

procedure TOrbisSaver.SetFileName(aValue: string);
begin
   FFileName := aValue;
end;

function  TOrbisSaver.QGoInside(Element: TElement): boolean;
begin
   result := Element is TEntity;
end;

function  TOrbisSaver.QProcessAfter(Element: TElement): boolean;
begin
   result := true;
end;

procedure TOrbisSaver.Process(Element: TElement);
var
   i: integer;
   PropInfo: PPropInfo;
begin
   if Element.ID > BaseID then exit;
   Element.ID := FIndex;
   inc(FIndex);
   if (FIndex mod 100) = 0 then
   begin
      //F_GaugeForm.Progress := FIndex;
      Progress := FIndex;
      ProgressText := 'Uloeno '+ IntToStr(FIndex)+ ' element';
      Application.ProcessMessages;
   end;
   FActualElement := Element;
   FInstance.Instance := Element;
   if (not CheckClass) and (not PrepareClassesBefore)then
      SaveClass(OrbisClasses.ActualClass);
   NewElement;
   SaveProperties;
   SaveElements(not Element.NoElements);
end;


procedure TOrbisSaver.DoIt;
begin
   FInstance := TOrbisInstance.Create;
   AddElement(FInstance);
   if Input = nil then exit;
   StartGauge;
   Prepare;
   FIndex := 0;
   SaveInfo;
   {!ShowMessage('After SaveInfo');}
   inherited DoIt;
   EndGauge;
end;

procedure   TOrbisSaver.Prepare;
begin
   if PrepareClassesBefore then PrepareClasses
                           else ResetModelIDs;
end;


procedure   TOrbisSaver.ResetModelIDs;
begin
   Input.ResetID;
   OrbisClasses := TOrbisClasses.Create;
   //PostTextMsg(tm_AddToProtocol,'BaseID: '+IntToStr(BaseID),nil);
end;

procedure   TOrbisSaver.PrepareClasses;
var
  RC: TResetAndListClasses;
begin
   RC := TResetAndListClasses.Create;
   RC.Input := Input;
   RC.DoIt;
   OrbisClasses := TOrbisClasses(RC.Output);
   RC.Free;
   SaveClasses;
end;


procedure   TOrbisSaver.SaveClasses;
var
   i: longInt;
begin
   for i := 0 to OrbisClasses.Count -1 do
   begin
      SaveClass(TOrbisClass(OrbisClasses.Elements[i]));
   end;
end;


function   TOrbisSaver.CheckClass: boolean;
var
   C: TElementClass;
   S: string;
   WC: TOrbisClasses;
   OCC: TOrbisClass;
begin
   result := true;
   C := TElementClass(ActualElement.ClassType);
   S := ActualElement.ClassName;
   if (OrbisClasses.ActualClass = nil) or (not OrbisClasses.ActualClass.IsClassName(S)) then
   if OrbisClasses.GetClass(S) = nil then
   begin
      WC  := TOrbisClasses(World.ObjectClasses);
      OCC := WC.GetOrbisClassByName(S);
      if OCC = nil then
      begin
         WC.AddClass(C);
         OCC := WC.ActualClass;
      end;
      OrbisClasses.AddOrbisClass(OCC);
      result := false;
   end;
end;

procedure TOrbisSaver.SaveProperties;
var
   i: integer;
   PropInfo: PPropInfo;
begin
   for i := 0 to Instance.PropCount - 1 do
   begin
      ActualPropertyIndex := i;
      if not FInstance.PropertyStored[i] then Continue;
      PropInfo := FInstance.PropList^[i];
      case Instance.PropertyTypeKind[i] of
         tkInteger,tkEnumeration: SaveInteger(GetOrdProp(ActualElement, PropInfo));
         tkChar: SaveChar (Chr(GetOrdProp(ActualElement, PropInfo)));
         tkFloat: SaveFloat(GetFloatProp(ActualElement, PropInfo));
         tkString
         {$IFDEF Win32}
         ,tkLString
         {$ENDIF}
         {$IFDEF UNICODE}
         , tkUString
         {$ENDIF}
                   : SaveString(GetStrProp(ActualElement, PropInfo));
         tkClass:
         try
            SaveElement(TElement(GetOrdProp(ActualElement, PropInfo)));
         except
            SaveInteger(-1);
         end;
      end;
   end;
end;

{ TStreamSaver }


procedure   TStreamSaver.DoIt;
begin

   Stream := TFileStream.Create(FileName, fmCreate);
   {
   Stream := TMemoryStream.Create;
   }

   Writer := TWriter.Create(Stream, 4096);
   {!ShowMessage('Before DoIt');}
   inherited DoIt;
   {!ShowMessage('After DoIt');}
   Writer.WriteListEnd;
   Writer.Free;
   {
   TMemoryStream(Stream).SaveToFile(FileName);
   }
   Stream.Free;
end;

procedure   TStreamSaver.SaveInfo;
const
   Magic: array[0..3] of Char = 'ORB0';
begin
   Writer.Write(Magic, SizeOf(Magic));
   Writer.WriteString(ModulName);
   Writer.WriteString(version);
   if Comment = '' then Comment := 'No comment';
   Writer.WriteString(Comment);
   Writer.WriteListBegin;
end;

procedure   TStreamSaver.SaveClass(aClass: TOrbisClass);
var
   i: integer;
begin
   Writer.WriteInteger(-1);        { oznaen tdy }
   Writer.Writestring(aClass.Name);
   Writer.WriteInteger(aClass.ID);
   Writer.WriteListBegin;
   for i := 0 to aClass.PropCount - 1 do
   begin
      {
      if OrbisClasses.ActualClass.PropertyStored[i] then
      }
      begin
         Writer.WriteString(Instance.PropertyName[i]);
         Writer.WriteInteger(integer(Instance.PropertyTypeKind[i]));
      end;
   end;
   Writer.WriteListEnd;
end;

procedure   TStreamSaver.NewElement;
begin
   Writer.WriteInteger(OrbisClasses.ActualClass.ID);
   Writer.WriteInteger(ActualElement.ID);
end;

procedure   TStreamSaver.SaveProperties;
begin
   Writer.WriteListBegin;
   inherited SaveProperties;
   Writer.WriteListEnd;
end;



procedure   TStreamSaver.SaveElements(realy: boolean);
var
   E: TEntity;
   El: TElement;
   i: longInt;
begin
   if realy then Writer.WriteInteger(1)
   else
   begin
      Writer.WriteInteger(0);
      exit;
   end;
   Writer.WriteListBegin;
   E := TEntity(ActualElement);
   for i := 0 to E.Count - 1 do
   begin
      El := E.Elements[i];
      Writer.WriteInteger(El.ID);
      //!!!!!!!!!!!!!!!!!!!
      if El.ID < 0 then
      begin
         PostTextMsg(tm_AddToProtocol,El.Name+': '+IntToStr(El.ID)+#13,nil);
      end;
   end;
   Writer.WriteListEnd;
end;

procedure   TStreamSaver.SaveInteger(aValue: integer);
begin
   Writer.WriteInteger(ActualPropertyIndex);
   Writer.WriteInteger(aValue);
end;

procedure   TStreamSaver.SaveChar(aValue: char);
begin
   Writer.WriteInteger(ActualPropertyIndex);
   Writer.WriteInteger(ord(aValue));
end;

procedure   TStreamSaver.SaveFloat(const aValue: extended);
begin
   Writer.WriteInteger(ActualPropertyIndex);
   Writer.WriteFloat(aValue);
end;

procedure   TStreamSaver.SaveString(const aValue: string);
begin
   Writer.WriteInteger(ActualPropertyIndex);
   Writer.WriteString(aValue);
end;

procedure   TStreamSaver.SaveElement(aValue: TElement);
begin
   Writer.WriteInteger(ActualPropertyIndex);
   if AValue = nil then Writer.WriteInteger(-1)
                   else Writer.WriteInteger(aValue.ID);
end;


{ TSaver }

constructor TSaver.Create;
begin
   inherited Create;
   Name := 'ORBIS.ORB';
end;

constructor TSaver.CreateAll(aFile: string);
begin
   inherited Create;
   Name := aFile;
end;


function  TSaver.QGoInside(Element: TElement): boolean;
begin
   result := Element is TEntity;
end;

function  TSaver.QProcessAfter(Element: TElement): boolean;
begin
   result := true;
end;

procedure TSaver.Process(Element: TElement);
begin
   if Element.ID > BaseID then exit;
   Element.ID := FIndex;
   inc(FIndex);
   Element.Save(Writer);
end;

procedure TSaver.DoIt;
var
   i: longInt;
begin
   Screen.Cursor := crHourGlass;
   Stream := TFileStream.Create(Name, fmCreate);
   Writer := TWriter.Create(Stream, 64000);
   FIndex := 0;
   Writer.WriteSignature;
   Writer.WriteListBegin;
   inherited DoIt;
   Writer.WriteListEnd;
   Writer.Free;
   Stream.Free;
   Screen.Cursor := crDefault;
end;


BEGIN
   RegisterOrbisClass(TSaver);
   RegisterOrbisClass(TSave);
END.