unit e_3D;

interface
uses
   Elements,
   e_Geometry,
   e_QTree,
   g_Elems,
   t_Sort,
   t_QTreeTools,
   Tools;

type

TPoint3DElement = class(TPointElement)
  private
    FOrigin: TElement;
    FaEnd: boolean;
    procedure SetOrigin(const Value: TElement);
    procedure SetaEnd(const Value: boolean);
private
   FZ: double;
   procedure SetZ(const Value: double);
   property  Origin: TElement read FOrigin write SetOrigin;
   property  aEnd: boolean read FaEnd write SetaEnd;
published
   property Z: double read FZ write SetZ;
end;

TEdge3DElement = class(TRectEntity)
private
    FLength: double;
    FSO: boolean;
    procedure SetLength(const Value: double);
    procedure SetSO(const Value: boolean);
published
   property Length: double read FLength write SetLength;
   property SO: boolean read FSO write SetSO;
end;

TSortEdges3D = class(TSortElements)
protected
   function CompareElements(E1,E2: TElement): integer; override;
end;

// Create_TIN
// vstup:
//   Lines  - vrstevnice
//   Points - body
// props:
//   Vertexes:        3DBody vrcholy
//   Edges:           Hrany potencialni
//   SameOriginEdges:     -  "  -       stejneho puvodu (stejna vyska nebo linie)
//   RealEdges:       Hrany realne
//   Z_LIN:           Nazev property souradnice Z linii
//   Z_PNT:           Nazev property souradnice Z bodu
// princip:
//   1 Vytvori seznam Vertexu z linii a bodu
//     - spojnice vrstevnic ulozi do realnych hran
//   2 pro kazdy vertex nalezne spojnice s vert. lezicimi v toleranci -> do Edges
//   3 seradi edges podle velikosti a od nejkratsi porovnava s RealEd.
//     - pokud jsou nekolizni tak
//     - spojnice steneho puvodu (linie) odklada do SOEdges ostatni do RealEdges
//   4 Nakonec seradi a vyresi SameOriginEdges

// error
//   1 Chybi LINES
//   2 Z_LIN neni definovano
//   3 Z_NPT neni definovano
TCreate_TIN = class(TTool)
  private
    FTolerance: double;
    FPoints: TEntity;
    FLines: TEntity;
    FVertexes: TRectEntity;
    FEdges: TRectEntity;
    FSameOriginEdges: TRectEntity;
    FRealEdges: TRectEntity;
    FVertexesQTree: TRectEntityQTree;
    FRealEdgesQTree: TRectEntityQTree;
    FZ_PNT: string;
    FZ_LIN: string;
    FComment: string;
    FEdgeTolerance: double;
    FUseRectFilter: boolean;
    FFilterMinY: double;
    FFilterMaxX: double;
    FFilterMinX: double;
    FFilterMaxY: double;
    FVertexesTime: Cardinal;
    FNEdges: integer;
    FNVertexes: integer;
    FNSOEdges: integer;
    FNRealEdges: integer;
    FMinZ: double;
    procedure SetEdges(const Value: TRectEntity);
    procedure SetLines(const Value: TEntity);
    procedure SetPoints(const Value: TEntity);
    procedure SetRealEdges(const Value: TRectEntity);
    procedure SetRealEdgesQTree(const Value: TRectEntityQTree);
    procedure SetSameOriginEdges(const Value: TRectEntity);
    procedure SetTolerance(const Value: double);
    procedure SetVertexes(const Value: TRectEntity);
    procedure SetVertexesQTree(const Value: TRectEntityQTree);
    procedure SetZ_LIN(const Value: string);
    procedure SetZ_PNT(const Value: string);
    procedure SetComment(const Value: string);
    procedure SetEdgeTolerance(const Value: double);
    procedure SetFilterMaxX(const Value: double);
    procedure SetFilterMaxY(const Value: double);
    procedure SetFilterMinX(const Value: double);
    procedure SetFilterMinY(const Value: double);
    procedure SetUseRectFilter(const Value: boolean);
    procedure SetNEdges(const Value: integer);
    procedure SetNRealEdges(const Value: integer);
    procedure SetNSOEdges(const Value: integer);
    procedure SetNVertexes(const Value: integer);
    procedure SetVertexesTime(const Value: Cardinal);
    procedure SetMinZ(const Value: double);
public
   procedure DoIt; override;
   property  Vertexes: TRectEntity read FVertexes write SetVertexes;
   property  Edges: TRectEntity read FEdges write SetEdges;
   property  SameOriginEdges: TRectEntity read FSameOriginEdges write SetSameOriginEdges;
   property  RealEdges: TRectEntity read FRealEdges write SetRealEdges;
   property  VertexesQTree: TRectEntityQTree read FVertexesQTree write SetVertexesQTree;
   property  RealEdgesQTree: TRectEntityQTree read FRealEdgesQTree write SetRealEdgesQTree;
   procedure CreateEntities;
   procedure CreateVertexes;
   procedure CreateQTree;
   procedure ProcessVertexes;
   procedure FreeVertexes;
   procedure ProcessEdges;
   function  TestEdges(P1,P2: TPoint3DElement; E1: TEdge3DElement): boolean;
   function  TestEdge(P1,P2: TPoint3DElement; E1: TEdge3DElement): boolean;
published
   property Lines: TEntity read FLines write SetLines;
   property Points: TEntity read FPoints write SetPoints;
   property Z_LIN: string read FZ_LIN write SetZ_LIN;
   property Z_PNT: string read FZ_PNT write SetZ_PNT;
   property Tolerance: double read FTolerance write SetTolerance;
   property EdgeTolerance: double read FEdgeTolerance write SetEdgeTolerance;
   property Comment: string read FComment write SetComment;
   property FilterMinX: double read FFilterMinX write SetFilterMinX;
   property FilterMinY: double read FFilterMinY write SetFilterMinY;
   property FilterMaxX: double read FFilterMaxX write SetFilterMaxX;
   property FilterMaxY: double read FFilterMaxY write SetFilterMaxY;
   property UseRectFilter: boolean read FUseRectFilter write SetUseRectFilter;
   property NVertexes: integer read FNVertexes write SetNVertexes;
   property NEdges: integer read FNEdges write SetNEdges;
   property NSOEdges: integer read FNSOEdges write SetNSOEdges;
   property NRealEdges: integer read FNRealEdges write SetNRealEdges;
   property VertexesTime: Cardinal read FVertexesTime write SetVertexesTime;
   property MinZ: double read FMinZ write SetMinZ;
end;

implementation
uses
   Forms,
   SysUtils,
   Windows,
   e_Class,
   e_GeometryElement,
   MathLib,
   u_String;

{ TPoint3DElement }

procedure TPoint3DElement.SetaEnd(const Value: boolean);
begin
  FaEnd := Value;
end;

procedure TPoint3DElement.SetOrigin(const Value: TElement);
begin
  FOrigin := Value;
end;

procedure TPoint3DElement.SetZ(const Value: double);
begin
   FZ := Value;
end;

{ TCreate_TIN }

procedure TCreate_TIN.CreateEntities;
begin
   SameOriginEdges := TRectEntity.Create;
   SameOriginEdges.Name := 'SameOriginEdges';
   Edges := TRectEntity.Create;
   Edges.Name := 'Edges';
   RealEdges := TRectEntity.Create;
   RealEdges.Name := 'RealEdges';
   Vertexes := TRectEntity.Create;
   Vertexes.Name := 'VERTEXES';
end;

procedure TCreate_TIN.CreateQTree;
var
   TQ: TAddEntityToQTree;
begin
   Progress := 0;
   ProgressText := '* Vertexes QTree';
   Application.ProcessMessages;
   VertexesQTree :=TRectEntityQTree.Create;
   VertexesQTree.Name := 'VertexesQTree';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Vertexes;
   TQ.Output := VertexesQTree;
   TQ.DoIt;
   TQ.Free;
   ProgressText := '* Edges QTree';
   Application.ProcessMessages;
   RealEdgesQTree :=TRectEntityQTree.Create;
   RealEdgesQTree.Name := 'RealEdgesQTree';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RealEdges;
   TQ.Output := RealEdgesQTree;
   TQ.DoIt;
   TQ.Free;
end;

procedure TCreate_TIN.CreateVertexes;
var
   i,j,li: integer;
   e: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   L: TPolyLineElement;
   P: TPointElement;
   C: T2DCoordinate;
   P3D,P3D1,P3D0: TPoint3DElement;
   E3D: TEdge3DElement;
   aZ: Double;
begin
   if MinZ = 0 then
      MinZ := 1;
   ProgressText := '* Vertexes LIN';
   MaxProgress := Lines.LastIndex;
   Application.ProcessMessages;
   for i := 0 to Lines.LastIndex do
   begin
      //if i mod 50 = 0 then
      //begin
         Progress := i;
         Application.ProcessMessages;
      //end;
      e := Lines.Elements[i];
      if not (e is TGeometryElement) then
         Continue;
      GE := TGeometryElement(e);
      G := GE.Geometry;
      if G = nil then
         Continue;
      if not (G is TPolyLineElement) then
         Continue;
      aZ := StrToExtended(GE.GetPropertyStringByName(Z_LIN));
      if aZ < MinZ then
         continue;
      L := TPolyLineElement(G);
      P3D1 := nil;
      li := L.LastIndex;
      P3D0 := nil;
      for j := 0 to li do
      begin
         C := T2DCoordinate(L.Elements[j]);
         if UseRectFilter then
         if (C.X < FilterMinX) or (C.X > FilterMaxX) or (C.Y < FilterMinY) or (C.Y > FilterMaxY)  then
         begin
            P3D1 := nil;
            Continue;
         end;
         if (j = li) and (P3D0 <> nil) and (P3D0.X = C.X) and (P3D0.Y = C.Y) then
         begin
            P3D := P3D0        //prvni a posledni bod shodny, pouzijeme prvni
         end
         else
         begin
            P3D := TPoint3DElement.Create;
            P3D.X := C.X;
            P3D.Y := C.Y;
            P3D.Z := aZ;
            P3D.Origin := GE;
            P3D.aEnd := (j = 0) or (j = li);
            Vertexes.AddAndUpdateRect(P3D);
         end;
         if j = 0 then
            P3D0 := P3D;       // schovani prvniho bodu
         if P3D1 <>  nil then // hrana
         begin
            E3D := TEdge3DElement.Create;
            E3D.AddAndUpdateRect(P3D1);
            E3D.AddAndUpdateRect(P3D);
            E3D.Length := Distance(P3D1.X,P3D1.Y,P3D.X,P3D.Y);
            RealEdges.AddAndUpdateRect(E3D);
         end;
         P3D1 := P3D;
      end;
   end;
   if (Points <> nil) and (Points.Count > 0) then
   begin
      ProgressText := '* Vertexes PNT';
      Progress := 0;
      MaxProgress := Points.LastIndex;
      Application.ProcessMessages;
      for i := 0 to Points.LastIndex do
      begin
         Progress := i;
         Application.ProcessMessages;
         e := Points.Elements[i];
         if not (e is TGeometryElement) then
            Continue;
         GE := TGeometryElement(e);
         G := GE.Geometry;
         if G = nil then
            Continue;
         if not (G is TPointElement) then
            Continue;
         aZ := StrToExtended(GE.GetPropertyStringByName(Z_PNT));
         if aZ < MinZ then
            Continue;
         P := TPointElement(G);
         if UseRectFilter then
            if (P.X < FilterMinX) or (P.X > FilterMaxX) or (P.Y < FilterMinY) or (P.Y > FilterMaxY)  then
               Continue;
         P3D := TPoint3DElement.Create;
         P3D.X := P.X;
         P3D.Y := P.Y;
         P3D.Z := aZ;
         P3D.Origin := GE;
         Vertexes.AddAndUpdateRect(P3D);
      end;
   end;
   NVertexes := Vertexes.Count;  // pocet vertexu
end;

procedure TCreate_TIN.DoIt;
begin
   Error := 0;
   if (Lines = nil) or (Lines.Count = 0) then
   begin
      Error := 1;
      exit;
   end;
   if Z_LIN = '' then
   begin
      Error := 2;
      exit;
   end;
   if (Points <> nil) and (Points.Count > 0) then
   if Z_PNT = '' then
   begin
      Error := 3;
      exit;
   end;
   if Comment = '' then
      Comment := Name;
   StartProgress(Comment,0,0);
   Application.ProcessMessages;
   CreateEntities;
   CreateVertexes;
   CreateQTree;
   ProcessVertexes;
   FreeVertexes;
   ProcessEdges;
end;

procedure TCreate_TIN.FreeVertexes;
begin
   ProgressText := 'Vertexes free';
   Application.ProcessMessages;
   VertexesQTree := nil;
   Vertexes := nil;
end;

procedure TCreate_TIN.ProcessEdges;
var
   Sort: TSortEdges3D;
   i: integer;
   E3D: TEdge3DElement;
   procedure proced(const S: string; EDS: TEntity);
   var
      i: integer;
   begin
      // trideni hran
      ProgressText := S+' Sorting';
      Application.ProcessMessages;
      Sort := TSortEdges3D.Create;
      Sort.Input := EDS;
      //Sort.PropertyName := 'Length';
      Sort.Down := true;
      Sort.DoIt;
      Sort.Free;
      ProgressText := S+' processing';
      MaxProgress := EDS.LastIndex;
      Progress := 0;
      for i := EDS.LastIndex downto 0 do
      begin
         Progress := i;
         Application.ProcessMessages;
         E3D := TEdge3DElement(EDS.Elements[i]);
         if TestEdges(nil,nil,E3D) then
         begin
            RealEdges.AddAndUpdateRect(E3D);
            RealEdgesQTree.NextRectEntity := E3D;
            EDS.RemoveElement(E3D);
         end
         else
         begin
            E3D.Free;
         end;
      end;
   end;
begin
   Proced('Edges',Edges);
   Proced('S.O.Edges',SameOriginEdges);
   NRealEdges := RealEdges.Count;
end;

procedure TCreate_TIN.ProcessVertexes;
var
   i,j,pni,eni,nd,ind: integer;
   P3D, P3D1: TPoint3DElement;
   he: TElement;
   E3D: TEdge3DElement;
   e: TElement;
   EE: TEntity;
   W,EW: TQTreeWalker;
   OK: boolean;
   X1,Y1,X2,Y2: double;
   ad,adX,adY,dX,dY: double;
   S: string;
   T1,T2: Cardinal;
begin
   T1 := GetTickCount;
   S := 'Vertex processing ';
   ProgressText := S;
   MaxProgress := Vertexes.LastIndex;
   Progress := 0;
   Application.ProcessMessages;
   W := TQTreeWalker.Create;
   if Tolerance < 10 then
      Tolerance := 1000;
   if EdgeTolerance < 1 then
      EdgeTolerance := 100;
   for i := 0 to Vertexes.LastIndex do
   begin
      Progress := i;
      //ProgressText := 'Vertex processing '+IntToStr(i);
      Application.ProcessMessages;
      e := Vertexes.Elements[i];
      if not (e is TPoint3DElement) then
         Continue;
      P3D := TPoint3DElement(e);
      pni := -1;
      e := W.GetFirstElementForRectangle(VertexesQTree,P3D.X,P3D.Y,P3D.X,P3D.Y,Tolerance);
      while e <> nil do
      begin
         Inc(pni);
         //ProgressText := 'Vertex processing '+IntToStr(i)+' P '+IntToStr(pni);
         //Application.ProcessMessages;
         P3D1 := TPoint3DElement(e);
         // steny bod
         if P3D1 = P3D then    // stejny bod
         begin
            e := W.GetNextElement;
            Continue;
         end;
         OK := true;
         // spolecna hrana
         for j := 0 to P3D1.OwnersCount - 1 do   // spolecna hrana?
         begin
            EE := P3D1.Owners[j];
            if (EE is TEdge3DElement) and P3D.IsMyOwner(EE) then
            begin
               OK := false;
               break;
            end;
         end;
         if not OK then   // spolecna hrana
         begin
            e := W.GetNextElement;
            Continue;
         end;
         if TestEdges(P3D,P3D1,nil) then   // neni kolize, budeme delat hranu
         begin
            E3D := TEdge3DElement.Create;
            E3D.AddAndUpdateRect(P3D);
            E3D.AddAndUpdateRect(P3D1);
            E3D.Length := Distance(P3D1.X,P3D1.Y,P3D.X,P3D.Y);
            if P3D.Z = P3D1.Z then
            begin
               if P3D.aEnd and P3D1.aEnd  and (E3D.Length < 2*EdgeTolerance) then
                  Edges.AddAndUpdateRect(E3D)            // stena vyska ale konce vrs
               else
               begin
                  SameOriginEdges.AddAndUpdateRect(E3D); // stena vyska
                  E3D.SO := true;
               end;
            end
            else
               Edges.AddAndUpdateRect(E3D); //jinak do hran
         end;
         e := W.GetNextElement;
      end;
   end;
   W.Free;
   T2 := GetTickCount;
   VertexesTime := T2 - T1;
   NEdges := Edges.Count;
   NSOEdges := SameOriginEdges.Count;
end;

procedure TCreate_TIN.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreate_TIN.SetEdges(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FEdges),Value);
end;

procedure TCreate_TIN.SetEdgeTolerance(const Value: double);
begin
  FEdgeTolerance := Value;
end;

procedure TCreate_TIN.SetFilterMaxX(const Value: double);
begin
  FFilterMaxX := Value;
end;

procedure TCreate_TIN.SetFilterMaxY(const Value: double);
begin
  FFilterMaxY := Value;
end;

procedure TCreate_TIN.SetFilterMinX(const Value: double);
begin
  FFilterMinX := Value;
end;

procedure TCreate_TIN.SetFilterMinY(const Value: double);
begin
  FFilterMinY := Value;
end;

procedure TCreate_TIN.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TCreate_TIN.SetMinZ(const Value: double);
begin
  FMinZ := Value;
end;

procedure TCreate_TIN.SetNEdges(const Value: integer);
begin
  FNEdges := Value;
end;

procedure TCreate_TIN.SetNRealEdges(const Value: integer);
begin
  FNRealEdges := Value;
end;

procedure TCreate_TIN.SetNSOEdges(const Value: integer);
begin
  FNSOEdges := Value;
end;

procedure TCreate_TIN.SetNVertexes(const Value: integer);
begin
  FNVertexes := Value;
end;

procedure TCreate_TIN.SetPoints(const Value: TEntity);
begin
  FPoints := Value;
end;

procedure TCreate_TIN.SetRealEdges(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FRealEdges),Value);
end;

procedure TCreate_TIN.SetRealEdgesQTree(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FRealEdgesQTree),Value);
end;

procedure TCreate_TIN.SetSameOriginEdges(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FSameOriginEdges),Value);
end;

procedure TCreate_TIN.SetTolerance(const Value: double);
begin
   FTolerance := Value;
end;

procedure TCreate_TIN.SetUseRectFilter(const Value: boolean);
begin
  FUseRectFilter := Value;
end;

procedure TCreate_TIN.SetVertexes(const Value: TRectEntity);
begin
   ExchangeFieldElement(TElement(FVertexes),Value);
end;

procedure TCreate_TIN.SetVertexesQTree(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FVertexesQTree),Value);
end;

procedure TCreate_TIN.SetVertexesTime(const Value: Cardinal);
begin
  FVertexesTime := Value;
end;

procedure TCreate_TIN.SetZ_LIN(const Value: string);
begin
  FZ_LIN := Value;
end;

procedure TCreate_TIN.SetZ_PNT(const Value: string);
begin
  FZ_PNT := Value;
end;

function TCreate_TIN.TestEdge(P1, P2: TPoint3DElement; E1: TEdge3DElement): boolean;
var
   P3,P4: TPoint3DElement;
begin
   result := false;
   if P1.IsMyOwner(E1) or P2.IsMyOwner(E1) then   // jeden z bodu je v hrane, OK
   begin
      result := true;
      exit;
   end;
   if E1.Count < 2 then exit; // to je divny
   P3 := TPoint3DElement(E1.Elements[0]);
   P4 := TPoint3DElement(E1.Elements[1]);
   // je prusecik usecek?
   if not ProtnutyUsecky(P1.X,P1.Y,P2.X,P2.Y,P3.X,P3.Y,P4.X,P4.Y) then
      result := true;    // ne
end;

function TCreate_TIN.TestEdges(P1, P2: TPoint3DElement; E1: TEdge3DElement): boolean;
var
   i,j,pni,eni,nd,ind: integer;
   P3D, P3D1: TPoint3DElement;
   he: TElement;
   E3D: TEdge3DElement;
   e: TElement;
   EE: TEntity;
   W: TQTreeWalker;
   X1,Y1,X2,Y2: double;
   ad,adX,adY,dX,dY: double;
   S: string;
begin
   result := false;
   if (P1 = nil) or (P2 = nil) then
   begin
      if E1 = nil then exit;
      if E1.Count < 2 then exit;
      P3D := TPoint3DElement(E1.Elements[0]);
      P3D1:= TPoint3DElement(E1.Elements[1]);
   end
   else
   begin
      P3D := P1;
      P3D1 := P2;
   end;
   // hrany
   result := true;
   dX := P3D1.X - P3D.X;  // delta X,Y
   dY := P3D1.Y - P3D.Y;
   adX := abs(dX);        // abs hodnoty
   adY := abs(dY);
   ad := adX;
   if adY > adX then
      ad := adY;          // max delta
   nd := Trunc(ad / EdgeTolerance) + 1;  // pocet kroku
   if adX > 0 then
      dX := dX / nd;      // krok X
   if adY > 0 then
      dY := dY / nd;      // krok Y
   ad := ad /nd;          // realna tolerance
   X1 := P3D.X + dX/2;
   Y1 := P3D.Y + dY/2;
   W := TQTreeWalker.Create;
   for ind := 1 to nd do
   begin
      {
      X1 := P3D.X;
      X2 := P3D1.X;
      Order(X1,X2);
      Y1 := P3D.Y;
      Y2 := P3D1.Y;
      Order(Y1,Y2);
      }
      eni := -1;
      he := W.GetFirstElementForRectangle(RealEdgesQTree,X1,Y1,X1,Y1,ad+1);
      X1 := X1 + dX;
      Y1 := Y1 + dY;
      while he <> nil do
      begin
         inc(eni);
         //ProgressText := 'Vertex processing '+IntToStr(i)+' P '+IntToStr(pni)+' E '+IntToStr(eni);
         //Application.ProcessMessages;
         E3D := TEdge3DElement(he);
         if not TestEdge(P3D,P3D1,E3D) then
         begin
            result := false;  // je prusecik s hranou, jedeme dal
            break;
         end;
         he := W.GetNextElement;
      end;
      if not result then
         break;
   end;
   W.Free;
end;

{ TEdge3DElement }

procedure TEdge3DElement.SetLength(const Value: double);
begin
  FLength := Value;
end;

procedure TEdge3DElement.SetSO(const Value: boolean);
begin
  FSO := Value;
end;


{ TSortEdges2D }

function TSortEdges3D.CompareElements(E1, E2: TElement): integer;
begin
   result := -1;
   if TEdge3DElement(E1).Length > TEdge3DElement(E2).Length then
      result := 1
   else
   if TEdge3DElement(E1).Length = TEdge3DElement(E2).Length then
      result := 0;
end;

INITIALIZATION
   RegisterOrbisClass(TPoint3DElement);
   RegisterOrbisClass(TEdge3DElement);
   RegisterOrbisClass(TCreate_TIN);
END.
