unit e_CompareModel;

interface
uses
   e_InhMdl,
   e_Type,
   Elements,
   Tools;

type

TCompareStatus = (csOK,csLeftNotFound,csRightNotFound,csDifference,csClassNotFound);

TCompareNodes = class(TOwnerEntity)
protected
    function  GetName: string; override;
end;

TCompareModel = class;

TCompareNode = class(TOwnerEntity)
private
   FCompareModel: TCompareModel;
   FModel2: TElement;
   FModel1: TElement;
   FStatus: TCompareStatus;
   FErrProps: TStringListElement;
   FNodes: TCompareNodes;
    FIsProgress: boolean;
    FProgressPart: Double;
    FGlobalStatus: TCompareStatus;
   function GetCompareNode(aIndex: integer): TCompareNode;
   function GetErrProps: TStringListElement;
   function GetNodes: TCompareNodes;
   procedure SetCompareModel(const Value: TCompareModel);
   procedure SetCompareNode(aIndex: integer; const Value: TCompareNode);
   procedure SetModel1(const Value: TElement);
   procedure SetModel2(const Value: TElement); virtual;
   procedure SetStatus(const Value: TCompareStatus);
   function GetModel1Name: string;
   function GetModel2Name: string;
    procedure SetIsProgress(const Value: boolean);
    procedure SetProgressPart(const Value: double);
    procedure SetGlobalStatus(const Value: TCompareStatus);
public
   function  FindSameElement(aElement: TElement; aEntity: TEntity): TElement;
   procedure CompareModels;
   procedure CompareProperties;
   function  CompareKeyProperties(aElement1,aElement2: TElement): boolean;
   function  NewNode: TCompareNode; virtual;
   property  CompareModel: TCompareModel read FCompareModel write SetCompareModel;
   property  Node[aIndex: integer]: TCompareNode read GetCompareNode write SetCompareNode;
   property  IsProgress: boolean read FIsProgress write SetIsProgress;
   property  ProgressPart: double read FProgressPart write SetProgressPart;
published
   property Status: TCompareStatus read FStatus write SetStatus;
   property GlobalStatus: TCompareStatus read FGlobalStatus write SetGlobalStatus;
   property Model1: TElement read FModel1 write SetModel1;
   property Model2: TElement read FModel2 write SetModel2;
   property Model1Name: string read GetModel1Name stored false;
   property Model2Name: string read GetModel2Name stored false;
   property ErrProps: TStringListElement read GetErrProps stored false;
   property Nodes: TCompareNodes read GetNodes stored false;
end;

TCompareClassModels = class;
TCompareStops=class;
TCompareType = (ctCompareNames, ctCompareNameAndClass, ctCompareAll);

TCompareModel = class(TCompareNode)
private
   FProgressTool: TTool;
   FCompareThruDifference: boolean;
   FCompareType: TCompareType;
   FActualIndex: double;
   FCompareScript: TCompareClassModels;
   FStopClasses: TCompareStops;
    FLastProgr: integer;
    FDifferences: integer;
    FComment: string;
    FDifferencesText: string;
   procedure SetModel2(const Value: TElement); override;
   procedure SetCompareThruDifference(const Value: boolean);
   procedure SetCompareType(const Value: TCompareType);
   function getTool: TTool;
   procedure SetActualIndex(const Value: double);
   procedure SetCompareScript(const Value: TCompareClassModels);
   function GetCompareScript: TCompareClassModels;
    function GetStopClasses: TCompareStops;
    procedure SetStopClasses(const Value: TCompareStops);
    procedure SetLastProgr(const Value: integer);
    procedure SetDifferences(const Value: integer);
    procedure SetComment(const Value: string);
    procedure SetDifferencesText(const Value: string);
public
   constructor Create; override;
   property    ProgressTool: TTool read getTool;
   property    ActualIndex: Double read FActualIndex write SetActualIndex;
   property    LastProgr: integer read FLastProgr write SetLastProgr;
published
   property CompareScript: TCompareClassModels read GetCompareScript write SetCompareScript;
   property StopClasses: TCompareStops read GetStopClasses write SetStopClasses;
   property CompareType: TCompareType read FCompareType write SetCompareType;
   property CompareThruDifference: boolean read FCompareThruDifference write SetCompareThruDifference;
   property Differences: integer read FDifferences write SetDifferences;
   property Comment: string read FComment write SetComment;
   property DifferencesText: string read FDifferencesText write SetDifferencesText;
end;

TCompareClassModel = class(TInheritanceModel)
private
   FKeyProperties: TStringListElement;
   FProperties: TStringListElement;
   function GetKeyProperties: TStringListElement;
   function GetProperties: TStringListElement;
   procedure SetNextKeyProperty(const Value: string);
   procedure SetNextProperty(const Value: string);
published
   property KeyProperties: TStringListElement read GetKeyProperties stored false;
   property Properties: TStringListElement read GetProperties stored false;
   property NextKeyProperty: string write SetNextKeyProperty stored false;
   property NextProperty: string write SetNextProperty stored false;
end;

TCompareClassModels = class(TInheritanceModelEntity)
private
    FLastClassModel: TCompareClassModel;
    FLastClass: TElementClass;
   function  GetActualClassModel: TCompareClassModel;
    procedure SetLastClass(const Value: TElementClass);
    procedure SetLastClassModel(const Value: TCompareClassModel);
public
   function  GetItemClass: TInheritanceModelClass; override;
   function  CheckClassModel(aElement: TElement): TCompareClassModel;
   property  LastClass: TElementClass read FLastClass write SetLastClass;
   property  LastClassModel: TCompareClassModel read FLastClassModel write SetLastClassModel;
published
   property ActualClassModel: TCompareClassModel read GetActualClassModel stored false;
end;

TCompareStops = class(TCompareClassModels);

implementation

uses
   Forms,
   SysUtils,
   e_Class;

{ TCompareNode }

function TCompareNode.CompareKeyProperties(aElement1,aElement2: TElement): boolean;
var
   i: integer;
   S1,S2,Sn: string;
   KProps: TStringListElement;
begin
   result := false;
   KProps := CompareModel.CompareScript.LastClassModel.KeyProperties;
   for i := 0 to Kprops.LastIndex do
   begin
      Sn := KProps.Names[i];
      S1 := aElement1.GetPropertyStringByName(Sn);
      S2 := aElement2.GetPropertyStringByName(Sn);
      if CompareText(S1,S2) <> 0 then exit;
   end;
   result := true;
end;

procedure TCompareNode.CompareModels;
var
   Progress1: double;
   Progress2: double;
   NPart: double;
   i: integer;
   E: TEntity;
   El: TElement;
   N: TCompareNode;
begin
   Status := csOK;
   if Model1 = nil then
   begin
      Status := csLeftNotFound;
      if Model2 <> nil then Name := Model2.Name;
      exit;
   end;
   Progress1 := CompareModel.ActualIndex;
   Progress2 := Progress1 + ProgressPart;
   Name := Model1.Name;
   if Model2 = nil then
   begin
      Status := csRightNotFound;
      CompareModel.ActualIndex := Progress2;
      exit;
   end;
   if CompareModel.CompareType = ctCompareAll then
      CompareProperties;
   if (Status <> csOK) and not CompareModel.CompareThruDifference then
   begin

      CompareModel.Differences := CompareModel.Differences + 1;
      CompareModel.ActualIndex := Progress2;
      exit;
   end;
   if CompareModel.StopClasses.CheckClassModel(Model1) <> nil then
   begin
      CompareModel.ActualIndex := Progress2;
      exit;
   end;
   if not (Model1 is TEntity) then
   begin
      CompareModel.ActualIndex := Progress2;
      exit;
   end;
   E := TEntity(Model1);
   if E.NoElements then
   begin
      CompareModel.ActualIndex := Progress2;
      exit;
   end;
   NPart := ProgressPart / e.Count;
   for i := 0 to E.LastIndex do
   begin
      N := NewNode;
      N.ProgressPart := NPart;
      N.Model1 := E.Elements[i];
      N.Model2 := FindSameElement(N.Model1,TEntity(Model2));
      N.CompareModels;
      if (N.Status <> csOK) or (N.GlobalStatus <> csOK) then
         GlobalStatus := csDifference;
   end;
   CompareModel.ActualIndex := Progress2;
   i := Round(CompareModel.ActualIndex);
   if i > CompareModel.LastProgr then
   begin
      CompareModel.LastProgr := i;
      CompareModel.ProgressTool.ProgressText := CompareModel.DifferencesText + ' '+IntToStr(COmpareModel.Differences);
      CompareModel.ProgressTool.Progress := i;
      Application.ProcessMessages;
   end;
   if not (Model2 is TEntity) then exit;
   E := TEntity(Model2);
   if E.NoElements then exit;
   for i := 0 to E.LastIndex do
   begin
      El := E.Elements[i];
      if El.FindOwnerClass(TCompareNode) <> nil then Continue;
      N := NewNode;
      N.Model2 := El;
      N.CompareModels;
   end;
end;

procedure TCompareNode.CompareProperties;
var
   i,res: integer;
   S1,S2,Sn: string;
   aProps: TStringListElement;
begin
   if CompareModel.CompareScript.CheckClassModel(Model1)= nil then
   begin
      Status := csClassNotFound;
      exit;
   end;
   aProps := CompareModel.CompareScript.LastClassModel.Properties;
   for i := 0 to aProps.LastIndex do
   begin
      Sn := aProps.Names[i];
      S1 := Model1.GetPropertyStringByName(Sn);
      S2 := Model2.GetPropertyStringByName(Sn);
      res := CompareText(S1,S2);
      if res <> 0 then
      begin
         Status := csDifference;
         Errprops.AddNameString(Sn,IntToStr(res));
         exit;
      end;
   end;
   Status := csOK;
end;

function TCompareNode.FindSameElement(aElement: TElement;
  aEntity: TEntity): TElement;
var
   i: integer;
   E: TElement;
begin
   result := nil;
   if CompareModel.CompareType = ctCompareNames then
   begin
      result := aEntity.FindElementName(aElement.Name);
      exit;
   end
   else if CompareModel.CompareType = ctCompareAll then
   begin
      if CompareModel.CompareScript.CheckClassModel(aElement) = nil then
      begin
         Status := csClassNotFound;
         exit;
      end;
   end;
   for i := 0 to aEntity.LastIndex do
   begin
      E := aEntity.Elements[i];
      if CompareModel.CompareType = ctCompareNameAndClass then
      begin
         if (CompareText(E.Name,aElement.Name) = 0) and (E.ClassType = aElement.ClassType) then
         begin
            result := E;
            exit;
         end;
      end
      else if CompareModel.CompareType = ctCompareAll then
      begin
         if not (E.ClassType = aElement.ClassType) then Continue;
         if CompareKeyProperties(aElement,E) then
         begin
            result := E;
            exit;
         end;
      end;
   end;
end;

function TCompareNode.GetCompareNode(aIndex: integer): TCompareNode;
begin
   //result := nil;
   //if NoNodes then exit;
   result := TCompareNode(Nodes.Elements[aIndex]);
end;

function TCompareNode.GetErrProps: TStringListElement;
begin
   if FErrProps = nil then
   begin
      FErrProps := TStringListElement(FindElementClass(TStringListElement));
      if FErrProps = nil then
      begin
         FErrProps := TStringListElement.Create;
         FErrProps.Name := 'Differences';
         AddElement(FErrProps);
      end;
   end;
   result := FErrProps;
end;

function TCompareNode.GetModel1Name: string;
begin
   result := '';
   if Model1 <> nil then result := Model1.Name;
end;

function TCompareNode.GetModel2Name: string;
begin
   result := '';
   if Model2 <> nil then result := Model2.Name;
end;

function TCompareNode.GetNodes: TCompareNodes;
begin
   if FNodes = nil then
   begin
      FNodes := TCompareNodes(FindElementClass(TCompareNodes));
      if FNodes = nil then
      begin
         FNodes := TCompareNodes.Create;
         AddElement(FNodes);
      end;
   end;
   result := FNodes;
end;

function TCompareNode.NewNode: TCompareNode;
begin
   result := TCompareNode.Create;
   result.CompareModel := CompareModel;
   Nodes.AddElement(result);
end;


procedure TCompareNode.SetCompareModel(const Value: TCompareModel);
begin
   FCompareModel := Value;
end;

procedure TCompareNode.SetCompareNode(aIndex: integer;
  const Value: TCompareNode);
begin
   Nodes.InsertElement(aIndex, Value);
end;


procedure TCompareNode.SetGlobalStatus(const Value: TCompareStatus);
begin
  FGlobalStatus := Value;
end;

procedure TCompareNode.SetIsProgress(const Value: boolean);
begin
   FIsProgress := Value;
end;

procedure TCompareNode.SetModel1(const Value: TElement);
begin
  FModel1 := Value;
  AddElement(Fmodel1);
end;

procedure TCompareNode.SetModel2(const Value: TElement);
begin
  FModel2 := Value;
  AddElement(Fmodel2);
end;



procedure TCompareNode.SetProgressPart(const Value: double);
begin
  FProgressPart := Value;
end;

procedure TCompareNode.SetStatus(const Value: TCompareStatus);
begin
  FStatus := Value;
end;

{ TCompareNodes }

function TCompareNodes.GetName: string;
begin
   result := 'NODES';
end;


{ TCompareModel }

constructor TCompareModel.Create;
begin
   inherited Create;
   CompareThruDifference := false;
   CompareType := ctCompareNames;
   CompareModel := Self;
end;

function TCompareModel.GetCompareScript: TCompareClassModels;
begin
   if FCompareScript = nil then
   begin
      FCompareScript := TCompareClassModels(FindElementClass(TCompareClassModels));
      if FCompareScript = nil then
      begin
         FCompareScript := TcompareClassmodels.Create;
         FCompareScript.Name := 'Compare Script';
         AddElement(FCompareScript);
      end;
   end;
   result := FCompareScript;
end;

function TCompareModel.GetStopClasses: TCompareStops;
begin
   if FStopClasses = nil then
   begin
      FStopClasses := TCompareStops(FindElementClass(TCompareStops));
      if FStopClasses = nil then
      begin
         FStopClasses := TCompareStops.Create;
         FStopClasses.Name := 'STOPC';
         AddElement(FStopClasses);
      end;
   end;
   result := FStopClasses;
end;

function TCompareModel.getTool: TTool;
begin
   if FProgressTool = nil then
   begin
      FProgressTool := TTool.Create;
      FProgressTool.Name := 'Progress';
      AddElement(FProgressTool);
   end;
   result := FProgressTool;
end;

procedure TCompareModel.SetActualIndex(const Value: double);
begin
  FActualIndex := Value;
end;

procedure TCompareModel.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCompareModel.SetCompareScript(const Value: TCompareClassModels);
begin
   ExchangeFieldElement(TElement(FCompareScript),Value);
end;

procedure TCompareModel.SetCompareThruDifference(const Value: boolean);
begin
  FCompareThruDifference := Value;
end;

procedure TCompareModel.SetCompareType(const Value: TCompareType);
begin
  FCompareType := Value;
end;

procedure TCompareModel.SetDifferences(const Value: integer);
begin
  FDifferences := Value;
end;

procedure TCompareModel.SetDifferencesText(const Value: string);
begin
  FDifferencesText := Value;
end;

procedure TCompareModel.SetLastProgr(const Value: integer);
begin
  FLastProgr := Value;
end;

procedure TCompareModel.SetModel2(const Value: TElement);
var
   i: integer;
begin
   i := 0;
   LastProgr := 0;
   Differences := 0;
   inherited SetModel2(Value);
   ProgressPart := 100;
   if Comment = '' then
      Comment := 'Comparing elements: ';
   if DifferencesText = '' then
      DifferencesText := 'Difs: ';
   if Model1 is TEntity then
   if Model1 <> nil then i := 100
                    else i := 0;
   ProgressTool.StartProgress(Comment, 0, i);
   CompareModels;
   ProgressTool.EndProgress;
   Application.ProcessMessages;
end;

procedure TCompareModel.SetStopClasses(const Value:  TCompareStops);
begin
   ExchangeFieldElement(TElement(FStopClasses),Value);
end;

{ TCompareClassModel }

function TCompareClassModel.GetKeyProperties: TStringListElement;
var
   S: string;
begin
   if FKeyProperties = nil then
   begin
      S := 'Key Properties';
      FKeyProperties := TStringListElement(FindElementName(S));
      if FKeyProperties = nil then
      begin
         FKeyProperties := TStringListElement.Create;
         FKeyProperties.Name := S;
         AddElement(FKeyProperties);
      end;
   end;
   result := FKeyProperties;
end;

function TCompareClassModel.GetProperties: TStringListElement;
var
   S: string;
begin
   if FProperties = nil then
   begin
      S := 'Properties';
      FProperties := TStringListElement(FindElementName(S));
      if FProperties = nil then
      begin
         FProperties := TStringListElement.Create;
         FProperties.Name := S;
         AddElement(FProperties);
      end;
   end;
   result := FProperties;
end;

procedure TCompareClassModel.SetNextKeyProperty(const Value: string);
begin
   KeyProperties.AddNameString(Value,'');
end;

procedure TCompareClassModel.SetNextProperty(const Value: string);
begin
   Properties.AddNameString(Value,'');
end;

{ TCompareClassModels }

function TCompareClassModels.CheckClassModel(
  aElement: TElement): TCompareClassModel;
begin
   if not (aElement.ClassType = LastClass) then
   begin
      LastClass := TElementClass(aElement.ClassType);
      LastClassModel := TCompareClassModel(GetItemForClass(TElementClass(aElement.ClassType)));
   end;
   result := LastClassModel;
end;

function TCompareClassModels.GetActualClassModel: TCompareClassModel;
begin
   result := TCompareClassModel(ActualItem);
end;

function TCompareClassModels.GetItemClass: TInheritanceModelClass;
begin
   result := TCompareClassModel;
end;

procedure TCompareClassModels.SetLastClass(const Value: TElementClass);
begin
  FLastClass := Value;
end;

procedure TCompareClassModels.SetLastClassModel(
  const Value: TCompareClassModel);
begin
  FLastClassModel := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCompareModel);
   RegisterOrbisClass(TCompareNode);
   RegisterOrbisClass(TCompareNodes);
END.
