unit e_Terrain;

interface
uses
   e_GeometryElement;

type

TTerrainGeometryElement = class(TPropertyGeometryElement)
private
    FOrientation: double;
    FMaxZ: double;
    FSlope: double;
    FMinZ: double;
    FColor: integer;
    procedure DoCalculate(const Value: boolean);
    procedure SetColor(const Value: integer);
    procedure SetMaxZ(const Value: double);
    procedure SetMinZ(const Value: double);
    procedure SetOrientation(const Value: double);
    procedure SetSlope(const Value: double);
    function GetOrientation_DEG: double;
    function GetSlope_DEG: double;
    procedure SetOrientation_DEG(const Value: double);
    procedure SetSlope_DEG(const Value: double);
    function GetZ: double;
published
   property Color: integer read FColor write SetColor;
   property Calculate: boolean write DoCalculate stored false;
   property Slope: double read FSlope write SetSlope;
   property Orientation: double read FOrientation write SetOrientation;
   property Slope_DEG: double read GetSlope_DEG write SetSlope_DEG stored false;
   property Orientation_DEG: double read GetOrientation_DEG write SetOrientation_DEG stored false;
   property MinZ: double read FMinZ write SetMinZ;
   property MaxZ: double read FMaxZ write SetMaxZ;
   property Z: double read GetZ stored false;
end;

implementation
uses
   Math,
   MathLib,
   e_Class,
   e_Geometry,
   g_Elems;

{ TTerrainGeometryElement }

procedure TTerrainGeometryElement.DoCalculate(const Value: boolean);
var
   C1,C2,C3,CCC: T3DCoordinate;
   C: T2DCoordinate;
   X2,Y2: double;   // skutecne souradnice posunuteho bodu C3 (druhy bod ridici usecky)
   pX,pY: double;   // pata kolmice bodu C1 na ridici usecku
   d,dz,dz1: double;    // delty Z  a tak
   AA,BB: TDoubleCoord;
   A: TAreaElement;
   G: TGeometry;
   i,cc: integer;
begin
   Slope := 0;
   Orientation := 0;
   MinZ := 0;
   MaxZ := 0;
   G := Geometry;
   if G = nil then exit;
   if not (G is TAreaElement) then exit;
   A := TAreaElement(G);
   cc := A.CoordsCount;
   if cc < 3 then exit;
   C := A.GetOrientedCoordElement(0,true);
   if not (C is T3DCoordinate) then exit;
   C1 := T3DCoordinate(C);
   C := A.GetOrientedCoordElement(1,true);
   if not (C is T3DCoordinate) then exit;
   C2 := T3DCoordinate(C);
   C := A.GetOrientedCoordElement(2,true);
   if not (C is T3DCoordinate) then exit;
   C3 := T3DCoordinate(C);
   //serazeni podle Z sestupne
   if SameValue(C1.Z, C2.Z, 0.0001) and SameValue(C1.Z, C3.Z, 0.0001) then // placka
   begin
      MinZ := C1.Z;
      MaxZ := MinZ;
      exit;
   end;
   CCC := C2;
   if CCC.Z > C1.Z then
   begin
      C2 := C1;
      C1 := CCC;
   end;
   CCC := C3;
   if CCC.Z > C1.Z then
   begin
      C3 := C2;
      C2 := C1;
      C1 := CCC;
   end
   else
   if CCC.Z > C2.Z then
   begin
      C3 := C2;
      C2 := CCC;
   end;
   MaxZ := C1.Z;
   MinZ := C3.Z;
   // Do C1 samostatny bod, C2 a C3 ridici usecka nebo jeji zaklad
   // C1 by mel mit vetsi odstup deltaZ od C2 nez C3
   if abs(C3.Z - C2.Z) > abs(C1.Z - C2.Z) then
   begin
      CCC := C3;
      C3  := C1;
      C1  := CCC;
   end;
   if SameValue(C2.Z, C3.Z, 0.0001) then  // C2 a C3 tvori primo ridici usecku
   begin
      X2 := C3.X;
      Y2 := C3.Y;
   end
   else                                   // C2 a C3 maj ruzne Z ridici usecku je treba najit
   begin
      d := Distance(C1.X,C1.Y,C3.X,C3.Y);
      dz := abs(C1.Z-C3.Z);
      dz1 := abs(C2.Z-C3.Z);
      GiveAbscPointByLength(C3.X, C3.Y, C1.X, C1.Y, d/dz*dz1,0,X2,Y2);
   end;
   DejPatuKolmice(C2.X, C2.Y, X2, Y2, C1.X, C1.Y, PX, PY);
   d := Distance(C1.X,C1.Y,PX,PY);
   dz := abs(C1.Z-C2.Z);
   if C1.Z > C2.Z then
   begin
      AA.X := C1.X;
      AA.Y := C1.Y;
      BB.X := PX;
      BB.Y := PY;
   end
   else
   begin
      AA.X := PX;
      AA.Y := PY;
      BB.X := C1.X;
      BB.Y := C1.Y;
   end;
   Slope := ATan2M(d,dz);
   XYToPolar(AA,BB,d,dz);
   if dz < 0 then
      dz := 2*PI+dz;
   Orientation := dz;
end;

function TTerrainGeometryElement.GetOrientation_DEG: double;
begin
   result := RadToDeg(Orientation);
end;

function TTerrainGeometryElement.GetSlope_DEG: double;
begin
   result := RadToDeg(Slope);
end;

function TTerrainGeometryElement.GetZ: double;
begin
   result := (MinZ + MaxZ)/2;
end;

procedure TTerrainGeometryElement.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TTerrainGeometryElement.SetMaxZ(const Value: double);
begin
  FMaxZ := Value;
end;

procedure TTerrainGeometryElement.SetMinZ(const Value: double);
begin
  FMinZ := Value;
end;

procedure TTerrainGeometryElement.SetOrientation(const Value: double);
begin
  FOrientation := Value;
end;

procedure TTerrainGeometryElement.SetOrientation_DEG(const Value: double);
begin
   Orientation := DegToRad(Value);
end;

procedure TTerrainGeometryElement.SetSlope(const Value: double);
begin
  FSlope := Value;
end;

procedure TTerrainGeometryElement.SetSlope_DEG(const Value: double);
begin
   Slope := DegToRad(Value);
end;

INITIALIZATION
   RegisterOrbisClass(TTerrainGeometryElement);
END.
