unit i_TileLayer;

interface
uses
   Classes,
   Graphics,
   Elements,
   e_Proje,
   e_Page,
   i_Basic;

type
TGis_OrbisTilesLayerInterpret = class(TRectInterpret)
private
    FProj: TPageProjection;
    FPage: T_Page;
    FTiles: TStringList;
    FColor: TColor;
    FUseColor: boolean;
    procedure SetTiles(const Value: TStringList);
    procedure SetColor(const Value: TColor);
    procedure SetUseColor(const Value: boolean);
protected
   function  IsItsClass(AObject: TObject): boolean; override;
   function  QGoInside(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   Property  Tiles: TStringList read FTiles write SetTiles;
   property  Proj: TPageProjection read FProj;
   property  Page: T_Page read FPage;
public
   constructor Create; override;
   destructor  Destroy; override;
published
   property Color: TColor read FColor write SetColor;
   property UseColor: boolean read FUseColor write SetUseColor;
end;

implementation
uses
   Forms,
   SysUtils,
   Types,
   vcl.imaging.pngimage,
   G_Elems,
   e_Class,
   e_Device,
   e_TileLayer;

{ TGis_OrbisTilesLayerInterpret }

constructor TGis_OrbisTilesLayerInterpret.Create;
begin
  inherited;
  Tiles := TStringList.Create;
  Tiles.Sorted := true;
  FProj := TPageProjection.Create;
  AddElement(FProj);
  FPage := T_Page.Create;
  AddElement(FPage);
end;

destructor TGis_OrbisTilesLayerInterpret.Destroy;
begin
   if FTiles <> nil then
      FTiles.Free;
   inherited;
end;


function TGis_OrbisTilesLayerInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := aObject is TGis_OrbisTilesLayer;
end;

procedure TGis_OrbisTilesLayerInterpret.Process(Element: TElement);
var
   i,Xi,Yi,xi1,xi2,yi1,yi2: integer;
   X1,Y1,X2,Y2: double;
   Revi: integer;
   TileIndex: integer;
   S: string;
   bm1,bms: TBitmap;//TGraphic;
   aCanvas: TCanvas;
   DestRect,SrcRect,RR: TRect;
   dX: Single;
   MapRec: TGis_OrbisTilesLayer;
   MX: TGis_OrbisTilesMatrix;
   Picture: TPicture;
   aGraphic: TGraphic;
   SrcModel: TDoubleRect;
   IsPNG: boolean;
   SrcPng: TPngImage;
   bmx,bmy: integer;
   aColor: TColor;
//   R: TDoubleRect;
begin
   if Device = nil then exit;
   if not(Element is TGis_OrbisTilesLayer) then exit;
   MapRec := TGis_OrbisTilesLayer(ELement);
   if MapRec.Error <> 0 then exit; // neco je spatne .... rychle pryc
    Application.ProcessMessages;
    // nalezeni spravneho Tilematrix pro projekci
    // Barva
    aColor := clBlack;
    if UseColor then
       aColor := Color;
    MX := nil;
    IsPNG := UpperCase(MapRec.Ext) = 'PNG';
    for i := 0 to MapRec.LastIndex do
    begin
       MX := MapRec.TileMatrix[i];
       if MX = nil then exit;
       dx := Device.Projection.GetPageLength(MX.TileWidth);
       if dx <= 500 then break;
    end;
    if MX = nil then exit;
    X1 := MX.MinX;
    if x1 < MinX then
    begin
       xi1 := Trunc((Minx - X1) / (MX.TileWidth - MX.OverX));
       X1 := X1 + (xi1 * (MX.TileWidth - MX.OverX));
    end
    else
    begin
       xi1 := 0;
    end;
    X2 := X1 + MX.TileWidth;
    for Xi := xi1 to MX.MatrixWidth - 1 do
    begin
       Y1 := MX.MinY;
       if y1 < MinY then
       begin
          yi1 := Trunc((Miny - y1) / (MX.TileHeight - MX.OverY));
          Y1 := Y1 + (yi1 * (MX.TileHeight - MX.OverY));
       end
       else
       begin
          yi1 := 0;
       end;
       Y2 := Y1 + MX.TileHeight;
       for Yi := yi1 to MX.MatrixHeight - 1 do
       begin
          if (X1 < MaxX) and (Y1 < MaxY) and
             (X2 > MinX) and (Y2 > MinY)then
          begin
             S := MapRec.MapDir + MX.Matrix_ID + '_'+IntToStr(Xi)+'-'+IntToStr(Yi)+'.'+MapRec.Ext;
             if FileExists(S) then
             begin
                try
                Picture := TPicture.Create;
                Picture.LoadFromFile(S);
                aGraphic := Picture.Graphic;
                Application.ProcessMessages;
                bms := nil;
                if IsPNG then
                begin
                   SrcPng := TPngImage(aGraphic);
                   bms := TBitmap.Create;
                   bms.Width := SrcPng.Width;
                   bms.Height := SrcPng.Height;
                   bms.Canvas.Brush.Color:= aColor;
                   bms.Canvas.Brush.Style:= bsSolid;
                   RR := System.Classes.Rect(0,0,bms.Width,bms.Height);
                   bms.Canvas.FillRect(RR);
                   bms.Canvas.CopyMode := cmSrcPaint;
                   bms.Canvas.CopyRect(RR,SrcPng.Canvas,RR);
                   bms.Canvas.CopyMode := cmSrcCopy;
//                   bms.Transparent := true;
//                   bms.TransparentColor := SrcPng.TransparentColor;
                   aGraphic := bms;
//                   for bmx := 0 to SrcPng.Width - 1 do
//                   begin
//                       for bmy := 0 to SrcPng.Height - 1 do
//                       begin
//                          if SrcPng.Pixels[bmx,bmy] <> SrcPng.TransparentColor then
//                             SrcPng.Pixels[bmx,bmy] := $000000FF;
//                       end;
//                   end;
                end;

                // CLipping tad nejak ... jaksi ... jakoze ... neni! ... ale bude asi
                // kontrola orezu
                SrcRect  := System.Classes.Rect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
                DestRect := System.Classes.Rect(Device.Projection.GetPageX(X1),Device.Projection.GetPageY(Y2),Device.Projection.GetPageX(X2),Device.Projection.GetPageY(Y1));
                bm1 := nil;
                if (X1 < MinX) or (X2 > MaxX) or (Y1 < MinY) or (Y2 > MaxY)   then
                begin
//                   Page.SetRect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
                   Page.SetRect(0,0,Picture.Graphic.Width,Picture.Graphic.Height);
                   Page.SetResolution(Page.Width / MX.TileWidth,Page.Height / MX.TileHeight);
                   SrcModel.MinX := X1;
                   SrcModel.MinY := Y1;
                   SrcModel.MaxX := X2;
                   SrcModel.MaxY := Y2;
                   Proj.ChangeProjectionByRect(SrcModel, Page);
                   if (X1  < MinX) then
                   begin
                      SrcRect.Left := Proj.GetPageX(MinX);
                      DestRect.Left := Device.Projection.GetPageX(MinX);
                   end;
                   if (X2 > MaxX) then
                   begin
                      SrcRect.Right := Proj.GetPageX(MaxX);
                      DestRect.Right := Device.Projection.GetPageX(MaxX);
                   end;
                   if (Y1 < MinY) then
                   begin
                      SrcRect.Bottom := Proj.GetPageY(MinY);
                      DestRect.Bottom := Device.Projection.GetPageY(MinY);
                   end;
                   if (Y2 > MaxY) then
                   begin
                      SrcRect.Top := Proj.GetPageY(MaxY);
                      DestRect.Top := Device.Projection.GetPageY(MaxY);
                   end;
//                   if IsPNG then
//                   begin
//                      bm1 := TPngImage.CreateBlank(SrcPng.Header.ColorType,SrcPng.Header.BitDepth,SrcRect.Right - SrcRect.Left,SrcRect.Bottom - SrcRect.Top);
//                      TPngImage(bm1).Transparent := SrcPng.Transparent;
////                      TPngImage(bm1).TransparencyMode := SrcPng.TransparencyMode;
//                      TPngImage(bm1).TransparentColor := SrcPng.TransparentColor;
//                      aCanvas := TPngImage(bm1).Canvas;
//                      aCanvas.Brush.Color:= SrcPng.TransparentColor;
//                      aCanvas.Brush.Style:= bsSolid;
//                      aCanvas.FillRect(System.Classes.Rect(0,0,bm1.Width,bm1.Height));
//                   end
//                   else
//                   begin
//                      bm1 := TBitmap.Create;
//                      Bm1.Width := SrcRect.Right - SrcRect.Left;
//                      Bm1.Height := SrcRect.Bottom - SrcRect.Top;
//                      aCanvas := TBitmap(bm1).Canvas;
//                   end;
                      bm1 := TBitmap.Create;
                      if IsPNG then
                      begin
                         bm1.Transparent := true;
                         bm1.TransparentColor := SrcPng.TransparentColor;
                      end;
                      Bm1.Width := SrcRect.Right - SrcRect.Left;
                      Bm1.Height := SrcRect.Bottom - SrcRect.Top;
                      aCanvas := TBitmap(bm1).Canvas;
                   aCanvas.Draw(0 - SrcRect.Left,0 - SrcRect.Top,aGraphic);
                   aGraphic := Bm1;
                end
                else
                if IsPNG then
                begin
                   bms.Transparent := true;
                   bms.TransparentColor := SrcPng.TransparentColor;
                end;


                Device.StretchDrawDevice(DestRect.Left,DestRect.Top,DestRect.Width,DestRect.Height,0,0,aGraphic.Width,aGraphic.Height,aGraphic);
//                Device.StretchDrawDevice(DestRect.Left,DestRect.Top,DestRect.Width,DestRect.Height,SrcRect.Left,SrcRect.Top,SrcRect.Width,SrcRect.Height,Picture.Graphic);
                finally
                   //G.Dispose;
                   Picture.Free;
                   if Assigned(bm1)then
                      FreeAndNil(bm1);
                   if IsPNG and Assigned(bms) then
                      FreeAndNil(bms);
                end;
                Application.ProcessMessages;
             end;
          end;
          Y1 := Y2 - MX.OverY;
          if Y1 > MaxY then Break;
          Y2 := Y1 + MX.TileHeight;
       end;
       X1 := X2 - MX.OverX;
       if X1 > MaxX then Break;
       X2 := X1 + MX.TileWidth;
    end;
end;

function TGis_OrbisTilesLayerInterpret.QGoInside(Element: TElement): boolean;
begin
   result := false;
end;

procedure TGis_OrbisTilesLayerInterpret.SetColor(const Value: TColor);
begin
  FColor := Value;
end;

procedure TGis_OrbisTilesLayerInterpret.SetTiles(const Value: TStringList);
begin
   FTiles := Value;
end;

procedure TGis_OrbisTilesLayerInterpret.SetUseColor(const Value: boolean);
begin
  FUseColor := Value;
end;

initialization
   RegisterOrbisClass(TGis_OrbisTilesLayerInterpret);
end.
