unit t_LHCFinderFrame;

interface
uses
   Elements,
   m_Intera,
   f_LHCFinderForm,
   r_LHCFinderFrame,
   t_ForestModelSelector,
   t_ParcelModelSelector,
   t_OrbisFrame;

type
TLHCFinderFrame = class(TOrbisFrame)
  private
    FFMSelector: TForestModelSelector;
    FPMSelector: TParcelModelSelector;
    FForm: TLHCFinder_Form;
    FFormLeft: integer;
    FFormTop: integer;
    FFindByZAK: boolean;
    FPAR: boolean;
    function GetLFFrame: TLHCFinder_Frame;
    procedure SetForm(const Value: TLHCFinder_Form);
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
    procedure SetFormLeft(const Value: integer);
    procedure SetFormTop(const Value: integer);
    function GetFMSelector: TForestModelSelector;
    procedure SetFindByZAK(const Value: boolean);
    function GetPMSelector: TParcelModelSelector;
    procedure SetPAR(const Value: boolean);
public
   constructor Create; override;
   destructor  Destroy; override;
   procedure   CreateFrame; override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   on_tm_Hide(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property    LFFrame: TLHCFinder_Frame read GetLFFrame;
   property    Form: TLHCFinder_Form read FForm write SetForm;
   property    FMSelector: TForestModelSelector read GetFMSelector;
   property    PMSelector: TParcelModelSelector read GetPMSelector;
published
   property    Active: boolean Read GetActive write SetActive;
   property    FormLeft: integer read FFormLeft write SetFormLeft;
   property    FormTop: integer read FFormTop write SetFormTop;
   property    FindByZAK: boolean read FFindByZAK write SetFindByZAK;
   property    PAR: boolean read FPAR write SetPAR;
end;

implementation
uses
   Forms,
   m_Orbis,
   e_Class;

{ TLHCFinderFrame }

constructor TLHCFinderFrame.Create;
begin
  inherited;

end;

procedure TLHCFinderFrame.CreateFrame;
begin
   aFrame := TLHCFinder_Frame.Create(nil);
   LFFrame.Tool := Self;
end;

destructor TLHCFinderFrame.Destroy;
begin
  if Active then
     Form.Free;
  inherited;
end;

function TLHCFinderFrame.GetActive: boolean;
begin
   result := Form <> nil;
end;

function TLHCFinderFrame.GetFMSelector: TForestModelSelector;
begin
   if FFMSelector = nil then
   begin
      FFMSelector := TForestModelSelector.Create;
      if not FindByZAK then
         FFMSelector.FrameClassName := 'Tgis_PorostniSkupina'
      else
         FFMSelector.FrameClassName := 'Tgis_Porost';
      FFMSelector.FindInput;
      AddElement(FFMSelector);
   end;
   result := FFMSelector;
end;

function TLHCFinderFrame.GetLFFrame: TLHCFinder_Frame;
begin
   result := TLHCFinder_Frame(aFrame);
end;

function TLHCFinderFrame.GetPMSelector: TParcelModelSelector;
begin
   if FPMSelector = nil then
   begin
      FPMSelector := TParcelModelSelector.Create;
      {
      if not FindByZAK then
         FFMSelector.FrameClassName := 'Tgis_PorostniSkupina'
      else
         FFMSelector.FrameClassName := 'Tgis_Porost';
      }
      FPMSelector.FindInput;
      FPMSelector.SelectByParcely := true;
      AddElement(FPMSelector);
   end;
   result := FPMSelector;
end;

procedure TLHCFinderFrame.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_Hide:   on_tm_Hide(aMessage,Handled);
   end;
end;

procedure TLHCFinderFrame.on_tm_Hide(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   if aMessage.Sender = Self then
   begin
      Active := false;
      Handled := true;
   end;
end;

procedure TLHCFinderFrame.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   aX,aY: double;
begin
   if not Active then exit;
   aX:= Device.Projection.GetModelX(aMessage.x);
   aY:= Device.Projection.GetModelY(aMessage.y);
   if PAR then
   begin
      if PMSelector.Busy then exit;
      PMSelector.X := aX;
      PMSelector.Y := aY;
      PMSelector.DoIt;
      if PMSelector.ActualParCast <> nil then
      begin
         Form.l_KU.Caption := PMSelector.ActualParCast.GetPropertyStringByName('KU_KOD');
         Form.l_SKP.Caption := PMSelector.ActualParCast.GetPropertyStringByName('Skupar_Cis');
         Form.l_PAR.Caption := PMSelector.ActualParcast.Name;
      end
      else
      if PMSelector.ActualKatastr <> nil then
      begin
         Form.l_KU.Caption := PMSelector.ActualKatastr.Name;
         Form.l_SKP.Caption := PMSelector.ActualKatastr.Nazev;
         Form.l_PAR.Caption := '';
      end
      else
      begin
         Form.l_KU.Caption := '?';
         Form.l_SKP.Caption := '?';
         Form.l_PAR.Caption := '?';
      end;
   end
   else
   begin
      if FMSelector.Busy then exit;
      FMSelector.X := aX;
      FMSelector.Y := aY;
      FMSelector.DoIt;
      if FMSelector.ActualFrame <> nil then
      begin
         Form.l_LHC.Caption := FMSelector.ActualFrame.GetPropertyStringByName('LHC');
         Form.l_LS.Caption := FMSelector.ActualFrame.GetPropertyStringByName('LS');
         Form.l_Oddeleni.Caption := FMSelector.ActualFrame.GetPropertyStringByName('Oddeleni');
         Form.l_Dilec.Caption := FMSelector.ActualFrame.GetPropertyStringByName('Dilec');
         Form.l_Porost.Caption := FMSelector.ActualFrame.GetPropertyStringByName('Porost');
         Form.l_Skupina.Caption := FMSelector.ActualFrame.GetPropertyStringByName('Skupina');
      end
      else
      begin
         Form.l_LHC.Caption := '?';
         Form.l_LS.Caption := '?';
         Form.l_Oddeleni.Caption := '?';
         Form.l_Dilec.Caption := '?';
         Form.l_Porost.Caption := '?';
         Form.l_Skupina.Caption := '?';
      end;
   end;
end;

procedure TLHCFinderFrame.SetActive(const Value: boolean);
begin
   if Value then
   begin
      if Form = nil then
         Form := TLHCFinder_Form.Create(Application);
      PAR := false;
      Form.Left := FormLeft;
      Form.Top := FormTop;
      Form.Tool := Self;
      Form.Visible := true;
      LFFrame.But.Down := true;
      FMSelector.FindInput;
      if FMSelector.ForestModel <> nil then
         FMSelector.ForestModel.UpdateAll := true;
      Form.ForestModel := FMSelector.ForestModel;
      PMSelector.FindInput;
      PMSelector.SelectByParcely := true;
      if PMSelector.ParcelModel <> nil then
         PMSelector.ParcelModel.UpdateAll := true;
      Form.ParcelModel := PMSelector.ParcelModel;
      if not FMSelector.ForestModel.NoElements then
      begin
         Form.ed_LHC.Text := FMSelector.ForestModel.Elements[0].Name;
         Form.ed_LHCClick(nil);
      end;
   end
   else
   begin
      if Form <> nil then
      begin
         FormLeft := Form.Left;
         FormTop := Form.Top;
         Form.Free;
      end;
      Form := nil;
      LFFrame.But.Down := false;
   end;
end;

procedure TLHCFinderFrame.SetFindByZAK(const Value: boolean);
begin
  FFindByZAK := Value;
  if not Value then
     FFMSelector.FrameClassName := 'Tgis_PorostniSkupina'
  else
     FFMSelector.FrameClassName := 'Tgis_Porost';
end;

procedure TLHCFinderFrame.SetForm(const Value: TLHCFinder_Form);
begin
  FForm := Value;
end;

procedure TLHCFinderFrame.SetFormLeft(const Value: integer);
begin
  FFormLeft := Value;
end;

procedure TLHCFinderFrame.SetFormTop(const Value: integer);
begin
  FFormTop := Value;
end;

procedure TLHCFinderFrame.SetPAR(const Value: boolean);
begin
  FPAR := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TLHCFinderFrame);
END.
