unit T_MapManagerFrame;

interface
uses
   e_ThematicMap,
   r_MapManagerFrame,
   T_OrbisFrame;
type

TMapManagerFrame = class(TOrbisFrame)
private
    FMaps: TThematicMaps;
    FMapsSetting: boolean;
    function GetMMFrame: TMapManager_Frame;
    procedure SetMaps(const Value: TThematicMaps);
    procedure SetMapChange(const Value: boolean);
    procedure SetSetting(const Value: boolean);
    procedure SetMapsSetting(const Value: boolean);
public
   procedure CreateFrame; override;
   property MMFrame: TMapManager_Frame read GetMMFrame;
   property MapsSetting: boolean read FMapsSetting write SetMapsSetting;
published
   property Maps: TThematicMaps read FMaps write SetMaps;
   property MapChange: boolean write SetMapChange stored false;
   property Setting: boolean write SetSetting stored false;
end;

implementation
uses
   Forms,
   SysUtils,
   Elements,
   e_Class,
   m_Orbis,
   t_ThematicMapsEditor;

{ TMapManagerFrame }

procedure TMapManagerFrame.CreateFrame;
begin
   aFrame := TMapManager_Frame.Create(nil);
   MMFrame.Tool := Self;
end;

function TMapManagerFrame.GetMMFrame: TMapManager_Frame;
begin
   result := TMapManager_Frame(aFrame);
end;

procedure TMapManagerFrame.SetSetting(const Value: boolean);
var
   ME: TTHematicMapsEditor;
begin
   if not Value then exit;
   ME := TTHematicMapsEditor.Create;
   ME.Input := Maps;
   ME.PostTool := Self;
   ME.DoIt;
   ME.Free;
end;

procedure TMapManagerFrame.SetMapChange(const Value: boolean);
var
   e: TElement;
begin
   if Value then
   begin
      e := Maps.Elements[MMFrame.Combo.ItemIndex];
      if not MapsSetting then
         if AnsiSameText(Maps.MapName,e.Name) then exit;// mapa zustava stejna
      Maps.MapName := e.Name;
      PostMsgToOwners(tm_CHangeThematic,e,false);
      Application.ProcessMessages;
      PostMsgToOwners(tm_BreakAndReView,nil,false);
   end;
end;

procedure TMapManagerFrame.SetMaps(const Value: TThematicMaps);
var
   i: integer;
   e: TElement;
begin
   FMaps := Value;
   MMFrame.Combo.Clear;
   if Value = nil then exit;
   for i := 0 to Value.LastIndex do
   begin
      e := Value.Elements[i];
      if not (e is TThematicMap) then continue;
      MMFrame.Combo.Items.Add(e.Name);
   end;
   MMFrame.Combo.ItemIndex := MMFrame.Combo.Items.IndexOf(Value.MapName);
   MapsSetting := true;
   MapChange := True;
   MapsSetting := false;
end;

procedure TMapManagerFrame.SetMapsSetting(const Value: boolean);
begin
  FMapsSetting := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMapManagerFrame);
END.
