unit f_GeometryListForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  f_OrbisForm,
  Elements, ComCtrls,
  Tools,
  t_GraphicForm, StdCtrls, Buttons, ExtCtrls;

type
  TGeometryList_Form = class(TORBISForm)
    List: TListView;
    Panel: TPanel;
    b_1: TBitBtn;
    b_2: TBitBtn;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ListSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure b_1Click(Sender: TObject);
    procedure b_2Click(Sender: TObject);
  private
    FLayer: TEntity;
    FGraphicForm: TGraphicForm;
    FItemText: string;
    FB1Tool: TTool;
    FB2Tool: TTool;
    procedure SetLayer(const Value: TEntity);
    procedure SetGraphicForm(const Value: TGraphicForm);
    procedure SetItemText(const Value: string);
    procedure SetB1Tool(const Value: TTool);
    procedure SetB2Tool(const Value: TTool);
    { Private declarations }
  public
     procedure CheckTools;
     property Layer: TEntity read FLayer write SetLayer;
     property GraphicForm: TGraphicForm read FGraphicForm write SetGraphicForm;
     property ItemText: string read FItemText write SetItemText;
     property B1Tool: TTool read FB1Tool write SetB1Tool;
     property B2Tool: TTool read FB2Tool write SetB2Tool;
  end;


implementation
uses
   g_Elems,
   m_Orbis;

{$R *.DFM}

{ TGeometryList_Form }

procedure TGeometryList_Form.SetLayer(const Value: TEntity);
var
   i: integer;
   E: TElement;
   L: TListItem;
begin
   FLayer := Value;
   List.Items.Clear;
   if Value = nil then exit;
   E := Value;
   if not (E is TEntity) then exit;
   //List.ViewStyle := vsList;
   List.Items.BeginUpdate;
   for i := 0 to Value.LastIndex do
   begin
      E := Value.Elements[i];
      L := List.Items.Add;
      L.Data := E;
      L.Caption := IntToStr(i+1)+'.'+ItemText;
   end;
   //List.ViewStyle := vsReport;
   List.Items.EndUpdate;
end;

procedure TGeometryList_Form.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
   inherited;
   if Tool <> nil then Tool.postMsg(tm_QuitTask,Tool,nil,false);
   CanClose := true;
end;

procedure TGeometryList_Form.SetGraphicForm(const Value: TGraphicForm);
begin
   FGraphicForm := Value;
end;

procedure TGeometryList_Form.ListSelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
var
   E: TElement;
   R: TRectEntity;
begin
   inherited;
   if not Selected then exit;
   if GraphicForm = nil then exit;
   //if List.Selected = nil then exit;
   E := TElement(Item.Data);
   R := TRectEntity.Create;
   R.UpdateRect(E);
   R.MinX := R.MinX - 10;
   R.MinY := R.MinY - 10;
   R.MaxX := R.MaxX + 10;
   R.MaxY := R.MaxY + 10;
   GraphicForm.ElementForProjection := R;
   Application.ProcessMessages;
   R.Free;
end;

procedure TGeometryList_Form.SetItemText(const Value: string);
begin
  FItemText := Value;
end;

procedure TGeometryList_Form.SetB1Tool(const Value: TTool);
begin
   if Value = nil then exit;
   FB1Tool := Value;
   b_1.Caption := Value.Name;
   b_1.Visible := true;
end;

procedure TGeometryList_Form.SetB2Tool(const Value: TTool);
begin
   if Value = nil then exit;
   FB2Tool := Value;
   b_2.Caption := Value.Name;
   b_2.Visible := true;
end;

procedure TGeometryList_Form.CheckTools;
begin
   if (B1Tool <> nil) or (B2Tool <> nil) then Panel.Visible := true;
end;

procedure TGeometryList_Form.b_1Click(Sender: TObject);
begin
   if B1Tool = nil then exit;
   B1Tool.DoIt;
   Layer := Layer;
end;

procedure TGeometryList_Form.b_2Click(Sender: TObject);
begin
   if B2Tool = nil then exit;
   B2Tool.DoIt;
   Layer := Layer;
end;

end.
