unit t_ExtendedLinkTool;

interface
uses
   ActnList,
   Windows,
   // KERNEL
   e_ToolModel,
   Elements,
   t_InterfaceTool,
   Tools;

type

TExtendedLinkTool = class;

TExtendedLinkToolGroup = class(TTechnology)
private
   FActualAction: TAction;
   FActionList: TActionList;
    FExtLinkTool: TExtendedLinkTool;
   procedure SetActionList(const Value: TActionList);
    function GetActualCaption: string;
    function GetActualHint: string;
    function GetActualImageIndex: integer;
    function GetActualScript: string;
    function GetActualTool: TTool;
    function GetActualVisible: boolean;
    procedure SetActualCaption(const Value: string);
    procedure setActualHint(const Value: string);
    procedure SetActualImageIndex(const Value: integer);
    procedure SetActualScript(const Value: string);
    procedure SetActualTool(const Value: TTool);
    procedure SetActualVisible(const Value: boolean);
    procedure SetFindActionName(const Value: string);
    procedure SetNewScriptAction(const Value: string);
    procedure SetNewToolAction(const Value: string);
    procedure SetEmpty(const Value: boolean);
    procedure SetNewSeparator(const Value: string);
    function GetEmpty: boolean;
    function GetActualShortCut: string;
    procedure SetActualShortCut(const Value: string);
    procedure SetExtLinkTool(const Value: TExtendedLinkTool);
public
   procedure CreateSelfByModel; override;
   function  CreateToolByModel(aModel: TToolModel): TTool; override;
   procedure AddToolsByModel(aTool: TTool); virtual;
   procedure AddToolByModel(aTool: TTool); virtual;
   procedure DisconnectActionList; virtual;
   procedure ActionExecute(Sender: TObject);
   procedure ActionPopup(Sender: TObject; MousePos: TPoint; var Handled: Boolean);
   function  FindActionByTool(aTool: TTool): TAction;
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property  ActionList: TActionList read FActionList write SetActionList;
   property  Empty: boolean read GetEmpty write SetEmpty;
   property  ExtLinkTool: TExtendedLinkTool read FExtLinkTool write SetExtLinkTool;
published
   // For manual setting only:
   property   NewScriptAction: string write SetNewScriptAction stored false;
   property   NewToolAction: string write SetNewToolAction stored false;
   property   NewSeparator: string write SetNewSeparator stored false;
   property   FindActionName: string write SetFindActionName stored false;
   property   ActualVisible: boolean read GetActualVisible write SetActualVisible stored false;
   property   ActualScript: string read GetActualScript write SetActualScript stored false;
   property   ActualTool: TTool read GetActualTool write SetActualTool stored false;
   property   ActualImageIndex: integer read GetActualImageIndex write SetActualImageIndex stored false;
   property   ActualCaption: string read GetActualCaption write SetActualCaption stored false;
   property   ActualHint: string read GetActualHint write setActualHint stored false;
   property   ActualShortCut: string read GetActualShortCut write SetActualShortCut stored false;
end;

TExtendedLinkTool = class(TInterfaceTool)
private
   FLinkTools: TExtendedLinkToolGroup;
   FMonitors: TTechnology;
   FFrameSelectors: TTechnology;
   procedure  SetNewGroupName(const Value: string);
   function   GetGroups: TTools;
   procedure  SetGroups(const Value: TTools);
   function   GetMonitors: TTechnology;
   function   GetLinkTools: TExtendedLinkToolGroup;
   function   GetActiveGroup: TExtendedLinkToolGroup;
   procedure  SetActiveGroup(const Value: TExtendedLinkToolGroup);
   function   GetFrameSelectors: TTechnology;
   function   GetActiveFrameSelector: TTool;
   procedure  SetActiveFrameSelector(const Value: TTool);
protected
   function    GetActiveTool: TTool; override;
   procedure   SetActiveTool(aTool: TTool); override;
public
   procedure   CreateDynamicPart; override;
   function    AddTool(aTool: TTool): TTool; override;
   function    GetText: string; virtual;
   procedure   SetText(const Value: string); virtual;
   procedure   RemoveLinkTools; virtual;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   on_tm_ShowText(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   BeginUpdate; virtual;
   procedure   EndUpdate; virtual;
   property    ActiveGroup: TExtendedLinkToolGroup read GetActiveGroup write SetActiveGroup;
   property    ActiveFrameSelector: TTool read GetActiveFrameSelector write SetActiveFrameSelector;
published
   property    Text: string read GetText write SetText;
   property    Groups: TTools read GetGroups write SetGroups stored false;
   property    NewGroupName: string write SetNewGroupName stored false;
   property    LinkTools: TExtendedLinkToolGroup read GetLinkTools stored false;
   property    Monitors: TTechnology read GetMonitors stored false;
   property    FrameSelectors: TTechnology read GetFrameSelectors stored false;
end;

implementation
uses
   {!!!}Dialogs,
   Classes,
   Controls,
   Forms,
   Graphics,
   Menus,
   SysUtils,
   //ToolWin,
   // KERNEL
   e_Class,
   e_Type,
   m_Orbis,
   u_String,
   // APP
   c_Actions,
   t_Device,
   t_FrameSelect,
   t_OrbisFrame;

{ TExtendedLinkTool }


function TExtendedLinkTool.AddTool(aTool: TTool): TTool;
begin
   result := inherited AddTool(aTool);
end;


function TExtendedLinkTool.GetActiveFrameSelector: TTool;
begin
   result := nil;
   if FrameSelectors = nil then exit;
   result := FrameSelectors.ActiveTool;
end;

function TExtendedLinkTool.GetActiveGroup: TExtendedLinkToolGroup;
var
   T: TTool;
begin
   result := nil;
   T := inherited GetActiveTool;
   if not (T is TExtendedLinkToolGroup) then exit;
   result := TExtendedLinkToolGroup(T);
end;

function TExtendedLinkTool.GetActiveTool: TTool;
var
   G: TExtendedLinkToolGroup;
begin
   result := nil;
   G := ActiveGroup;
   if G = nil then exit;
   result := G.ActiveTool;
end;

function TExtendedLinkTool.GetFrameSelectors: TTechnology;
begin
   if FFrameSelectors = nil then
   begin
      FFrameSelectors := TTechnology.Create;
      FFrameSelectors.Name := 'FRAMESELECTORS';
      AddTool(FFrameSelectors);
   end;
   result := FFrameSelectors;
end;

function TExtendedLinkTool.GetGroups: TTools;
begin
   result := Tools;
end;

function TExtendedLinkTool.GetMonitors: TTechnology;
begin
   if FMonitors = nil then
   begin
      FMonitors := TTechnology.Create;
      FMonitors.Name := 'MONITORS';
      AddTool(FMonitors);
   end;
   result := FMonitors;
end;

function TExtendedLinkTool.GetLinkTools: TExtendedLinkToolGroup;
begin
   if FLinkTools = nil then
   begin
      FLinkTools := TExtendedLinkToolGroup.Create;
      FLinkTools.Name := 'LINKTOOLS';
      AddTool(FLinkTools);
      FLinkTools.ExtLinkTool := Self;
   end;
   result := FLinkTools;
end;

function TExtendedLinkTool.GetText: string;
begin

end;

procedure TExtendedLinkTool.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
var
   i: integer;
begin
   //if aMessage.Message = tm_Resize then
   //   PostTextMsg(tm_AddToProtocol,Name + '.on_Resize',nil);
   Handled := false;
   try
     if (Monitors <> nil) and not(Monitors.NoTools) then
     begin
        for i := 0 to Monitors.Tools.LastIndex do
        begin
           Monitors.Tool[i].HandleMessage(aMessage, Handled);
        end;
     end;
     if (ActiveTool <> nil) then
     begin
        case aMessage.Message  of
           tm_Review, tm_Cursor: ;// udlosti, kter se nepedvaj aktivnmu nstroji
        else
           ActiveTool.HandleMessage(aMessage, Handled);
        end;
     end;
   except
   end;
   if Handled then exit;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      //tm_ExternalTaskSelected: on_tm_ExternalTaskSelected(aMessage, Handled);
      //tm_TaskSelected:         on_tm_TaskSelected(aMessage, Handled);
      tm_ToolSelected:         on_tm_ToolSelected(aMessage, Handled);
      tm_ShowText:             on_tm_ShowText(aMessage, Handled);
   end;
end;


procedure TExtendedLinkTool.on_tm_ShowText(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   if not (aMessage is TOrbisTextMessage) then exit;
   Text := TOrbisTextMessage(aMessage).Text;
end;


procedure TExtendedLinkTool.on_tm_ToolSelected(var aMessage: TOrbisMessage;var Handled: boolean);
var
   G: TExtendedLinkToolGroup;  // skupina noveho nastroje
   T: TTool;               // novy nastroj
   IT: TInterfaceTool;     // novy nastroj jako IT (Interface tool)
   A: TAction;             // akce noveho nastroje
   IsAT: boolean;          // Aktivni nastroj je nenilovy
   IsIAT: boolean;         // Aktivni nastroj je nenilovy a je to IT
   AT: TTool;              // Aktivni nastroj
   IAT: TInterfaceTool;    // Aktivni nastroj jako IT
   Flash: boolean;
   FlashReturn: boolean;
   FlashChange: boolean;
   FlashAbort: boolean;
   NoAction: boolean;
   NoActualAction: boolean;
begin
   Flash := false;
   FlashReturn := false;
   FlashChange := false;
   FlashAbort := false;
   NoAction := false;
   NoActualAction := true;
   T := TTool(aMessage.Data);
   // deaktivace
   if T = nil then
   begin
      if ActiveTool = nil then exit;
      if ActiveGroup.FActualAction <> nil then
         ActiveGroup.FActualAction.Checked := false;
      ActiveTool := nil;
      Cursor := crDefault;
      exit;
   end;
   // naplneni promennych pro pristup k Aktivnimu nastroji
   AT := ActiveTool;
   IsAT := AT <> nil;
   IsIAT := IsAt and (AT is TInterfaceTool);
   if IsIAT then IAT := TInterfaceTool(AT) else IAT := nil;

   G := TExtendedLinkToolGroup(T.FindOwnerClass(TExtendedLinkToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToprotocol,'Tool: '+ T.Name + ' is not component of '+ Name,nil);
      exit;
   end;
   A := G.FindActionByTool(T);
   NoAction := A = nil;
   (*
   if G = Tasks then // Tool T je TASK
   begin
      Tasks.FActualAction.Checked := false;
      if (T = ActiveTask) then
      begin
         Tasks.FActualAction.Checked := true;
         exit;
      end;
      if IsIAT and not IAT.IsPossibleToDeactivate then
      begin
         if not NoAction then
         begin
            A.Checked := true;
            A.Checked := false;
         end;
         Tasks.FActualAction.Checked := true;
         exit;
      end;
      Tasks.FActualAction := A;
      PostMsg(tm_TaskSelected,Self,T,false);
      exit;
   end;
   *)
   if IsAt then NoActualAction := ActiveGroup.FActualAction = nil;

   if not (T is TInterfaceTool) then
   begin
      if not NoAction then A.Checked := true;
      T.DoIt;
      if not NoAction then A.Checked := false;
      if not NoActualAction then
         if IsAT then ActiveGroup.FActualAction.Checked := true;
      exit;
   end
   else
   if isAT then
   begin
      if not NoActualAction then
         ActiveGroup.FActualAction.Checked := false;
      if T = AT then
      begin
         if not NoActualAction then
            ActiveGroup.FActualAction.Checked := true;
         exit;
      end;
      // een flash voln
      IT := TInterfaceTool(T);
      if IAT.AllowFlash and IT.CanBeFlash then
         Flash := true // bude flash tool
      else
      if IAT.CanBeFlash and (IAT.FlashTool <> nil) and IT.AllowFlash then
         if (IAT.FlashTool = IT) then
            FlashReturn := true // flash return
         else
            FlashAbort := true // flashAbort
      else
      if IAT.CanBeFlash and (IAT.FlashTool <> nil) and IT.CanBeFlash then
         FlashChange := true;
      // Zeptame se Aktivniho nastroje, jestli se mu chce koncit
      if not Flash then
      if (FlashAbort and not IAT.FlashTool.IsPossibleToDeactivate)
         or not IAT.IsPossibleToDeActivate then
      begin
         // ...asi nechce
         if not NoAction then
         begin
           A.Checked := true;
           A.Checked := false;
         end;
         if not NoActualAction then
            ActiveGroup.FActualAction.Checked := true;
         exit;
      end;
   end;
   if Flash then
   begin
      IAT.FlashTool := IT;
      IT.FlashTool := IAT;
      IAT.StartFlashTool;
   end;
   if FlashChange then
   begin
      IT.FlashTool := IAT.FlashTool;
      IT.FlashTool.FlashTool := IT;
   end;
   if FlashReturn then
   begin
      IAT.FlashTool := nil;
      IT.FlashTool := nil;
   end;
   if not NoAction then
   begin
      A.Checked := true;
      if (Flash or FlashChange) then
         Text := A.Hint + ' > F'
      else
         Text := A.Hint;
   end;

   G.FActualAction := A;
   IT := TInterfaceTool(T);
   if FlashAbort then
      IAT.FlashTool.On_DeActivate; // deaktivace zkl. nstroje
   if not Flash and IsIAT then              // pokud je Aktivni nastroj Interface Tool...
      IAT.On_DeActivate;      // ... tak ho deaktivujeme

   Cursor := crDefault;
   ActiveTool := IT;
   //Cursor := IT.Cursor;
   if not FlashReturn then
   begin
      IT.Device := Device;
      IT.Input  := Self.Input;
      if IT is TFrameTool then
      begin
         if TFrameTool(IT).FrameName <> '' then
         begin
            if FrameSelectors <> nil then
               ActiveFrameSelector := FrameSelectors.GetToolByName(TFrameTool(IT).FrameName);
         end;
         TFrameTool(IT).FrameSelector := TFrameSelector(ActiveFrameSelector);
      end;
      IT.On_Activate;             // a noveho chlapce aktivujeme
   end
   else
      IT.EndFlashTool;
   Handled := true;
   //CheckMenuItem(ActiveTool.ClassName);
   //FormCursor := TInterActiveTool(ActiveTool).Cursor;
end;


procedure TExtendedLinkTool.RemoveLinkTools;
begin
   if LinkTools = nil then exit;
   LinkTools.Empty := true;
end;

procedure TExtendedLinkTool.SetActiveFrameSelector(const Value: TTool);
begin
   if FrameSelectors = nil then exit;
   FrameSelectors.ActiveTool := Value;
end;

procedure TExtendedLinkTool.SetActiveGroup(const Value: TExtendedLinkToolGroup);
begin
   inherited SetActiveTool(Value);
end;

procedure TExtendedLinkTool.SetActiveTool(aTool: TTool);
var
   G: TExtendedLinkToolGroup;
begin
   if aTool = nil then
   begin
      ActiveGroup := nil;
      exit;
   end;
   G := TExtendedLinkToolGroup(aTool.FindOwnerClass(TExtendedLinkToolGroup));
   if G = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Tool '+aTool.Name+' is not component of '+ Name,nil);
      exit;
   end;
   ActiveGroup := G;
   G.ActiveTool := aTool;
end;

procedure TExtendedLinkTool.SetGroups(const Value: TTools);
begin
   Tools := Value;
end;

procedure TExtendedLinkTool.SetNewGroupName(const Value: string);
var
   G: TExtendedLinkToolGroup;
begin
   if GetToolByName(Value) <> nil then
   begin
      //PostTextMsg(tm_AddToProtocol, 'Group named '+ Value +' already exists !',nil);
      exit;
   end;
   G := TExtendedLinkToolGroup.Create;
   G.Name := Value;
   AddTool(G);
end;


procedure TExtendedLinkTool.SetText(const Value: string);
begin

end;

procedure TExtendedLinkTool.CreateDynamicPart;
begin
   //ExtForm.TopBar.DisableAlign;
   inherited;
   //ExtForm.TopBar.EnableAlign;
//   PostMsg(tm_resize, self, nil, false)
end;


procedure TExtendedLinkTool.BeginUpdate;
begin

end;

procedure TExtendedLinkTool.EndUpdate;
begin

end;

{ TExtendedLinkToolGroup }

procedure TExtendedLinkToolGroup.ActionExecute(Sender: TObject);
begin
   PostMsgToOwners(tm_ToolSelected,TToolAction(Sender).Tool,false);
   //PostTextMsg(tm_AddToProtocol,'SelectedTool: '+TToolAction(Sender).Name,nil);
end;


procedure TExtendedLinkToolGroup.AddToolByModel(aTool: TTool);
var
   aModel: TToolModel;
begin
   {
   if aTool = nil then exit;
   AddTool(aTool);
   aModel := aTool.SelfModel;
   if aModel = nil then exit;
   NewToolAction := aTool.Name;
   ActualTool := aTool;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
   ActualShortCut := aModel.ShortCut;
   }
end;

procedure TExtendedLinkToolGroup.AddToolsByModel(aTool: TTool);
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
   T: TTechnology;
begin
   {
   if aTool = nil then exit;
   if not (aTool is TTechnology) then exit;
   T := TTechnology(aTool);
   if T.Tools.NoElements then exit;
   M := T.SelfModel;
   if M = nil then exit;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
   for i := 0 to T.Tools.LastIndex do
   begin
      AddToolByModel(T.Tool[i]);
   end;
   }
end;


procedure TExtendedLinkToolGroup.CreateSelfByModel;
var
   M: TToolModel;
   S: TStringListElement;
   i: integer;
begin
   {
   M := SelfModel;
   if M = nil then exit;
   inherited;
   if not M.NoImages then
   begin
      try
         S := TStringListElement(M.Images);
         for i := 0 to S.LastIndex do
         begin
            NewImage := S.Strings[i];
         end;
      except
      end;
   end;
   }
end;

function TExtendedLinkToolGroup.CreateToolByModel(aModel: TToolModel): TTool;
var
   ET: TExtendedLinkTool;
begin
   result := nil;
   {
   if aModel = nil then exit;
   if (aModel.Name = '.') or (aModel.Name = '|') then
   begin
      NewSeparator := aModel.Name;
      exit;
   end;
   result := inherited CreateToolByModel(aModel);
   if result = nil then exit;
   if result is TControlTool then exit;
   if result is TOrbisFrame then
   begin
      ET := TExtendedLinkTool(FindClassInOwners(TExtendedLinkTool));
      if ET <> nil then
         TOrbisFrame(result).Device := ET.Device;
      exit;
   end;
   NewToolAction := result.Name;
   ActualTool := result;
   ActualCaption := aModel.Alias;
   ActualHint := aModel.Comment;
   ActualImageIndex := aModel.ImageIndex;
   }
end;

procedure TExtendedLinkToolGroup.DisconnectActionList;
var
   i: integer;
   A: TAction;
   TA: TToolAction;
begin
   if FActionList = nil then exit;
   for i := 0 to FActionList.ActionCount - 1 do
   begin
      A := TAction(FActionList.Actions[i]);
      if not (A is TToolAction) then continue;
      TA := TToolAction(A);
      TA.Tool := nil;
      TA.OnExecute := nil;
   end;
   ActiveTool := nil;
end;

function TExtendedLinkToolGroup.FindActionByTool(aTool: TTool): TAction;
var
   i: integer;
   A: TAction;
begin
   result := nil;
   if ActionList = nil then exit;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if not (A is TToolAction) then continue;
      if TToolAction(A).Tool = aTool then
      begin
         result := A;
         exit;
      end;
   end;
end;

function TExtendedLinkToolGroup.GetActualCaption: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Name;
end;

function TExtendedLinkToolGroup.GetActualHint: string;
begin
   result := '';
   if FactualAction <> nil then result := FActualAction.Hint;
end;

function TExtendedLinkToolGroup.GetActualImageIndex: integer;
begin
   result := -1;
   if FactualAction <> nil then result := FActualAction.ImageIndex;
end;

function TExtendedLinkToolGroup.GetActualScript: string;
begin
   result := '';
   if FactualAction = nil then exit;
   if not (FActualAction is TScriptAction) then exit;
   result := TScriptAction(FActualAction).Script;
end;

function TExtendedLinkToolGroup.GetActualTool: TTool;
begin
   result := nil;
   if FactualAction = nil then exit;
   if not (FActualAction is TToolAction) then exit;
   result := TToolAction(FActualAction).Tool;
end;

function TExtendedLinkToolGroup.GetActualVisible: boolean;
begin
   result := false;
   if FactualAction <> nil then result := FActualAction.Visible;
end;

procedure TExtendedLinkToolGroup.HandleMessage(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   OwnersHandleMessage(aMessage,Handled);
end;

procedure TExtendedLinkToolGroup.SetActionList(const Value: TActionList);
var
   i: integer;
   A: TAction;
   TA: TToolAction;
begin
   if FActionList = Value then exit;
   if FActionList <> nil then Empty := true;
   FActionList := Value;
   if FActionList = nil then exit;
   for i := 0 to FActionList.ActionCount - 1 do
   begin
      A := TAction(FActionList.Actions[i]);
      if not (A is TToolAction) then continue;
      TA := TToolAction(A);
      TA.CreateTool;
      if TA.Tool = nil then continue;
      AddTool(TA.Tool);
      if TA.Tool is TDeviceTool then
         TDeviceTool(TA.Tool).Device := ExtLinkTool.Device;
      TA.OnExecute := ActionExecute;
      if TA.OrbisMonitor then
         ExtLinkTool.Monitors.AddTool(TA.Tool);
   end;
   ActiveTool := nil;
end;

procedure TExtendedLinkToolGroup.SetActualCaption(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Caption := Value;
end;

procedure TExtendedLinkToolGroup.setActualHint(const Value: string);
begin
   if FactualAction <> nil then FActualAction.Hint := Value;
end;

procedure TExtendedLinkToolGroup.SetActualImageIndex(const Value: integer);
begin
   if FactualAction <> nil then FActualAction.ImageIndex := Value;
end;

procedure TExtendedLinkToolGroup.SetActualScript(const Value: string);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TScriptAction) then exit;
   TScriptAction(FActualAction).Script := Value;
end;

procedure TExtendedLinkToolGroup.SetActualTool(const Value: TTool);
begin
   if FActualAction = nil then exit;
   if not(FActualAction is TToolAction) then exit;
   TToolAction(FActualAction).Tool := Value;
end;

procedure TExtendedLinkToolGroup.SetActualVisible(const Value: boolean);
begin
   if FactualAction <> nil then FActualAction.Visible := Value;
end;


procedure TExtendedLinkToolGroup.SetEmpty(const Value: boolean);
var
   V: boolean;
begin
   //ShowMessage('Free Tools');
   Tools.FreeAll;
   //ShowMessage('After Free Tools');
   FActualAction:= nil;
   ActiveTool:= nil
end;

procedure TExtendedLinkToolGroup.SetFindActionName(const Value: string);
var
   i: integer;
   A: TAction;
begin
   FActualAction := nil;
   if ActionList = nil then exit;
   for i := 0 to ActionList.ActionCount - 1 do
   begin
      A := TAction(ActionList.Actions[i]);
      if CompareText(Value, A.Name) = 0 then
      begin
         FActualAction := A;
         break;
      end;
   end;
end;


procedure TExtendedLinkToolGroup.SetNewScriptAction(const Value: string);
var
   A: TAction;
   //B: TToolButton;
begin
   {
   A := TScriptAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   //ToolBar.EdgeInner :=
   //ToolBar.EdgeOuter :=
   B.Action := A;
   B.Style := tbsCheck;
   A.DisableIfNoHandler := false;
   }
end;

procedure TExtendedLinkToolGroup.SetNewSeparator(const Value: string);
//var
//   B: TToolButton;
begin
   {
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   if Value = '|' then B.Style := tbsDivider
                  else B.Style := tbsSeparator;
   B.Width := 10;
   }
end;

procedure TExtendedLinkToolGroup.SetNewToolAction(const Value: string);
var
   A: TAction;
   //B: TToolButton;
begin
   {
   A := TToolAction.Create(ActionList);
   A.ActionList := ActionList;
   FActualAction := A;
   A.OnExecute := ActionExecute;
   B := TToolButton.Create(ToolBar);
   ToolBar.InsertControl(B);
   B.Action := A;
   B.Style := tbsCheck;
   B.Cursor := crHandPoint;
   B.OnContextPopup := ActionPopUp;
   A.DisableIfNoHandler := false;
   }
end;

function TExtendedLinkToolGroup.GetEmpty: boolean;
begin
   result := NoTools;
end;

function TExtendedLinkToolGroup.GetActualShortCut: string;
begin
   result := '';
   if FactualAction <> nil then
   begin
      result := ShortCutToText(FActualAction.ShortCut);
   end;
end;

procedure TExtendedLinkToolGroup.SetActualShortCut(const Value: string);
begin
   if FactualAction <> nil then FActualAction.ShortCut := TextToShortCut(Value);
end;

procedure TExtendedLinkToolGroup.ActionPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
   //ShowMessage('Hint: '+TToolAction(TToolButton(Sender).Action).Hint);
end;



procedure TExtendedLinkToolGroup.SetExtLinkTool(const Value: TExtendedLinkTool);
begin
   FExtLinkTool := Value;
end;

initialization
   RegisterOrbisClass(TExtendedLinkToolGroup);
   RegisterOrbisClass(TExtendedLinkTool);
end.

