unit t_GraphicLinkTool;

interface
uses
   Controls,
   ExtCtrls,
   Graphics,
   // KERNEL
   Elements,
   Tools,
   // DEVICE
   e_Proje,
   // INTERPRET
   e_IScrpt,
   // GUI
   m_Intera,
   // APP
   r_OrbisGraphicFrame,
   t_ExtendedLinkTool;

type

TOnInterpretDone = procedure of object;
TOnElementSelectEvent = procedure(const aElement: TElement) of object;
TOnGLTProjectionChanged = procedure(aX,aY,aScale,aVirtualScale: double) of object;
TOnMeasureChangedEvent = procedure(Value,Value1: double) of object;
{ TGraphicLinkTool }

TGraphicLinkTool = class(TExtendedLinkTool)
 private
   FImageW: integer;
   FImageH: integer;
   FGFrame: TOrbisGraphic_Frame;
    FOnSelect: TOnElementSelectEvent;
    FOnProjectionChanged: TOnGLTProjectionChanged;
    FOnAreaChanged: TOnMeasureChangedEvent;
    FOnLengthChanged: TOnMeasureChangedEvent;
    FOnInterpretDone: TOnInterpretDone;
   function    GetActiveToolName: string;
   function    GetActiveToolClassName: string;
   procedure   SetActiveToolName(aValue: string);
   procedure   SetActiveToolClassName(aValue: string);
   function    GetPageColor: TColor;
   procedure   SetPageColor(aValue: TColor);
   function    GetProjection: TProjection;
   function    GetInterpretScript: TInterpretScript;
   procedure   SetInterpretScript(aValue: TInterpretScript);
    procedure SetViewAll(const Value: boolean);
    procedure SetViewByElement(const Value: TElement);
    procedure SetShiftView(const Value: string);
    procedure SetZoom(const Value: string);
    procedure SetGFrame(const Value: TOrbisGraphic_Frame);
    procedure SetNewThematic(const Value: TElement);
    procedure SetElementForProjection(const Value: TElement);
    procedure SetOnSelect(const Value: TOnElementSelectEvent);
    procedure SetOnProjectionChanged(const Value: TOnGLTProjectionChanged);
    procedure SetElementForPattern(const Value: TElement);
    procedure SetOnAreaChanged(const Value: TOnMeasureChangedEvent);
    procedure SetOnLengthChanged(const Value: TOnMeasureChangedEvent);
    procedure SetNoResetReviewByElement(const Value: boolean);
    procedure SetOnInterpretDone(const Value: TOnInterpretDone);
 protected
   procedure   ShowScale(aVal: string); virtual;

   procedure   SetInput(aInput: TElement); override;
   procedure   CreateDevice; virtual;
   procedure   LinkDeviceToGFrame; virtual;
   procedure   CreateInterpret; virtual;
   procedure   LinkInterpretToDevice; virtual;
   procedure   CreateTools; virtual;
   procedure   SetActiveToolIndex(aIndex: longInt); override;
   procedure   SetNextTool(aTool: TTool); virtual;
   function    GetCaption: string; virtual;
   procedure   SetCaption(const aValue: string); virtual;
   procedure   SetBreakReView(aValue: boolean);
   procedure   SetElementForReView(aValue: TElement);
   procedure   SetElementForKillReView(aValue: TElement);
 public
   constructor Create; override;
   procedure   DoIt; override;

   function    GetText: string; override;
   procedure   SetText(const Value: string); override;
   function    GetCursor: TCursor; override;
   procedure   SetCursor(const Value: TCursor); override;
   function    AddTool(aTool: TTool): TTool; override;
   function    InsertTool(aIndex: longInt; aTool: TTool): TTool; override;
   procedure   RemoveTool(aTool: TTool); override;
   procedure   FreeTool(aTool: TTool); override;
   procedure   FreeTools;

   procedure   Show;
   procedure   ShowAll;
   procedure   on_tm_Open(var Handled: boolean); virtual;
   procedure   on_tm_SaveAs(var Handled: boolean); virtual;
   procedure   on_tm_QuitTask; override;
   procedure   on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   on_tm_ElementSelected(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   on_tm_Print(var Handled: boolean); virtual;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_ReviewByElement(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   On_tm_Resize(var Handled: boolean); override;
   procedure   On_tm_RePaint(var Handled: boolean); virtual;
   procedure   On_tm_NewScale(var Handled: boolean); virtual;
   procedure   on_tm_ShowAll(var Handled: boolean); virtual;
   procedure   On_tm_EndOfPrint(var Handled: boolean); virtual;
   procedure   On_tm_Cursor(var aMessage: TOrbisCursorEvent; var Handled: boolean); override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseWheelDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   On_tm_ChangeThematic(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   On_tm_ProjectionChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   On_tm_LengthChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   On_tm_AreaChanged(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure   BeginUpdate; override;
   procedure   EndUpdate; override;
   procedure   StartSizeMove;
   procedure   EndSizeMove;
   property    GFrame: TOrbisGraphic_Frame read FGFrame write SetGFrame;
   property    OnSelect: TOnElementSelectEvent read FOnSelect write SetOnSelect;
   property    OnProjectionChanged: TOnGLTProjectionChanged read FOnProjectionChanged write SetOnProjectionChanged;
   property    OnLengthChanged: TOnMeasureChangedEvent read FOnLengthChanged write SetOnLengthChanged;
   property    OnAreaChanged: TOnMeasureChangedEvent read FOnAreaChanged write SetOnAreaChanged;
   property    OnInterpretDone: TOnInterpretDone read FOnInterpretDone write SetOnInterpretDone;
published
   property    NextTool: TTool write SetNextTool;
   property    ActiveToolName: string read GetActiveToolName write SetActiveToolName;
   property    ActiveToolClassName: string read GetActiveToolClassName write SetActiveToolClassName;
   property    NewThematic: TElement write SetNewThematic;
   property    PageColor: TColor read GetPageColor write SetPageColor;
   property    Projection: TProjection read GetProjection;
   property    InterpretScript: TInterpretScript read GetInterpretScript write SetInterpretScript;
   property    BreakReView: boolean write SetBreakReView stored false;
   property    ElementForReView: TElement write SetElementForReView;
   property    ElementForKillReView: TElement write SetElementForKillReView;
   property    ElementForProjection: TElement write SetElementForProjection;
   property    ElementForPattern: TElement write SetElementForPattern;
   property    ReViewByElement: TElement write SetViewByElement;
   property    ShiftView: string write SetShiftView;
   property    Zoom: string write SetZoom;
   property    ViewAll: boolean write SetViewAll;
   property    NoResetReviewByElement: boolean write SetNoResetReviewByElement;
end;


implementation
uses
   Classes,
   Dialogs,
   Forms,
   SysUtils,
   Windows,
   // KERNEL
   e_Class,
   e_Type,
   m_ORBIS,
   Worlds,
   u_String,
   // GLIB
   G_Elems,
   // DEVICE
   d_StdImg,
   t_InterfaceTool,
   t_Print,
   // INTERPRET
   i_SelectedGeometry,
   i_Uni;

{ TGraphicLinkTool }

constructor TGraphicLinkTool.Create;
begin
   inherited Create;
   //CreateForm;
   CreateDevice;
   //LinkDeviceToImage;
   CreateInterpret;
   LinkInterpretToDevice;
   CreateTools;
end;


procedure   TGraphicLinkTool.SetBreakReView(aValue: boolean);
begin
   if Interpret.Busy then Interpret.BreakWork := true;
end;


procedure   TGraphicLinkTool.SetElementForReView(aValue: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, aValue, DontKill);
end;


procedure   TGraphicLinkTool.SetElementForKillReView(aValue: TElement);
begin
   if Idle then exit;
   PostMsg(tm_ReView, Self, aValue, KillIt);
end;




function    TGraphicLinkTool.GetInterpretScript: TInterpretScript;
begin
   result := nil;
   if Interpret is TUniversalInterpret then result := TUniversalInterpret(Interpret).Script;
end;


procedure   TGraphicLinkTool.SetInterpretScript(aValue: TInterpretScript);
//var
   //aTool: TUniversalTopolScriptCreator;
   //aScript: TInterpretScript;
begin
   (*
   if aValue = nil then
   begin
   {$IFNDEF NOTIS}
      aTool := TUniversalTopolScriptCreator.Create;
      aTool.DoIt;
      aScript:= TInterpretScript(aTool.Output);
      TUniversalInterpret(FInterpret).Script := aScript;
      aTool.Free;
   {$ENDIF}
   end
   else
   *)
   begin
     if ((TElement(aValue) is TInterpretScript) and (Interpret is TUniversalInterpret))
        then TUniversalInterpret(Interpret).Script := aValue;
   end;
end;


function    TGraphicLinkTool.GetPageColor: TColor;
begin
   try
      result := Device.Page.Color;
   except
      result := clBlack;
   end;
end;

procedure   TGraphicLinkTool.SetPageColor(aValue: TColor);
begin
   try
      Device.Page.Color := aValue;
   except
   end;
end;

function    TGraphicLinkTool.GetProjection: TProjection;
begin
   if Device <> nil then result := Device.Projection
                    else result := nil;
end;


function    TGraphicLinkTool.GetActiveToolName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.Name;
end;

function    TGraphicLinkTool.GetActiveToolClassName: string;
var
   T: TTool;
begin
   result := '';
   T := ActiveTool;
   if T = nil then exit;
   result := T.ClassName;
end;

procedure   TGraphicLinkTool.SetActiveToolName(aValue: string);
var
   T: TTool;
begin
   T := TTool(LinkTools.Tools.FindElementName(aValue));
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
   Application.ProcessMessages;
end;

procedure   TGraphicLinkTool.SetActiveToolClassName(aValue: string);
var
   T: TTool;
begin
   T := nil;
   try
      T := TTool(Tools.FindElementClass(FindOrbisClass(aValue)));
   except
   end;
   if T = nil then exit;
   T.PostMsgToOwners(tm_ToolSelected, T, DontKill);
end;


procedure   TGraphicLinkTool.DoIt;
begin
   Show;
   if Idle then exit;
   PostMsg(tm_ReView, Self, nil, DontKill);
end;

procedure   TGraphicLinkTool.SetInput(aInput: TElement);
var
   Envelope: TRectEntity;
begin
   if FInput <> nil then
   begin
      FreeElement(FInput);
      FInput := nil;
   end;
   if aInput = nil then exit;
   Envelope := TRectEntity.Create;
   Envelope.Name := 'Oblka';      // p.k.17.1.99
   Envelope.AddAndUpdateRect(aInput{.CreateEnvelope});
   AddElement(Envelope);
   try
      if aInput is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(aInput).Rect, Device.Page);
      end
      else
      if aInput is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(aInput).GetRegion, Device.Page);
      end;
   except
   end;
   inherited SetInput(Envelope);
end;

procedure   TGraphicLinkTool.CreateDevice;
begin
   Device := TImageCanvasDevice.Create;
end;

procedure   TGraphicLinkTool.LinkDeviceToGFrame;
begin
  if GFrame = nil then exit;
   with TImageCanvasDevice(Device) do
   begin
      //Canvas := Form.Image.Canvas;
      Bitmap := GFrame.Image.Picture.Bitmap;
   end;
end;

procedure   TGraphicLinkTool.CreateInterpret;
begin
   Interpret := TUniversalInterpret.Create;
end;



procedure   TGraphicLinkTool.LinkInterpretToDevice;
begin
   Interpret.Device := Device;
end;

procedure   TGraphicLinkTool.CreateTools;
begin
end;

procedure   TGraphicLinkTool.SetActiveToolIndex(aIndex: longInt);
begin
   if aIndex = FActiveToolIndex then exit;
   inherited SetActiveToolIndex(aIndex);
end;

procedure   TGraphicLinkTool.SetNextTool(aTool: TTool);
begin
   {
   aTool.Device := Device;
   aTool.Manager:= Self;
   AddTool(aTool);
   ActiveTool := aTool;
   }
   //CheckMenuItem(aTool.ClassName);
end;




function    TGraphicLinkTool.GetCaption: string;
begin
  result:= '';
  {
  if Form = nil then exit;

   result := Form.Caption;
   }
end;

procedure   TGraphicLinkTool.SetCaption(const aValue: string);
begin
   {
  if Form = nil then exit;
   Form.Caption := aValue;
   }
end;




function    TGraphicLinkTool.AddTool(aTool: TTool): TTool;
begin
   result := inherited AddTool(aTool);
end;


function    TGraphicLinkTool.InsertTool(aIndex: longInt; aTool: TTool): TTool;
begin
   result :=  inherited InsertTool(aIndex, aTool);
end;

procedure   TGraphicLinkTool.RemoveTool(aTool: TTool);
begin
   inherited RemoveTool(aTool);
end;

procedure   TGraphicLinkTool.FreeTool(aTool: TTool);
begin
   inherited FreeTool(aTool);
end;

procedure   TGraphicLinkTool.FreeTools;
var
   i: longInt;
begin
   for i := 0 to Tools.Count do
   begin
      FreeTool(TTool(Tools.Elements[i]));
   end;
end;

procedure   TGraphicLinkTool.Show;
begin
   //Visible := true;
end;


procedure   TGraphicLinkTool.ShowAll;
begin
  if FInput = nil then exit;
  try
     if FInput is TRectEntity then
     begin
        Device.Projection.ChangeProjectionByRectangle(TRectEntity(FInput).Rect, Device.Page);
     end
     else
     if FInput is TCoordinate then
     begin
        Device.Projection.ChangeProjectionByRect(TCoordinate(FInput).GetRegion, Device.Page);
     end;
  except
  end;
end;



procedure   TGraphicLinkTool.on_tm_Open(var Handled: boolean);
begin
end;
(*
var
   Loader: TLoader;
   SelEl: TElement;
   SwapG: boolean;
begin
   if Form.OpenDialog.Execute then
   begin
      SwapG := true;
      Loader := TLoader.CreateAll(Form.OpenDialog.FileName);
      try
         Form.OpenDialog.InitialDir := ExtractFilePath(Form.OpenDialog.FileName);
         Loader.DoIt;
         Input := Loader.Output;
         Loader.Output.Free;
         PostMsg(tm_Review, Self, nil, DontKill);
      except
      end;
      Loader.Free;
   end
end;
*)

procedure   TGraphicLinkTool.on_tm_SaveAs(var Handled: boolean);
begin
end;
(*
var
   Saver: TSaver;
begin
   Form.SaveDialog.InitialDir := Form.OpenDialog.InitialDir;
   Form.SaveDialog.FileName   := Form.OpenDialog.FileName;
   if Form.SaveDialog.Execute then
   begin
      Form.OpenDialog.InitialDir := ExtractFilePath(Form.SaveDialog.FileName);
      Form.OpenDialog.FileName := Form.SaveDialog.FileName;
      if Input = nil then exit;
      Input.ResetID;
      Saver := TSaver.CreateAll(Form.SaveDialog.FileName);
      Saver.Input := Input;
      Saver.DoIt;
      Saver.Free
   end;
end;
*)



procedure   TGraphicLinkTool.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);

begin
  {
  if Form = nil then exit;
    Handled := true;
    begin
       Coord := Device.Projection.GetModelPoint(aMessage.x, aMessage.y);
       Form.l_x.Caption := FloatToStrF(Coord.x, ffFixed, 12, 2);
       Form.l_y.Caption := FloatToStrF(Coord.y, ffFixed, 12, 2);
    end;
   }
end;

procedure TGraphicLinkTool.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
{
var
   P: Wintypes.TPoint;
}
begin
  (*
  if Form = nil then exit;
   Handled := true;
   if aMessage.Button = mbRight then
   begin
      {
      if Interpret.Busy then
      begin
         PostMouseEvent(tm_MouseDown, Self, aMessage.Button,
                        aMessage.Shift, aMessage.X, aMessage.Y, aMessage.ActiveDevice);
         exit;
      end;
      }
      P := Point(aMessage.x, aMessage.y);
//      P := Form.PaintBox.ClientToScreen(P);
      P := Form.Image.ClientToScreen(P);
      Form.PopupMenu1.PopUp(P.x,P.y);
   end;
   *)
end;



procedure TGraphicLinkTool.On_tm_MouseWheelDown(var aMessage: TOrbisMouseEvent;  var Handled: boolean);
var
   dx,dy: integer;
   _X,_Y: double;
begin
   if Handled then exit;
   Handled := true;
   dx := aMessage.X - (Device.Page.Width div 2);
   dy := aMessage.Y - (device.Page.Height div 2);
   _X := Device.Projection.GetModelX(aMessage.x);
   _Y := Device.Projection.GetModelY(aMessage.y);
   Scale:= Scale * 1.5;
   Device.Projection.Center_X := _X - Device.Projection.GetModelLengthHoriz(dX);
   Device.Projection.Center_Y := _Y + Device.Projection.GetModelLengthVert(dY);
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
end;

procedure TGraphicLinkTool.On_tm_MouseWheelUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   dx,dy: integer;
   _X,_Y: double;
begin
   if Handled then exit;
   Handled := true;
   dx := aMessage.X - (Device.Page.Width div 2);
   dy := aMessage.Y - (device.Page.Height div 2);
   _X := Device.Projection.GetModelX(aMessage.X);
   _Y := Device.Projection.GetModelY(aMessage.Y);
   Scale:= Scale * 0.75;
   Device.Projection.Center_X := _X - Device.Projection.GetModelLengthHoriz(dX);
   Device.Projection.Center_Y := _Y + Device.Projection.GetModelLengthVert(dY);
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
end;

procedure   TGraphicLinkTool.on_tm_QuitTask;
begin
   if Interpret.Busy then
   begin
      Status := Status + [st_DIE];
      Interpret.BreakWork := true;
   end
   else
   begin
     {
     if Form <> nil then
        Form.Free;
      }
      inherited on_tm_QuitTask;
   end;
end;

procedure   TGraphicLinkTool.on_tm_Done(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := true;
   if aMessage.Sender is TUniversalInterpret then
   begin
      if st_DIE in Status then
      begin
        {
        if Form <> nil then
           Form.Free;
         }
         inherited on_tm_QuitTask;
      end
      else
      if st_BREAK_AND_DOIT in Status then
      begin
         Status := Status - [st_BREAK_AND_DOIT];
         DoIt;
      end;
      if Assigned(OnInterpretDone) then
         OnInterpretDone;
   end;
end;


procedure   TGraphicLinkTool.on_tm_ToolSelected(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   inherited;
   (*
   if not (aMessage.Data is TInterActiveTool) then exit;

   if ActiveTool is TInterActiveTool then
      TInterActiveTool(ActiveTool).On_DeActivate;

   ActiveTool := TInterActiveTool(aMessage.Data);
   TInterActiveTool(ActiveTool).Device := Device;
   ActiveTool.Input  := Self.Input;
   TInterActiveTool(ActiveTool).On_Activate;
   Handled := true;
   CheckMenuItem(ActiveTool.ClassName);
   FormCursor := TInterActiveTool(ActiveTool).Cursor;
   *)
end;

procedure   TGraphicLinkTool.on_tm_Print(var Handled: boolean);

var
   P: TPrint;

begin

   P := TPrint.Create;
   P.Input := Interpret;
   P.DoIt;
   P.Free;

end;


procedure TGraphicLinkTool.ShowScale(aVal: string);
begin
{
   try
       if (Form <> nil) and (Form.l_m <> nil) then
          Form.l_M.Caption := aVal;
   except
   end;
}
end;



procedure   TGraphicLinkTool.On_tm_ReView(var aMessage: TOrbisMessage; var Handled: boolean);
//var
   // s: string;
   //aX,aY,aScale: double;
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReView(aMessage, Handled);
{
   if Scale < 10 then
   begin
      Str(Device.Projection.Scale:0:3,S);
   end
   else S := IntToStr(round(Device.Projection.Scale));

   ShowScale(S);
   if CLoseAfterPrinting then PostMsg(tm_Print, Self, nil, DontKill);
}
   if ActiveTool <> nil then PostMsg(tm_Review,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);

   //aX := Device.Projection.Center_X;
   //aY := Device.Projection.Center_Y;
   //aScale := Device.Projection.Scale;
   {
   if (aX <> 0) and (aY <> 0) and (aScale <> 0) then
   begin
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('X')).AsFloat := aX;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('Y')).AsFloat := aY;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('Scale')).AsFloat := aScale;
      TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('VirtualScale')).AsFloat := Device.Projection.VirtualScale;;
      Form.Maps.ItemIndex := Form.Maps.Items.IndexOf(TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('MAP')).Value);

      //Form.Maps.Text :=  TStringElement(TEntity(World.SystemRegister.FindElementName('Projection')).FindElementName('MAP')).Value;
   end;
   }
   //TDeviceTool(Tool).Device.Projection.Scale := aScale;
   //Tool.PostMsg(tm_ModelChanged, nil, nil, false);
end;

procedure TGraphicLinkTool.On_tm_ReviewByElement(var aMessage: TOrbisMessage;
  var Handled: boolean);
//var
   //s: string;
begin
   if Idle then exit;
   if Interpret.Busy then
   begin
      Status := Status + [st_BREAK_AND_DOIT];
      Interpret.BreakWork := true;
      exit;
   end;
   Device.Pen.Mode:= pmCopy; { novinka 10.6.97 }

   inherited On_tm_ReViewByElement(aMessage, Handled);
{
   if Scale < 10 then
   begin
      Str(Device.Projection.Scale:0:3,S);
   end
   else S := IntToStr(round(Device.Projection.Scale));

   ShowScale(S);
   if CLoseAfterPrinting then PostMsg(tm_Print, Self, nil, DontKill);
}
   //if ActiveTool <> nil then PostMsg(tm_ReviewByElement,ActiveTool,nil,dontkill);
   Application.ProcessMessages;
   PostMsg(tm_Done, nil, self, dontKill);
end;


procedure TGraphicLinkTool.On_tm_Resize(var Handled: boolean);
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   Device.SetupPage(nil);
   if Idle then exit;
   PostMsg(tm_BreakAndReView, Self, nil, DontKill);
end;

procedure TGraphicLinkTool.On_tm_RePaint(var Handled: boolean);
begin
   Handled := true;
   {
   PostMsg(tm_BreakInterpret, Self, nil, DontKill);
   }
   if assigned(GFrame) then
      GFrame.Image.Refresh;
end;



procedure TGraphicLinkTool.On_tm_EndOfPrint(var Handled: boolean);
begin
   Handled := true;
   {
   if CloseAfterPrinting then
   begin
      PostMsgToOwners(tm_EndOfPrint, nil, DontKill);
      on_tm_QuitTask;
   end;
   }
end;



procedure TGraphicLinkTool.On_tm_NewScale(var Handled: boolean);
(*
var
   aForm: TF_SetScale;
   D: double;
*)
begin
(*
  try
    Application.CreateForm(Tf_SetScale, aForm);
    try
      aForm.Combo.Text := IntToStr(round(Scale));
      case aForm.ShowModal of
         mrOK:
         begin
            D:= StrToExtended(aForm.Combo.Text);
            if D > 0 then
            begin
              Scale := D;
              PostMsg(tm_BreakAndReView, self, nil, DontKill);
              Handled:= true
            end
            else
              ShowAll
         end;
      end;
    finally
      aForm.Free;
    end
  except
  end
*)
end;


procedure   TGraphicLinkTool.On_tm_ShowAll(var Handled: boolean);
begin
  ShowAll;
  Handled:= true;
  if Idle then exit;
  PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;



procedure TGraphicLinkTool.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
var
   P: TPoint;
begin
   if Idle then exit;
   case aMessage.Key of
      VK_F5:
      begin
         if ssCtrl in aMessage.Shift then
            ViewAll := true
         else
         begin
            Scale:= Scale / 2;
            PostMsg(tm_BreakAndReView, Self, nil, DontKill);
            PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
         end;
      end;
      VK_F6:
      begin
         Scale:= Scale * 2;
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_F7:
      begin
         P := GFrame.Image.ScreenToClient(Mouse.CursorPos);
         Device.Projection.Center_X := Device.Projection.GetModelX(P.X);
         Device.Projection.Center_Y := Device.Projection.GetModelY(P.Y);
         PostMsg(tm_BreakAndReView, Self, nil, DontKill);
         PostMsg(tm_ProjectionChanged, Self, Device.Projection, false);
      end;
      VK_ESCAPE:
      begin
         PostMsg(tm_BreakInterpret, Self, nil, DontKill);
      end;
   end;
end;



procedure TGraphicLinkTool.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
  // if aMessage.Message = tm_Resize then
  //    PostTextMsg(tm_AddToProtocol,Name + '.on_Resize GF',nil);
  (*
   try
     if (ActiveTool <> nil) and (aMessage.Message <> tm_Review) then
        ActiveTool.HandleMessage(aMessage, Handled);
   except
   end;
   if Handled then exit;
   *)
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      //tm_ToolSelected: on_tm_ToolSelected(aMessage, Handled);
      tm_Print:        on_tm_Print(Handled);
      tm_EndOfPrint:   on_tm_EndOfPrint(Handled);
      tm_Open:         on_tm_Open(Handled);
      tm_SaveAs:       on_tm_SaveAs(Handled);
      tm_NewScale:     on_tm_NewScale(Handled);
      tm_ShowAll:      on_tm_ShowAll(Handled);
      tm_RePaint:      on_tm_RePaint(Handled);
      tm_ChangeThematic:  on_tm_CHangeThematic(aMessage, Handled);
      tm_ElementSelected: on_tm_ElementSelected(aMessage, Handled);
      tm_ProjectionCHanged: on_tm_ProjectionChanged(aMessage, Handled);
      tm_LengthCHanged: on_tm_LengthChanged(aMessage, Handled);
      tm_AreaCHanged: on_tm_AreaChanged(aMessage, Handled);
   end;
end;





procedure TGraphicLinkTool.SetViewAll(const Value: boolean);
begin
   if not Value then exit;
   ShowAll;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicLinkTool.On_tm_Cursor(var aMessage: TOrbisCursorEvent;
  var Handled: boolean);
begin
   Cursor := aMessage.Cursor;
end;

function TGraphicLinkTool.GetCursor: TCursor;
begin
  result:= 0;
  if GFrame = nil then exit;
  result := GFrame.Image.Cursor;
end;

procedure TGraphicLinkTool.SetCursor(const Value: TCursor);
begin
   if GFrame = nil then exit;
   if GFrame.Image.Dragging then GFrame.Image.DragCursor := Value
                            else GFrame.Image.Cursor := Value;
end;

procedure TGraphicLinkTool.SetViewByElement(const Value: TElement);
var
   R: TRectEntity;
begin
   if not (Value is TRectEntity) then exit;
   R := TRectEntity(Value);
   Projection.ModelView := R.Rect.AsDoubleRect;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
end;

procedure TGraphicLinkTool.SetShiftView(const Value: string);
var
   DX,DY: double;
begin
   if Idle then exit;
   DX := Projection.ModelView.MaxX - Projection.ModelView.MinX;
   DY := Projection.ModelView.MaxY - Projection.ModelView.MinY;
   if CompareText(Value,'UP')=0 then
      Projection.Center_Y := Projection.Center_Y + DY * 0.75
   else
   if CompareText(Value,'DOWN')=0 then
      Projection.Center_Y := Projection.Center_Y - DY * 0.75
   else
   if CompareText(Value,'LEFT')=0 then
      Projection.Center_X := Projection.Center_X - DX * 0.75
   else
   if CompareText(Value,'RIGHT')=0 then
      Projection.Center_X := Projection.Center_X + DX * 0.75
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

function TGraphicLinkTool.GetText: string;
begin
   result := '';
   {
   if Form = nil then exit;
   result := Form.StatusBar.SimpleText;
   }
end;

procedure TGraphicLinkTool.SetText(const Value: string);
begin
   {
   if Form = nil then exit;
   Form.StatusBar.SimpleText := Value;
   }
end;

procedure TGraphicLinkTool.SetZoom(const Value: string);
begin
   if Idle then exit;
   if CompareText(Value,'IN')=0 then
      Scale:= Scale / 1.5
   else
   if CompareText(Value,'OUT')=0 then
      Scale:= Scale * 1.5
   else
      exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicLinkTool.On_tm_ChangeThematic(var aMessage: TOrbisMessage; var Handled: boolean);
var
   aX,aY,aScale: double;
   T: TTool;
   IT: TInterfaceTool;
begin
   Handled := true;
   aX := Device.Projection.Center_X;
   aY := Device.Projection.Center_Y;
   aScale := Device.Projection.Scale;
   Input := aMessage.Data;
   // pas kvli ShiftImage apod. Ztrc se input a vbec
   try
      T := ActiveTool;
      if T <> nil then
      begin
         if (T is TInterfaceTool) then
         begin
            IT := TInterfaceTool(T);
            if IT.ShareInput then
               IT.Input := aMessage.Data;
         end;
      end;
   except
   end;
   // pas END
   if (aX <> 0) and (aY <> 0) then
   begin
      Device.Projection.Center_X := aX;
      Device.Projection.Center_Y := aY;
      Device.Projection.Scale := aScale;
   end
   else
      PostMsg(tm_ProjectionCHanged,Self,Device.Projection,false);
end;

procedure TGraphicLinkTool.BeginUpdate;
begin
   Idle := true;
   FImageW := GFrame.Image.Width;
   FImageH := GFrame.Image.Height;
end;

procedure TGraphicLinkTool.EndUpdate;
begin
   Idle := false;
   if (FImageW <> GFrame.Image.Width) or
      (FImageH <> GFrame.Image.Height) then
      PostMsg(tm_ReSize, Self, nil, false);
end;

procedure TGraphicLinkTool.SetGFrame(const Value: TOrbisGraphic_Frame);
begin
   FGFrame := Value;
   FGFrame.Tool := Self;
   LinkDeviceToGFrame;
end;

procedure TGraphicLinkTool.SetNewThematic(const Value: TElement);
begin
   PostMsg(tm_ChangeThematic,Self,Value,false);
   Application.ProcessMessages;
end;

procedure TGraphicLinkTool.SetElementForProjection(const Value: TElement);
begin
   if FInput = nil then exit;
   try
      if Value is TRectEntity then
      begin
         Device.Projection.ChangeProjectionByRectangle(TRectEntity(Value).Rect, Device.Page);
      end
      else
      if Value is TCoordinate then
      begin
         Device.Projection.ChangeProjectionByRect(TCoordinate(Value).GetRegion, Device.Page);
      end;
   except
   end;
   if Idle then exit;
   PostMsg(tm_BreakAndReView, self, nil, DontKill);
   PostMsg(tm_ProjectionChanged, self, Projection, DontKill);
end;

procedure TGraphicLinkTool.on_tm_ElementSelected(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   if Assigned(OnSelect) then
      OnSelect(aMessage.Data);
end;

procedure TGraphicLinkTool.SetOnSelect(const Value: TOnElementSelectEvent);
begin
   FOnSelect := Value;
end;

procedure TGraphicLinkTool.SetOnProjectionChanged(const Value: TOnGLTProjectionChanged);
begin
  FOnProjectionChanged := Value;
end;

procedure TGraphicLinkTool.On_tm_ProjectionChanged(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   if Assigned(OnProjectionChanged) then
      OnProjectionChanged(Projection.Center_X,Projection.Center_Y,Projection.Scale,Projection.VirtualScale);
end;

procedure TGraphicLinkTool.SetElementForPattern(const Value: TElement);
var
   GI: TSelectedGeometryInterpret;
   h: boolean;
begin
   GI := TSelectedGeometryInterpret.Create;
   GI.Input := Value;
   GI.Reset(Device);
   GI.DoIt;
   on_tm_Repaint(h);
   Application.ProcessMessages;
   GI.Free;
end;

procedure TGraphicLinkTool.SetOnAreaChanged(
  const Value: TOnMeasureChangedEvent);
begin
  FOnAreaChanged := Value;
end;

procedure TGraphicLinkTool.SetOnInterpretDone(const Value: TOnInterpretDone);
begin
  FOnInterpretDone := Value;
end;

procedure TGraphicLinkTool.SetOnLengthChanged(
  const Value: TOnMeasureChangedEvent);
begin
  FOnLengthChanged := Value;
end;

procedure TGraphicLinkTool.On_tm_AreaChanged(var aMessage: TOrbisMessage;
  var Handled: boolean);
var
   S1,S2: string;
   D,D1: double;
begin
   if Assigned(OnAreaChanged) then
   begin
      if not (aMessage is TOrbisTextMessage) then exit;
      StringAsTwoParams(TOrbisTextMessage(aMessage).Text,':',S1,S2);
      D := StrToFloatDef(S1,0);
      D1 := StrToFloatDef(S2,0);
      OnAreaChanged(D,D1);
   end;
end;

procedure TGraphicLinkTool.On_tm_LengthChanged(var aMessage: TOrbisMessage; var Handled: boolean);
var
   S1,S2: string;
   D,D1: double;
begin
   if Assigned(OnLengthChanged) then
   begin
      if not (aMessage is TOrbisTextMessage) then exit;
      StringAsTwoParams(TOrbisTextMessage(aMessage).Text,':',S1,S2);
      D := StrToFloatDef(S1,0);
      D1 := StrToFloatDef(S2,0);
      OnLengthChanged(D,D1);
   end;
end;

procedure TGraphicLinkTool.SetNoResetReviewByElement(const Value: boolean);
begin
   if Interpret <> nil then
      Interpret.NoResetReviewByElement := Value;
end;



procedure TGraphicLinkTool.EndSizeMove;
begin
   GFrame.EndSizeMove;
end;

procedure TGraphicLinkTool.StartSizeMove;
begin
   GFrame.StartSizeMove;
end;

end.
