unit t_OrbisControlElementsEx;

interface
uses
   ComCtrls,
   Elements,
   t_ControlTool;

type

TOrbis_TreeView = class(TControlTool)
 private
    FSelectedElement: TElement;
    FShowOwners: boolean;
    FStatusText: string;
    FOnSelectScript: string;
    FUpdating: boolean;
    FStopClasses: Array of TClass;
    FUseStopClasses: boolean;
    FStopClassList: string;
    function GetTreeView: TTreeView;
    procedure SetStatusText(const Value: string);
    function GetSelectedElement: TElement;
    procedure SetOnSelectScript(const Value: string);
    procedure SetCheck(const Value: boolean);
    procedure SetUpdating(const Value: boolean);
    procedure SetStopClassList(const Value: string);
    function GetSelectedPath: string;
protected
   procedure SetInput(aInput: TElement); override;
   procedure TreeViewChange(Sender: TObject; Node: TTreeNode);
   procedure TreeViewCollapsed(Sender: TObject;Node: TTreeNode);
   procedure TreeViewExpanded(Sender: TObject;Node: TTreeNode);
   procedure TreeViewExpanding(Sender: TObject;Node: TTreeNode; var AllowExpansion: Boolean);
   procedure ReCheckNode(aNode: TTreeNode);
   procedure CheckNode(aNode: TTreeNode);
public
   destructor Destroy; override;
   procedure CreateControl; override;
   property  TreeView: TTreeView read GetTreeView;
published
   property ShowOwners: boolean read FShowOwners write FShowOwners;
   property StatusText: string read FStatusText write SetStatusText;
   property SelectedElement: TElement read GetSelectedElement stored false;
   property OnSelectScript: string read FOnSelectScript write SetOnSelectScript;
   property Check: boolean write SetCheck stored false;
   property Updating: boolean read FUpdating write SetUpdating;
   property StopClassList: string read FStopClassList write SetStopClassList;
   property SelectedPath: string read GetSelectedPath stored false;
end;


TOrbis_ListColumn = class(TOwnerElement)
  private
    FPropertyName: string;
    FWidth: integer;
    FPropertyType: integer;
    procedure SetPropertyName(const Value: string);
    procedure SetWidth(const Value: integer);
    procedure SetPropertyType(const Value: integer);
published
   property PropertyName: string read FPropertyName write SetPropertyName;
   property PropertyType: integer read FPropertyType write SetPropertyType;
   property Width: integer read FWidth write SetWidth;
end;

TOrbis_ListColumns = class(TOwnerEntity)
  private
    function GetLastColumn: TOrbis_ListColumn;
    procedure SetNextColumnName(const Value: string);
published
   property NextColumnName: string write SetNextColumnName;
   property LastColumn: TOrbis_ListColumn read GetLastColumn;
end;

TOrbis_ListView = class(TControlTool)
private
    FOnSelectScript: string;
    FOnClickScript: string;
    FOnChangeScript: string;
    FChangedItem: TListItem;
    FChangedItemCHecked: boolean;
    FIgnoreChanges: boolean;
    FCHeckByPropertyReverse: boolean;
    FCheckByProperty: boolean;
    FCheckPropertyName: string;
    FColumns: TOrbis_ListColumns;
    FNoAutoNameColumn: boolean;
    FAutoSort: boolean;
    FSortOrder: boolean;
    FSortBy: string;
    FSortByIndex: integer;
    FActualColumn: TOrbis_ListColumn;
    FOnColumnClickScript: string;
   function GetListView: TListView;
    function GetViewStyle: TViewStyle;
    procedure SetViewStyle(const Value: TViewStyle);
    function GetCheckBoxes: boolean;
    procedure SetCheckBoxes(const Value: boolean);
    function GetColumnAutoSize: boolean;
    procedure SetColumnAutoSize(const Value: boolean);
    function GetSelectedElement: TElement;
    procedure SetSelectedElement(const Value: TElement);
    procedure SetOnSelectScript(const Value: string);
    procedure SetOnClickScript(const Value: string);
    procedure SetOnChangeScript(const Value: string);
    procedure SetIgnoreChanges(const Value: boolean);
    function GetChangedElement: TElement;
    procedure SetCheckAll(const Value: boolean);
    procedure SetCheckByProperty(const Value: boolean);
    procedure SetCHeckByPropertyReverse(const Value: boolean);
    procedure SetCheckPropertyName(const Value: string);
    function GetChangedIsChecked: boolean;
    function GetColumns: TOrbis_ListColumns;
    function GetReadOnly: boolean;
    procedure SetNoAutoNameColumn(const Value: boolean);
    procedure SetReadOnly(const Value: boolean);
    procedure SetAutoSort(const Value: boolean);
    procedure SetSortBy(const Value: string);
    procedure SetSortOrder(const Value: boolean);
    procedure SetSortByIndex(const Value: integer);
    procedure SetActualColumn(const Value: TOrbis_ListColumn);
    procedure SetOnColumnClickScript(const Value: string);
protected
   procedure SetInput(aInput: TElement); override;
   procedure OnSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean); virtual;
   procedure OnClick(Sender: TObject); virtual;
   procedure OnChange(Sender: TObject; Item: TListItem; Change: TItemChange);virtual;
   procedure OnChanging(Sender: TObject; Item: TListItem;Change: TItemChange; var AllowChange: Boolean); virtual;
   procedure OnColumnClick(Sender: TObject; Column: TListColumn); virtual;
   property  IgnoreChanges: boolean read FIgnoreChanges write SetIgnoreChanges;
public
   procedure CreateControl; override;
   property  ListView: TListView read GetListView;
published
   property SelectedElement: TElement read GetSelectedElement write SetSelectedElement stored false;
   property ChangedElement: TElement read GetChangedElement;
   property OnSelectScript: string read FOnSelectScript write SetOnSelectScript;
   property OnClickScript: string read FOnClickScript write SetOnClickScript;
   property OnColumnClickScript: string read FOnColumnClickScript write SetOnColumnClickScript;
   property OnChangeScript: string read FOnChangeScript write SetOnChangeScript;
   property ViewStyle: TViewStyle read GetViewStyle write SetViewStyle;
   property CheckBoxes: boolean read GetCheckBoxes write SetCheckBoxes;
   property ColumnAutoSize: boolean read GetColumnAutoSize write SetColumnAutoSize;
   property CheckAll: boolean write SetCheckAll;
   property CheckByProperty: boolean read FCheckByProperty write SetCheckByProperty;
   property CHeckByPropertyReverse: boolean read FCHeckByPropertyReverse write SetCHeckByPropertyReverse;
   property CheckPropertyName: string read FCheckPropertyName write SetCheckPropertyName;
   property CHangedIsChecked: boolean read GetChangedIsChecked;
   property Columns: TOrbis_ListColumns read GetColumns;
   property ReadOnly: boolean read GetReadOnly write SetReadOnly;
   property NoAutoNameColumn: boolean read FNoAutoNameColumn write SetNoAutoNameColumn;
   property AutoSort: boolean read FAutoSort write SetAutoSort;
   property SortByIndex: integer read FSortByIndex write SetSortByIndex;
   property SortBy: string read FSortBy write SetSortBy;
   property SortOrder: Boolean read FSortOrder write SetSortOrder;
   property ActualColumn: TOrbis_ListColumn read FActualColumn write SetActualColumn;
end;

implementation
uses
   Classes,
   Dialogs,
   Forms,
   SysUtils,
 { TODO -oPPK -cOpravit : sputn scriptu je pomoc tm_Run, kter je v m_Proje, to je na hlavu }
   m_Proj,
   e_Class;

{ TOrbis_TreeView }

procedure TOrbis_TreeView.CreateControl;
begin
   Control := TTreeView.Create(Application);
   TreeView.HideSelection := false;
   TreeView.RightClickSelect := true;
   TreeView.OnChange := TreeViewChange;
   TreeView.OnCollapsed := TreeViewCollapsed;
   //TreeView.OnDragOver := TreeViewDragOver;
   TreeView.OnExpanded := TreeViewExpanded;
   TreeVIew.OnExpanding := TreeViewExpanding;
end;

function TOrbis_TreeView.GetTreeView: TTreeView;
begin
   result := nil;
   if Control <> nil then
      result := TTreeView(Control);
end;

procedure TOrbis_TreeView.TreeViewCollapsed(Sender: TObject;Node: TTreeNode);
begin
   Node.DeleteChildren;
   CheckNode(Node);
end;

procedure TOrbis_TreeView.TreeViewExpanded(Sender: TObject;Node: TTreeNode);
var
   E: TEntity;
   El: TElement;
   i: integer;
   N: TTreeNode;
   Envelope: boolean;
begin
   El := TElement(Node.Data);
   if not ShowOwners then
   begin
      if not (TElement(El) is TEntity) then exit;
      E := TEntity(El);
      if E.NoElements then exit;
      TreeView.Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      for i := 1 to E.LastIndex do
      begin
         El := E.Elements[i];
         N := TreeView.Items.AddChildObject(Node, El.Name, El);
         CheckNode(N);
      end;
      TreeView.Items.EndUpdate;
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
      end
      else if El.NoOwners then exit;
      TreeView.Items.BeginUpdate;
      if Node.Count > 0 then CheckNode(Node.Item[0]);
      if not Envelope then
      begin
         for i := 1 to El.OwnersCount-1 do
         begin
            E := El.Owners[i];
            N := TreeView.Items.AddChildObject(Node, E.Name, E);
            CheckNode(N);
         end;
      end;
      TreeView.Items.EndUpdate;
   end;
end;

procedure TOrbis_TreeView.TreeViewExpanding(Sender: TObject;
  Node: TTreeNode; var AllowExpansion: Boolean);
begin
   AllowExpansion := true;
end;

procedure TOrbis_TreeView.TreeViewChange(Sender: TObject; Node: TTreeNode);
var
   N: TTreeNode;
   L: TListItem;
   E: TEntity;
   El: TElement;
   i,j: integer;
   LC: TListColumn;
   //PM: TRegisterPropertyModel;
   S: string;
   Envelope: boolean;
   VS: TViewStyle;
   RS: string;
procedure AddEToList;
var
   ONumber: integer;
begin
   L.Data := E;
   L.Caption := E.Name;
   {
   CheckClassModel(E);
   if ClassModel <> nil then
   begin
      L.ImageIndex := ClassModel.ImageIndex;
      //L.StateIndex := ClassModel.ImageIndex;
   end;
   }
   // podmnka ... nsledujc provdt jen v ppad detailu
   L.SubItems.Add(IntToStr(E.Count));
   if E is TEnvelopeEntity then
   begin
      ONumber := 0;
      if TEnvelopeEntity(E).Owner <> nil then ONumber := 1;
   end
   else ONumber := E.OwnersCount;
   L.SubItems.Add(IntToStr(ONumber));
end;

procedure BeginUpd;
begin
   {
   ListView.Items.BeginUpdate;
   VS := ListView.ViewStyle;
   if VS = vsReport then ListView.ViewStyle := vsList;
   }
end;

procedure EndUpd;
begin
   {
   if ListView.ViewStyle <> VS then ListView.ViewStyle := VS;
   ListView.Items.EndUpdate;
   }
end;

begin
   N := Node;//TreeView.Selected;
   if N.Deleting then exit;
   El := TElement(N.Data);
   if El = nil then
   begin
      FSelectedElement := El;
      exit;
   end;
   if Updating then exit;
   // !!!!!!!!!!!!!!!!
   {
   if W <> nil then
   begin
      W.C.ActiveElement := El;
      W.Run;
   end;
   }
   if GetOrbisClass(El.ClassName) <> nil then RS := ' R+'
                                         else RS := ' R-';
   StatusText := El.Name + ' ('+El.ClassName + ')'+RS;
   FSelectedElement := El;
   if OnSelectScript <> '' then
      PostTextMsg(tm_Run,OnSelectScript,nil);
   {
   if DontShowList then exit;
   if not ShowOwners then
   begin
      BeginUpd;
      ListView.Items.Clear;
      CheckClassModel(El);
      if not CheckSubClassModel then
      begin
         Memo.Lines.Clear;
         ListView.Columns.Clear;
         LC := ListView.Columns.Add;
         LC.Caption := 'Name';
         LC.Width := -2;
         if SubClassModel <> nil then
         begin
            for i := 0 to SubClassModel.PropertyModels.LastIndex do
            begin
               PM := TRegisterPropertyModel(SubClassModel.PropertyModels.Elements[i]);
               Memo.Lines.Add(PM.Name);
               LC := ListView.Columns.Add;
               LC.Caption := PM.Alias;
               LC.Width := -2;
            end;
         end;
      end;
      //ListView.Items.EndUpdate;
      if not (TElement(El) is TEntity) then
      begin
         EndUpd;
         exit;
      end;
      E := TEntity(El);
      if E.NoElements then
      begin
         EndUpd;
         exit;
      end;

      //LIstView.Items.BeginUpdate;
      for i := 0 to E.LastIndex do
      begin
         //Caption := IntToStr(i);
         L := ListView.Items.Add;
         El := E.Elements[i];
         L.Data := El;
         L.Caption := El.Name;
         CheckClassModel(El);
         if ClassModel <> nil then
         begin
            L.ImageIndex := ClassModel.ImageIndex;
            //L.StateIndex := ClassModel.ImageIndex;
         end;
         // podmnka ... nsledujc provdt jen v ppad detailu
         for j := 0 to Memo.Lines.Count - 1 do
         begin
            S := El.GetPropertyStringByName(Memo.Lines.Strings[j]);
            L.SubItems.Add(S);
         end;
      end;
      EndUpd;
   end
   else
   begin
      Envelope := (El is TEnvelopeEntity);
      if Envelope  then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
      end
      else if El.NoOwners then exit;

      BeginUpd;
      ListView.Items.Clear;
      if not Envelope then
      begin
         for i := 0 to El.OwnersCount-1 do
         begin
            L := ListView.Items.Add;
            E := El.Owners[i];
            AddEToList;
         end;
      end
      else
      begin
            L := ListView.Items.Add;
            E := TEnvelopeEntity(El).Owner;
            AddEToList;
      end;
      EndUpd;
   end;
   }
end;

procedure TOrbis_TreeView.CheckNode(aNode: TTreeNode);
var
   E: TEntity;
   El: TElement;
   Envelope: boolean;
   i: integer;
begin
   if aNode.Data = nil then exit;
   El := aNode.Data;
   {
   CheckClassModel(El);
   if ClassModel <> nil then
   begin
     aNode.ImageIndex := ClassModel.ImageIndex;
     aNode.SelectedIndex := ClassModel.ImageSelectedIndex;
   end;
   }
   if not ShowOwners then
   begin
      if not(El is Tentity) then exit;
      if FUseStopClasses then
      for i := 0 to High(FStopClasses) do
      begin
         if El.ClassType = FStopClasses[i] then
            exit;
      end;
      E := TEntity(El);
      if E.NoElements then exit;
      El := E.Elements[0];
      TreeView.Items.AddChildObject(aNode, El.Name, El);
   end
   else
   begin
      Envelope := El is TEnvelopeEntity;
      if Envelope then
      begin
         if TEnvelopeEntity(El).Owner = nil then exit;
         E := TEnvelopeEntity(El).Owner;
      end
      else
      begin
         if El.NoOwners then exit;
         E := El.Owners[0];
      end;
      TreeView.Items.AddChildObject(aNode, E.Name, E);
   end;
end;

procedure TOrbis_TreeView.SetInput(aInput: TElement);
var
   N: TTreeNode;
begin
   Inherited;
   if aInput = nil then
   begin
      TreeView.Items.Clear;
      exit;
   end;
   N := TReeView.Items.AddObject(nil, aInput.Name, aInput);
   CheckNode(N);
   TreeViewChange(nil, N);
end;

procedure TOrbis_TreeView.SetStatusText(const Value: string);
begin
  FStatusText := Value;
end;

function TOrbis_TreeView.GetSelectedElement: TElement;
begin
   result := FSelectedElement;
end;

procedure TOrbis_TreeView.SetOnSelectScript(const Value: string);
begin
   FOnSelectScript := Value;
end;

procedure TOrbis_TreeView.SetCheck(const Value: boolean);
begin
   if not Value then exit;
   TreeView.Items.BeginUpdate;
   try
      RecheckNode(TReeView.Items[0]);
   except
      ShowMessage('TreeView hyba!');
   end;
   TreeView.Items.EndUpdate;
   if TreeView.Selected <> nil then
   if TreeView.Selected.Data <> FSelectedElement then
   begin
      FSelectedElement := TElement(TreeView.Selected.Data);
      if OnSelectScript <> '' then
         PostTextMsg(tm_Run,OnSelectScript,nil);
   end;
end;

procedure TOrbis_TreeView.ReCheckNode(aNode: TTreeNode);
var
   E: TElement;
   En: TEntity;
   i: integer;
   aN,N: TTreeNode;
begin
   E := nil;
   En := nil;
   E := TElement(aNode.Data);
   if (E is TEntity) then En := TEntity(E);
   if aNode.Expanded then
   begin
      if (En = nil) or En.NoElements then
      begin
         aNode.DeleteChildren;
         exit;
      end;
      // zatim jen ubirani elementu;
      for i := 0 to En.LastIndex do
      begin
         E := En.Elements[i];
         if i > aNode.Count - 1 then
         begin
            N := TreeView.Items.AddChildObject(aNode, E.Name, E);
            CheckNode(N);
         end
         else
         begin
            N := aNode.Item[i];
            while (N <> nil) and (N.Data <> E) do
            begin
               aN  := N;
               N := N.getNextSibling;
               aN.Data := nil;
               aN.Delete;
            end;
            if N = nil then
            begin
               N := TreeView.Items.AddChildObject(aNode, E.Name, E);
               CheckNode(N);
            end
            else
               RecheckNode(N);
         end;
      end;
      // a odendn pebytench nod
      if aNode.Count > En.Count then
      begin
         for i := aNode.Count - 1 downto En.Count do
         begin
            N := aNode.Item[i];
            N.Delete;
         end;
      end;
   end
   else
   begin
      aNode.DeleteChildren;
      CheckNode(aNode);
   end;
end;

procedure TOrbis_TreeView.SetUpdating(const Value: boolean);
begin
  FUpdating := Value;
end;

procedure TOrbis_TreeView.SetStopClassList(const Value: string);
var
   SL: TStringList;
   i: integer;
   OC: TClass;
begin
  FStopClassList := Value;
  SL := TStringList.Create;
  SL.CommaText := Value;
  SetLength(FStopClasses,SL.Count);
  for i := 0 to SL.Count - 1 do
  begin
     OC := GetOrbisClass(SL.Strings[i]);
     FStopClasses[i] := OC;
     FUseStopClasses := true;
  end;
  SL.Free;
end;

destructor TOrbis_TreeView.Destroy;
begin
   FStopClasses := nil;
   inherited;
end;

function TOrbis_TreeView.GetSelectedPath: string;
var
   N: TTreeNode;
   AddChar: string;
begin
   result := '';
   AddChar := '';
   N := TreeView.Selected;
   while (N <> nil) do
   begin
      result :=  N.Text + AddChar + result;
      AddChar := '.';
      if N.IsFirstNode then exit;
      N := N.Parent;
   end;
end;

{ TOrbis_ListView }

procedure TOrbis_ListView.CreateControl;
begin
   Control := TListView.Create(Application);
   ListView.HideSelection := false;
   ListView.OnSelectItem := OnSelectItem;
   ListView.OnClick := OnClick;
   ListView.OnChange := OnChange;
   ListView.OnChanging := OnChanging;
   ListView.OnColumnClick := OnColumnClick;
   //ListView
   //ListView.OnCollapsed := TreeViewCollapsed;
   //TreeView.OnDragOver := TreeViewDragOver;
end;

function TOrbis_ListView.GetChangedElement: TElement;
begin
   result := nil;
   if FChangedItem = nil then exit;
   result := TElement(FChangedItem.Data);
end;

function TOrbis_ListView.GetChangedIsChecked: boolean;
begin
   result := false;
   if FChangedItem = nil then exit;
   result := FChangedItem.Checked;

end;

function TOrbis_ListView.GetCheckBoxes: boolean;
begin
   result := false;
   if Control <> nil then
      result := ListView.Checkboxes;
end;

function TOrbis_ListView.GetColumnAutoSize: boolean;
begin
   result := false;
   if Control <> nil then
      if ListView.Columns.Count > 0 then
         result := ListView.Column[0].AutoSize;
end;

function TOrbis_ListView.GetColumns: TOrbis_ListColumns;
var
   e: TElement;
   S: string;
begin
   if FColumns = nil then
   begin
      S := 'COLUMNS';
      e := FindElementName(S);
      if (e is TOrbis_ListColumns) then
      begin
         FColumns := TOrbis_ListColumns(e);
      end
      else
      begin
         FColumns := TOrbis_ListColumns.Create;
         FColumns.Name := S;
         AddElement(FColumns);
      end;
   end;
   result := FColumns;
end;

function TOrbis_ListView.GetListView: TListView;
begin
   result := nil;
   if Control <> nil then
      result := TListView(Control);
end;

function TOrbis_ListView.GetReadOnly: boolean;
begin
   result := ListView.ReadOnly;
end;

function TOrbis_ListView.GetSelectedElement: TElement;
var
   LI: TListItem;
begin
   result := nil;
   if Control = nil then exit;
   LI := ListView.Selected;
   if LI = nil then exit;
   result := TElement(LI.Data);
end;

function TOrbis_ListView.GetViewStyle: TViewStyle;
begin
   result := vsList;
   if Control <> nil then
      result := ListView.ViewStyle;
end;

procedure TOrbis_ListView.OnChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
   if IgnoreChanges then exit;
   if not (CHange = ctState) then exit;
   if FChangedItem <> Item then exit;
   if FChangedItemChecked = Item.Checked then exit;
   if OnChangeScript <> '' then
      PostTextMsg(tm_Run,OnChangeScript,nil);
end;

procedure TOrbis_ListView.OnChanging(Sender: TObject; Item: TListItem; Change: TItemChange; var AllowChange: Boolean);
begin
   if IgnoreChanges then exit;
   if not (CHange = ctState) then exit;
   FChangedItem := Item;
   FChangedItemChecked := Item.Checked;
end;

procedure TOrbis_ListView.OnClick(Sender: TObject);
begin
   if OnClickScript <> '' then
      PostTextMsg(tm_Run,OnClickScript,nil);
end;

procedure TOrbis_ListView.OnColumnClick(Sender: TObject; Column: TListColumn);
var
  aIx: integer;
  Cols: TOrbis_ListColumns;
begin
   if not AutoSort then exit;
   aIx := Column.Index;
   Cols := Columns;
   if (Cols.NoElements or not NoAutoNameColumn) then
      aIx := aIx -1;
   if aIx < 0 then
      ActualColumn := nil
   else
      ActualColumn := TOrbis_ListColumn(cols.Elements[aIx]);
   SortOrder := not SortOrder;
   SortByIndex := aIx;
   if OnColumnClickScript <> '' then
   begin
      PostTextMsg(tm_Run,OnColumnClickScript,nil);
      Application.ProcessMessages;
   end;
end;

procedure TOrbis_ListView.OnSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
   if Selected then
   begin
      if OnSelectScript <> '' then
      begin
         PostTextMsg(tm_Run,OnSelectScript,nil);
         Application.ProcessMessages;
      end;
   end;
end;

procedure TOrbis_ListView.SetActualColumn(const Value: TOrbis_ListColumn);
begin
  FActualColumn := Value;
end;

procedure TOrbis_ListView.SetAutoSort(const Value: boolean);
begin
  FAutoSort := Value;
end;

procedure TOrbis_ListView.SetCheckAll(const Value: boolean);
var
   i: integer;
begin
   for i := 0 to ListView.Items.Count - 1 do
   begin
      ListView.Items.Item[i].Checked := Value;
   end;
end;

procedure TOrbis_ListView.SetCheckBoxes(const Value: boolean);
begin
   if Control <> nil then
      ListView.Checkboxes := Value;
end;

procedure TOrbis_ListView.SetCheckByProperty(const Value: boolean);
begin
  FCheckByProperty := Value;
end;

procedure TOrbis_ListView.SetCHeckByPropertyReverse(const Value: boolean);
begin
  FCHeckByPropertyReverse := Value;
end;

procedure TOrbis_ListView.SetCheckPropertyName(const Value: string);
begin
  FCheckPropertyName := Value;
end;

procedure TOrbis_ListView.SetColumnAutoSize(const Value: boolean);
var
   i: integer;
begin
   if Control = nil then exit;
   if ListView.Columns.Count > 0 then
   for i := 0 to ListView.Columns.Count - 1 do
   begin
      ListView.Column[i].AutoSize := Value;
   end;
end;

procedure TOrbis_ListView.SetIgnoreChanges(const Value: boolean);
begin
  FIgnoreChanges := Value;
end;

procedure TOrbis_ListView.SetInput(aInput: TElement);
var
   En: TEntity;
   e: TElement;
   i,j,j1: integer;
   ListItem: TListItem;
   Column: TListColumn;
   B: boolean;
   Cols: TOrbis_ListColumns;
   Col: TOrbis_ListColumn;
   S: string;
begin
   IgnoreChanges := true;
   Inherited;
   ListView.Items.Clear;
   ListView.Columns.Clear;
   IgnoreChanges := false;
   if aInput = nil then exit;
   if not (aInput is TEntity) then exit;

   // columnDefinition
   Cols := Columns;
   j1 := 1;
   if (Cols.NoElements or not NoAutoNameColumn) then
   begin
      Column := ListView.Columns.Add;
      Column.Caption := 'Name';
      Column.Width := -2;
      j1 := 0;
      //Column.AutoSize := true;
   end;
   if not Cols.NoElements then
   begin
      for i := 0 to Cols.LastIndex do
      begin
         Col := TOrbis_ListColumn(Cols.Elements[i]);
         Column := ListView.Columns.Add;
         Column.Caption := Col.Name;
         if Col.Width <> 0 then
            Column.Width := Col.Width
         else
            Column.Width := -1;
         //Column.AutoSize := true;
      end;
   end;

   IgnoreChanges := true;
   En := TEntity(aInput);
   for i := 0 to En.LastIndex do
   begin
      e := En.Elements[i];
      ListItem := ListView.Items.Add;
      if j1 > 0 then
         ListItem.Caption := E.GetPropertyStringByName(TOrbis_ListColumn(Cols.Elements[0]).PropertyName)
      else
         ListItem.Caption := e.Name;
      ListItem.Data := e;
      if ListView.Checkboxes and CheckByProperty then
      begin
         B := E.GetPropertyBooleanByName(CheckPropertyName);
         if CHeckByPropertyReverse then
            B := not B;
         ListItem.Checked := B;
      end;
      //columns
      if not Cols.NoElements then
      begin
         for j := j1 to Cols.LastIndex do
         begin
            Col := TOrbis_ListColumn(Cols.Elements[j]);
            S := '';
            if Col.PropertyName <> '' then
               S := E.GetPropertyStringByName(Col.PropertyName);
            ListItem.SubItems.Add(S);
         end;
      end;

   end;
   IgnoreChanges := false;
end;

procedure TOrbis_ListView.SetNoAutoNameColumn(const Value: boolean);
begin
  FNoAutoNameColumn := Value;
end;

procedure TOrbis_ListView.SetOnChangeScript(const Value: string);
begin
  FOnChangeScript := Value;
end;

procedure TOrbis_ListView.SetOnClickScript(const Value: string);
begin
  FOnClickScript := Value;
end;

procedure TOrbis_ListView.SetOnColumnClickScript(const Value: string);
begin
  FOnColumnClickScript := Value;
  ListView.ColumnClick := Value <> '';
end;

procedure TOrbis_ListView.SetOnSelectScript(const Value: string);
begin
  FOnSelectScript := Value;
end;

procedure TOrbis_ListView.SetReadOnly(const Value: boolean);
begin
   ListView.ReadOnly := Value;
end;

procedure TOrbis_ListView.SetSelectedElement(const Value: TElement);
var
   LI: TListItem;
begin
   if Control = nil then exit;
   LI := ListView.FindData(0,Value,true,false);
   if LI = nil then exit;
   ListView.Selected := LI;
end;

procedure TOrbis_ListView.SetSortBy(const Value: string);
begin
  FSortBy := Value;
end;

procedure TOrbis_ListView.SetSortByIndex(const Value: integer);
begin
  FSortByIndex := Value;
end;

procedure TOrbis_ListView.SetSortOrder(const Value: boolean);
begin
  FSortOrder := Value;
end;

procedure TOrbis_ListView.SetViewStyle(const Value: TViewStyle);
begin
   if Control <> nil then
      ListView.ViewStyle := Value;
end;

{ TOrbis_ListColumn }

procedure TOrbis_ListColumn.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

procedure TOrbis_ListColumn.SetPropertyType(const Value: integer);
begin
  FPropertyType := Value;
end;

procedure TOrbis_ListColumn.SetWidth(const Value: integer);
begin
  FWidth := Value;
end;

{ TOrbis_ListColumns }

function TOrbis_ListColumns.GetLastColumn: TOrbis_ListColumn;
begin
   result := nil;
   if NoElements then exit;
   result := TOrbis_ListColumn(Elements[LastIndex]);
end;

procedure TOrbis_ListColumns.SetNextColumnName(const Value: string);
var
   Col: TOrbis_ListColumn;
begin
   Col := TOrbis_ListColumn.Create;
   Col.Name := Value;
   AddElement(Col);
end;

INITIALIZATION
   RegisterOrbisClass(TOrbis_TreeView);
   RegisterOrbisClass(TOrbis_ListView);
   RegisterOrbisClass(TOrbis_ListColumn);
   RegisterOrbisClass(TOrbis_ListColumns);
END.

