unit t_SelFil;

interface
uses
// KERNEL
   e_Type,
   Tools;

type

TSelectFileDialog = class(TTool)
private
   FFilter: string;
   FinitialDir: string;
   FFileName: string;
    FCaption: string;
    FResult: boolean;
    FMultiSelect: Boolean;
    FFiles: TStringListElement;
    procedure SetCaption(const Value: string);
    procedure SetResult(const Value: boolean);
    procedure SetMultiSelect(const Value: Boolean);
    function GetFiles: TStringListElement;
public
   procedure DoIt; override;
published
   property Filter: string read FFilter write FFilter;
   property InitialDir: string read FinitialDir write FInitialDir;
   property FileName: string read FFileName write FFileName;
   property Caption: string read FCaption write SetCaption;
   property Result: boolean read FResult write SetResult;
   property MultiSelect: Boolean read FMultiSelect write SetMultiSelect;
   property Files: TStringListElement read GetFiles;
end;

implementation
uses
Dialogs,
Forms,
// KERNEL
Worlds,
t_MainTl,
e_Class,
u_String;

procedure TSelectFileDialog.DoIt;
var
   D: TOpenDialog;
begin
   Result := false;
   if Assigned(TOldMainTool(World.MainTool).On_SelectFile_Event) then
   begin
      result := TOldMainTool(World.MainTool).On_SelectFile_Event(Caption,InitialDir,Filter,FFileName);
      exit;
   end;
   D := TOpenDialog.Create(Application);
   D.Filter := Filter;
   D.InitialDir := InitialDir;
   D.FileName := FileName;
   D.Title := Caption;
   Files.FreeAll;
   if MultiSelect then
      D.Options := D.Options + [ofAllowMultiSelect];
   if D.Execute then
   begin
      FFileName := D.FileName;
      Files.AddStringList(D.Files);
      Result := true;
   end;
   D.Free;
end;

function TSelectFileDialog.GetFiles: TStringListElement;
begin
   if FFiles = nil then
   begin
      FFiles := TStringListElement.Create;
      FFiles.Name := 'FILES';
      AddElement(FFiles);
   end;
   result := FFiles;
end;

procedure TSelectFileDialog.SetCaption(const Value: string);
begin
  FCaption := Value;
end;

procedure TSelectFileDialog.SetMultiSelect(const Value: Boolean);
begin
  FMultiSelect := Value;
end;

procedure TSelectFileDialog.SetResult(const Value: boolean);
begin
  FResult := Value;
end;

initialization
   RegisterOrbisClass(TSelectFileDialog);
end.
