unit e_DelphiSystem;
interface
uses
   Classes,
// KERNEL
   e_Files,
   Elements,
   t_Recurs;
type


TDelphiPackageElement = class;
TDelphiUnitElement = class;
TDelphiProjectElement = class;


TDelphiSystemEntity = class(TOwnerEntity)
protected
   procedure SetNewItem(const Value: string); virtual;
public
   procedure AddItemFile(const Value: string); virtual;
published
   property NewItem: string write SetNewItem stored false;
end;

TDelphiUnits = class(TDelphiSystemEntity)
  private
    function Get_Unit(aName: string): TDelphiUnitElement;
protected
   procedure SetNewItem(const Value: string); override;
public
   function NewUnitName(const aName: string): TDelphiUnitElement;
   property _Unit[aName: string]: TDelphiUnitElement read Get_Unit;
end;

TDelphiProjects = class(TDelphiSystemEntity)
protected
   procedure SetNewItem(const Value: string); override;
end;


TDelphiPackages = class(TDelphiSystemEntity)
private
   FUnits: TDelphiUnits;
   procedure SetUnits(const Value: TDelphiUnits);
   function GetUnits: TDelphiUnits;
   function Get_Package(aName: string): TDelphiPackageElement;
protected
   procedure SetNewItem(const Value: string); override;
public
   function NewPackageName(const aName: string): TDelphiPackageElement;
   property _Package[aName: string]: TDelphiPackageElement read Get_Package;
published
   property Units: TDelphiUnits read GetUnits write SetUnits;
end;

TDelphiUnitElement = class(TFileBasedEntity)
private
   function GetPackageName: string;
public
   procedure LoadFromFile(aFileName: string); override;
published
   property PackageName: string read GetPackageName;
end;

TDelphiPackageElement = class(TFileBasedEntity)
private
   FLoaded: boolean;
   procedure Set_Contains(const Value: TOwnerEntity);
   procedure Set_Requires(const Value: TOwnerEntity);
   function  Get_Contains: TOwnerEntity;
   function  Get_Requires: TOwnerEntity;
   function  GetPackages: TDelphiPackages;
   procedure SetLoaded(const Value: boolean);
    function GetUnits: TDelphiUnits;
public
   procedure LoadFromFile(aFileName: string); override;
   procedure LoadContains(SL: TStringList; var i: integer);
   procedure LoadRequires(SL: TStringList; var i: integer);
published
   property Loaded: boolean read FLoaded write SetLoaded;
   property Packages: TDelphiPackages read GetPackages;
   property Units: TDelphiUnits read GetUnits;
   property _Contains: TOwnerEntity read Get_Contains write Set_Contains;
   property _Requires: TOwnerEntity read Get_Requires write Set_Requires;
end;

TDelphiProjectElement = class(TFileBasedEntity)
end;


TDelphiSystem = class(TFileBasedEntity)
private
   FPackages: TDelphiPackages;
   FProjects: TDelphiProjects;
   FUnits: TDelphiUnits;
    FDirectory: string;
   procedure SetPackages(const Value: TDelphiPackages);
   procedure SetProjects(const Value: TDelphiProjects);
   procedure SetUnits(const Value: TDelphiUnits);
   function GetPackages: TDelphiPackages;
   function GetProjects: TDelphiProjects;
   function GetUnits: TDelphiUnits;
    procedure SetDirectory(const Value: string);
published
   property Directory: string read FDirectory write SetDirectory;
   property Units: TDelphiUnits read GetUnits write SetUnits;
   property Packages: TDelphiPackages read GetPackages write SetPackages;
   property Projects: TDelphiProjects read GetProjects write SetProjects;
end;


TDelphiSystemReader = class(TRecursor)
private
   FIx: integer;
   FDelphiSystem: TDelphiSystem;
    FActualDirectory: string;
   procedure SetDelphiSystem(const Value: TDelphiSystem);
    procedure SetActualDirectory(const Value: string);
protected
   function  QProcessBefore(Element: TElement): boolean; override;
public
   procedure Process(Element: TElement); override;
   property ActualDirectory: string read FActualDirectory write SetActualDirectory;
   property DelphiSystem: TDelphiSystem read FDelphiSystem write SetDelphiSystem;
end;

implementation
uses
   Dialogs,
   Forms,
   SysUtils,
// KERNEL
   e_Class,
   m_Orbis,
   u_String;

const
   c_Interface: string = 'interface';
   c_Implementation: string = 'implementation';
   c_Uses: string = 'uses';
   c_Contains: string = 'Contains';
   c_Requires: string = 'Requires';
   c_Units: string    = 'Units';
   c_End: string      = 'End';

{ TDelphiPackageElement }

function TDelphiPackageElement.GetPackages: TDelphiPackages;
begin
   result := TDelphiPackages(FindOwnerClass(TDelphiPackages));
end;

function TDelphiPackageElement.GetUnits: TDelphiUnits;
var
   P: TDelphiPackages;
begin
   P := Packages;
   if P <> nil then result := Packages.Units
               else result := nil;
end;

function TDelphiPackageElement.Get_Contains: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName(c_Contains));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := c_Contains;
      AddElement(result);
   end;
end;

function TDelphiPackageElement.Get_Requires: TOwnerEntity;
begin
   result := TOwnerEntity(FindElementName(c_Requires));
   if result = nil then
   begin
      result := TOwnerEntity.Create;
      result.Name := c_Requires;
      AddElement(result);
   end;
end;

procedure TDelphiPackageElement.LoadContains(SL: TStringList;
  var i: integer);
var
   Done: boolean;
   S,SS: string;
   Delimiters: TDelimiters;
   Delimiter: char;
   Position: integer;
   SE: TDelphiUnitElement;
   U: TDelphiUnits;
begin
   Done := false;
   repeat
      S := SL.Strings[i];
      Inc(i);
      Position := 1;
      Delimiters := [''''];
      SS := ReadToDelimiterWithoutWS(S, Delimiters, Position,
                                     Delimiter, Position);
      SS := ReadToDelimiterWithoutWS(S, Delimiters, Position,
                                    Delimiter, Position);
      U := Units;
      if U <> nil then SE := Units._Unit[SS]
      else
      begin
         SE := TDelphiUnitElement.Create;
         SE.Name := SS;
      end;
      if SE <> nil then _Contains.AddElement(SE);
      Delimiters := [',',';'];
      SS := ReadToDelimiterWithoutWS(S, Delimiters, Position,
                                    Delimiter, Position);
      if Delimiter = ';' then Done := true;
   until Done or (i >=SL.Count);
end;

procedure TDelphiPackageElement.LoadFromFile(aFileName: string);
var
   SL: TStringList;
   S: string;
   i: integer;
begin
   SL := TStringList.Create;
   Name := ExtractFileName(aFileName);
   SL.LoadFromFile(aFileName);
   i := 0;
   repeat
      S := SL.Strings[i];
      Inc(i);
      S := BezVnejsichMezer(S);
      if CompareText(S,c_Contains) = 0 then LoadContains(SL,i)
      else
      if CompareText(S,c_Requires) = 0 then LoadRequires(SL,i);
   until i >= SL.Count;
   SL.Free;
   Loaded := true;
end;

procedure TDelphiPackageElement.LoadRequires(SL: TStringList;
  var i: integer);
var
   Done: boolean;
   S,SS: string;
   Delimiters: TDelimiters;
   Delimiter: char;
   Position: integer;
   SE: TDelphiPackageElement;
   P: TDelphiPackages;
begin
   Done := false;
   repeat
      S := SL.Strings[i];
      Inc(i);
      Position := 1;
      Delimiters := [',',';'];
      SS := ReadToDelimiterWithoutWS(S, Delimiters, Position,
                                     Delimiter, Position);
      if Delimiter = ';' then Done := true;
      if SS <> '' then
      begin
         P := Packages;
         if P <> nil then SE := Packages._Package[SS]
         else
         begin
            SE := TDelphiPackageElement.Create;
            SE.Name := SS;
         end;
         if SE <> nil then _Requires.AddElement(SE);
      end;
   until Done or (i >=SL.Count);
end;

procedure TDelphiPackageElement.SetLoaded(const Value: boolean);
begin
  FLoaded := Value;
end;


procedure TDelphiPackageElement.Set_Contains(const Value: TOwnerEntity);
var
   e: TElement;
begin
   e := FindElementName(c_Contains);
   if e <> nil then FreeElement(e);
   AddElement(Value);
end;

procedure TDelphiPackageElement.Set_Requires(const Value: TOwnerEntity);
var
   e: TElement;
begin
   e := FindElementName(c_Requires);
   if e <> nil then FreeElement(e);
   AddElement(Value);
end;

{ TDelphiPackages }


function TDelphiPackages.GetUnits: TDelphiUnits;
begin
   result := FUnits;
end;

function TDelphiPackages.Get_Package(aName: string): TDelphiPackageElement;
var
   S: string;
begin
   S := aName;
   S := ChangeFileExt(S,'.dpk');
   result := TDelphiPackageElement(FindElementName(S));
   if result = nil then result := NewPackageName(S);
end;

function TDelphiPackages.NewPackageName(
  const aName: string): TDelphiPackageElement;
begin
   result := TDelphiPackageElement.Create;
   result.Name := aName;
   AddElement(result);
end;

procedure TDelphiPackages.SetNewItem(const Value: string);
var
   E: TDelphiPackageElement;
begin
   E := TDelphiPackageElement.Create;
   AddElement(E);
   E.Name := ExtractFileName(Value);
   E.FileName := Value;
end;

procedure TDelphiPackages.SetUnits(const Value: TDelphiUnits);
begin
   FUnits := Value;
end;



{ TDelphiSystem }

function TDelphiSystem.GetPackages: TDelphiPackages;
begin
   if FPackages = nil then
   begin
      FPackages := TDelphiPackages.Create;
      FPackages.Name := 'Delphi Packages';
      FPackages.Units := Units;
      AddElement(FPackages);
   end;
   result := FPackages;
end;

function TDelphiSystem.GetProjects: TDelphiProjects;
begin
   if FProjects = nil then
   begin
      FProjects := TDelphiProjects.Create;
      FProjects.Name := 'Delphi Projects';
      AddElement(FProjects);
   end;
   result := FProjects;
end;

function TDelphiSystem.GetUnits: TDelphiUnits;
begin
   if FUnits = nil then
   begin
      FUnits := TDelphiUnits.Create;
      FUnits.Name := 'Delphi Units';
      AddElement(FUnits);
   end;
   result := FUnits;
end;

procedure TDelphiSystem.SetDirectory(const Value: string);
var
   D: TDirectoryElement;
   DSR: TDelphiSystemReader;
   i: integer;
begin
   FDirectory := Value;
   DSR := TDelphiSystemReader.Create;
   DSR.StartProgress('Delphi system - ten sloky:',0,0);
   DSR.ProgressText := Value;
   Application.ProcessMessages;
   D := TDirectoryElement.Create;
   //PostTextMsg(tm_ShowText,'Reading directory: '+ Value,nil);
   D.FileName := Value;
   DSR.EndProgress;
   //AddElement(D);
   DSR.DelphiSystem := Self;
   DSR.Input := D;
   i := D.WholeCount;
   DSR.StartProgress('Delphi system - analza sloky '+Value+ ' ('+IntToStr(i)+')Soubor',0,i);
   DSR.DoIt;
   D.Free;
   DSR.EndProgress;
   DSR.Free;
end;

procedure TDelphiSystem.SetPackages(const Value: TDelphiPackages);
begin
   if FPackages <> nil then
   begin
      FreeElement(FPackages);
      FPackages := nil;
   end;
   FPackages := Value;
end;

procedure TDelphiSystem.SetProjects(const Value: TDelphiProjects);
begin
   if FProjects <> nil then
   begin
      FreeElement(FProjects);
      FProjects := nil;
   end;
   FProjects := Value;
end;

procedure TDelphiSystem.SetUnits(const Value: TDelphiUnits);
begin
   if FUnits <> nil then
   begin
      FreeElement(FUnits);
      FUnits := nil;
   end;
   FUnits := Value;
end;

{ TDelphiSystemReader }

procedure TDelphiSystemReader.Process(Element: TElement);
var
   S: string;
   FE: TFileElement;
begin
   if not (Element is TFileElement) then exit;
   FE := TFileElement(Element);
   inc(FIx);
   if FE.AttrDirectory then
   begin
      ActualDirectory := FE.FileName;
      //PostTextMsg(tm_ShowText,ActualDirectory,nil);
      Progress := FIx;
      ProgressText := ActualDirectory;
      Application.ProcessMessages;
      //PostTextMsg(tm_AddToProtocol,'Scaning directory: '+ActualDirectory,nil);
      //PostTextMsg(tm_AddToProtocol,'directory: '+ActualDirectory,nil);
      exit;
   end;

   S := FE.FileExt;
   //PostTextMsg(tm_AddToProtocol,'   File: '+FE.Name,nil);
   S := UpperCase(S);
   if S = 'PAS' then
   begin
      //PostTextMsg(tm_AddToProtocol,'Directory: '+ActualDirectory+ ' adding Unit '+FE.Name,nil);
      DelphiSystem.Units.AddItemFile(FE.FileName);
      //ProgressText := ActualDirectory+ ' Unit:.....'+FE.Name;
   end
   else
   if S = 'DPK' then
   begin
      //PostTextMsg(tm_AddToProtocol,'Directory: '+ActualDirectory+ ' adding Package '+FE.Name,nil);
      DelphiSystem.Packages.AddItemFile(FE.FileName);
      //ProgressText := ActualDirectory+ ' Package:..'+FE.Name;
   end
   else
   if S = 'DPR' then
   begin
      //PostTextMsg(tm_AddToProtocol,'Directory: '+ActualDirectory+ ' adding Project '+FE.Name,nil);
      DelphiSystem.Projects.AddItemFile(FE.FileName);
      //ProgressText := ActualDirectory+ ' Project:..'+FE.Name;
   end;
   Application.ProcessMessages;
end;

function TDelphiSystemReader.QProcessBefore(Element: TElement): boolean;
begin
   result := true;
end;

procedure TDelphiSystemReader.SetActualDirectory(const Value: string);
begin
  FActualDirectory := Value;
end;

procedure TDelphiSystemReader.SetDelphiSystem(const Value: TDelphiSystem);
begin
  FDelphiSystem := Value;
end;

{ TDelphiSystemEntity }

procedure TDelphiSystemEntity.AddItemFile(const Value: string);
var
   i: integer;
   FBE: TFileBasedEntity;
   E: TElement;
   S: string;
   ResultElement: TFileBasedEntity;
begin
   S := ExtractFileName(Value);
   ResultElement := nil;
   for i := 0 to lastIndex do
   begin
      E := Elements[i];
      if not (E is TFileBasedEntity) then Continue;
      FBE := TFileBasedEntity(E);
      if CompareText(S,FBE.Name) <> 0 then Continue;
      if FBE.FileName = '' then
      begin
         FBE.FileName := Value;
         ResultElement := FBE;
         Break;
      end
      else
      begin
         if CompareText(Value,FBE.FileName) = 0 then
         begin
            ResultElement := FBE;
            Break;
         end;
      end;
   end;
   if ResultElement = nil then
   begin
      SetNewItem(Value);
   end;
end;



procedure TDelphiSystemEntity.SetNewItem(const Value: string);
begin

end;



{ TDelphiUnits }

function TDelphiUnits.Get_Unit(aName: string): TDelphiUnitElement;
var
   S: string;
begin
   S := aName;
   result := TDelphiUnitElement(FindElementName(S));
   if result = nil then result := NewUnitName(S);
end;

function TDelphiUnits.NewUnitName(const aName: string): TDelphiUnitElement;
begin
   result := TDelphiUnitElement.Create;
   result.Name := aName;
   AddElement(result);
end;

procedure TDelphiUnits.SetNewItem(const Value: string);
var
   E: TDelphiUnitElement;
begin
   E := TDelphiUnitElement.Create;
   AddElement(E);
   E.Name := ExtractFileName(Value);
   E.FileName := Value;
end;

{ TDelphiProjects }

procedure TDelphiProjects.SetNewItem(const Value: string);
var
   E: TDelphiProjectElement;
begin
   E := TDelphiProjectElement.Create;
   AddElement(E);
   E.Name := ExtractFileName(Value);
   E.FileName := Value;
end;

{ TDelphiUnitElement }

function TDelphiUnitElement.GetPackageName: string;
var
   P: TDelphiPackageElement;
begin
   result := '';
   P := TDelphiPackageElement(FindClassInOwners(TDelphiPackageElement));
   if P <> nil then result := P.Name;
end;

procedure TDelphiUnitElement.LoadFromFile(aFileName: string);
begin
   inherited;

end;

INITIALIZATION
   RegisterOrbisClass(TDelphiSystem);
   RegisterOrbisClass(TDelphiUnits);
   RegisterOrbisClass(TDelphiPackages);
   RegisterOrbisClass(TDelphiProjects);
   RegisterOrbisClass(TDelphiUnitElement);
   RegisterOrbisClass(TDelphiPackageElement);
   RegisterOrbisClass(TDelphiProjectElement);
END.
