unit E_device;
{
[Unit_Description]
Description=Tdy zazen ORBISu
Admin=Petr
TechTopic=Devices
Topic=Zobrazen dat
Version=1.0
}

interface

uses
   Wintypes,
   Forms,
   Vcl.Graphics,
   Printers_f,
   // KERNEL
   Elements,
   // GLIB
   g_Elems,
   // DEVICE
   e_Page,
   e_Proje;


const
  SizeOfInch = 25.4; {mm}
type
  TLengthItem = integer;
  TLengthArray = array[0..10000]of TLengthItem;


TBorderBuffer = class(TRectangle)
{ star se o vytvoen a oezn buffru hranice.
  Nastavuj se property MinX .. MaxY, RectEntity a ClipRect. Ped pouitm
  Buffer .. PointNo zavolat ClipIt }
private
  FProjection: TPageProjection;
  FRectEntity: TRectEntity;
  FPointNo: integer;
  FClipRect: TRect;
  FMinPointNo: integer;
  procedure SetEntity(aVal: TRectEntity);
public
  FBuffer: pointer;
  FBufferSize: integer;
  destructor Destroy; override;
  procedure ClipIt;

  property Buffer: pointer read FBuffer write FBuffer;
  property ClipRect: TRect write FClipRect;
published
  property Projection: TPageProjection write FProjection;
  property RectEntity: TRectEntity write SetEntity;
  property BufferSize: integer read FBufferSize write FBufferSize;
  property PointNo: integer read FPointNo write FPointNo;
  property MinPointNo: integer read FMinPointNo write FMinPointNo;
end;



TDevice = class(TOwnerEntity)
    //FAngle: double;

 protected
    FANWCS: byte;
    FPage: T_Page;
    FProjection: TPageProjection;
    FVisible: boolean;
    function GetRealWidth: double;
    function GetRealHeight: double;
    procedure DoAction(aAction: string);

    function GetBrush: TBrush; virtual;
    function GetFont: TFont; virtual;
    function GetPen: TPen; virtual;
    function GetPenWidth: double;
    function GetPixel(X, Y: double): TColor; virtual;
    function GetHandle: HDC; virtual;
    procedure SetBrush(Value: TBrush); virtual;
    procedure SetFont(Value: TFont); virtual;
    procedure SetPen(Value: TPen); virtual;
    procedure SetPenWidth(Value: double); virtual;
    { nastaven tlouky ry v 1/1000 mm }
    procedure SetPixel(X, Y: double; Value: TColor); virtual;
    procedure SetVisible(aVisible: boolean); virtual;
    procedure SetCaption(aCaption: string); virtual;
    function GetTextColor: integer; virtual;
    procedure SetTextColor(const Value: integer); virtual;
 public
    constructor Create; override;
    procedure Clear; virtual;
    // ppk 10.2009
    procedure TransparencyBegin(Value: integer); virtual;
    procedure TransparencyEnd; virtual;
    procedure TransformIn(var X, Y: double);
    procedure TransformOut(var X, Y: double);
    procedure FillRect(const Rect: TDoubleRect); virtual;
    procedure FrameRect(const Rect: TDoubleRect); virtual;

    procedure WriteText(aText: string); virtual;
    procedure WriteLnText(aText: string); virtual;

    procedure SetTextOrientation(aOrientation: double); virtual;
    procedure LineTo(X, Y: double); virtual;
    procedure MoveTo(X, Y: double); virtual;
    procedure LineToDevice(X, Y: integer); virtual;
    procedure MoveToDevice(X, Y: integer); virtual;
    procedure PrepareForPolyPolygon(const Entita: TEntity;
              var PCoords,PLengths: pointer; var CrdSize,LNumber: integer); virtual;
    procedure Polyline(const Entita: TEntity); virtual;
    procedure Polygon(const Entita: TEntity); virtual;
    procedure PolyPolygon(const Entita: TEntity); virtual;
    procedure Rectangle(X1, Y1, X2, Y2: double); virtual;
    procedure TextOut(X, Y: double; const Text: string); virtual;
    procedure TextOutDevice(X, Y: integer; const Text: string); virtual;
    function  TextHeight(const Text: string): Integer; virtual;
    function  TextWidth(const Text: string): Integer; virtual;
    function  TextModelHeight(const Text: string): double; virtual;
    function  TextModelWidth(const Text: string): double; virtual;
    procedure Draw(X, Y: double; Graphic: Vcl.Graphics.TGraphic); virtual;
    procedure StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); virtual;
    procedure DrawDevice(X, Y: integer; Graphic: Vcl.Graphics.TGraphic); virtual;
    procedure StretchDrawDevice(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); virtual;
    procedure Reset; virtual;
    procedure Close; virtual;
    procedure Adjust; virtual;
    procedure SetupPage(Sender: TObject); virtual;
  { properties }
    property Page: T_Page read FPage;
    property Projection: TPageProjection read FProjection;
    property RealWidth: double read GetRealWidth;
    property RealHeight: double read GetRealHeight;
    property Pixels[X, Y: double]: TColor read GetPixel write SetPixel;
    property Brush: TBrush read GetBrush write SetBrush;
    property Font: TFont read GetFont write SetFont;
    property Pen: TPen read GetPen write SetPen;
    property PenWidth: double write SetPenWidth;
    property ANWCS: byte read FANWCS write FANWCS;
    property Visible: boolean read FVisible write SetVisible;
    property Caption: string write SetCaption;
    property Handle: HDC read GetHandle;
    property Action: string write DoAction;
    property TextColor: integer read GetTextColor write SetTextColor;
end;


TDevice_Device = class(TDevice)
  private
    FDevice: TDevice;
    procedure SetDevice(const Value: TDevice); // device for encapsulation of standard device
    //FAngle: double;

 protected
    function GetBrush: TBrush; override;
    function GetFont: TFont; override;
    function GetPen: TPen; override;
    function GetPixel(X, Y: double): TColor; override;
    function GetHandle: HDC; override;
    procedure SetBrush(Value: TBrush); override;
    procedure SetFont(Value: TFont); override;
    procedure SetPen(Value: TPen); override;
    { nastaven tlouky ry v 1/1000 mm }
    procedure SetPixel(X, Y: double; Value: TColor); override;
    procedure SetVisible(aVisible: boolean); override;
    procedure SetCaption(aCaption: string); override;
    function GetTextColor: integer; override;
    procedure SetTextColor(const Value: integer); override;
 public
    constructor Create; override;
    procedure Clear; override;
    procedure FillRect(const Rect: TDoubleRect); override;
    procedure FrameRect(const Rect: TDoubleRect); override;

    procedure WriteText(aText: string); override;
    procedure WriteLnText(aText: string); override;

    procedure SetTextOrientation(aOrientation: double); override;
    procedure LineTo(X, Y: double); override;
    procedure MoveTo(X, Y: double); override;
    procedure LineToDevice(X, Y: integer); override;
    procedure MoveToDevice(X, Y: integer); override;
    procedure PrepareForPolyPolygon(const Entita: TEntity;
              var PCoords,PLengths: pointer; var CrdSize,LNumber: integer); override;
    procedure Polyline(const Entita: TEntity); override;
    procedure Polygon(const Entita: TEntity); override;
    procedure PolyPolygon(const Entita: TEntity); override;
    procedure Rectangle(X1, Y1, X2, Y2: double); override;
    procedure TextOut(X, Y: double; const Text: string); override;
    procedure TextOutDevice(X, Y: integer; const Text: string); override;
    function  TextHeight(const Text: string): Integer; override;
    function  TextWidth(const Text: string): Integer; override;
    function  TextModelHeight(const Text: string): double; override;
    function  TextModelWidth(const Text: string): double; override;
    procedure Draw(X, Y: double; Graphic: Vcl.Graphics.TGraphic); override;
    procedure StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure DrawDevice(X, Y: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure StretchDrawDevice(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure Reset; override;
    procedure Close; override;
    procedure Adjust; override;
    procedure SetupPage(Sender: TObject); override;
published
    property Device: TDevice read FDevice write SetDevice;
end;


TDevices = class(TOwnerEntity)
 protected
   function     GetDevice(Index: longint): TDevice;
   procedure    PutDevice(Index: longint; Item: TDevice);
 public
   property Device[Index: longint]: TDevice read GetDevice write PutDevice;
end;



TCanvasDevice = class(TDevice)
  private
    FNoPageSetup: boolean;
    procedure SetNoPageSetup(const Value: boolean);
 protected
    FOrientation: double;
    FCanvas: TCanvas;
    ox,oy: longInt;
    function GetBrush: TBrush; override;
    function GetFont: TFont; override;
    function GetPen: TPen; override;
    function GetPixel(X, Y: double): TColor; override;
    function GetHandle: HDC; override;
    procedure SetBrush(Value: TBrush); override;
    procedure SetFont(Value: TFont); override;
    procedure SetPen(Value: TPen); override;
    procedure SetPixel(X, Y: double; Value: TColor); override;
    procedure SetVisible(aVisible: boolean); override;
    procedure SetCanvas(aCanvas: TCanvas); virtual;
    function GetTextColor: integer; override;
    procedure SetTextColor(const Value: integer); override;
 public
    procedure SetTextOrientation(aOrientation: double); override;
    procedure Clear; override;
    procedure Reset; override;
    procedure FillRect(const Rect: TDoubleRect);  override;
    procedure FrameRect(const Rect: TDoubleRect);  override;
    procedure LineTo(X, Y: double);  override;
    procedure MoveTo(X, Y: double);  override;
    procedure LineToDevice(X, Y: integer);  override;
    procedure MoveToDevice(X, Y: integer);  override;
    procedure FillBuffer(Entita: TEntity;
                         var Buffer: pointer;
                         var BufferSize: integer;
                         var PointNo: integer);
    procedure PrepareForPolyPolygon(const Entita: TEntity;
              var PCoords,PLengths: pointer; var CrdSize,LNumber: integer); override;
    procedure Polyline(const Entita: TEntity); override;
    procedure Polygon(const Entita: TEntity);  override;
    procedure PolyPolygon(const Entita: TEntity);  override;
    procedure Rectangle(X1, Y1, X2, Y2: double);  override;
    function  TextHeight(const Text: string): Integer;  override;
    function  TextModelHeight(const Text: string): double; override;
    procedure TextOut(X, Y: double; const Text: string);  override;
    procedure TextOutDevice(X, Y: integer; const Text: string); override;
    function  TextWidth(const Text: string): Integer;  override;
    function  TextModelWidth(const Text: string): double; override;
    procedure Draw(X, Y: double; Graphic: Vcl.Graphics.TGraphic); override;
    procedure StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure DrawDevice(X, Y: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure StretchDrawDevice(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;
    procedure SetupPage(Sender: TObject); override;

    property  Canvas: TCanvas read FCanvas write SetCanvas;
    property  NoPageSetup: boolean read FNoPageSetup write SetNoPageSetup;
    property CurX: longInt read ox write ox;
    property CurY: longInt read oy write oy;
end;

TBitmapDevice = class(TCanvasDevice)
private
   FBitmap: TBitmap;
    FKillBitmap: boolean;
   procedure SetBitmap(const Value: Vcl.Graphics.TBitmap);
    procedure SetKillBitmap(const Value: boolean);
public
   destructor Destroy; override;
   property Bitmap: TBitmap read FBitmap write SetBitmap;
   property KillBitmap: boolean read FKillBitmap write SetKillBitmap;
end;


TileRecord = record
   TileX:   integer;
   TileY:   integer;
   DeviceX: integer;
   DeviceY: integer;
   X,Y:     double;
end;


TTileDevice = class(TBitmapDevice)
private
   FDevice: TDevice;
   FLastTile: boolean;
   FTilesCount: integer;
   FTile: integer;
    FDeviceY: integer;
    FDeviceX: integer;
   procedure SetDevice(const Value: TDevice); virtual;
   procedure SetLastTile(const Value: boolean);
   procedure SetTilesCount(const Value: integer);
   procedure SetTile(const Value: integer);
    procedure SetDeviceY(const Value: integer);
    procedure SetDeviceX(const Value: integer);
protected
   property  Tile: integer read FTile write SetTile;
   property  DeviceY: integer read FDeviceY write SetDeviceY;
   property  DeviceX: integer read FDeviceX write SetDeviceX;
public
   procedure Reset; override;
   procedure Close; override;
   procedure ReFreshDevice; virtual;
   procedure TransparencyBegin(Value: integer); override;
   procedure TransparencyEnd; override;
   procedure NextTile; virtual;
   //***
   property Device: TDevice read FDevice write SetDevice;
   property TilesCount: integer read FTilesCount write SetTilesCount;
   property LastTile: boolean read FLastTile write SetLastTile;
end;


TTransparentTileDevice = class(TTileDevice)
private
    FBitmapT: TBitmap;
    FTransparencyMode: boolean;
    FTransparency: integer;
    Tiles: Array of TileRecord;
    FTileY: integer;
    FTileX: integer;
    FFrame: TDoubleRect;
    FByFrame: boolean;
    FStartY: integer;
    FStartX: integer;
    procedure SetDevice(const Value: TDevice); override;
    procedure SetBitmapT(const Value: Vcl.Graphics.TBitmap);
    procedure SetTransparency(const Value: integer);
    procedure SetTransparencyMode(const Value: boolean);
    procedure SetTileX(const Value: integer);
    procedure SetTileY(const Value: integer);
    procedure SetFrame(const Value: TDoubleRect);
    procedure SetFrameElement(const Value: TElement);
    procedure SetByFrame(const Value: boolean);
    procedure SetStartX(const Value: integer);
    procedure SetStartY(const Value: integer);
protected
    property StartX: integer read FStartX write SetStartX;
    property StartY: integer read FStartY write SetStartY;
    property TileX: integer read FTileX write SetTileX;
    property TileY: integer read FTileY write SetTileY;
    property Frame: TDoubleRect read FFrame write SetFrame;
    property ByFrame: boolean read FByFrame write SetByFrame;
public
   destructor Destroy; override;
   procedure  Close; override;
   procedure  ReFreshDevice; override;
   procedure  TransparencyBegin(Value: integer); override;
   procedure  TransparencyEnd; override;
   procedure  NextTile; override;
   property   Transparency: integer read FTransparency write SetTransparency;
   property   TransparencyMode: boolean read FTransparencyMode write SetTransparencyMode;
   property   BitmapT: TBitmap read FBitmapT write SetBitmapT;
   property   FrameELement: TElement write SetFrameElement;
end;

TFormDevice = class(TCanvasDevice)
 protected
   FForm: TForm;
   procedure SetVisible(aVisible: boolean); override;
   procedure SetCaption(aCaption: string); override;
 public
   constructor Create; override;
   destructor  Destroy; override;
   procedure CreateForm; virtual;

   property Form: TForm read FForm;
end;

{ Dekompozice tluste linie }
{ ************************ }
{ CeloLinie }
{ ********* }
TLineFront = record
   Center : TPoint;   { Bod ridici cary }
   Left   : TPoint;   { Levy obrysovy bod }
   Right  : TPoint;   { Pravy obrysovy bod }
end;

{ Zaznam tluste linie }
{ ******************* }
TThickLine = record
   A: TLineFront;
   B: TLineFront;
end;




TPrintDevice = class(TCanvasDevice)
private
   FRLO: boolean;
    FExternalPrint: boolean;
    procedure SetExternalPrint(const Value: boolean);
protected
   procedure BeginDocToFile(aFileName: string);
   procedure SetCaption(aCaption: string); override;
   function  GetOrientation: TPrinterOrientation;
public
   constructor CreateExternalPrint; virtual;
   constructor Create; override;
   constructor CreateTitle(const aTitle: string); virtual;
   destructor  Destroy; override;
   procedure Reset; override;
   procedure Close; override;
   procedure LineTo(X, Y: double);  override;
   procedure MoveTo(X, Y: double);  override;
   procedure LineToDevice(X, Y: integer);  override;
   procedure MoveToDevice(X, Y: integer);  override;
   procedure ThickLine(x1,y1,x2,y2: integer);
   procedure StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;
   procedure StretchDrawDevice(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic); override;

   property  ExternalPrint: boolean read FExternalPrint write SetExternalPrint;
   property  Orientation: TPrinterOrientation read GetOrientation;
   property  RasterizeLinesByOrbis: boolean read FRLO write FRLO default false;
end;


implementation

uses
   Classes,
   Dialogs,
   SysUtils,
   WinProcs,
   JPEG,
//{$IFNDEF WIN64}
//   GIFImage,
//{$ENDIF}
   vcl.imaging.GIFImg,
   vcl.imaging.pngimage,
   {!T_Main,}
   // GLIB
   AnalInt,
   ClipArea,
   MathLib,
   m_Orbis,
   e_Memory,
   e_Type,
   u_String,
   u_Type,
   Worlds;


function CreateCoordEntity(const Value: TEntity; aDevice: TDevice): TEntity;
var
   i: integer;
   CC,CCn: T2DCoordinate;
   E: TElement;
   aX,aY: double;
begin
   result := TRectEntity.Create;
   if Value = nil then exit;
   if Value.NoElements then exit;
   for i := 0 to Value.LastIndex do
   begin
      E := Value.Elements[i];
      if not (E is T2DCoordinate) then Continue;
      CC := T2DCoordinate(E);
      CCn := T2DCoordinate.Create;
      aX := CC.x;
      aY := CC.y;
      aDevice.TransformIn(aX,aY);
      CCn.x := aX;
      CCn.y := aY;
      TRectEntity(result).AddAndUpdateRect(CCn);
   end;
end;

destructor TBorderBuffer.Destroy;
begin
  if Buffer <> nil then freeMem(Buffer, BufferSize);
  inherited Destroy;
end;



procedure TBorderBuffer.SetEntity(aVal: TRectEntity);
var
  I: integer;
  Point: TPoint;
  X,Y,MaxV,MinV: integer;
begin
  MaxV := High(Integer);
  MinV := Low(Integer);
  FRectEntity:= aVal;
  Buffer:= nil;
  BufferSize:= aVal.Count * sizeOf(TPoint);
  if BufferSize > MaxBufferLen then exit;
  {Zaplneni buffru dle Listu}
  GetMem(FBuffer, BufferSize);

  PointNo:= 0;
  for I:= 0 to aVal.Count-1 do
  begin
    if TElement(aVal.Elements[I]) is T2DCoordinate then
    begin
      X := FProjection.GetPageX(T2DCoordinate(aVal.Elements[I]).X);
      Y := FProjection.GetPageY(T2DCoordinate(aVal.Elements[I]).Y);
      if X > MaxV then X := MaxV
         else if X < MinV then X := MinV;
      if Y > MaxV then Y := MaxV
         else if Y < MinV then Y := MinV;
      Point.X := X;
      Point.Y := Y;
      if (I=0) or
         ((I>0) and
          ((Point.X <> TMaxBuffer(Buffer^)[PointNo-1].X) or
           (Point.Y <> TMaxBuffer(Buffer^)[PointNo-1].Y))) then
      begin
        TMaxBuffer(Buffer^)[PointNo]:= Point;
        inc(FPointNo)
      end
    end
  end;
  if MinPointNo = 0 then
     MinPointNo:= 3;
  if PointNo < MinPointNo then
  begin
    freeMem(FBuffer, BufferSize);
    FBuffer:= nil;
    BufferSize:= 0;
    FPointNo:= 0;
  end;
end;




procedure TBorderBuffer.ClipIt;
var
  Orezavac: TOrezavacPloch;

begin
  if Buffer = nil then exit;

//O.S. 9.9.89  with FProjection.ModelView do
  if FRectEntity.IsInsideRegion(MinX, MinY, MaxX, MaxY) then
     exit;

  Orezavac := TOrezavacPloch.Create(FClipRect, Buffer, BufferSize, PointNo);
  Orezavac.Orezavej;
  Orezavac.DejKopiiPoleVelikostAPocet(FBuffer, FBufferSize, FPointNo);
  Orezavac.Free;
end;




{ TDevice }

constructor TDevice.Create;
begin
   inherited Create;
   FPage := T_Page.Create;
   AddElement(Page);
   FProjection := TPageProjection.Create;
   AddElement(FProjection);
end;

procedure TDevice.Adjust;
begin

end;

procedure TDevice.Clear;
begin

end;

procedure TDevice.Close;
begin
end;


procedure TDevice.DoAction(aAction: string);
begin
  try
  if CompareText(aAction, 'ADJUST') = 0 then
     Adjust
  else
  if CompareText(aAction, 'CLEAR') = 0 then
     Clear
  else
  if CompareText(aAction, 'CLOSE') = 0 then
     Close
  else
  if CompareText(aAction, 'RESET') = 0 then
     Reset
  except
    on E:Exception do
       ShowMessage('Device "' + Name + '"' + #13 + #10 +
                   E.Message)
  end
end;



procedure TDevice.TransformIN(var X, Y: double);
var
  R, Alfa: double;
begin
  try
  if Projection.Angle <> 0 then
  begin
    R:= sqrt(sqr(X-Projection.RotateCenter_X) + sqr(Y-Projection.RotateCenter_Y));
    Alfa:= Atan2M(Y-Projection.RotateCenter_Y, X-Projection.RotateCenter_X);
    X:= Projection.RotateCenter_X + R*sin(Projection.Angle + Alfa);
    Y:= Projection.RotateCenter_Y + R*cos(Projection.Angle + Alfa);
  end
  except
  end
end;

procedure TDevice.TransformOut(var X, Y: double);
var
  R, Alfa: double;
begin
  try
  if Projection.Angle <> 0 then
  begin
    R:= sqrt(sqr(X-Projection.RotateCenter_X) + sqr(Y-Projection.RotateCenter_Y));
    //Alfa:= Atan2M(X-Projection.RotateCenter_X, Y-Projection.RotateCenter_Y);
    Alfa:= Atan2M(Y-Projection.RotateCenter_Y, X-Projection.RotateCenter_X);
    X:= Projection.RotateCenter_X + R*sin(Alfa-Projection.Angle);
    Y:= Projection.RotateCenter_Y + R*cos(Alfa-Projection.Angle);
  end
  except
  end
end;



function TDevice.GetRealWidth: double; { m }
begin
  result:= 0;
  if Page = nil then exit;
  result:= Page.Width {bod} / Page.HRes{b/m};
end;

function TDevice.GetRealHeight: double; { m }
begin
  result:= 0;
  if Page = nil then exit;
  result:= Page.Height {bod} / Page.VRes{b/m};
end;


function TDevice.GetBrush: TBrush;
begin
   result := nil;
end;


function TDevice.GetFont: TFont;
begin
   result := nil;
end;

function TDevice.GetHandle: HDC;
begin
   result := 0;
end;

function TDevice.GetPen: TPen;
begin
   result := nil;
end;

function TDevice.GetPixel(X, Y: double): TColor;
begin
   result := 0;
end;

procedure TDevice.SetBrush(Value: TBrush);
begin
end;

procedure TDevice.SetFont(Value: TFont);
begin
end;

procedure TDevice.SetPen(Value: TPen);
begin
end;

procedure TDevice.SetPixel(X, Y: double; Value: TColor);
begin
end;

procedure TDevice.FillRect(const Rect: TDoubleRect);
begin
end;

procedure TDevice.FrameRect(const Rect: TDoubleRect);
begin
end;

procedure TDevice.LineTo(X, Y: double);
begin
end;

procedure TDevice.MoveTo(X, Y: double);
begin
end;

procedure TDevice.Draw(X, Y: double; Graphic: Vcl.Graphics.TGraphic);
begin
end;

procedure TDevice.StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic);
begin
end;



procedure TDevice.SetTextOrientation(aOrientation: double);
begin
end;

procedure TDevice.SetupPage(Sender: TObject);
begin

end;

procedure TDevice.SetVisible(aVisible: boolean);
begin

end;

procedure TDevice.SetPenWidth(Value: double);
var
  aWidth: longInt;
begin
   if Pen = nil then exit;
   { HRes .. pixl na m
     Value .. ka v m
     Width .. pixly }
   aWidth:= Projection.GetPageLengthHoriz1(Value);
   if aWidth < 1 then aWidth:= 1;
   Pen.Width:= aWidth
end;


function TDevice.GetPenWidth: double;
begin
   result := 0;
   if Pen = nil then exit;
   result:= Projection.GetModelLength(Pen.Width);
end;


procedure TDevice.LineToDevice(X, Y: integer);
begin
end;

procedure TDevice.MoveToDevice(X, Y: integer);
begin
end;

procedure TDevice.PolyLine(const Entita: TEntity);
begin
end;

procedure TDevice.Rectangle(X1, Y1, X2, Y2: double);
begin
end;

procedure TDevice.Reset;
begin

end;

procedure TDevice.Polygon(const Entita: TEntity);
begin
end;

procedure TDevice.PolyPolygon(const Entita: TEntity);
begin
end;


function TDevice.TextHeight(const Text: string): Integer;
begin
   result := 0;
end;

procedure TDevice.TextOut(X, Y: double; const Text: string);
begin
end;

procedure TDevice.TextOutDevice(X, Y: integer; const Text: string);
begin
end;

function TDevice.TextWidth(const Text: string): Integer;
begin
   result := 0;
end;

function  TDevice.TextModelHeight(const Text: string): double;
begin
   result := 0;
end;

function  TDevice.TextModelWidth(const Text: string): double;
begin
   result := 0;
end;


procedure TDevice.SetCaption(aCaption: string);
begin
end;


procedure TDevice.PrepareForPolyPolygon(const Entita: TEntity; var PCoords,
  PLengths: pointer; var CrdSize, LNumber: integer);
begin
   PCoords := nil;
   PLengths := nil;
   CrdSize := 0;
   LNumber := 0;
end;

procedure TDevice.StretchDrawDevice(X, Y, Width, Height, srcX, srcY,
  srcWidth, srcHeight: integer; Graphic: TGraphic);
begin
end;

function TDevice.GetTextColor: integer;
begin
   result := 0;
end;

procedure TDevice.SetTextColor(const Value: integer);
begin

end;

procedure TDevice.DrawDevice(X, Y: integer; Graphic: TGraphic);
begin

end;

procedure TDevice.TransparencyBegin(Value: integer);
begin

end;

procedure TDevice.TransparencyEnd;
begin

end;

procedure TDevice.WriteLnText(aText: string);
begin

end;

procedure TDevice.WriteText(aText: string);
begin

end;

{ TDevices }

function     TDevices.GetDevice(Index: longint): TDevice;
begin
   result := (Get(Index) as TDevice);
end;


procedure    TDevices.PutDevice(Index: longint; Item: TDevice);
begin
   Put(Index, Item);
end;

{ TCanvasDevice }




procedure TCanvasDevice.Clear;
begin
   Brush.Color:= Page.Color;
   Brush.Style:= bsSolid;
   if Page.Clipping then
      Canvas.FillRect(Page.ClipRegion)
   else
      Canvas.FillRect(Canvas.ClipRect);
end;

procedure TCanvasDevice.Reset;
begin
   Clear;
end;

procedure TCanvasDevice.SetVisible(aVisible: boolean);
begin
end;


procedure TCanvasDevice.SetCanvas(aCanvas: TCanvas);
begin
   FCanvas := aCanvas;
   if aCanvas = nil then exit;
   {
   FCanvas.OnChange := SetupPage;
   }
   if not NoPageSetup then
      SetupPage(nil);
end;


procedure TCanvasDevice.SetupPage(Sender: TObject);
var
   HResX, HResY: double;
   AuxRegion: TRect;
begin
   if not assigned(FPage) then
   begin
      FPage := T_Page.Create;
      AddElement(FPage);
   end;
   Page.Rect := FCanvas.ClipRect;
   AuxRegion:= FCanvas.ClipRect;
   (*
   with AuxRegion do
   begin
     Left:= Left+10;
     Right:= Right-10;
     Top:= Top+10;
     Bottom:= Bottom-10
   end;
   *)
   Page.ClipRegion:= AuxRegion;

   HResX := 1000;
   HResY := 1000;
   HResX := HResX * GetDeviceCaps(Canvas.Handle, LOGPIXELSX) / SizeOfInch;
   HResY := HResY * GetDeviceCaps(Canvas.Handle, LOGPIXELSY) / SizeOfInch;
   Page.SetResolution(HResX, HResY);
   Projection.ChangeProjection
    (Projection.CenterPoint, Projection.Scale, Page)
end;


function TCanvasDevice.GetBrush: TBrush;
begin
   result := FCanvas.Brush;
end;

function TCanvasDevice.GetFont: TFont;
begin
   result := FCanvas.Font;
end;


function TCanvasDevice.GetPen: TPen;
begin
   result := FCanvas.Pen;
end;

function TCanvasDevice.GetPixel(X, Y: double): TColor;
begin
   result := FCanvas.Pixels[Projection.GetPageX(x), Projection.GetPageY(y)];
end;

function TCanvasDevice.GetHandle: HDC;
begin
   result := Canvas.Handle;
end;


procedure TCanvasDevice.SetBrush(Value: TBrush);
begin
   FCanvas.Brush := Value;
end;

procedure TCanvasDevice.SetFont(Value: TFont);
begin
   FCanvas.Font := Value;
end;

procedure TCanvasDevice.SetPen(Value: TPen);
begin
   FCanvas.Pen := Value;
end;

procedure TCanvasDevice.SetPixel(X, Y: double; Value: TColor);
begin
   FCanvas.Pixels[Projection.GetPageX(x), Projection.GetPageY(y)] := Value;
end;

procedure TCanvasDevice.FillRect(const Rect: TDoubleRect);
begin
   FCanvas.FillRect(Projection.GetPageRect(Rect.MinX, Rect.MinY, Rect.MaxX, Rect.MaxY));
end;

procedure TCanvasDevice.FrameRect(const Rect: TDoubleRect);
begin
   FCanvas.FrameRect(Projection.GetPageRect(Rect.MinX, Rect.MinY, Rect.MaxX, Rect.MaxY));
end;

procedure TCanvasDevice.LineTo(X, Y: double);
var
   aX,aY, ClX1, ClY1, ClX2, ClY2: longInt;
   FullyInside, Clipped, Visible: boolean;

begin
   TransformIn(X, Y);
   aX := Projection.GetPageX(X);
   aY := Projection.GetPageY(Y);

   if (aX <> OX) or (aY <> OY) then
   begin
      if Page.Clipping then
      begin
        RelRegionLine(Page.ClipRegion, OX, OY, aX, aY,
                      FullyInside, Clipped);
        if FullyInside then
        begin
          OX := aX;
          OY := aY;
          FCanvas.LineTo(OX, OY)
        end
        else
        if Clipped then
        begin
          try
            ClipLine(Page.ClipRegion, OX, OY, aX, aY,
                     ClX1, ClY1, ClX2, ClY2, Visible);
            OX := aX;
            OY := aY;
            if Visible then
            begin
              FCanvas.MoveTo(ClX1, ClY1);
              FCanvas.LineTo(ClX2, ClY2)
            end
          except
            FCanvas.MoveTo(OX, OY);
          end
        end
        else
        { usecka zcela mimo }
        begin
          OX := aX;
          OY := aY;
        end
      end
      else
      begin
        OX := aX;
        OY := aY;
        FCanvas.LineTo(OX, OY)
      end
   end;
end;

procedure TCanvasDevice.MoveTo(X, Y: double);
begin
   TransformIn(X, Y);
   OX := Projection.GetPageX(X);
   OY := Projection.GetPageY(Y);
   FCanvas.MoveTo(OX, OY);
end;


procedure TCanvasDevice.LineToDevice(X, Y: integer);
begin
   FCanvas.LineTo(x, y);
end;

procedure TCanvasDevice.MoveToDevice(X, Y: integer);
begin
   FCanvas.MoveTo(x, y);
end;


procedure TCanvasDevice.FillBuffer(Entita: TEntity;
                             var Buffer: pointer;
                             var BufferSize: integer;
                             var PointNo: integer);
const
  PomInt = 32000;
var
  I: longInt;
  Point: TPoint;
  X,Y: longInt;
begin
  Buffer:= nil;
  BufferSize:= Entita.Count * sizeOf(TPoint);
  if BufferSize > MaxBufferLen then exit;
  {Zaplneni buffru dle Listu}
  GetMem(Buffer, BufferSize);

  PointNo:= 0;
  for I:= 0 to Entita.Count-1 do
  begin
    if TElement(Entita.Elements[I]) is T2DCoordinate then
    begin
      X := Projection.GetPageX(T2DCoordinate(Entita.Elements[I]).X);
      Y := Projection.GetPageY(T2DCoordinate(Entita.Elements[I]).Y);
      if X > PomInt then X := PomInt
         else if X < -PomInt then X := -PomInt;
      if Y > PomInt then Y := PomInt
         else if Y < -PomInt then Y := -PomInt;
      Point.X := X;
      Point.Y := Y;
      if (I=0) or
         ((I>0) and
          ((Point.X <> TMaxBuffer(Buffer^)[PointNo-1].X) or
           (Point.Y <> TMaxBuffer(Buffer^)[PointNo-1].Y))) then
      begin
        TMaxBuffer(Buffer^)[PointNo]:= Point;
        inc(PointNo)
      end
    end
  end
end;

procedure TCanvasDevice.Polyline(const Entita: TEntity);
var
  BB: TBorderBuffer;
  ClipModelReg: TDoubleRect;

begin
  if not (Entita is TRectEntity) then exit;

  with Page.ClipRegion do
  begin
    ClipModelReg:= Projection.GetModelRect(Left, Top, Right, Bottom)
  end;

  BB:= TBorderBuffer.Create;
  BB.Projection:= Projection;

  BB.MinPointNo:= 2;
  BB.RectEntity:= TRectEntity(Entita);
  BB.MinX:= ClipModelReg.MinX;
  BB.MinY:= ClipModelReg.MinY;
  BB.MaxX:= ClipModelReg.MaxX;
  BB.MaxY:= ClipModelReg.MaxY;
  BB.ClipRect:= Page.ClipRegion;
  BB.ClipIt;

  if BB.Buffer <> nil then
      WinProcs.Polyline(FCanvas.Handle, BB.Buffer^, BB.PointNo);

  BB.Free
end;

procedure TCanvasDevice.Polygon(const Entita: TEntity);
var
  BB: TBorderBuffer;
  ClipModelReg: TDoubleRect;
  EE: TEntity;
  KillEE: boolean;
begin
  if not (Entita is TRectEntity) then exit;

  with Page.ClipRegion do
  begin
    ClipModelReg:= Projection.GetModelRect(Left, Top, Right, Bottom)
  end;

  //EE := nil;
  KillEE := false;
  if Projection.Angle <> 0 then
  begin
     EE := CreateCoordEntity(Entita,Self);
     KillEE := true;
  end
  else
     EE := Entita;


  BB:= TBorderBuffer.Create;
  BB.Projection:= Projection;

  BB.MinPointNo:= 3;
  BB.RectEntity:= TRectEntity(EE);
  BB.MinX:= ClipModelReg.MinX;
  BB.MinY:= ClipModelReg.MinY;
  BB.MaxX:= ClipModelReg.MaxX;
  BB.MaxY:= ClipModelReg.MaxY;
  BB.ClipRect:= Page.ClipRegion;
  BB.ClipIt;
  if KillEE then EE.Free;
  if BB.Buffer <> nil then
      WinProcs.Polygon(FCanvas.Handle, BB.Buffer^, BB.PointNo);

  BB.Free
end;

(*
procedure TCanvasDevice.PolyPolygon(const Entita: TEntity);
type
  TPointArray = array[0..20] of TPoint;
  TWordArray = array[0..20]of word;
var
  CrdBuffer: TPointArray;
  LengthBuffer: TWordArray;
  I, Position: longInt;
  PointNo, Size: word;
begin
  CrdBuffer[0].X:= 10;
  CrdBuffer[0].Y:= 10;
  CrdBuffer[1].X:= 70;
  CrdBuffer[1].Y:= 10;
  CrdBuffer[2].X:= 70;
  CrdBuffer[2].Y:= 100;
  CrdBuffer[3].X:= 10;
  CrdBuffer[3].Y:= 100;
  CrdBuffer[4].X:= 10;
  CrdBuffer[4].Y:= 10;

  CrdBuffer[5].X:= 30;
  CrdBuffer[5].Y:= 40;
  CrdBuffer[6].X:= 50;
  CrdBuffer[6].Y:= 40;
  CrdBuffer[7].X:= 50;
  CrdBuffer[7].Y:= 70;
  CrdBuffer[8].X:= 30;
  CrdBuffer[8].Y:= 70;
  CrdBuffer[9].X:= 30;
  CrdBuffer[9].Y:= 40;

  LengthBuffer[0]:= 5;
  LengthBuffer[1]:= 5;

  WinProcs.PolyPolygon(FCanvas.Handle, CrdBuffer[0], LengthBuffer[0], 2);
end;
*)


procedure TCanvasDevice.PolyPolygon(const Entita: TEntity);
var
  Holes: TEntity;
  CrdBuffer: pointer;
  LengthBuffer: pointer;
  Size: integer;
  LNumber: integer;

begin

  Holes:= TEntity(Entita.GetPropertyIntByName('HOLES'));
  if (Holes = nil) or (Holes.Count < 1) then
  begin
    Polygon(Entita);
    exit;
  end;
  PrepareForPolyPolygon(Entita,CrdBuffer,LengthBuffer,Size,LNumber);
  if not ((CrdBuffer = nil) or (LengthBuffer = nil)) then
  if not WinProcs.PolyPolygon(FCanvas.Handle, TLargeByteArray(CrdBuffer^)[0], TLengthArray(LengthBuffer^)[0], {HoleBuffers.Count}LNumber) then
  begin
//    MessageBeep(0);
  end;

  freeMem(CrdBuffer, Size);
  freeMem(LengthBuffer, LNumber*sizeOf(TLengthItem));

  //HoleBuffers.Free;
end;



procedure TCanvasDevice.Rectangle(X1, Y1, X2, Y2: double);
begin
   FCanvas.Rectangle ( Projection.GetPageX(X1), Projection.GetPageY(Y1),
                       Projection.GetPageX(X2), Projection.GetPageY(Y2));
end;


procedure TCanvasDevice.SetTextOrientation(aOrientation: double);
var
  LogFont: TLogFont;

begin
  if Canvas = nil then exit;
{  if aOrientation = 0 then exit; }

  with LogFont do
  begin
    lfHeight := Font.Height;
{    lfHeight := Font.Size; }

    lfWidth := 0; { have font mapper choose }
    lfEscapement := round(aOrientation*10);
    lfOrientation := 0{round(aOrientation*10)};
    if fsBold in Font.Style then
      lfWeight := FW_BOLD
    else
      lfWeight := FW_NORMAL;
    lfItalic := Byte(fsItalic in Font.Style);
    lfUnderline := Byte(fsUnderline in Font.Style);
    lfStrikeOut := Byte(fsStrikeOut in Font.Style);
    lfCharSet := Font.CharSet;
    StrPCopy(lfFaceName, Font.Name);
    lfQuality := DEFAULT_QUALITY;
    { Everything else as default }
    lfOutPrecision := OUT_DEFAULT_PRECIS;
    lfClipPrecision := CLIP_DEFAULT_PRECIS;
    lfPitchAndFamily := byte(Font.Pitch);
  end;
{  DeleteObject(Canvas.Font.Handle); }
  Canvas.Font.Handle := CreateFontIndirect(LogFont);
  FOrientation:= aOrientation;
end;




function TCanvasDevice.TextHeight(const Text: string): Integer;
begin
   result := FCanvas.TextHeight(Text);
end;

function TCanvasDevice.TextModelHeight(const Text: string): double;
begin
   result := Projection.GetModelLength(FCanvas.TextHeight(Text));
end;


procedure TCanvasDevice.TextOut(X, Y: double; const Text: string);
begin
   FCanvas.Brush.Style:= bsClear;
   FCanvas.TextOut(Projection.GetPageX(X), Projection.GetPageY(Y), Text);
end;

procedure TCanvasDevice.TextOutDevice(X, Y: integer; const Text: string);
begin
   FCanvas.Brush.Style:= bsClear;
   FCanvas.TextOut(X, Y, Text);
end;

function TCanvasDevice.TextWidth(const Text: string): Integer;
begin
   result := FCanvas.TextWidth(Text);
end;

function TCanvasDevice.TextModelWidth(const Text: string): double;
begin
   result := Projection.GetModelLength(FCanvas.TextWidth(Text));
end;


procedure TCanvasDevice.Draw(X, Y: double; Graphic: Vcl.Graphics.TGraphic);
begin
   FCanvas.Draw(Projection.GetPageX(X), Projection.GetPageY(Y), Graphic)
end;

procedure TCanvasDevice.StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic);
var
  aRect: TRect;
begin
   aRect.Left := Projection.GetPageX(X);
   aRect.Top  := Projection.GetPageY(Y);
   aRect.Right:= Projection.GetPageLengthHoriz1(Width) + aRect.Left;
   aRect.Bottom:= Projection.GetPageLengthVert1(Height) + aRect.Top;

   Canvas.StretchDraw (aRect, {Bitmap}Graphic);
end;

procedure TCanvasDevice.StretchDrawDevice(X, Y, Width, Height, srcX, srcY,
  srcWidth, srcHeight: integer; Graphic: TGraphic);
var
  aRect: TRect;
begin
   aRect.Left := X;
   aRect.Top  := Y;
   aRect.Right:= Width + aRect.Left;
   aRect.Bottom:= Height + aRect.Top;

   Canvas.StretchDraw (aRect, {Bitmap}Graphic);
end;

procedure TCanvasDevice.PrepareForPolyPolygon(const Entita: TEntity;
  var PCoords, PLengths: pointer; var CrdSize, LNumber: integer);
var
  ClipModelReg: TDoubleRect;
  Holes: TEntity;
  Hole: TRectEntity;
  HoleBuffers: TEntity;
  BB, BBHole: TBorderBuffer;
  CrdBuffer: pointer;
  LengthBuffer: pointer;
  I, Position: integer;
  //PointNo,
  Size: integer;
  EE: TEntity;
  KillEE: boolean;

begin
  inherited;
  Holes:= TEntity(Entita.GetPropertyIntByName('HOLES'));
  {
  if (Holes = nil) or (Holes.Count < 1) then
  begin
    Polygon(Entita);
    exit;
  end;
  }
  { Vytvoen oezvaccho obdlnku }
  with Page.ClipRegion do
  begin
    ClipModelReg:= Projection.GetModelRect(Left, Top, Right, Bottom)
  end;

  //EE := nil;
  KillEE := false;
  if Projection.Angle <> 0 then
  begin
     EE := CreateCoordEntity(Entita,Self);
     KillEE := true;
  end
  else
     EE := Entita;
  { Vytvoen buffru vnj hranice }
  BB:= TBorderBuffer.Create;


  BB.MinPointNo:= 3;
  BB.Projection:= Projection;
  BB.RectEntity:= TRectEntity(EE);
  BB.MinX:= ClipModelReg.MinX;
  BB.MinY:= ClipModelReg.MinY;
  BB.MaxX:= ClipModelReg.MaxX;
  BB.MaxY:= ClipModelReg.MaxY;
  BB.ClipRect:= Page.ClipRegion;
  BB.ClipIt;
  if BB.Buffer = nil then
  begin
    BB.Free;
    exit;
  end;
  if KillEE then EE.Free;
  HoleBuffers:= TEntity.Create;
  HoleBuffers.AddElement(BB);
  { pidn dr do buffru dr }
  if (Holes <> nil) and (Holes.Count > 0) then
  for I:= 0 to Holes.Count-1 do
  begin
    Hole:= TRectEntity(Holes.Elements[I]);
    if Hole = nil then continue;

    { Vytvoen buffru vnj hranice }

    //EE := nil;
    KillEE := false;
    if Projection.Angle <> 0 then
    begin
       EE := CreateCoordEntity(Hole,Self);
       KillEE := true;
    end
    else
       EE := Hole;

    BBHole:= TBorderBuffer.Create;


    BBHole.MinPointNo:= 3;
    BBHole.Projection:= Projection;
    BBHole.RectEntity:= TRectEntity(EE);
    BBHole.MinX:= ClipModelReg.MinX;
    BBHole.MinY:= ClipModelReg.MinY;
    BBHole.MaxX:= ClipModelReg.MaxX;
    BBHole.MaxY:= ClipModelReg.MaxY;
    BBHole.ClipRect:= Page.ClipRegion;
    BBHole.ClipIt;

    if KillEE then EE.Free;

    if BBHole.Buffer = nil then
    begin
      BBHole.Free;
      continue
    end;
    HoleBuffers.AddElement(BBHole);
  end;

  { Celkov dlka }
  //PointNo:= 0;
  Size:= 0;
  for I:= 0 to HoleBuffers.Count-1 do
  begin
     //PointNo:= PointNo + TBorderBuffer(HoleBuffers.Elements[I]).PointNo;
     Size:= Size + TBorderBuffer(HoleBuffers.Elements[I]).PointNo*sizeOf(TBodL);
  end;
  getMem(CrdBuffer, Size);
  getMem(LengthBuffer, HoleBuffers.Count*sizeOf(TLengthItem));

  Position:= 0;

  for I:= 0 to HoleBuffers.Count-1 do
  begin

    Move(TBorderBuffer(HoleBuffers.Elements[I]).FBuffer^,
         TLargeByteArray(CrdBuffer^)[Position],
         TBorderBuffer(HoleBuffers.Elements[I]).PointNo*sizeOf(TBodL));

    Position:= Position + TBorderBuffer(HoleBuffers.Elements[I]).PointNo*sizeOf(TBodL);

    TLengthArray(LengthBuffer^)[I]:= TBorderBuffer(HoleBuffers.Elements[I]).PointNo;

  end;
  PCoords := CrdBuffer;
  PLengths:= LengthBuffer;
  CrdSize := Size;
  LNumber := HoleBuffers.Count;
  {
  if not WinProcs.PolyPolygon(FCanvas.Handle, TByteArray(CrdBuffer^)[0], TLengthArray(LengthBuffer^)[0], HoleBuffers.Count) then
  begin
    MessageBeep(0);
  end;

  freeMem(CrdBuffer, Size);
  freeMem(LengthBuffer, HoleBuffers.Count*sizeOf(TLengthItem));
   }
  HoleBuffers.Free;
end;


function TCanvasDevice.GetTextColor: integer;
begin
   result := Font.Color;
end;

procedure TCanvasDevice.SetTextColor(const Value: integer);
begin
   Font.Color := Value;
end;

procedure TCanvasDevice.DrawDevice(X, Y: integer; Graphic: TGraphic);
begin
   FCanvas.Draw(X, Y, Graphic)
end;

procedure TCanvasDevice.SetNoPageSetup(const Value: boolean);
begin
  FNoPageSetup := Value;
end;

{ TFormDevice }


procedure   TFormDevice.SetVisible(aVisible: boolean);
begin
   Visible      := aVisible;
   Form.Visible := aVisible;
end;

procedure TFormDevice.SetCaption(aCaption: string);
begin
  Form.Caption:= aCaption
end;


constructor TFormDevice.Create;
begin
   inherited Create;
   CreateForm;
end;


destructor TFormDevice.Destroy;
begin
   Form.Free;
   inherited Destroy;
end;



procedure   TFormDevice.CreateForm;
begin
   FForm:= TForm.Create(Application);
   Canvas := Form.Canvas;
end;


{ TPrintDevice}

constructor TPrintDevice.CreateExternalPrint;
begin
  inherited Create;
  Name:= 'extPRINTER';
  //Printer.Title:= 'ORBIS MAPPER';
{  BeginDocToFile('G:\P1.PRN'); }
  //Printer.BeginDoc;
  Page.Color := $FFFFFF;
  Canvas:= Printer.Canvas;
  ExternalPrint := true;
end;


constructor TPrintDevice.CreateTitle(const aTitle: string);
begin
  inherited Create;
  Name:= 'PRINTER';
  Printer.Title:= aTitle;
{  BeginDocToFile('G:\P1.PRN'); }
  Printer.BeginDoc;
  Page.Color := $FFFFFF;
  Canvas:= Printer.Canvas;
end;

constructor TPrintDevice.Create;
begin
  inherited Create;
  Name:= 'PRINTER';
  Printer.Title:= AppString;
{  BeginDocToFile('G:\P1.PRN'); }
  Printer.BeginDoc;
  Page.Color := $FFFFFF;
  Canvas:= Printer.Canvas;
end;


procedure TPrintDevice.BeginDocToFile(aFileName: string);
var
  CTitle: array[0..31] of Char;
  PFileName: array[0..70] of Char;
  DocInfo: TDocInfo;

begin
{  CheckPrinting(False); }
{  SetState(psHandleDC); }
  Printer.Canvas.Refresh;
{  FPrinting := True; }
{  FAborted := False; }
{  FPageNumber := 1; }
  StrPLCopy(CTitle, Printer.Title, SizeOf(CTitle) - 1);
  StrPCopy(PFileName, aFileName);
  with DocInfo do
  begin
    cbSize := SizeOf(DocInfo);
    lpszDocName := CTitle;
    lpszOutput := PFileName;
  end;
{  SetAbortProc(Printer.Handle, AbortProc); }
  StartDoc(Printer.Handle, DocInfo);
  StartPage(Printer.Handle);
end;


destructor  TPrintDevice.Destroy;
begin
  if not ExternalPrint then
  if Printer.Printing then  // pokud byl abort uz EndDoc bylo
     Printer.EndDoc;
  inherited Destroy
end;

procedure TPrintDevice.SetCaption(aCaption: string);
begin
  Printer.Title:= aCaption
end;

function  TPrintDevice.GetOrientation: Printers_f.TPrinterOrientation;
begin
   result := Printer.Orientation;
end;

procedure TPrintDevice.Reset;
begin
{  Printer.BeginDoc }
end;


procedure TPrintDevice.Close;
begin
{  Printer.EndDoc }
end;



procedure TPrintDevice.LineTo(X, Y: double);
var
   aX,aY, ClX1, ClY1, ClX2, ClY2: longInt;
   FullyInside, Clipped, Visible: boolean;

begin
   TransformIn(X, Y);
   aX := Projection.GetPageX(X);
   aY := Projection.GetPageY(Y);

   if (aX <> OX) or (aY <> OY) then
   begin
      if Page.Clipping then
      begin
        RelRegionLine(Page.ClipRegion, OX, OY, aX, aY,
                      FullyInside, Clipped);
        if FullyInside then
        begin
          if RasterizeLinesByOrbis then
          begin
             ThickLine(ox,oy,ax,ay);
             OX := aX;
             OY := aY;
          end
          else
          begin
             OX := aX;
             OY := aY;
             FCanvas.LineTo(OX, OY)
          end;
        end
        else
        if Clipped then
        begin
          try
             ClipLine(Page.ClipRegion, OX, OY, aX, aY,
                     ClX1, ClY1, ClX2, ClY2, Visible);
             OX := aX;
             OY := aY;
             if Visible then
             begin
               if RasterizeLinesByOrbis then
               begin
                  ThickLine(ClX1, ClY1, ClX2, ClY2);
               end
               else
               begin
                  FCanvas.MoveTo(ClX1, ClY1);
                  FCanvas.LineTo(ClX2, ClY2)
               end
             end
          except
             FCanvas.MoveTo(OX, OY);
          end
        end
        else
        { usecka zcela mimo }
        begin
           OX := aX;
           OY := aY;
        end
      end
      else
      begin
         if RasterizeLinesByOrbis then
         begin
            ThickLine(ox,oy,ax,ay);
            OX := aX;
            OY := aY;
         end
         else
         begin
            OX := aX;
            OY := aY;
            FCanvas.LineTo(OX, OY)
         end;
      end
   end;
end;

procedure TPrintDevice.MoveTo(X, Y: double);
begin
   TransformIn(X, Y);
   OX := Projection.GetPageX(X);
   OY := Projection.GetPageY(Y);
   FCanvas.MoveTo(OX, OY);
end;


procedure TPrintDevice.LineToDevice(X, Y: integer);
begin

  if RasterizeLinesByOrbis then
  begin
     if (Pen.Color = clBlack) or (Pen.Color = clWhite) then
     begin
       FCanvas.LineTo(x, y);
       ox := x;
       oy := y;
       exit;
     end;
     ThickLine(ox,oy,x,y);
     OX := X;
     OY := Y;
  end
  else

  begin
    FCanvas.LineTo(x, y);
    ox := x;
    oy := y
  end
end;


procedure TPrintDevice.MoveToDevice(X, Y: integer);
begin
   FCanvas.MoveTo(x, y);
   ox := x;
   oy := y;
end;

procedure  TPrintDevice.ThickLine(x1,y1,x2,y2: integer);
var
   aLine: TThickLine;
   LineLength: Double; { ??? }{ Delka zakladnii linie }
   deltax, deltay: double;
   deltaxL, deltayL, PomI: integer;  { sour rozdily bodu Left }
   HalfWidth: integer;
begin
   { vypocet tela linie }
   { ------------------ }
   { Vypocet Left bodu ... to bude makacka }
   { ------------------------------------- }
   { }
   if (Pen.Color = clBlack) or (Pen.Color = clWhite) then
   begin
      FCanvas.MoveTo(x1, y1);
      FCanvas.LineTo(x2, y2);
      exit;
   end;

   HalfWidth := Pen.Width div 2;
   if HalfWidth < 1 then HalfWidth := 1;
   { sour. rozdily }
   deltax      := x2 - x1;
   deltay      := y2 - y1;

   { delka linie }
   LineLength := (SQRT(SQR(deltax) + SQR(deltay)));

   if LineLength < 1 then exit;
   { sour rozdily polovicni tloustky pera ... haha co kdyz je to 2 ? }
   deltaxL := round(deltax * HalfWidth  / LineLength);
   deltayL := round(deltay * HalfWidth  / LineLength);

   { vypocet kolmice }
   PomI    := deltaXL;
   deltaxL := -deltayL;
   deltayL := PomI;


   aLine.A.Left.x  := x1 + deltaxL;
   aLine.A.Left.y  := y1 + deltayL;
   aLine.A.Right.x := x1 - deltaxL;
   aLine.A.Right.y := y1 - deltayL;
   aLine.B.Left.x  := x2 + deltaxL;
   aLine.B.Left.y  := y2 + deltayL;
   aLine.B.Right.x := x2 - deltaxL;
   aLine.B.Right.y := y2 - deltayL;
   {
   LineArray[0] := aLine.A.Right;
   LineArray[1] := aLine.A.Left;
   LineArray[2] := aLine.B.Left;
   LineArray[3] := aLine.B.Right;
   }
   Pen.Style := psClear;
   Brush.Color := Pen.Color;
   Brush.Style := bsSolid;
   with aLine do FCanvas.Polygon([A.Right,A.Left,B.Left,B.Right]);
   Pen.Style  := psSolid;
end;


function MemGAlloc(Size: Longint; var Handle: THandle): Pointer;
begin
  if Size < 65535 then
    GetMem(Result, Size)
  else
  begin
    Handle := GlobalAlloc(HeapAllocFlags, Size);
    Result := GlobalLock(Handle);
  end;
end;


procedure FreeGMem(var Handle: THandle; var P: Pointer; Size: longInt);
begin
   if Size < 65535 then
   begin
      FreeMem(P, Size);
   end
   else
   begin
      GlobalUnlock(Handle);
      GlobalFree(Handle);
   end;
end;

procedure TPrintDevice.StretchDraw(X, Y, Width, Height: double; srcX, srcY, srcWidth, srcHeight: integer; Graphic: Vcl.Graphics.TGraphic);
var
  aRect: TRect;
  aBitmap,PomBitmap: Vcl.Graphics.TBitmap;
  BitmapInfo: ^TBitmapInfo;
  Bits: Pointer;
  InfoHeaderSize: Cardinal;
  ImageSize: Cardinal;
  cX,cY,cW,cH: longInt;
  Hi,Hb: THandle;
begin
   PomBitmap := nil;
   if (Graphic is TJPEGImage) or (Graphic is TGIFImage) or (Graphic is TPNGImage) then
   begin
      PomBitmap := Vcl.Graphics.TBitmap.Create;
      PomBitmap.Assign(Graphic);
      Graphic := PomBitmap;
   end;
   if Graphic is Vcl.Graphics.TBitmap then
   begin
      aBitmap := Vcl.Graphics.TBitmap(Graphic);
      //aBitmap.SaveToFile('e:\kos\rastr.bmp');
      cX := Projection.GetPageX(X);
      cY := Projection.GetPageY(Y);
      cW := Projection.GetPageLengthHoriz1(Width);
      cH := Projection.GetPageLengthVert1(Height);
     {print the bitmap}

      GetDIBSizes(aBitmap.Handle, InfoHeaderSize, ImageSize);
      BitMapInfo := MemGAlloc(InfoHeaderSize, Hi);
      Bits := MemGAlloc(ImageSize, Hb);
      GetDIB(aBitmap.Handle, aBitmap.Palette ,BitmapInfo^, Bits^);
      if (srcX = 0) and (srcY = 0)and (srcWidth = 0) and (srcHeight = 0) then
      begin
         srcWidth := aBitmap.Width;
         srcHeight := aBitmap.Height;
      end;
      StretchDIBits(Canvas.Handle, cX, cY, cW, cH,
                    srcX, srcY, srcWidth, srcHeight, Bits, BitmapInfo^, DIB_RGB_COLORS, SRCCOPY);
      FreeGMem(Hi, Pointer(BitMapInfo), InfoHeaderSize);
      FreeGMem(Hb, Bits, ImageSize);
   end
   else

   begin
      aRect.Left := Projection.GetPageX(X);
      aRect.Top  := Projection.GetPageY(Y);
      aRect.Right:= Projection.GetPageLengthHoriz1(Width) + aRect.Left;



      aRect.Bottom:= Projection.GetPageLengthVert1(Height) + aRect.Top;

      Canvas.StretchDraw (aRect, {Bitmap}Graphic);
   end;
   if PomBitmap <> nil then PomBitmap.Free;
end;

procedure TPrintDevice.StretchDrawDevice(X, Y, Width, Height, srcX, srcY,
  srcWidth, srcHeight: integer; Graphic: TGraphic);
var
  aRect: TRect;
  aBitmap,PomBitmap: Vcl.Graphics.TBitmap;
  BitmapInfo: ^TBitmapInfo;
  Bits: Pointer;
  InfoHeaderSize: Cardinal;
  ImageSize: Cardinal;
  Hi,Hb: THandle;
begin
   PomBitmap := nil;
   if (Graphic is TJPEGImage) {$IFNDEF WIN64}or (Graphic is TGIFImage){$ENDIF}or (Graphic is TPngImage) then
   begin
      PomBitmap := Vcl.Graphics.TBitmap.Create;
      PomBitmap.Assign(Graphic);
      Graphic := PomBitmap;
   end;
   if Graphic is Vcl.Graphics.TBitmap then
   begin
      aBitmap := Vcl.Graphics.TBitmap(Graphic);
      //aBitmap.SaveToFile('e:\kos\rastr.bmp');
     {print the bitmap}

      GetDIBSizes(aBitmap.Handle, InfoHeaderSize, ImageSize);
      BitMapInfo := MemGAlloc(InfoHeaderSize, Hi);
      Bits := MemGAlloc(ImageSize, Hb);
      GetDIB(aBitmap.Handle, aBitmap.Palette ,BitmapInfo^, Bits^);
      if (srcX = 0) and (srcY = 0)and (srcWidth = 0) and (srcHeight = 0) then
      begin
         srcWidth := aBitmap.Width;
         srcHeight := aBitmap.Height;
      end;
      StretchDIBits(Canvas.Handle, X, Y, Width, Height,
                    srcX, srcY, srcWidth, srcHeight, Bits, BitmapInfo^, DIB_RGB_COLORS, SRCCOPY);
      FreeGMem(Hi, Pointer(BitMapInfo), InfoHeaderSize);
      FreeGMem(Hb, Bits, ImageSize);
   end
   else
   begin
      aRect.Left := X;
      aRect.Top  := Y;
      aRect.Right:= Width + aRect.Left;
      aRect.Bottom:= Height + aRect.Top;

      Canvas.StretchDraw (aRect, {Bitmap}Graphic);
   end;
   if PomBitmap <> nil then PomBitmap.Free;
end;

procedure TPrintDevice.SetExternalPrint(const Value: boolean);
begin
  FExternalPrint := Value;
end;



{ TBitmapDevice }

destructor TBitmapDevice.Destroy;
begin
   if Bitmap <> nil then
      if KillBitmap then Bitmap.Free;
   inherited;
end;

procedure TBitmapDevice.SetBitmap(const Value: Vcl.Graphics.TBitmap);
begin
   if FBitmap <> nil then
   begin
      if KillBitmap then FBitmap.Free;
      FBitmap := nil;
   end;
   FBitmap := Value;
   if Value <> nil then
      Canvas := FBitmap.Canvas
   else
      Canvas := nil;
end;

procedure TBitmapDevice.SetKillBitmap(const Value: boolean);
begin
   FKillBitmap := Value;
end;

{ TTileDevice }

procedure TTileDevice.Close;
var
   CDevice: TDevice;
begin
   if Device = nil then exit;
   CDevice := Device;
   if (CDevice is TPrintDevice) then
   begin
      {
      Bitmap.Canvas.Pen.Style := psSolid;
      Bitmap.Canvas.Pen.Color := 0;
      Bitmap.Canvas.Rectangle(0,0,100,100);
      }
      CDevice.StretchDrawDevice(0,DeviceY,Bitmap.Width,Bitmap.Height,0,0,Bitmap.Width,Bitmap.Height,Bitmap);
   end
   else
   begin
      CDevice.DrawDevice(0,DeviceY,Bitmap);
      {
      CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                              Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
      }
   end;

end;

procedure TTileDevice.NextTile;
var
   //Pos: integer;
   TileY: integer;
   Center: TDoubleCoord;
   BMH: integer;
   CY1,CY2: double;
begin
   Tile := Tile + 1;
   DeviceY := DeviceY + Bitmap.Height;
   LastTile := Tile >= TilesCount;
   if LastTile then
   begin
      Bitmap := nil;
      exit;
   end;
   BMH := Bitmap.Height;
   {
   if (DeviceY + Bitmap.Height) > (Device.Page.Height - 1) then
   begin                                                                   // pridal sem -1!
      TileY := Bitmap.Height -((DeviceY + Bitmap.Height) - (Device.Page.Height-1));
      Bitmap.Height := TileY;
   end;
   }
   if (DeviceY + Bitmap.Height) > (Device.Page.Height) then
   begin
      TileY := Bitmap.Height -((DeviceY + Bitmap.Height) - (Device.Page.Height));
      Bitmap.Height := TileY;
   end;
   Center.X := Device.Projection.Center_X;
   if BitMap.Height mod 2 = 0 then
      Center.Y := Device.Projection.GetModelY((Tile * BMH) + (Bitmap.Height div 2))
   else
   begin
      CY1 := Device.Projection.GetModelY((Tile * BMH) + (Bitmap.Height div 2)+ (Bitmap.Height mod 2));
      CY2 := Device.Projection.GetModelY((Tile * BMH) + (Bitmap.Height div 2));
      Center.Y := CY1 + ((CY2-CY1)/2);
   end;
   Projection.ChangeProjection(Center,Device.Projection.Scale,Page);
end;

procedure TTileDevice.ReFreshDevice;
var
   CDevice: TDevice;
begin
   if Device = nil then exit;
   CDevice := Device;
   if not (CDevice is TPrintDevice) then
   begin
      CDevice.DrawDevice(0,DeviceY,Bitmap);
      {
      CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                              Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
      }
   end;

end;

procedure TTileDevice.Reset;
begin
   Clear;
end;

procedure TTileDevice.SetDevice(const Value: TDevice);
var
   //aPage: T_Page;
   aByte: integer;
   BMx, BMy, TileY: integer;
   ME: TMemoryElement;
   RasInfo: TStringListElement;
   MemoryDC: HDC;
   BM: HBitmap;
   MA: longWord;
   BasicMem: integer;
   Center: TDoubleCoord;
   BMP: Vcl.Graphics.TBitmap;
   CY1,CY2: double;
begin
   FDevice := Value;

   TilesCount := 1;
   BMx := FDevice.Page.Width;
   BMy := FDevice.Page.Height;
   TileY := BMy;
   if Bitmap <> nil then Bitmap.Free;
   BMP := Vcl.Graphics.TBitmap.Create;
   BMP.Width := 1;
   BMP.Height := 1;
   BMP.Pixelformat := pf24Bit;
   aByte := BMx * 3;

   MemoryDC := CreateCompatibleDC(BMP.Canvas.Handle);
   // Kontrola memory
   BasicMem := 20000000;
   ME := TMemoryElement.Create;
   RasInfo := nil;
   if World <> nil then
      RasInfo := TStringListElement(World.SystemRegister.FindElementName('RASTER'));
   if RasInfo = nil then MA := BasicMem
   else
   begin
      MA := RasInfo.GetIntegerByName('MEMORY_TILE');
      if MA = 0 then MA := BasicMem;
   end;
   if ME.AvailMemory < MA then MA := ME.AvailMemory;
   if MA < 2000000 then
   begin
      //PostTextMsg(tm_AddToProtocol,'TILE... kritick stav pamti: '+IntToStr(MA),nil);
      if ME.AvailPageFile > 2000000 then
      begin
         MA := 2000000;
      end
      else
      begin
         PostTextMsg(tm_AddToProtocol,'TILE... Zoufale kritick stav pamti ... uvolnte msto na swapovacm disku!',nil);
         MA := 0;
      end;
   end;
   ME.Free;
   if MA > 0 then
   begin
      while (aByte * TileY) > MA do
      //while TilesCount < 5 do
      begin
         TilesCount := TilesCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
   end;
   BM := CreateCompatibleBitmap(BMP.Canvas.Handle,BMx,TileY);
   while BM = 0 do
   begin
      TilesCount := TilesCount * 2;
      TileY := (TileY div 2) + (TileY mod 2);
      BM := CreateCompatibleBitmap(BMP.Canvas.Handle,BMx,TileY);
   end;
   // prava potu dladic
   while not(((TileY * TilesCount) - TileY) < BMy) do Dec(FTilesCount);
   PostTextMsg(tm_ShowText,'TILE... Poet dladic '+ IntToStr(TilesCount),nil);
   Application.ProcessMessages;
   SelectObject(MemoryDC,BM);
   BMP.Handle := BM;
   BMP.Canvas.Handle := MemoryDC;
   Application.ProcessMessages;
   KillBitmap := true;
   Bitmap := BMP;
 // Konec vytvoreni Bitmapy
   Page.SetResolution(Device.Page.HRes, Device.Page.VRes);
   Center.X := Device.Projection.Center_X;                // pridano mod
   if TileY mod 2 = 0 then
      Center.Y := Device.Projection.GetModelY(TileY div 2)
   else
   begin
      CY1 := Device.Projection.GetModelY((TileY div 2) + (TileY mod 2));
      CY2 := Device.Projection.GetModelY(TileY div 2);
      Center.Y := CY1 + ((CY2-CY1)/2);
   end;
   Projection.ChangeProjection(Center,Device.Projection.Scale,Page);
   Projection.VirtualScale := Value.Projection.VirtualScale;
   (*
   PostTextMsg(tm_AddToProtocol,'TILE... Scale: '+ FloatToStr(Device.Projection.Scale),nil);
   PostTextMsg(tm_AddToProtocol,'TILE..D HRes: '+ FloatToStr(Device.Page.Hres),nil);
   PostTextMsg(tm_AddToProtocol,'TILE..D VRes: '+ FloatToStr(Device.Page.Vres),nil);
   PostTextMsg(tm_AddToProtocol,'TILE... HRes: '+ FloatToStr(Page.Hres),nil);
   PostTextMsg(tm_AddToProtocol,'TILE... VRes: '+ FloatToStr(Page.Vres),nil);
   *)
   Page.Color := Device.Page.Color;
   Tile := 0;
   DeviceY := 0;
   LastTile := false;
end;

procedure TTileDevice.SetDeviceX(const Value: integer);
begin
  FDeviceX := Value;
end;

procedure TTileDevice.SetDeviceY(const Value: integer);
begin
  FDeviceY := Value;
end;

procedure TTileDevice.SetLastTile(const Value: boolean);
begin
  FLastTile := Value;
end;



procedure TTileDevice.SetTile(const Value: integer);
begin
   FTile := Value;
end;

procedure TTileDevice.SetTilesCount(const Value: integer);
begin
   FTilesCount := Value;
end;

procedure TTileDevice.TransparencyBegin(Value: integer);
begin

end;

procedure TTileDevice.TransparencyEnd;
begin

end;

{ TDevice_Device }

procedure TDevice_Device.Adjust;
begin
   if Device <> nil then
      Device.Adjust;
end;

procedure TDevice_Device.Clear;
begin
   if Device <> nil then
      Device.Clear;
end;

procedure TDevice_Device.Close;
begin
   if Device <> nil then
      Device.Close;
end;

constructor TDevice_Device.Create;
begin
   inherited;
end;

procedure TDevice_Device.Draw(X, Y: double; Graphic: TGraphic);
begin
   if Device <> nil then
      Device.Draw(X,Y,Graphic);
end;

procedure TDevice_Device.DrawDevice(X, Y: integer; Graphic: TGraphic);
begin
   if Device <> nil then
      Device.DrawDevice(X, Y, Graphic);
end;

procedure TDevice_Device.FillRect(const Rect: TDoubleRect);
begin
   if Device <> nil then
      Device.FillRect(Rect);
end;

procedure TDevice_Device.FrameRect(const Rect: TDoubleRect);
begin
   if Device <> nil then
      Device.FrameRect(Rect);
end;

function TDevice_Device.GetBrush: TBrush;
begin
   result := nil;
   if Device <> nil then
      result := Device.GetBrush;
end;

function TDevice_Device.GetFont: TFont;
begin
   result := nil;
   if Device <> nil then
      result := Device.GetFont;
end;

function TDevice_Device.GetHandle: HDC;
begin
   result := 0;
   if Device <> nil then
      result := Device.GetHandle;
end;

function TDevice_Device.GetPen: TPen;
begin
   result := nil;
   if Device <> nil then
      result := Device.GetPen;
end;

function TDevice_Device.GetPixel(X, Y: double): TColor;
begin
   result := 0;
   if Device <> nil then
      result := Device.GetPixel(X, Y);
end;

function TDevice_Device.GetTextColor: integer;
begin
   result :=0;
   if Device <> nil then
      result := Device.GetTextColor;
end;

procedure TDevice_Device.LineTo(X, Y: double);
begin
   if Device <> nil then
      Device.LineTo(X, Y);
end;

procedure TDevice_Device.LineToDevice(X, Y: integer);
begin
   if Device <> nil then
      Device.LineToDevice(X, Y);
end;

procedure TDevice_Device.MoveTo(X, Y: double);
begin
   if Device <> nil then
      Device.MoveTo(X, Y);
end;

procedure TDevice_Device.MoveToDevice(X, Y: integer);
begin
   if Device <> nil then
      Device.MoveToDevice(X, Y);
end;

procedure TDevice_Device.Polygon(const Entita: TEntity);
begin
   if Device <> nil then
      Device.Polygon(Entita);
end;

procedure TDevice_Device.Polyline(const Entita: TEntity);
begin
   if Device <> nil then
      Device.Polyline(Entita);
end;

procedure TDevice_Device.PolyPolygon(const Entita: TEntity);
begin
   if Device <> nil then
      Device.PolyPolygon(Entita);
end;

procedure TDevice_Device.PrepareForPolyPolygon(const Entita: TEntity;
  var PCoords, PLengths: pointer; var CrdSize, LNumber: integer);
begin
   if Device <> nil then
      Device.PrepareForPolyPolygon(Entita, PCoords, PLengths, CrdSize, LNumber);
end;

procedure TDevice_Device.Rectangle(X1, Y1, X2, Y2: double);
begin
   if Device <> nil then
      Device.Rectangle(X1, Y1, X2, Y2);
end;

procedure TDevice_Device.Reset;
begin
   if Device <> nil then
      Device.Reset;
end;

procedure TDevice_Device.SetBrush(Value: TBrush);
begin
   if Device <> nil then
      Device.SetBrush(Value);
end;

procedure TDevice_Device.SetCaption(aCaption: string);
begin
   if Device <> nil then
      Device.SetCaption(aCaption);
end;

procedure TDevice_Device.SetDevice(const Value: TDevice);
begin
   FDevice := Value;
   if Value = nil then exit;
   FPage := Value.Page;
   FProjection := Value.Projection;
   FANWCS := Value.ANWCS;
   FVisible := Value.Visible;
end;

procedure TDevice_Device.SetFont(Value: TFont);
begin
   if Device <> nil then
      Device.SetFont(Value);
end;

procedure TDevice_Device.SetPen(Value: TPen);
begin
   if Device <> nil then
      Device.SetPen(Value);
end;

procedure TDevice_Device.SetPixel(X, Y: double; Value: TColor);
begin
   if Device <> nil then
      Device.SetPixel(X, Y, Value);
end;

procedure TDevice_Device.SetTextColor(const Value: integer);
begin
   if Device <> nil then
      Device.SetTextColor(Value);
end;

procedure TDevice_Device.SetTextOrientation(aOrientation: double);
begin
   if Device <> nil then
      Device.SetTextOrientation(aOrientation);
end;

procedure TDevice_Device.SetupPage(Sender: TObject);
begin
   if Device <> nil then
      Device.SetupPage(Sender);
end;

procedure TDevice_Device.SetVisible(aVisible: boolean);
begin
   if Device <> nil then
   begin
      Device.SetVisible(aVisible);
      FVisible := aVisible;
   end;
end;

procedure TDevice_Device.StretchDraw(X, Y, Width, Height: double; srcX,
  srcY, srcWidth, srcHeight: integer; Graphic: TGraphic);
begin
   if Device <> nil then
      Device.StretchDraw(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight, Graphic);
end;

procedure TDevice_Device.StretchDrawDevice(X, Y, Width, Height, srcX, srcY,
  srcWidth, srcHeight: integer; Graphic: TGraphic);
begin
   if Device <> nil then
      Device.StretchDrawDevice(X, Y, Width, Height, srcX, srcY, srcWidth, srcHeight, Graphic);
end;

function TDevice_Device.TextHeight(const Text: string): Integer;
begin
   result := 0;
   if Device <> nil then
      result := Device.TextHeight(Text);
end;

function TDevice_Device.TextModelHeight(const Text: string): double;
begin
   result := 0;
   if Device <> nil then
      result := Device.TextModelHeight(Text);
end;

function TDevice_Device.TextModelWidth(const Text: string): double;
begin
   result := 0;
   if Device <> nil then
      result := Device.TextModelWidth(Text);
end;

procedure TDevice_Device.TextOut(X, Y: double; const Text: string);
begin
   if Device <> nil then
      Device.TextOut(X, Y, Text);
end;

procedure TDevice_Device.TextOutDevice(X, Y: integer; const Text: string);
begin
   if Device <> nil then
      Device.TextOutDevice(X, Y, Text);
end;

function TDevice_Device.TextWidth(const Text: string): Integer;
begin
   result := 0;
   if Device <> nil then
      result := Device.TextWidth(Text);
end;

procedure TDevice_Device.WriteLnText(aText: string);
begin
   if Device <> nil then
      Device.WritelnText(aText);
end;

procedure TDevice_Device.WriteText(aText: string);
begin
   if Device <> nil then
      Device.WriteText(aText);
end;

{ TTransparentTileDevice }

procedure TTransparentTileDevice.Close;
var
   CDevice: TDevice;
begin
   if Device = nil then exit;
   CDevice := Device;
   if (CDevice is TPrintDevice) then
   begin
      {
      Bitmap.Canvas.Pen.Style := psSolid;
      Bitmap.Canvas.Pen.Color := 0;
      Bitmap.Canvas.Rectangle(0,0,100,100);
      }
      CDevice.StretchDrawDevice(DeviceX,DeviceY,TileX,TileY,0,0,TileX,TileY,Bitmap);
   end
   else
   begin
      CDevice.DrawDevice(DeviceX,DeviceY,Bitmap);
      {
      CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                              Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
      }
   end;

end;

destructor TTransparentTileDevice.Destroy;
begin
   if BitmapT <> nil then
      Bitmap.Free;
   Tiles := nil;
   inherited;
end;

procedure TTransparentTileDevice.NextTile;
var
   Center: TDoubleCoord;
begin
   Tile := Tile + 1;
   LastTile := Tile >= TilesCount;
   if LastTile then
   begin
      Bitmap := nil;
      exit;
   end;
   DeviceX  := Tiles[Tile].DeviceX;
   DeviceY  := Tiles[Tile].DeviceY;
   Center.X := Tiles[Tile].X;
   Center.Y := Tiles[Tile].Y;
   TileX    := Tiles[Tile].TileX;
   TileY    := Tiles[Tile].TileY;
   Projection.ChangeProjection(Center,Device.Projection.Scale,Page);
end;

procedure TTransparentTileDevice.ReFreshDevice;
var
   CDevice: TDevice;
begin
   if Device = nil then exit;
   CDevice := Device;
   if not (CDevice is TPrintDevice) then
   begin
      CDevice.DrawDevice(DeviceX,DeviceY,Bitmap);
      {
      CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                              Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
      }
   end;

end;

procedure TTransparentTileDevice.SetBitmapT(const Value: Vcl.Graphics.TBitmap);
begin
   if FBitmapT <> nil then
   begin
      FBitmapT.Free;
      FBitmapT := nil;
   end;
   FBitmapT := Value;
end;

procedure TTransparentTileDevice.SetByFrame(const Value: boolean);
begin
  FByFrame := Value;
end;

procedure TTransparentTileDevice.SetDevice(const Value: TDevice);
var
   //aPage: T_Page;
   aByte: integer;
   BMx, BMy, XCount, YCount: integer;
   ME: TMemoryElement;
   RasInfo: TStringListElement;
   MemoryDC,MemoryDCT: HDC;
   BM,BMT: HBitmap;
   MA: longWord;
   BasicMem: integer;
   Center: TDoubleCoord;
   BMP, BMPT: Vcl.Graphics.TBitmap;
   CX1,CX2,CY1,CY2: double;
   xi,yi,ti: integer;
   F: TDoubleRect;
begin
   FDevice := Value;
   Projection.VirtualScale := Value.Projection.VirtualScale;
   // inicializace promennych nebo co .....
   StartX     := 0;
   StartY     := 0;
   XCount     := 1;  // pocet dlazdic horizontalne
   YCount     := 1;  // totez vertikalne
   TilesCount := 1;
   if ByFrame then
   begin
      F := Frame;
      StartX := FDevice.Projection.GetPageX(F.MinX);
      StartY := FDevice.Projection.GetPageY(F.MaxY);
      BMX    := FDevice.Projection.GetPageLengthHoriz1(F.MaxX - F.MinX);
      BMY    := FDevice.Projection.GetPageLengthVert1(F.MaxY - F.MinY);
   end
   else
   begin
      BMx := FDevice.Page.Width;
      BMy := FDevice.Page.Height;
   end;
   TileX := BMX;
   TileY := BMy;
   Bitmap :=  nil;
   BitmapT := nil;
   // Normalni bitmapa
   BMP := Vcl.Graphics.TBitmap.Create;
   BMP.Pixelformat := pf24Bit;
   BMP.Width := 1;
   BMP.Height := 1;
   // bitmapa pro transparency
   BMPT := Vcl.Graphics.TBitmap.Create;
   BMPT.Pixelformat := pf24Bit;
   BMPT.Width := 1;
   BMPT.Height := 1;
   // Bytu bude dvakrat tolik ... az pri vypoctu .. nesmi prekrocit 32767
   // Uprava zakl.sirky bitmapy
   aByte := TileX * 3;
   while aByte  > 32767 do
   begin
      TileX  := (TileX div 2) + (TileX mod 2);  // Tile X je polovicni
      XCount := XCount * 2;                     // dvojnasobny pocet horizontalne
      aByte  := TileX  * 3;                     // a vypocet aByte
   end;
   // pocet pixelu nesmi prekrocit Maxint
   while ((MaxInt div aByte) < (TileY))do
   //***while TilesCount < 5 do
   begin
      if TileX > TileY then
      begin
         TileX  := (TileX div 2) + (TileX mod 2);  // Tile X je polovicni
         XCount := XCount * 2;                     // dvojnasobny pocet horizontalne
         aByte  := TileX  * 3;                     // a vypocet aByte
      end
      else
      begin
         YCount := YCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
   end;
   MemoryDC := CreateCompatibleDC(BMP.Canvas.Handle);
   MemoryDCT := CreateCompatibleDC(BMPT.Canvas.Handle);
   // Kontrola memory
   BasicMem := 20000000;
   ME := TMemoryElement.Create;
   RasInfo := nil;
   if World <> nil then
      RasInfo := TStringListElement(World.SystemRegister.FindElementName('RASTER'));
   if RasInfo = nil then MA := BasicMem
   else
   begin
      MA := RasInfo.GetIntegerByName('MEMORY_TILE');
      if MA = 0 then MA := BasicMem;
   end;
   if ME.AvailMemory < MA then MA := ME.AvailMemory;
   if MA < 2000000 then
   begin
      //PostTextMsg(tm_AddToProtocol,'TILE... kritick stav pamti: '+IntToStr(MA),nil);
      if ME.AvailPageFile > 2000000 then
      begin
         MA := 2000000;
      end
      else
      begin
         PostTextMsg(tm_AddToProtocol,'TILE... Zoufale kritick stav pamti ... uvolnte msto na swapovacm disku!',nil);
         MA := 0;
      end;
   end;
   ME.Free;
   // sirka dlazdice nesmi prekrocit 2^15/ 3  tj asi 10900 bodu    32767 / 3
   // soucin sirka(aByte) * vyska * 2 nesmi prekrocit 2^31 t.j. Max integer tedy asi  2147483647
   if MA > 0 then
   begin
      while (aByte * TileY) > (MA div 2) do
      //***while TilesCount < 5 do
      begin
         if TileX > TileY then
         begin
            TileX  := (TileX div 2) + (TileX mod 2);  // Tile X je polovicni
            XCount := XCount * 2;                     // dvojnasobny pocet horizontalne
            aByte  := TileX  * 3;                     // a vypocet aByte
         end
         else
         begin
            YCount := YCount * 2;
            TileY := (TileY div 2) + (TileY mod 2);
         end;
      end;
   end;
   BM := CreateCompatibleBitmap(BMP.Canvas.Handle,TileX,TileY);
   BMT := CreateCompatibleBitmap(BMPT.Canvas.Handle,TileX,TileY);
   if BMT = 0 then
   begin
      DeleteObject(BM);
      BM := 0;
   end;
   while BM = 0 do
   begin
      if TileX > TileY then
      begin
         TileX  := (TileX div 2) + (TileX mod 2);  // Tile X je polovicni
         XCount := XCount * 2;                     // dvojnasobny pocet horizontalne
         //aByte  := TileX  * 3;                     // a vypocet aByte
      end
      else
      begin
         YCount := YCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
      BM := CreateCompatibleBitmap(BMP.Canvas.Handle,TileX,TileY);
      BMT := CreateCompatibleBitmap(BMPT.Canvas.Handle,TileY,TileY);
      if BMT = 0 then
      begin
         DeleteObject(BM);
         BM := 0;
      end;
   end;
   // prava potu dladic
   while not(((TileX * XCount) - TileX) < BMx) do Dec(XCount);
   while not(((TileY * YCount) - TileY) < BMy) do Dec(YCount);
   TilesCount := XCount * YCount;
   PostTextMsg(tm_ShowText,'TILE... Poet dladic '+ IntToStr(TilesCount),nil);
   Application.ProcessMessages;
   SelectObject(MemoryDC,BM);
   BMP.Handle := BM;
   BMP.Canvas.Handle := MemoryDC;
   Application.ProcessMessages;
   // transparency
   SelectObject(MemoryDCT,BMT);
   BMPT.Handle := BMT;
   BMPT.Canvas.Handle := MemoryDCT;
   Application.ProcessMessages;
   KillBitmap := true;
   Bitmap := BMP;
   BitmapT := BMPT;
 // Konec vytvoreni Bitmapy
   //Definice dlazdic
   Page.SetResolution(Device.Page.HRes, Device.Page.VRes);
   SetLength(Tiles,TilesCount);
   ti := 0;
   DeviceX := 0;
   for xi := 0 to XCount -1 do
   begin
      DeviceY := 0;
      CX2 := Device.Projection.GetModelX(StartX + ((xi * TileX) + (TileX div 2)+ (TileX mod 2)));
      CX1 := Device.Projection.GetModelX(StartX + ((xi * TileX) + (TileX div 2)));
      Center.X := (CX1 + CX2)/2;
      for yi := 0 to YCount - 1 do
      begin
         CY1 := Device.Projection.GetModelY(StartY + ((yi * TileY) + (TileY div 2)+ (TileY mod 2)));
         CY2 := Device.Projection.GetModelY(StartY + ((yi * TileY) + (TileY div 2)));
         Center.Y := (CY1 + CY2)/2;
         Tiles[ti].X := Center.X;
         Tiles[ti].Y := Center.Y;
         Tiles[ti].DeviceX := StartX + DeviceX;
         Tiles[ti].DeviceY := StartY + DeviceY;

         if (DeviceX + TileX) > BMx then
            Tiles[ti].TileX := BMx - DeviceX
         else
            Tiles[ti].TileX := TileX;

         if (DeviceY + TileY) > BMy then
            Tiles[ti].TileY := BMy - DeviceY
         else
            Tiles[ti].TileY := TileY;

         Inc(ti);
         DeviceY := DeviceY + TileY;
      end;
      DeviceX := DeviceX + TileX;
   end;
   Page.Color := Device.Page.Color;
   Tile    := -1;
   DeviceY := 0;
   LastTile := false;
   NextTile;
end;


procedure TTransparentTileDevice.SetFrame(const Value: TDoubleRect);
begin
  FFrame := Value;
end;

procedure TTransparentTileDevice.SetFrameElement(const Value: TElement);
var
   R: TRectEntity;
begin
   ByFrame := false;
   if Value = nil then exit;
   if not (Value is TRectEntity) then exit;
   R := TRectEntity(Value);
   FFrame.MinX := R.MinX;
   FFrame.MinY := R.MinY;
   FFrame.MaxX := R.MaxX;
   FFrame.MaxY := R.MaxY;
   ByFrame    := true;
end;

procedure TTransparentTileDevice.SetStartX(const Value: integer);
begin
  FStartX := Value;
end;

procedure TTransparentTileDevice.SetStartY(const Value: integer);
begin
  FStartY := Value;
end;

procedure TTransparentTileDevice.SetTileX(const Value: integer);
begin
  FTileX := Value;
end;

procedure TTransparentTileDevice.SetTileY(const Value: integer);
begin
  FTileY := Value;
end;

procedure TTransparentTileDevice.SetTransparency(const Value: integer);
begin
  FTransparency := Value;
end;

procedure TTransparentTileDevice.SetTransparencyMode(const Value: boolean);
begin
  FTransparencyMode := Value;
end;

procedure TTransparentTileDevice.TransparencyBegin(Value: integer);
var
   aX,aY: integer;
begin
   //***
   if Value = 0 then exit;
   if Value = 100 then exit;
   NoPageSetup := true;
   Canvas := FBitmapT.Canvas;
   NoPageSetup := false;
   aX := Bitmap.Width;
   aY := Bitmap.Height;
   BitmapT.Canvas.CopyRect(Classes.Rect(0,0,aX,aY), Bitmap.Canvas,Classes.Rect(0,0,aX,aY));
   Transparency := Value;
   TransparencyMode := true;
end;

procedure TTransparentTileDevice.TransparencyEnd;
var
   W,H: integer;
   PB1,PB2: PByteArray;
   i,j: integer;
begin
   if not TransparencyMode then exit;
   TransparencyMode := false;
   NoPageSetup := true;
   Canvas := FBitmap.Canvas;
   NoPageSetup := false;
   W := FBitmap.Width;
   H := FBitmap.Height;
   i := 0;
   j := 0;
   try
   H := H - 1;
   W := W * 3 - 1;
   for i := 0 to H do
   begin
      PB1 := FBitmap.ScanLine[i];
      PB2 := FBitmapT.ScanLine[i];
      for j := 0 to W do
      begin
         if (PB1[j] <> PB2[j]) then
            PB1[j] := (PB1[j]*Transparency div 100) + (PB2[j]*(100 - Transparency) div 100)
      end;
   end;
   except
      on E: Exception do
         ShowMessage('Transp i:'+IntToStr(i)+' j:'+IntToStr(j)+' '+E.Message);
   end;
   Transparency := 0;
   Application.ProcessMessages;
end;

END.
