unit t_CreateIslandAreas;

interface
uses
   Elements,
   e_Geometry,
   Tools;

type

// Input - Entita Ploch
// Output - Entita (AreaString) Ostrov (prostorov slouench ploch)
TCreateIslandAreas = class(TTool)
  private
    FRefAreaClassName: string;
    FRefAreaClass: TElementClass;
    FClearHoles: boolean;
    procedure SetRefAreaClass(const Value: TElementClass);
    procedure SetRefAreaClassName(const Value: string);
    procedure SetClearHoles(const Value: boolean);
public
   procedure DoIt; override;
   procedure FreeInnerLines(Lines: TEntity);
   function  TestLine(aLine: TPolyLineElement): boolean;
   procedure SetIslandNumbers;
   property  RefAreaClass: TElementClass read FRefAreaClass write SetRefAreaClass;
published
   property RefAreaClassName: string read FRefAreaClassName write SetRefAreaClassName;
   property ClearHoles: boolean read FClearHoles write SetClearHoles;
end;


implementation
uses
   Forms,
   SysUtils,
   e_Class,
   t_AreMak,
   t_LineExtractor;

{ TCreateIslandAreas }

procedure TCreateIslandAreas.DoIt;
var
   LE: TLineExtractor;
   MA: TAreasFromClearLines;
   Lines: TPolyLineString;
   i: integer;
begin
   Output := nil;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   if Input.NoElements then exit;
   if RefAreaClass = nil then exit;
   Lines := TPolyLineString.Create;
   Lines.Name := 'Lines';
   AddElement(Lines);
   StartProgress('Tvorba ostrov',0,0);
   ProgressText := 'Extrakce lini';
   Application.ProcessMessages;
   LE := TLineExtractor.Create;
   LE.Input := Input;
   LE.Output := Lines;
   LE.NoDuplicates := true;
   LE.DoIt;
   LE.Free;
   ProgressText := 'Eliminace vnitnch lini';
   Application.ProcessMessages;
   FreeInnerLines(Lines);
   ProgressText := 'Plochy...';
   Application.ProcessMessages;
   Lines.Update;
   MA := TAreasFromClearLines.Create;
   MA.Input := Lines;
   MA.DoIt;
   Output := MA.Output;
   SetIslandNumbers;
   MA.Free;
   Lines.Free;
end;

procedure TCreateIslandAreas.FreeInnerLines(Lines: TEntity);
var
   i: integer;
   e: TElement;
   L: TPolyLineElement;
begin
   if Lines.NoElements then exit;
   Maxprogress := Lines.LastIndex;
   for i := Lines.LastIndex downto 0 do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := Lines.Elements[i];
      if not (e is TPolyLineElement) then continue;
      L := TPolyLineElement(e);
      if TestLine(L) then
      begin
         Lines.FreeElement(L);
      end;
   end;
end;

procedure TCreateIslandAreas.SetClearHoles(const Value: boolean);
begin
  FClearHoles := Value;
end;

procedure TCreateIslandAreas.SetIslandNumbers;
var
   E: TEntity;
   i: integer;
begin
   E := TEntity(Output);
   if E.NoElements then exit;
   for i := 0 to E.LastIndex do
   begin
      E.Elements[i].Name := IntToStr(i+1);
   end;
end;

procedure TCreateIslandAreas.SetRefAreaClass(const Value: TElementClass);
begin
  FRefAreaClass := Value;
end;

procedure TCreateIslandAreas.SetRefAreaClassName(const Value: string);
begin
   FRefAreaClassName := Value;
   RefAreaClass := GetOrbisClass(Value);
end;

function TCreateIslandAreas.TestLine(aLine: TPolyLineElement): boolean;
var
   L: TPolyLineElement;
   i: integer;
   E: TEntity;
   P1,P2: TElement;
   Are: TPolyLineAreaElement;
begin
   Result := True;
   if aLine = nil then exit;
   L := aLine;
   P1 := nil;
   P2 := nil;
   for i := 0 to L.OwnersCount - 1 do
   begin
      E := L.Owners[i];
      if not (E is TPolyLineAreaElement) then continue;
      Are := TPolyLineAreaElement(E);
      if Are.IsHole then
      begin
         Are := TPolyLineAreaElement(Are.HoleOwnerArea);
         if Are <> nil then
            P2 := Are.FindClassInOwners(RefAreaClass);
      end
      else
         P2 := E.FindClassInOwners(RefAreaClass);
      if P2 <> nil then
      begin
         if P1 = nil then
         begin
            P1 := P2;
            P2 := nil;
         end
         else
            Break;
      end;
   end;
   if P1 = nil then exit;
   if P2 = nil then
   begin
      result := false;
      exit;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateIslandAreas);
END.
