unit t_GeometryCHecks;

interface
uses
   Elements,
   Tools;
type

TGeometryCheckTool = class(TTool)
private
    FLayer: TEntity;
    procedure SetLayer(const Value: TEntity);
private
   FLayerName: string;
   procedure SetLayerName(const Value: string);
   function FindLayer(aLayer: string): TEntity;
public
published
   property Layer: TEntity read FLayer write SetLayer;
end;

TClearNoValidLines = class(TResultTool)
  private
    FErrorCount: integer;
    FWithoutPrompt: boolean;
    procedure SetErrorCount(const Value: integer);
    procedure SetWithoutPrompt(const Value: boolean);
protected
   function NumberToPropertyName(aNumber: longInt): string; override;
public
   procedure DoIt; override;
   property  ErrorCount: integer read FErrorCount write SetErrorCount;
published
   property WithoutPrompt: boolean read FWithoutPrompt write SetWithoutPrompt;
end;

TFindAreaPseudoBorders = class(TTool)
  private
    FRefAreaClassName: string;
    FRefAreaClass: TElementClass;
    FRefAreasLayer: TEntity;
    procedure SetRefAreaClass(const Value: TElementClass);
    procedure SetRefAreaClassName(const Value: string);
    procedure SetRefAreasLayer(const Value: TEntity);
public
   procedure DoIt; override;
   property RefAreaClass: TElementClass read FRefAreaClass write SetRefAreaClass;
published
   property RefAreasLayer: TEntity read FRefAreasLayer write SetRefAreasLayer;
   property RefAreaClassName: string read FRefAreaClassName write SetRefAreaClassName;
end;

implementation
uses
   Dialogs,
   Forms,
   Math,
   SysUtils,
   e_Class,
   e_GeometryElement,
   e_Geometry,
   t_CmdLn;


{ TGeometryCheckTool }

function TGeometryCheckTool.FindLayer(aLayer: string): TEntity;
var
   OIL: TCommandLine;
   E: TElement;
begin
   result := nil;
   OIL := TCommandLine.Create;
   OIL.Line := aLayer;
   OIL.DoIt;
   {$IFDEF WIN64}
   E := TElement(StrToInt64Def(OIL.ResultLine, 0));
   {$ELSE}
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   {$ENDIF}
   OIL.Free;
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;

procedure TGeometryCheckTool.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

procedure TGeometryCheckTool.SetLayerName(const Value: string);
begin
   FLayerName := Value;
   Layer := FindLayer(Value);
end;

{ TClearNoValidLines }

procedure TClearNoValidLines.DoIt;
var
   i: integer;
   E: TEntity;
   El: TElement;
   G: TGeometryElement;
   L: TPolyLineElement;
   LastI: integer;
   X1,Y1,X2,Y2: double;
begin
   ErrorCount := 0;
   if Input = nil then exit;
   if not(Input is TEntity) then exit;
   E := TEntity(Input);
   LastI := E.LastIndex;
   if E.Count > 100 then
      StartProgress(Name,0,LastI);
   for i := E.LastIndex downto 0 do
   begin
      if i mod 50 = 0 then
      begin
         Progress := LastI-i;
         ProgressText := intToStr(LastI-i)+'/'+IntToStr(ErrorCount);
         Application.ProcessMessages;
      end;
      El := E.Elements[i];
      G := nil;
      if(El is TGeometryElement) then
      begin
         G := TGeometryElement(El);
         if G.NoGeometry then
         begin
            G.Free;
            ErrorCount := ErrorCount + 1;
            Continue;
         end;
         if not (G.Geometry is TPolyLineElement) then continue;
         L := TPolyLineElement(G.Geometry);
      end
      else
      begin
         if not (El is TPolyLineElement) then continue;
         L := TPolyLineElement(El);
      end;
      if L.Count < 2 then
      begin
         L.Free;
         if G <> nil then
            G.Free;
         ErrorCount := ErrorCount + 1;
      end
      else
      if L.Count = 2 then
      begin
         X1 := L.Xi[0];
         Y1 := L.Yi[0];
         X2 := L.Xi[1];
         Y2 := L.Yi[1];
         if abs(X2-X1) < 0.01 then
         if SameValue(X1,X2,1E-8) and SameValue(Y1,Y2,1E-8) then
         begin
            L.Free;
            if G <> nil then
               G.Free;
            ErrorCount := ErrorCount + 1;
         end;
      end
      else
      if L.Length = 0 then
      begin
         L.Free;
         if G <> nil then
            G.Free;
         ErrorCount := ErrorCount + 1;
      end;
   end;
   Result := IntToStr(ErrorCount);
   if not WithoutPrompt then
      ShowMessage('Bylo odstranno '+IntToStr(ErrorCount)+' chybnch lin!');
end;


function TClearNoValidLines.NumberToPropertyName(aNumber: Integer): string;
begin
  case aNumber of
     0: result := 'WithoutPrompt';
  end;
end;

procedure TClearNoValidLines.SetErrorCount(const Value: integer);
begin
  FErrorCount := Value;
end;

procedure TClearNoValidLines.SetWithoutPrompt(const Value: boolean);
begin
  FWithoutPrompt := Value;
end;

{ TFindAreaPseudoBorders }

procedure TFindAreaPseudoBorders.DoIt;
var
   En,OE,Ow: TEntity;
   E: TElement;
   GE: TGeometryElement;
   G: TGeometry;
   i,j: integer;
   A1,A2,Are: TAreaElement;
   E1,E2: TEntity;
begin
   OE := TOwnerEntity.Create;
   Output := OE;
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   StartProgress('Hledn pseudohranic ploch',0,En.LastIndex);
   for i := 0 to En.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := En.Elements[i];
      if not (E is TGeometryElement) then continue;
      GE := TGeometryElement(E);
      if GE.NoGeometry then continue;
      G := GE.Geometry;
      A1 := nil;
      A2 := nil;
      E1 := nil;
      E2 := nil;
      for j := 0 to G.OwnersCount - 1 do
      begin
         Ow := G.Owners[j];
         if not (Ow is TAreaElement) then Continue;
         Are := TAreaElement(Ow);
         if Are.IsHole then
            Are := TAreaElement(Are.HoleOwnerArea);
         if A1 = nil then
         begin
            A1 := Are;
            E1 := A1.FindClassWithOwnerInOwners(RefAreaClass,RefAreasLayer);
            if E1 = nil then
            begin
               A1 := nil;
               E1 := nil;
            end;
         end
         else
         begin
            A2 := Are;
            E2 := A2.FindClassWithOwnerInOwners(RefAreaClass,RefAreasLayer);
            if E2 = nil then
            begin
               A2 := nil;
               E2 := nil;
            end
            else
            begin
               if E1 = E2 then
               begin
                  OE.AddElement(GE);
                  Break;
               end;
            end;
         end;
      end;
   end;
end;

procedure TFindAreaPseudoBorders.SetRefAreaClass(const Value: TElementClass);
begin
  FRefAreaClass := Value;
end;

procedure TFindAreaPseudoBorders.SetRefAreaClassName(const Value: string);
begin
  FRefAreaClassName := Value;
  RefAreaClass := GetOrbisClass(Value);
end;

procedure TFindAreaPseudoBorders.SetRefAreasLayer(const Value: TEntity);
begin
   FRefAreasLayer := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryCheckTool);
   RegisterOrbisClass(TClearNoValidLines);
   RegisterOrbisClass(TFindAreaPseudoBorders);
END.
