unit t_GraphicSelectors;

interface
uses
   Types,
   Elements,
   e_QTree,
   m_Intera,
   t_CmdLn,
   t_FrameSelect,
   t_QTreeTools;

type
(*
   TGraphicSelector
   LevelName: vrstva pro proveden vbru pokud se nepouv Frame
   LocalLevelName: lokln vrstva v nalezenm Frame
   SelectLevelName: Vsledn vrstva s vbrem - ukld se tam vsledek
*)

TGraphicSelector = class(TFrameTool)
private
   FPixelTolerance: integer;
   FLocalLevelName: string;
   FMultiSelect: boolean;
   FSelectLevelName: string;
    FLevelName: string;
    FSelectLevel: TEntity;
    FLevel: TEntity;
    FDialogScript: string;
    FOnSelectScript: string;
    FInterpretAlways: boolean;
    FSelX: double;
    FSelY: double;
   procedure SetPixelTolerance(const Value: integer);
   procedure SetLocalLevelName(const Value: string);
   procedure SetMultiSelect(const Value: boolean);
   procedure SetSelectLevelName(const Value: string);
    procedure SetLevelName(const Value: string);
    procedure SetLevel(const Value: TEntity);
    procedure SetSelectLevel(const Value: TEntity);
    procedure SetDialogScript(const Value: string);
    procedure SetOnSelectScript(const Value: string);
    procedure SetInterpretAlways(const Value: boolean);
    procedure SetSelX(const Value: double);
    procedure SetSelY(const Value: double);
protected
    FShR: TRect;
    FDragging: boolean;
public
   procedure ProcessSelect(SelectedElement: TElement; MultiSel: boolean); virtual;
   procedure ProcessSelectForRect(X1,Y1,X2,Y2: double; mode: integer); virtual;
   function  FindLevel(aLevel: string): TEntity;
   procedure On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure On_Activate  ; override;
   procedure On_DeActivate; override;
   function  IsPossibleToDeactivate: boolean; override;
   property  Level: TEntity read FLevel write SetLevel;
   property  SelectLevel: TEntity read FSelectLevel write SetSelectLevel;
published
   property LevelName: string read FLevelName write SetLevelName;
   property LocalLevelName: string read FLocalLevelName write SetLocalLevelName;
   property SelectLevelName: string read FSelectLevelName write SetSelectLevelName;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property MultiSelect: boolean read FMultiSelect write SetMultiSelect;
   property InterpretAlways: boolean read FInterpretAlways write SetInterpretAlways;
   property OnSelectScript: string read FOnSelectScript write SetOnSelectScript;
   property DialogScript: string read FDialogScript write SetDialogScript;
   property SelX: double read FSelX write SetSelX;
   property SelY: double read FSelY write SetSelY;
end;

TGraphicSelectorNoFrame = class(TGraphicSelector)
private
   FQTFinder: TFindRectEntityInQTreeByXY;
   FLevelQT: TRectEntityQTree;
    FCursor1Name: string;
    FCursor0Name: string;
    FCursor2Name: string;
   function GetQTFinder: TFindRectEntityInQTreeByXY;
   procedure SetLevelQT(const Value: TRectEntityQTree);
    procedure SetCursor0Name(const Value: string);
    procedure SetCursor1Name(const Value: string);
    procedure SetCursor2Name(const Value: string);
public
   procedure  RegisterCursors; override;
   procedure  ProcessSelectForRect(X1,Y1,X2,Y2: double; mode: integer); override;
   procedure  On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure  On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure PrepareQTree; virtual;
   property  QTFinder: TFindRectEntityInQTreeByXY read GetQTFinder;
   property  LevelQT: TRectEntityQTree read FLevelQT write SetLevelQT;
published
   property Cursor0Name: string read FCursor0Name write SetCursor0Name;
   property Cursor1Name: string read FCursor1Name write SetCursor1Name;
   property Cursor2Name: string read FCursor2Name write SetCursor2Name;
end;

TPolyLineSelectorNoFrame = Class(TGraphicSelectorNoFrame)
public
   procedure  RegisterCursors; override;
   procedure  On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure  On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;

TQTreeSelector = class(TGraphicSelector)
private
    FQTree: TRectEntityQTree;
    procedure SetQTree(const Value: TRectEntityQTree);
public
   property QTree: TRectEntityQTree read FQTree write SetQTree;
end;

TDoubleQTreeSelector = class(TQTreeSelector)
private
   FQTree1: TRectEntityQTree;
   procedure SetQTree1(const Value: TRectEntityQTree);
public
   property QTree1: TRectEntityQTree read FQTree1 write SetQTree1;
end;

TPolyLineSelector = Class(TGraphicSelector)
public
   procedure  RegisterCursors; override;
   procedure  On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


TAreaSelector = class(TGraphicSelector)
public
   procedure  RegisterCursors; override;
   procedure  On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;

TPointSelector = class(TGraphicSelector)
public
   procedure  RegisterCursors; override;
   procedure  On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
end;


implementation
uses
   Classes,
   Controls,
   Graphics,
   Forms,
   SysUtils,
   e_Class,
   e_Device,
   //e_Space,
   e_Geometry,
   e_GeometryElement,
   g_Elems,
   m_Orbis,
   m_Proj,
   Tools,
   t_PickUpGeometry,
   t_SpatialAnalysis;

{ TGraphicSelector }


procedure TGraphicSelector.ProcessSelect(SelectedElement: TElement;
  MultiSel: boolean);
var
   E: TEntity;
   Change: boolean;
   procedure InterpretIt(aE: TElement);
   begin
      if Interpret = nil then exit;
      Interpret.Reset(Device);
      Interpret.Input := aE;
      Interpret.DoIt;
      Interpret.Close;
      PostMsgToOwners(tm_Repaint,nil,DontKill);
      Application.ProcessMessages;
   end;
begin
   Change := false;
   if SelectLevelName = '' then
   begin
      if SelectedElement <> Output then
      begin
         if InterpretAlways then
         begin
            if Output <> nil then
               InterpretIt(Output);
            Output := SelectedElement;
            InterpretIt(Output);
         end;
         if OnSelectScript <> '' then
            PostTextMsg(tm_Run,OnSelectScript,nil);
      end;
      exit;
   end;
   E := FindLevel(SelectLevelName);
   if E = nil then exit;
   if not MultiSel then
   begin
      InterpretIt(E);
      if E.Count > 0 then Change := true;
      E.FreeAll;
   end;
   if SelectedElement <> nil then
   begin
      if E.IndexOf(SelectedElement) >= 0 then
      begin
         E.FreeElement(SelectedElement);
      end
      else
         E.AddElement(SelectedElement);
      CHange := true;
      if E is TRectEntity then
         TRectEntity(E).ResetAndUpdate;
      InterpretIt(SelectedElement);
   end;
   if Change then
      PostMsg(tm_ModelChanged,nil,nil,false);
end;

procedure TGraphicSelector.SetSelectLevelName(const Value: string);
begin
   FSelectLevelName := Value;
end;

procedure TGraphicSelector.SetSelX(const Value: double);
begin
  FSelX := Value;
end;

procedure TGraphicSelector.SetSelY(const Value: double);
begin
  FSelY := Value;
end;

procedure TGraphicSelector.SetLocalLevelName(const Value: string);
begin
   FLocalLevelName := Value;
end;

procedure TGraphicSelector.SetMultiSelect(const Value: boolean);
begin
   FMultiSelect := Value;
end;


procedure TGraphicSelector.SetPixelTolerance(const Value: integer);
begin
   FPixelTolerance := Value;
end;

function TGraphicSelector.FindLevel(aLevel: string): TEntity;
var
   OIL: TCommandLine;
   E: TElement;
begin
   result := nil;
   OIL := TCommandLine.Create;
   OIL.Line := aLevel;
   OIL.DoIt;
   {$IFDEF WIN64}
   E := TElement(StrToInt64Def(OIL.ResultLine, 0));
   {$ELSE}
   E := TElement(StrToIntDef(OIL.ResultLine, 0));
   {$ENDIF}
   OIL.Free;
   if not (E is TEntity) then exit;
   result := TEntity(E);
end;

procedure TGraphicSelector.SetLevelName(const Value: string);
begin
  FLevelName := Value;
end;

procedure TGraphicSelector.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TGraphicSelector.SetSelectLevel(const Value: TEntity);
begin
  FSelectLevel := Value;
end;


procedure TGraphicSelector.SetDialogScript(const Value: string);
begin
  FDialogScript := Value;
end;

procedure TGraphicSelector.SetOnSelectScript(const Value: string);
begin
  FOnSelectScript := Value;
end;

procedure TGraphicSelector.SetInterpretAlways(const Value: boolean);
begin
  FInterpretAlways := Value;
end;

procedure TGraphicSelector.On_tm_Review(var aMessage: TOrbisMessage;  var Handled: boolean);
begin
   Output := nil;
   if OnSelectScript <> '' then
      PostTextMsg(tm_Run,OnSelectScript,nil);
end;


procedure TGraphicSelector.On_Activate;
begin
   inherited;
   Output := nil;
   if OnSelectScript <> '' then
      PostTextMsg(tm_Run,OnSelectScript,nil);
end;


procedure TGraphicSelector.On_DeActivate;
begin
   if (SelectLevelName = '') and InterpretALways and (Output <> nil) then
      ProcessSelect(nil,MultiSelect);
   inherited;
   Output := nil;
   CursorIndex := -1;
   if OnSelectScript <> '' then
      PostTextMsg(tm_Run,OnSelectScript,nil);
end;

function TGraphicSelector.IsPossibleToDeactivate: boolean;
begin
   result := true;
   On_DeActivate;
   Application.ProcessMessages;
end;

procedure TGraphicSelector.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   if not MultiSelect then exit;
   FSHR.Left:= aMessage.x;
   FSHR.Top := aMessage.y;
   FSHR.Right := aMessage.X;
   FSHR.Bottom := aMessage.Y;
end;

procedure TGraphicSelector.On_tm_DragEnd(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   aX1,aY1,aX2,aY2: double;
   mode: integer;
begin
   //PostTextMsg(tm_AddToProtocol,'D End',nil);
   //Application.ProcessMessages;
   //inherited;
   if not FDragging then exit;
   FDragging := false;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);

   aX1 := Device.Projection.GetModelX(FSHR.Left);
   aY1 := Device.Projection.GetModelY(FSHR.Bottom);
   aX2 := Device.Projection.GetModelX(FSHR.Right);
   aY2 := Device.Projection.GetModelY(FSHR.Top);
   mode := 0;
   if ssShift in aMessage.Shift then
      mode := 1
   else
   if ssCtrl in aMessage.Shift then
      mode := 2;
   ProcessSelectForRect(aX1,aY1,aX2,aY2,mode);
end;

procedure TGraphicSelector.On_tm_DragOver(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   //inherited;
   if not FDragging then exit;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   FSHR.Right := aMessage.X;
   FSHR.Bottom := aMessage.Y;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
end;

procedure TGraphicSelector.On_tm_StartDrag(var aMessage: TOrbisMouseEvent;var Handled: boolean);
begin
   //PostTextMsg(tm_AddToProtocol,'D Start',nil);
   //Application.ProcessMessages;
   //inherited;
   if not MultiSelect then exit;
   FDragging := true;
   FSHR.Right :=FSHR.Left+2;
   FSHR.Bottom := FSHR.Top+2;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
end;

procedure TGraphicSelector.ProcessSelectForRect(X1, Y1, X2, Y2: double; mode: integer);
begin

end;

{ TPolyLineSelector }

procedure TPolyLineSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   pick: TPickUpPolyLineElement;
   aX, aY: double;
   E: TElement;
   Line: TPolyLineElement;
begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       if FrameSelector = nil then exit;
       aX := Device.Projection.GetModelX(aMessage.x);
       aY := Device.Projection.GetModelY(aMessage.y);
       FrameSelector.X := aX;
       FrameSelector.Y := aY;
       FrameSelector.DoIt;
       if FrameSelector.Output = nil then exit;
       Pick:= TPickUpPolyLineElement.Create;
       Pick.Tolerance:= Device.Projection.GetModelLength(PixelTolerance);
       Pick.X:= aX;
       Pick.Y:= aY;
       E := nil;
       if FrameSelector.Output is TEntity then
       begin
          if LocalLevelName <> '' then
          begin
             E := TEntity(FrameSelector.Output).FindElementName(LocalLevelName);
             if E = nil then exit;
             Pick.Input := E;
          end;
       end;
       if E = nil then Pick.Input := FrameSelector.Output;
       Pick.DoIt;
       Line:= TPolyLineElement(Pick.Output);
       Pick.Free;
       ProcessSelect(Line, MultiSelect and (ssShift in aMessage.Shift));
       if Line <> nil then
       begin
          Output := Line;
          PostMsgToOwners(tm_ElementSelected,Line,false);
       end;
      except
        on E: Exception do
         PostTextMsg(tm_AddToProtocol,'Line selector: ' + E.Message,nil);
      end;
    end;
end;

procedure TPolyLineSelector.RegisterCursors;
begin
   NewCursorName('LINESELECT');
end;

{ TAreaSelector }

procedure TAreaSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   pick: TPickUpAreaElement;
{ TODO -cOpravit : Upravit na vbr GeometrzElementu }
   aX, aY: double;
   E: TElement;
   Area: TAreaElement;
begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       if FrameSelector = nil then exit;
       aX := Device.Projection.GetModelX(aMessage.x);
       aY := Device.Projection.GetModelY(aMessage.y);
       FrameSelector.X := aX;
       FrameSelector.Y := aY;
       FrameSelector.DoIt;
       if FrameSelector.Output = nil then exit;
       Pick:= TPickUpAreaElement.Create;
       Pick.Tolerance:= 0;
       Pick.X:= aX;
       Pick.Y:= aY;
       E := nil;
       if FrameSelector.Output is TEntity then
       begin
          if LocalLevelName <> '' then
          begin
             E := TEntity(FrameSelector.Output).FindElementName(LocalLevelName);
             if E = nil then exit;
             Pick.Input := E;
          end;
       end;
       if E = nil then Pick.Input := FrameSelector.Output;
       Pick.DoIt;
       Area:= TAreaElement(Pick.Output);
       Pick.Free;
       ProcessSelect(Area, MultiSelect and (ssShift in aMessage.Shift));
       if Area <> nil then
       begin
          Output := Area;
          PostMsgToOwners(tm_ElementSelected,Area,false);
       end;
      except
        on E: Exception do
         PostTextMsg(tm_AddToProtocol,'Area selector: ' + E.Message,nil);
      end;
    end;
end;

procedure TAreaSelector.RegisterCursors;
begin
   NewCursorName('AREASELECT');
end;

{ TPointSelector }

procedure TPointSelector.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   pick: TPickUpPointElement;
   aX, aY: double;
   E: TElement;
   Point: TPointElement;
begin
    Handled := true;
    if aMessage.Button = mbLeft then
    begin
      try
       if FrameSelector = nil then exit;
       aX := Device.Projection.GetModelX(aMessage.x);
       aY := Device.Projection.GetModelY(aMessage.y);
       FrameSelector.X := aX;
       FrameSelector.Y := aY;
       FrameSelector.DoIt;
       if FrameSelector.Output = nil then exit;
       Pick:= TPickUpPointElement.Create;
       Pick.Tolerance:= Device.Projection.GetModelLength(PixelTolerance);
       Pick.X:= aX;
       Pick.Y:= aY;
       E := nil;
       if FrameSelector.Output is TEntity then
       begin
          if LocalLevelName <> '' then
          begin
             E := TEntity(FrameSelector.Output).FindElementName(LocalLevelName);
             if E = nil then exit;
             Pick.Input := E;
          end;
       end;
       if E = nil then Pick.Input := FrameSelector.Output;
       Pick.DoIt;
       Point:= TPointElement(Pick.Output);
       Pick.Free;
       ProcessSelect(Point, MultiSelect and (ssShift in aMessage.Shift));
       if Point <> nil then
       begin
          Output := Point;
          PostMsgToOwners(tm_ElementSelected,Point,false);
       end;
      except
        on E: Exception do
         PostTextMsg(tm_AddToProtocol,'Point selector: ' + E.Message,nil);
      end;
    end;
end;

procedure TPointSelector.RegisterCursors;
begin
   NewCursorName('POINTSELECT');
end;

{ TQTreeSelector }

procedure TQTreeSelector.SetQTree(const Value: TRectEntityQTree);
begin
   FQTree := Value;
end;

{ TDoubleQTreeSelector }

procedure TDoubleQTreeSelector.SetQTree1(const Value: TRectEntityQTree);
begin
  FQTree1 := Value;
end;

{ TGraphicSelectorNoFrame }

function TGraphicSelectorNoFrame.GetQTFinder: TFindRectEntityInQTreeByXY;
begin
   if FQTFinder = nil then
   begin
      FQTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(FQTFinder);
   end;
   result := FQTFinder;
end;

procedure TGraphicSelectorNoFrame.On_Activate;
begin
   Error := 0;
   HighActivity := true;
   Level := FindLevel(LevelName);
   if Level = nil then Error := 1;
   SelectLevel := FindLevel(SelectLevelName);
   if SelectLevel = nil then Error := 1;
   PrepareQTree;
   QTFinder.Busy := false;
   inherited;
   HighActivity := false;
end;

procedure TGraphicSelectorNoFrame.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY, d: double;
  e: TElement;
begin
   aX:= Device.Projection.GetModelX(aMessage.x);
   aY:= Device.Projection.GetModelY(aMessage.y);
   d := Device.Projection.GetModelLength(PixelTolerance);
   if QTFInder.Busy then exit;
   QTFinder.Input := LevelQT;
   QTFinder.X := aX;
   QTFInder.Y := aY;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   e := QTFinder.Output;
   if e = nil then
   begin
      CursorIndex := 0;
   end
   else
   if SelectLevel <> nil then
   begin
      if SelectLevel.IndexOf(e) > -1 then
         CursorIndex := 2
      else
         CursorIndex := 1;
   end
   else
   begin
      if Output = e then
         CursorIndex := 2
      else
         CursorIndex := 1;
   end;
end;

procedure TGraphicSelectorNoFrame.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY,d: double;
   e: TElement;
begin
   if (Level = nil) then exit;
   Handled := true;
   if aMessage.Button = mbLeft then
   begin
      e := QTFinder.Output;
      ProcessSelect(e, MultiSelect and (ssShift in aMessage.Shift));
      Output := e;
   end
   else
   begin
      if DialogScript <> '' then
         PostTextMsg(tm_Run,DialogScript,nil);
   end;
end;

procedure TGraphicSelectorNoFrame.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if Level = nil then exit;
   StartProgress('Pprava vrstev...',0,0);
   LevelQT :=TRectEntityQTree.Create;
   LevelQT.Name := 'LEVEL QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := Level;
   TQ.Output := LevelQT;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
end;

procedure TGraphicSelectorNoFrame.ProcessSelectForRect(X1, Y1, X2, Y2: double; mode: integer);
var
   QTW: TQTreeWalker;
   i: integer;
   e: TElement;
   ee: TEntity;
   SelE: TEntity;
   IsSel: boolean;
   SAT: TSpatialAnalysisTool;
   ARE: TAreaElement;
   IsOver: boolean;
begin
   //Hledame Elementi
   if not MultiSelect then exit;
   ee := FindLevel(SelectLevelName);
   if ee = nil then exit;
   SelE := TQuickEntity.Create;
   SAT := TSpatialAnalysisTool.Create;
   ARE := TAreaElement.Create;
   ARE.AddXY(X1,Y1);
   ARE.AddXY(X2,Y1);
   ARE.AddXY(X2,Y2);
   ARE.AddXY(X1,Y2);
   ARE.Update;
   QTW := TQTreeWalker.Create;
   e := QTW.GetFirstElementForRectangle(LevelQT,X1,Y1,X2,Y2,0);
   while e <> nil do
   begin
      if SelE.IndexOf(e) < 0 then
      begin
         IsOver := false;
         if e is TGeometryElement then
            IsOver := SAT.Relate(ARE, TGeometryElement(e).Geometry)
         else
         if e is TGeometry then
            IsOver := SAT.Relate(ARE, TGeometry(e));
         if IsOver then
            SelE.AddElement(e);
      end;
      e := QTW.GetNextElement;
   end;
   ARE.Free;
   SAT.Free;
   QTW.Free;
   for i := SelE.LastIndex downto 0 do
   begin
      e := SelE.Elements[i];
      IsSel := e.IsMyOwner(ee);
      case mode of
         0:
         begin
            if not IsSel then
               ee.AddElement(e)
            else
               SelE.RemoveElement(e);
         end;
         1:
         begin
            if not IsSel then
               ee.AddElement(e)
            else
               ee.FreeElement(e);
         end;
         2:
         begin
            if IsSel then
               ee.FreeElement(e)
            else
               SelE.RemoveElement(e);
         end;
      end;
   end;
   if SelE.Count > 0 then
   begin
      if Interpret <> nil then
      begin
         Interpret.Reset(Device);
         Interpret.Input := SelE;
         Interpret.DoIt;
         Interpret.Close;
         PostMsgToOwners(tm_Repaint,nil,DontKill);
      end;
      PostMsg(tm_ModelChanged,nil,nil,false);
      Application.ProcessMessages;
      if ee is TRectEntity then
         TRectEntity(ee).ResetAndUpdate;
   end;
   SelE.Clear;
   SelE.Free;
end;

procedure TGraphicSelectorNoFrame.RegisterCursors;
var
   S: string;
begin
   S := 'NOSELECT';
   if Cursor0Name = '' then
      Cursor0Name := S;
   if Cursor1Name = '' then
      Cursor1Name := S;
   if Cursor2Name = '' then
      Cursor2Name := S;
   NewCursorName(Cursor0Name);
   NewCursorName(Cursor1Name);
   NewCursorName(Cursor2Name);
end;

procedure TGraphicSelectorNoFrame.SetCursor0Name(const Value: string);
begin
  FCursor0Name := Value;
end;

procedure TGraphicSelectorNoFrame.SetCursor1Name(const Value: string);
begin
  FCursor1Name := Value;
end;

procedure TGraphicSelectorNoFrame.SetCursor2Name(const Value: string);
begin
  FCursor2Name := Value;
end;

procedure TGraphicSelectorNoFrame.SetLevelQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLevelQT), Value);
end;

{ TPolyLineSelectorNoFrame }

procedure TPolyLineSelectorNoFrame.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY, d: double;
  e: TElement;
begin
   aX:= Device.Projection.GetModelX(aMessage.x);
   aY:= Device.Projection.GetModelY(aMessage.y);
   d := Device.Projection.GetModelLength(PixelTolerance);
   if QTFInder.Busy then exit;
   QTFinder.Input := LevelQT;
   QTFinder.X := aX;
   QTFInder.Y := aY;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   e := QTFinder.Output;
   if e = nil then
   begin
      CursorIndex := 0;
   end
   else
   if SelectLevel.IndexOf(e) > -1 then
   begin
      CursorIndex := 2;
   end
   else
      CursorIndex := 1;
end;

procedure TPolyLineSelectorNoFrame.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aX,aY,d: double;
   e: TElement;
begin
   if (Level = nil) or (SelectLevel = nil) then exit;
   Handled := true;
   if aMessage.Button = mbLeft then
   begin
      {
      aX := Device.Projection.GetModelX(aMessage.x);
      aY := Device.Projection.GetModelY(aMessage.y);
      d := Device.Projection.GetModelLength(PixelTolerance);
      QTFinder.Input := LevelQT;
      QTFinder.X := aX;
      QTFInder.Y := aY;
      QTFinder.Tolerance := d;
      QTFinder.DoIt;
      }
      e := QTFinder.Output;

      ProcessSelect(e, MultiSelect and (ssShift in aMessage.Shift));
      Output := e;
   end
   else
   begin
      if DialogScript <> '' then
         PostTextMsg(tm_Run,DialogScript,nil);
   end;
end;

procedure TPolyLineSelectorNoFrame.RegisterCursors;
begin
   NewCursorName('NOSELECT');
   NewCursorName('LINESELECT');
   NewCursorName('LINESELECTED');
end;

initialization
   RegisterOrbisClass(TGraphicSelectorNoFrame);
   RegisterOrbisClass(TPolyLineSelectorNoFrame);
   RegisterOrbisClass(TQTreeSelector);
   RegisterOrbisClass(TDoubleQTreeSelector);
   RegisterOrbisClass(TPolyLineSelector);
   RegisterOrbisClass(TAreaSelector);
   RegisterOrbisClass(TPointSelector);
end.
