unit t_NodeEditor;

interface
uses
   Types,
   Elements,
   e_Geometry,
   e_Node,
   e_QTree,
   g_Elems,
   m_Intera,
   Tools,
   t_GraphicEditors,
   t_Snap,
   t_QTreeTools;


type

TNodeEditor_SelStatus=(ness_None,ness_Node,ness_Line,ness_Point,ness_Insert);

TNodeEditor = class(TGraphicEditor)
  private
    FShR: TRect;
    FDragging: boolean;
    FDragBusy: boolean;
    FR: TRectEntity;
    FX: double;
    FY: double;
    FdX: double;  // coord difference
    FdY: double;
    FrX: double;  // reference coord
    FrY: double;
    FSetupCI: integer;
    FPixelTolerance: integer;
    FLevelName: string;
    FLevel: TEntity;
    FNodeQT: TRectEntityQTree;
    FTrainingNode: TRectEntity;
    FLineQT: TRectEntityQTree;
    FSnapTool: TPointLineSnapTool;
    FSelNodes: TEntity;
    FSelLines: TEntity;
    FSelPoints: TEntity;
    FSelP: integer;
    FSelN: TNode;
    FSelStatus: TNodeEditor_SelStatus;
    FSelL: TPolyLineElement;
    FSelShift: boolean;
    FSelSel: boolean;
    FIsSelect: boolean;
    FSnapLevelName: string;
    FSnapQT: TRectEntityQTree;
    FAuxProjectsName: string;
    FAuxProjects: TEntity;
    FLevel1Name: string;
    FLevel1: TEntity;
    FSelCtrl: boolean;
    FYIns: double;
    FXIns: double;
    FAllowEditTool: TBooleanTool;
    FVarningText: string;
    procedure SetLevel(const Value: TEntity);
    procedure SetLevelName(const Value: string);
    procedure SetNodeQT(const Value: TRectEntityQTree);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    function GetR: TRectEntity;
    procedure SetDragBusy(const Value: boolean);
    function GetNode: TNode;
    function GetTrainingNode: TRectEntity;
    procedure SetLineQT(const Value: TRectEntityQTree);
    procedure SetSnapTool(const Value: TPointLineSnapTool);
    function GetSelLines: TEntity;
    function GetSelNodes: TEntity;
    function GetSelPoints: TEntity;
    procedure SetSelL(const Value: TPolyLineElement);
    procedure SetSelN(const Value: TNode);
    procedure SetSelP(const Value: integer);
    procedure SetSelStatus(const Value: TNodeEditor_SelStatus);
    procedure SetSelShift(const Value: boolean);
    procedure SetSelSel(const Value: boolean);
    procedure SetIsSelect(const Value: boolean);
    function GetSnapping: boolean;
    procedure SetSnapLevelName(const Value: string);
    procedure SetSnapQT(const Value: TRectEntityQTree);
    procedure SetAuxProjectsName(const Value: string);
    procedure SetAuxProjects(const Value: TEntity);
    procedure SetLevel1Name(const Value: string);
    procedure SetLevel1(const Value: TEntity);
    procedure SetSelCtrl(const Value: boolean);
    procedure SetXIns(const Value: double);
    procedure SetYIns(const Value: double);
    procedure SetAllowEditTool(const Value: TBooleanTool);
    procedure SetVarningText(const Value: string);
public
   procedure   XORDraw; override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_StartDrag(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragOver(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_DragEnd(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean); override;
   procedure   On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure On_DeActivate; override;
   procedure RegisterCursors; override;
   procedure PrepareQTree; virtual;
   procedure PrepareSnapQTree; virtual;
   procedure CHangeSelectCursor;
   procedure CHangeSetupCursor;
   procedure PrepareTrainingNode;
   procedure SelectNode(aNode: TNode);
   procedure SelectLinePoint(aLine: TPolyLineElement; aIndex: integer);
   function  CheckNode(aNode: TNode): boolean;
   function  CheckLine(aLine: TPolyLineElement): boolean;
   function  IsSelectedNode(aNode: TNode): boolean;
   function  IsSelectedLine(aLine: TPolyLineElement): boolean;
   function  IsSelectedLinePoint(aLine: TPolyLineElement; aIndex: integer): boolean;
   procedure StoreNode;
   procedure ClearSelect;
   property NodeQT: TRectEntityQTree read FNodeQT write SetNodeQT;
   property LineQT: TRectEntityQTree read FLineQT write SetLineQT;
   property SnapQT: TRectEntityQTree read FSnapQT write SetSnapQT;
   property Level: TEntity read FLevel write SetLevel;
   property Level1: TEntity read FLevel1 write SetLevel1;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property XIns: double read FXIns write SetXIns;
   property YIns: double read FYIns write SetYIns;
   property R: TRectEntity read GetR;
   property DragBusy: boolean read FDragBusy write SetDragBusy;
   property Node: TNode read GetNode;
   property TrainingNode: TRectEntity read GetTrainingNode;
   property SelNodes: TEntity read GetSelNodes;
   property SelLines: TEntity read GetSelLines;
   property SelPoints: TEntity read GetSelPoints;
   property SelSel: boolean read FSelSel write SetSelSel;      // aktualni obj je selected
   property SelShift: boolean read FSelShift write SetSelShift;// mys jede se shiftem
   property SelCtrl: boolean read FSelCtrl write SetSelCtrl;   // mys jede s ctrl - body
   property SelN: TNode read FSelN write SetSelN;              // aktualni nod pro select
   property SelL: TPolyLineElement read FSelL write SetSelL;   // aktualni line pro select
   property SelP: integer read FSelP write SetSelP;            // dtto index bodu
   property SelStatus: TNodeEditor_SelStatus read FSelStatus write SetSelStatus; // co je aktualni
   property IsSelect: boolean read FIsSelect write SetIsSelect; // ve vyberu uz neco je
   property Snapping: boolean read GetSnapping;
   property AuxProjects: TEntity read FAuxProjects write SetAuxProjects;
published
   property LevelName: string read FLevelName write SetLevelName;
   property Level1Name: string read FLevel1Name write SetLevel1Name;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property SnapTool: TPointLineSnapTool read FSnapTool write SetSnapTool;
   property SnapLevelName: string read FSnapLevelName write SetSnapLevelName;
   property AuxProjectsName: string read FAuxProjectsName write SetAuxProjectsName;
   property AllowEditTool: TBooleanTool read FAllowEditTool write SetAllowEditTool;
   property VarningText: string read FVarningText write SetVarningText;
end;


TLineBorderZip = class(TGraphicEditor)
private
    FPixelTolerance: integer;
    FAuxProjectsName: string;
    FLevelName: string;
    FLevel1Name: string;
    FAuxProjects: TEntity;
    FLevelpName: string;
    FLevel1pName: string;
    FLevelp: TEntity;
    FLevel1p: TEntity;
    FLevel: TEntity;
    FLevel1: TEntity;
    FNode1QT: TRectEntityQTree;
    FLineQT: TRectEntityQTree;
    FLine1QT: TRectEntityQTree;
    FNodeQT: TRectEntityQTree;
    FSelStatus: integer;
    FIsSelect: boolean;
    FSelP: integer;
    FSelN: TNode;
    FSelL: TPolyLineElement;
    FSelLQT: TRectEntityQTree;
    FSelNQT: TRectEntityQTree;
    FSelNQT1: TRectEntityQTree;
    FSelLQT1: TRectEntityQTree;
    FY: double;
    FX: double;
    FR: TRectEntity;
    FP12: integer;
    FP21: integer;
    FP11: integer;
    FP22: integer;
    FN11: TNode;
    FN21: TNode;
    FN22: TNode;
    FN12: TNode;
    FL1: TPolyLineElement;
    FL2: TPolyLineElement;
    FSelEStatus: TNodeEditor_SelStatus;
    FOnlyLevel: boolean;
    procedure SetAuxProjects(const Value: TEntity);
    procedure SetAuxProjectsName(const Value: string);
    procedure SetLevel1Name(const Value: string);
    procedure SetLevelName(const Value: string);
    procedure SetPixelTolerance(const Value: integer);
    procedure SetLevel1pName(const Value: string);
    procedure SetLevelpName(const Value: string);
    procedure SetLevel(const Value: TEntity);
    procedure SetLevel1(const Value: TEntity);
    procedure SetLevel1p(const Value: TEntity);
    procedure SetLevelp(const Value: TEntity);
    procedure SetLine1QT(const Value: TRectEntityQTree);
    procedure SetLineQT(const Value: TRectEntityQTree);
    procedure SetNode1QT(const Value: TRectEntityQTree);
    procedure SetNodeQT(const Value: TRectEntityQTree);
    procedure SetSelStatus(const Value: integer);
    procedure SetIsSelect(const Value: boolean);
    procedure SetSelL(const Value: TPolyLineElement);
    procedure SetSelN(const Value: TNode);
    procedure SetSelP(const Value: integer);
    procedure SetSelLQT(const Value: TRectEntityQTree);
    procedure SetSelNQT(const Value: TRectEntityQTree);
    procedure SetSelLQT1(const Value: TRectEntityQTree);
    procedure SetSelNQT1(const Value: TRectEntityQTree);
    function GetR: TRectEntity;
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetL1(const Value: TPolyLineElement);
    procedure SetL2(const Value: TPolyLineElement);
    procedure SetN11(const Value: TNode);
    procedure SetN12(const Value: TNode);
    procedure SetN21(const Value: TNode);
    procedure SetN22(const Value: TNode);
    procedure SetP11(const Value: integer);
    procedure SetP12(const Value: integer);
    procedure SetP21(const Value: integer);
    procedure SetP22(const Value: integer);
    procedure SetSelEStatus(const Value: TNodeEditor_SelStatus);
    procedure SetOnlyLevel(const Value: boolean);
public
   procedure XORDraw; override;
   procedure On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure CHangeSelectCursor;
   procedure On_Activate; override;
   procedure On_DeActivate; override;
   procedure RegisterCursors; override;
   procedure PrepareQTree; virtual;
   procedure ClearSelect;
   procedure ZipIt;
   property NodeQT: TRectEntityQTree read FNodeQT write SetNodeQT;
   property Node1QT: TRectEntityQTree read FNode1QT write SetNode1QT;
   property LineQT: TRectEntityQTree read FLineQT write SetLineQT;
   property Line1QT: TRectEntityQTree read FLine1QT write SetLine1QT;
   property AuxProjects: TEntity read FAuxProjects write SetAuxProjects;
   property Level: TEntity read FLevel write SetLevel;
   property Levelp: TEntity read FLevelp write SetLevelp;
   property Level1: TEntity read FLevel1 write SetLevel1;
   property Level1p: TEntity read FLevel1p write SetLevel1p;
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property R: TRectEntity read GetR;
   property N11: TNode read FN11 write SetN11;
   property N21: TNode read FN21 write SetN21;
   property N12: TNode read FN12 write SetN12;
   property N22: TNode read FN22 write SetN22;
   property L1: TPolyLineElement read FL1 write SetL1;
   property P11: integer read FP11 write SetP11;
   property P12: integer read FP12 write SetP12;
   property L2: TPolyLineElement read FL2 write SetL2;
   property P21: integer read FP21 write SetP21;
   property P22: integer read FP22 write SetP22;
   property SelN: TNode read FSelN write SetSelN;              // aktualni nod pro select
   property SelL: TPolyLineElement read FSelL write SetSelL;   // aktualni line pro select
   property SelP: integer read FSelP write SetSelP;            // dtto index bodu
   property SelNQT: TRectEntityQTree read FSelNQT write SetSelNQT; // akt. NodeQT pro select
   property SelLQT: TRectEntityQTree read FSelLQT write SetSelLQT; // akt. LineQT pro select
   property SelNQT1: TRectEntityQTree read FSelNQT1 write SetSelNQT1; // akt. NodeQT 1 pro select
   property SelLQT1: TRectEntityQTree read FSelLQT1 write SetSelLQT1; // akt. LineQT 1 pro select
   property IsSelect: boolean read FIsSelect write SetIsSelect; // ve vyberu uz neco je
   property SelStatus: integer read FSelStatus write SetSelStatus;
   property SelEStatus: TNodeEditor_SelStatus read FSelEStatus write SetSelEStatus;
published
   property LevelName: string read FLevelName write SetLevelName;
   property LevelpName: string read FLevelpName write SetLevelpName;
   property Level1Name: string read FLevel1Name write SetLevel1Name;
   property Level1pName: string read FLevel1pName write SetLevel1pName;
   property PixelTolerance: integer read FPixelTolerance write SetPixelTolerance;
   property AuxProjectsName: string read FAuxProjectsName write SetAuxProjectsName;
   property OnlyLevel: boolean read FOnlyLevel write SetOnlyLevel;
end;

implementation
uses
   Classes,
   Controls,
   Dialogs,
   Forms,
   Graphics,
   Windows,
   SysUtils,
   Math,
   MathLib,
   Worlds,
   t_Applac,
   e_Class,
   e_Device,
   e_GeometryElement,
   e_Type,
   m_Orbis,
   t_FindBreakLineConnections,
   t_LineExtractor;

{ TNodeEditor }

function TNodeEditor.GetR: TRectEntity;
begin
   if FR = nil then
   begin
      FR := TRectEntity.Create;
      AddElement(FR);
   end;
   result := FR;
end;

procedure TNodeEditor.CHangeSelectCursor;
var
   d,dist,ddist: double;
   X1,Y1: double;
   CI: integer;
   aLine: TPolyLineElement;
   aNode: TNode;
   GE: TGeometryElement;
   e: TElement;
   i: integer;
   PataUvnitr: boolean;
   PataX, PataY: double;
begin
   SelN := nil;
   SelL := nil;
   SelP := -1;
   SelSel := false;
   SelStatus := ness_None;
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CI := 0;
   QTFinder.Input := NodeQT;
   QTFinder.X := X;
   QTFInder.Y := Y;
   QTFinder.Tolerance := d;
   QTFinder.DoIt;
   aNode := TNode(QTFinder.Output);
   if aNode <> nil then
   begin
      if not SelCtrl then // pokud neni manipulace s body....
      begin
         if not CheckNode(aNode) then
         begin
            CursorIndex := 13;         // neproslo kontrolou
            exit;
         end;
         SelN := aNode;
         SelStatus := ness_Node;
         if IsSelectedNode(aNode) then
            SelSel := true;
      end;
   end
   else
   begin
      QTFinder.Input := LineQT;
      QTFinder.X := X;
      QTFInder.Y := Y;
      QTFinder.Tolerance := d;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e <> nil then
      begin
         aLine := nil;
         if e is TPolyLineElement then
            aLine :=  TPolyLineElement(e)
         else
         begin
            if e is TGeometryElement then
            begin
               GE := TGeometryElement(e);
               if not GE.NoGeometry and (GE.Geometry is TPolyLineElement) then
                  aLine := TPolyLineElement(GE.Geometry);
            end;
         end;
         dist := d * 2;
         if (aLine <> nil) and not CheckLine(aLine) then
         begin
            CursorIndex := 13;         // neproslo kontrolou
            exit;
         end;
         // nalezeni bodu linie
         if (aLine <> nil) and (aLine.Count > 2) then // nejakou linii jsme nasli, tak jdem na body
         begin
            for i := 1 to aLine.LastIndex - 1 do
            begin
               X1 := aLine.Xi[i];
               Y1 := aLine.Yi[i];
               if InTolerance(X,Y,X1,Y1,d) then
               begin
                  ddist := distance(X,Y,X1,Y1);
                  if ddist < dist then
                  begin
                     SelL := aLine;
                     SelP := i;
                     dist := ddist;
                  end;
               end
               else
               if dist < d then
                  break;
            end;
            if SelL <> nil then
            begin
               SelStatus := ness_Point;
               if IsSelectedLinePoint(SelL,SelP) then
                  SelSel := true;
            end;
         end;
         if SelCtrl then // manipulace s body
         begin
            if SelL <> nil then
            begin
               if SelSel then // bod je vybran - nemuze se rusit
               begin
                  SelL := nil;
                  SelP := -1;
                  SelSel := false;
                  SelStatus := ness_None;
               end;
            end
            else           // bod neni - bude se vkladat
            begin
               if (aLine <> nil) and (aLine.Count > 1) then
               for i := 0 to aLine.LastIndex - 1 do
               begin
                  DejVztahyBod_Usecka(X,Y,aLine.Xi[i],aLine.Yi[i],aLine.Xi[i+1],aLine.Yi[i+1],
                                      PataUvnitr, PataX, PataY, dist);
                  if PataUvnitr and (dist < d) then
                  begin
                     SelStatus := ness_Insert;
                     SelL := aLine;
                     SelP := i + 1;
                     XIns := PataX;
                     YIns := PataY;
                     break;
                  end;
               end;
            end;
         end;
      end;
   end;
   if SelCtrl then
   begin
      case SelStatus of
         ness_Insert:  CI := 11;
         ness_Point:   CI := 12;
      end;
   end
   else
   begin
      case SelStatus of
         ness_Node:  CI := 2;
         ness_Point: CI := 1;
      end;
      if CI > 0 then // neco je pod kurzorem
      begin
         if SelShift then      // Shift - rezim vyberu
         begin
            if SelSel then
               CI := CI + 3;   // Objekt je uz ve vyberu
         end
         else                  // bez Shiftu - bud vyber nebo rovnou posun jednoho obj.
         begin
            if IsSelect then   // existuje vyber
            begin
               if SelSel then  // objekt ve vyberu je - muze se soupat
                  CI := CI + 6
               else            // neni ve vyberu takze nic
                  CI := 0;
            end
            else               // vyber neni, bude se rovnou posunovat
            begin
               CI := CI + 6
            end;
         end;
      end;
   end;
   FSetupCI := CI;
   CursorIndex := CI;
end;

procedure TNodeEditor.CHangeSetupCursor;
var
   d: double;
begin
   if Snapping then
   begin
      d := Device.Projection.GetModelLength(PixelTolerance);
      //SnapDraw(aMessage.X,aMessage.Y);
      SnapTool.X := X;
      SnapTool.Y := Y;
      SnapTool.SnapDistance := d;
      SnapTool.DoIt;
      if SnapTool.Snapped then
      begin
         X := SnapTool.SnapX;
         Y := SnapTool.SnapY;
         case SnapTool.SnapStatus of
            1: CursorIndex := 9;
            2: CursorIndex := 10;
         end;
      end
      else
         CursorIndex := FSetupCI;
   end;
   XORDraw;
   FdX := X - FrX;
   FdY := Y - FrY;
   //TrainingNode.X := X;
   //TrainingNode.Y := Y;
   XORDraw;
end;

procedure TNodeEditor.On_Activate;
begin
   EditorStatus := gesPrepare;
   HighActivity := true;
   if not Active then
      PrepareQTree;
   EditorStatus := gesSelect;
   QTFinder.Busy := false;
   if Color = 0 then Color := clSilver;
   inherited;
   HighActivity := false;
   IsSelect := false;
end;

procedure TNodeEditor.On_tm_DragEnd(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
   QTW: TQTreeWalker;
   aX1,aY1,aX2,aY2: double;
   ix,iy: double;
   N: TNode;
   L: TPolyLineElement;
   i: integer;
   e: TElement;
   Varning: boolean;

   procedure ShowVarning;
   begin
      if not Varning then
      begin
         ShowMessage(VarningText);
         Varning := true;
      end;
   end;
begin
   //PostTextMsg(tm_AddToProtocol,'D End',nil);
   //Application.ProcessMessages;
   //inherited;
   if not FDragging then exit;
   Varning := false;
   FDragging := false;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   XORDraw;
   aX1 := Device.Projection.GetModelX(FSHR.Left);
   aY1 := Device.Projection.GetModelY(FSHR.Bottom);
   aX2 := Device.Projection.GetModelX(FSHR.Right);
   aY2 := Device.Projection.GetModelY(FSHR.Top);
   //Hledame uzly
   QTW := TQTreeWalker.Create;
   e := QTW.GetFirstElementForRectangle(NodeQT,aX1,aY1,aX2,aY2,0);
   while e <> nil do
   begin
      if e is TNode then
      begin
         N := TNode(e);
         if not CheckNode(N) then
            ShowVarning
         else
         if not IsSelectedNode(N) then
            SelectNode(N);
      end;
      e := QTW.GetNextElement;
   end;
   QTW.Free;
   // Hledame linie a body
   QTW := TQTreeWalker.Create;
   e := QTW.GetFirstElementForRectangle(LineQT,aX1,aY1,aX2,aY2,0);
   while e <> nil do
   begin
      L := nil;
      if e is TPolyLineElement then
         L := TPolyLineElement(e)
      else
      if e is TGeometryElement then
      begin
         e := TGeometryElement(e).Geometry;
         if e is TPolyLineElement then
            L := TPolyLineElement(e)
      end;
      if L <> nil then
      begin
         if not CheckLine(L) then
         begin
            ShowVarning;
         end
         else
         begin
            if L.Count > 2 then
            for i := 1 to L.LastIndex - 1 do
            begin
               ix := L.Xi[i];
               iy := L.Yi[i];
               if (iX >= aX1) and (iX <= aX2) and (iY >= aY1) and (iY <= aY2) then
               begin
                  SelectLinePoint(L,i);
               end;
            end;
         end;
      end;
      e := QTW.GetNextElement;
   end;
   QTW.Free;
   XORDraw;
end;

procedure TNodeEditor.On_tm_DragOver(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   //inherited;
   if not FDragging then exit;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
   FSHR.Right := aMessage.X;
   FSHR.Bottom := aMessage.Y;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
end;

procedure TNodeEditor.On_tm_MouseDown(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   //PostTextMsg(tm_AddToProtocol,'M Down',nil);
   //Application.ProcessMessages;
   if EditorStatus <> gesSelect then exit;
   FSHR.Left:= aMessage.x;
   FSHR.Top := aMessage.y;
   FSHR.Right := aMessage.X;
   FSHR.Bottom := aMessage.Y;
end;

procedure TNodeEditor.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
var
  aX, aY: integer;
begin
   aX:= aMessage.x;
   aY:= aMessage.y;
   X:= Device.Projection.GetModelX(aX);
   Y:= Device.Projection.GetModelY(aY);
   SelShift := ssShift in aMessage.Shift;
   SelCtrl  := ssCtrl in aMessage.Shift;
   if not Active then exit;
   if EditorStatus = gesSelect then
      ChangeSelectCursor
   else
   if EditorStatus = gesSetup then
      CHangeSetupCursor;
   {
   else
   if EditorStatus = gesAdd then
      CHangeAddCursor;
   }
   {
   else
   begin
      if PolyElement = nil then exit;
      ChangeCursor;
   end;
   }
end;

procedure TNodeEditor.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   StartSetup: boolean;
begin
   //PostTextMsg(tm_AddToProtocol,'M UP',nil);
   //Application.ProcessMessages;
   if EditorStatus = gesSetup then
   begin
      // konec posunu, ulozeni uzlu
      XORDraw;
      if (aMessage.Button = mbLeft) then
      begin
         // pokud je leve tlacitko tak ulozeni
         StoreNode;
      end;
      ClearSelect;
      EditorStatus := gesSelect;
      CursorIndex := 0;
      exit;
   end;
   if (aMessage.Button = mbRight) then
   begin
      // tady asi bude dialog
      if (EditorStatus = gesSelect) and (SelStatus = ness_None) then
      begin
         XORDraw;
         ClearSelect;
      end;
      exit;
   end;
   if EditorStatus = gesSelect then
   begin
      StartSetup := false;
      if SelStatus <> ness_None then // neco je pod kurzorem
      begin
         if SelCtrl then       // manipulace s body
         begin
            XORDraw;
            case SelStatus of
               ness_Insert:  SelL.InsertXY(SelP,XIns,YIns);
               ness_Point:   SelL.FreeElementIndex(SelP);
            end;
            PostMsgToOwners(tm_ReviewByElement, SelL, false);
            Application.ProcessMessages;
            XORDraw;
         end
         else
         if SelShift then      // Shift - rezim vyberu
         begin
            if not SelSel then    // obj. jeste neni ve vyberu
            begin
               XORDraw;
               case SelStatus of
                  ness_Node:  SelectNode(SelN);
                  ness_Point: SelectLinePoint(SelL,SelP);
               end;
               XORDraw;
            end;
         end
         else                  // bez Shiftu - bud vyber nebo rovnou posun jednoho obj.
         begin
            if IsSelect then   // existuje vyber
            begin
               if SelSel then  // objekt ve vyberu je - bude se posunovat
               begin
                  StartSetup := true;
               end;
            end
            else               // vyber neni, bude se rovnou posunovat
            begin
               // Ulozeni aktualniho objektu a rovnou posun
               case SelStatus of
                  ness_Node:  SelectNode(SelN);
                  ness_Point: SelectLinePoint(SelL,SelP);
               end;
               XORDraw;
               StartSetup := true;
            end;
         end;
      end;
      if StartSetup then       // zaciname posunovat
      begin
         case SelStatus of
            ness_Node:
            begin
               FrX := SelN.X;
               FrY := SelN.Y;
            end;
            ness_Point:
            begin
               FrX := SelL.Xi[SelP];
               FrY := SelL.Yi[SelP];
            end;
         end;
         FdX := 0;
         FdY := 0;
         XORDraw;
         PrepareTrainingNode;
         EditorStatus := gesSetup;
         XORDraw;
      end;
      exit;

      // zaciname posunovat
      if Input = nil then exit;
      PrepareTrainingNode;
      XORDraw;
      EditorStatus := gesSetup;
      CursorIndex := 2;
   end;
end;

procedure TNodeEditor.On_tm_StartDrag(var aMessage: TOrbisMouseEvent;
  var Handled: boolean);
begin
   //PostTextMsg(tm_AddToProtocol,'D Start',nil);
   //Application.ProcessMessages;
   //inherited;
   if EditorStatus <> gesSelect then exit;
   FDragging := true;
   FSHR.Right :=FSHR.Left+2;
   FSHR.Bottom := FSHR.Top+2;
   Device.Pen.Mode  := pmXOR;
   Device.Pen.Color := clRed;
   Device.Pen.Style := psSolid;
   Device.Brush.Style := bsClear;
   TCanvasDevice(Device).Canvas.Pen.Width := 1;
   with FSHR do TCanvasDevice(Device).Canvas.Rectangle(Left, Top, Right, Bottom);
end;

procedure TNodeEditor.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   i: integer;
   FB: TFindBreakLineConnections;
   e: TElement;
   RE: TRectEntity;
   N: TNode;
   PomLevel: TRectEntity;

   procedure PrepareLevel(aLevel: TEntity);
   var
      LEX: TLineExtractor;
      HasLines: boolean;
   begin
      if aLevel = nil then exit;
      HasLines := false;
      if aLevel.Count > 0 then
      begin
         e := aLevel.Elements[0];
         if e is TGeometryElement then
         begin
            e := TGeometryElement(e).Geometry;
            HasLines := (e <> nil) and (e is TPolyLineElement);
         end
         else
         if e is TPolyLineElement then
            HasLines := true;
      end;
      if HasLines then
         PomLevel.AddAllFrom := aLevel
      else
      begin
         LEX := TLineExtractor.Create;
         LEX.Input := aLevel;
         LEX.Output := PomLevel;
         LEX.NoDuplicates := true;
         LEX.DoIt;
         LEX.Free;
      end;
   end;
begin
   if AuxProjectsName <> '' then
   begin
      AuxProjects := FindLevel(AuxProjectsName);
      if AuxProjects = nil then exit;
   end;
   PomLevel := TRectEntity.Create;
   AddElement(PomLevel);
   Level := FindLevel(LevelName);
   if Level = nil then exit;
   PrepareLevel(Level);
   if Level1Name <> '' then
   begin
      Level1 := FindLevel(Level1Name);
      PrepareLevel(Level1);
   end;
   PomLevel.Update;
   RE := TRectEntity.Create;
   FB := TFindBreakLineConnections.Create;
   FB.AllNodes := true;
   FB.Input := PomLevel;
   FB.Output := RE;
   FB.DoIt;
   FB.Free;
   if RE.NoElements then
   begin
      RE.Free;
      PomLevel.Free;
      exit;
   end;
   //RE.Update;
   for i := 0 to RE.LastIndex do
   begin
      e := RE.Elements[i];
      if not(e is TNode) then continue;
      N := TNode(e);
      N.MinX := N.X - 1;
      N.MaxX := N.X + 1;
      N.MinY := N.Y - 1;
      N.MaxY := N.Y + 1;
      N.AbsRefX := N.X;
      N.AbsRefY := N.Y;
      RE.UpdateRect(N);
   end;
   StartProgress('Pprava vrstev...',0,0);
   NodeQT :=TRectEntityQTree.Create;
   NodeQT.Name := 'Node QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := NodeQT;
   TQ.DoIt;
   TQ.Free;
   RE.Free;

   LineQT :=TRectEntityQTree.Create;
   LineQT.Name := 'Line QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := PomLevel;
   TQ.Output := LineQT;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
   PomLevel.Free;
   PrepareSnapQTree;
end;

procedure TNodeEditor.RegisterCursors;
begin
   NewCursorName('NOSELECT');     //0
   NewCursorName('POINTOLD');     //1
   NewCursorName('NODEOLD');      //2
   NewCursorName('LINESELECT');   //3
   NewCursorName('POINTSELECTED');//4
   NewCursorName('NODESELECTED'); //5
   NewCursorName('LINESELECTED'); //6
   NewCursorName('POINTOLDMOVE'); //7
   NewCursorName('NODEOLDMOVE');  //8
   NewCursorName('SNAP_LPOINT');  //9
   NewCursorName('SNAP_LBODY');   //10
   NewCursorName('POINTNEW');     //11
   NewCursorName('POINTDELETE');  //12
   NewCursorName('SELECTVAR');    //13
end;

procedure TNodeEditor.SetDragBusy(const Value: boolean);
begin
  FDragBusy := Value;
end;

procedure TNodeEditor.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TNodeEditor.SetLevelName(const Value: string);
begin
   FLevelName := Value;
end;

procedure TNodeEditor.SetNodeQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FNodeQT), Value);
end;

procedure TNodeEditor.SetPixelTolerance(const Value: integer);
begin
  FPixelTolerance := Value;
end;

procedure TNodeEditor.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TNodeEditor.SetY(const Value: double);
begin
  FY := Value;
end;

procedure TNodeEditor.XORDraw;
var
  aR: double;
  i,j,ni,li: integer;
  //K: integer;
  P: TPointElement;
  aX,aY,iX,iY: double;
  L: TPolyLineElement;
  ILE: TIntegerListElement;
  En: TEntity;
  N: TNode;
begin
   //exit; //!!!!!!!!!!!!!!!!!
   //if Input = nil then
   //   Exit;

  Device.Pen.Color:= Color;
  Device.Pen.Style:= psSolid;
  Device.Pen.Mode:= pmXOR;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsSolid;
  case EditorStatus of
     gesSelect:
     begin
        aR:= Device.Projection.GetModelLength(7);
        for i := 0 to SelNodes.LastIndex do
        begin
           N := TNode(SelNodes.Elements[i]);
           iX := N.X;
           iY := N.Y;
           Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
        end;
        aR:= Device.Projection.GetModelLength(5);
        for i := 0 to SelPoints.LastIndex do
        begin
           En := TEntity(SelPoints.Elements[i]);
           ILE := TIntegerListElement(En.FindElementClass(TIntegerListElement));
           L := TPolyLineElement(En.FindElementClass(TPolyLineElement));
           li := L.LastIndex;
           for j := 0 to ILE.LastIndex do
           begin
              ni := ILE.Integers[j];
              if (ni = 0) or (ni=li) then continue;
              iX := L.Xi[ni];
              iY := L.Yi[ni];
              Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
           end;
        end;
     end;
     gesSetup:
     begin
        for i := 0 to TrainingNode.LastIndex do
        begin
           En := TEntity(TrainingNode.Elements[i]);
           for j := 0 to En.LastIndex do
           begin
              P := TPointElement(En.Elements[j]);
              iX := P.X;
              iY := P.Y;
              if not P.IsDone then
              begin
                 iX := iX + FdX;
                 iY := iY + FdY;
              end;
              if j = 0 then
                 Device.MoveTo(iX,iY)
              else
                 Device.LineTo(iX,iY);
           end;
        end;
     end;
  else
     exit;
  end;
  exit;
  Device.Pen.Width:= 1;
  Device.Brush.Style:= bsClear;
  aX := TrainingNode.X;
  aY := TrainingNode.Y;
  aR:= Device.Projection.GetModelLength(3);
  For i := 0 to TrainingNode.LastIndex do
  begin
     P := TPointElement(TrainingNode.Elements[i]);
     Device.MoveTo(aX,aY);
     iX := P.X;
     iY := P.Y;
     Device.LineTo(iX,iY);
     Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
  end;
  {
  Device.Pen.Width:= 3;
  aR:= Device.Projection.GetModelLength(5);
  Device.Rectangle(aX-aR,aY-aR,aX+aR,aY+aR);

  PostMsgToOwners(tm_Repaint,nil,DontKill);
  }
end;

function TNodeEditor.GetNode: TNode;
begin
   result := TNode(Input);
end;

function TNodeEditor.GetTrainingNode: TRectEntity;
begin
   if FTrainingNode = nil then
   begin
      FTrainingNode := TRectEntity.Create;
      AddElement(FTrainingNode);
   end;
   result := FTrainingNode;
end;

procedure TNodeEditor.PrepareTrainingNode;
var
   i,j,ni,nii,li,lile,ninext: integer;
   //e: TElement;
   RE: TRectEntity;
   L: TPolyLineElement;
   ILE: TIntegerListElement;
   P: TPointElement;
   //aX,aY: double;
   SP: Tentity;
   En: TEntity;

   procedure InsertPoint(aIndex: integer; aDone: boolean);
   begin
      P := TPointElement.Create;
      P.X := L.Xi[aIndex];
      P.Y := L.Yi[aIndex];
      P.IsDone := aDone;   // kotva
      RE.AddAndUpdateRect(P);
   end;
begin
   R.Rect.Reset;
   TrainingNode.FreeAll;
   TrainingNode.X := FrX;
   TrainingNode.Y := FrY;
   SP := SelPoints;
   for i := 0 to SP.LastIndex do
   begin
      En := TEntity(SP.Elements[i]);
      L := TPolyLineElement(En.FindElementClass(TPolyLineElement));      // kontrola ????
      li := L.LastIndex;     // posledni index linie
      ILE := TIntegerListElement(En.FindElementClass(TIntegerListElement)); // kontrola ????
      lile := ILE.LastIndex; // posledni index bodu
      ninext := 0;
      RE := TRectEntity.Create;   // nova jako liniie
      TrainingNode.AddElement(RE);
      for j := 0 to lile do  // pro kazdy ulozeny index
      begin
         ni := ILE.Integers[j];
         if j = 0 then  // prvni cislo - kontrola indexu 0
         begin
            if ni > 0 then // neni to 0 vlozime kotvu
            begin
               nii := Pred(ni);
               InsertPoint(nii, true);
            end;
         end
         else
         if ni > ninext then // nesouvisla rada bodu - doplnime kotvy
         begin
            nii := ninext;              // kotva za
            InsertPoint(nii, true);
            TrainingNode.UpdateRect(RE);
            RE := TRectEntity.Create;   // nova jako liniie
            TrainingNode.AddElement(RE);
            nii := Pred(ni);            // kotva pred
            InsertPoint(nii, true);
         end;
         // vlozeni regulerniho bodu
         InsertPoint(ni, false);
         ninext := Succ(ni);
         if j = lile then // posledni bod
         if ni < li then  // a neni to posledni index linie - kotva
         begin
            InsertPoint(ninext, true);
         end;
      end;
      TrainingNode.UpdateRect(RE);
      R.UpdateRect(TrainingNode);
   end;
end;

procedure TNodeEditor.On_tm_Keydown(var aMessage: TOrbisKeyEvent; var Handled: boolean);
begin
   if EditorStatus = gesPrepare then exit;
   //if EditorStatus = gesSetup then
   begin
      case aMessage.Key of
         VK_ESCAPE:
         begin
            XORDraw;
            ClearSelect;
            EditorStatus := gesSelect;
            CursorIndex := 0;
          end;
      end;
   end;
   if EditorStatus = gesSetup then
   case aMessage.Key of
      VK_SHIFT:
      begin
         if SnapTool <> nil then
            SnapTool.SnapToLineBody := not SnapTool.SnapToLineBody;
         exit;
      end;
      VK_SPACE:
      begin
         if SnapTool <> nil then
            SnapTool.SnapToLine := not SnapTool.SnapToLine;
         exit;
      end;
    end;
end;

procedure TNodeEditor.On_tm_Review(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
  //if EditorStatus = gesSetup then
     XORDraw;
end;

procedure TNodeEditor.StoreNode;
var
   i,j,ni: integer;
   //NB: TNodeBranch;
   e: TElement;
   En: TEntity;
   L: TPolyLineElement;
   ILE: TIntegerListElement;
   aX,aY: double;
   C: T2DCoordinate;
   A: TAreaElement;
   AP: TAreaPointPlacer;
   aNode: TNode;
   SN: TEntity;
   SP: TEntity;
   Areas: TEntity;
   ProjE: TEntity;
   GE: TGeometryElement;
begin
   Areas := TOwnerEntity.Create;
   AddElement(Areas);
   SN := SelNodes;
   for i := SN.LastIndex downto 0 do
   begin
      aNode := TNode(SN.Elements[i]);
      RemoveElementOwners(aNode);
      aX := aNode.X;
      aY := aNode.Y;
      aX := aX + FdX;
      aY := aY + FdY;
      aNode.X := aX;
      aNode.Y := aY;
      aNode.MinX := aX - 1;
      aNode.MaxX := aX + 1;
      aNode.MinY := aY - 1;
      aNode.MaxY := aY + 1;
      aNode.AbsRefX := aX;
      aNode.AbsRefY := aY;
      if NodeQT <> nil then
         NodeQT.NextRectEntity := aNode;
   end;
   SP := SelPoints;
   for i := 0 to SP.LastIndex do
   begin
      En := TEntity(SP.Elements[i]);
      L := TPolyLineElement(En.FindElementClass(TPolyLineElement));
      ILE := TIntegerListElement(En.FindElementClass(TIntegerListElement));
      // nastaveni souradnic linie
      for j := 0 to ILE.LastIndex do
      begin
         ni := ILE.Integers[j];
         C := L.GetOrientedCoordElement(ni,true);
         aX := C.X + FdX;
         aY := C.Y + FdY;
         C.X := aX;
         C.Y := aY;
      end;
      L.RecalculateRect;       // dopocitani obd a pop.bodu linie
      L.CalculateLabelProps;
      R.UpdateRect(L);
      // Nalezeni ploch
      for j := 0 to L.OwnersCount - 1 do
      begin
         e := L.Owners[j];
         if not (e is TAreaElement) then continue;
         A := TAreaElement(e);
         if not A.IsMyOwner(Areas) then
            Areas.AddElement(A);
      end;
   end;
   // Uprava ploch
   for i := 0 to Areas.LastIndex do
   begin
      A := TAreaElement(Areas.Elements[i]);
      A.Update;
      AP := TAreaPointPlacer.Create;
      AP.Input := A;
      AP.DoIt;
      AP.Free;
      en := A.FindClassInOwners(TGeometryElement);
      if en <> nil then
      begin
         GE := TGeometryElement(en);
         GE.TimeStamp := Now;
         GE.UID := World.UID;
      end;
   end;
   Areas.Free;
   ProjE := nil;
   if AuxProjects <> nil then
      ProjE := Level.FindClassWithOwnerInOwners(TEntity,AuxProjects);
   PostMsg(tm_ModelChanged,nil,ProjE,false);
   PostMsgToOwners(tm_ReviewByElement, R, false);
end;

procedure TNodeEditor.On_DeActivate;
begin
   inherited;
   XORDraw;
   ClearSelect;
   NodeQT := nil;
   LineQT := nil;
end;

procedure TNodeEditor.SetLineQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLineQT), Value);
end;

procedure TNodeEditor.SetSnapTool(const Value: TPointLineSnapTool);
begin
   ExchangeFieldTool(TTool(FSnapTool), Value);
end;

function TNodeEditor.GetSelLines: TEntity;
begin
   if FSelLines = nil then
   begin
      FSelLines := TOwnerEntity.Create;
      FSelLines.Name := 'SelLines';
      AddElement(FSelLines);
   end;
   result := FSelLines;
end;

function TNodeEditor.GetSelNodes: TEntity;
begin
   if FSelNodes = nil then
   begin
      FSelNodes := TOwnerEntity.Create;
      FSelNodes.Name := 'SelNodes';
      AddElement(FSelNodes);
   end;
   result := FSelNodes;
end;

function TNodeEditor.GetSelPoints: TEntity;
begin
   if FSelPoints = nil then
   begin
      FSelPoints := TOwnerEntity.Create;
      FSelPoints.Name := 'SelPoints';
      AddElement(FSelPoints);
   end;
   result := FSelPoints;
end;

function TNodeEditor.IsSelectedLine(aLine: TPolyLineElement): boolean;
begin
   result := false;
   if aLine = nil then exit;
   result := aLine.IsMyOwner(SelLines);
end;

function TNodeEditor.IsSelectedLinePoint(aLine: TPolyLineElement; aIndex: integer): boolean;
var
   ILE: TIntegerListElement;
   //e: TElement;
   En: TEntity;
begin
   result := false;
   if not IsSelectedLine(aLine) then exit;
   En := aLine.FindClassWithOwnerInOwners(TOwnerEntity,SelPoints);
   if En = nil then exit;
   ILE := TIntegerListElement(En.FindElementClass(TIntegerListElement));
   if ILE = nil then exit;
   result := ILE.ValueExists(aIndex);
end;

function TNodeEditor.IsSelectedNode(aNode: TNode): boolean;
begin
   result := false;
   if aNode = nil then exit;
   result := aNode.IsMyOwner(SelNodes);
end;

procedure TNodeEditor.SelectLinePoint(aLine: TPolyLineElement; aIndex: integer);
var
   ILE: TIntegerListElement;
   SP: TEntity;
   SL: TEntity;
   En: TEntity;
begin
   SL := SelLines;
   SP := SelPoints;
   ILE := nil;
   if not aLine.IsMyOwner(SL) then
      SL.AddElement(aLine);
   En := aLine.FindClassWithOwnerInOwners(TOwnerEntity,SelPoints);
   if En = nil then
   begin
      En := TOwnerEntity.Create;
      En.Name := 'NELPNT';
      SP.AddElement(En);
      En.AddElement(aLine);
   end
   else
      ILE := TIntegerListElement(En.FindElementClass(TIntegerListElement));
   if ILE = nil then
   begin
      ILE := TIntegerListElement.Create;
      ILE.Sorted := true;
      En.AddElement(ILE);
   end;
   ILE.AddInteger(aIndex);
   IsSelect := true;
end;

procedure TNodeEditor.SelectNode(aNode: TNode);
var
   i,ni: integer;
   e: TElement;
   NB: TNodeBranch;
   L: TPolyLineElement;
begin
   if not aNode.IsMyOwner(SelNodes) then
      SelNodes.AddElement(aNode);
   for i := 0 to aNode.LastIndex do
   begin
      e := aNode.Elements[i];
      if not (e is TNodeBranch) then continue;
      NB := TNodeBranch(e);
      L := NB.Line;
      ni := 0;
      if not NB.LineBegin then
         ni := L.LastIndex;
      SelectLinePoint(L,ni);
   end;
   IsSelect := true;
end;

procedure TNodeEditor.SetSelL(const Value: TPolyLineElement);
begin
  FSelL := Value;
end;

procedure TNodeEditor.SetSelN(const Value: TNode);
begin
  FSelN := Value;
end;

procedure TNodeEditor.SetSelP(const Value: integer);
begin
  FSelP := Value;
end;

procedure TNodeEditor.SetSelStatus(const Value: TNodeEditor_SelStatus);
begin
  FSelStatus := Value;
end;

procedure TNodeEditor.SetSelShift(const Value: boolean);
begin
  FSelShift := Value;
end;

procedure TNodeEditor.SetSelSel(const Value: boolean);
begin
  FSelSel := Value;
end;

procedure TNodeEditor.ClearSelect;
begin
   IsSelect := false;
   SelNodes.FreeAll;
   SelLines.FreeAll;
   SelPoints.FreeAll;
end;

procedure TNodeEditor.SetIsSelect(const Value: boolean);
begin
  FIsSelect := Value;
end;

function TNodeEditor.GetSnapping: boolean;
begin
   result := false;
   if SnapTool <> nil then
   begin
      result := SnapTool.SnapToLine and (SnapTool.SnapToLinePoint or SnapTool.SnapToLineBody);
   end;
end;

procedure TNodeEditor.SetSnapLevelName(const Value: string);
begin
   FSnapLevelName := Value;
end;

procedure TNodeEditor.SetSnapQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FSnapQT), Value);
end;

procedure TNodeEditor.PrepareSnapQTree;
var
   RE: TRectEntity;
   E: TEntity;
   //i: integer;
   LEX: TLineExtractor;
   TQ: TAddEntityToQTree;
begin
   if SnapLevelName = '' then exit;
   if SnapTool = nil then exit;
   StartProgress('Pprava chapk...',0,0);
   RE := TRectEntity.Create;
   AddElement(RE);// uloen kvli lexu
   LEX := TLineExtractor.Create;
   E := FindLevel(SnapLevelName);
   if E <> nil then
   begin
      ProgressText := 'Hledn lini v ' + E.Name;
      Application.ProcessMessages;
      LEX.Input := E;
      LEX.Output := RE;
      LEX.DoIt;
   end;
   LEX.Free;
   if RE.NoElements then
   begin
      RE.Free;
      EndProgress;
      exit;
   end;
   ProgressText := 'chapci lezou na strom';
   Application.ProcessMessages;
   SnapQT :=TRectEntityQTree.Create;
   SnapQT.Name := 'SNAP QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := SnapQT;
   TQ.DoIt;
   if SnapTool <> nil then
      SnapTool.FrameQT := SnapQT;
   TQ.Free;
   RE.Free;
   EndProgress;
end;

procedure TNodeEditor.SetAuxProjectsName(const Value: string);
begin
  FAuxProjectsName := Value;
end;

procedure TNodeEditor.SetAuxProjects(const Value: TEntity);
begin
  FAuxProjects := Value;
end;

procedure TNodeEditor.SetLevel1Name(const Value: string);
begin
  FLevel1Name := Value;
end;

procedure TNodeEditor.SetLevel1(const Value: TEntity);
begin
  FLevel1 := Value;
end;

procedure TNodeEditor.SetSelCtrl(const Value: boolean);
begin
  FSelCtrl := Value;
end;

procedure TNodeEditor.SetXIns(const Value: double);
begin
  FXIns := Value;
end;

procedure TNodeEditor.SetYIns(const Value: double);
begin
  FYIns := Value;
end;

procedure TNodeEditor.SetAllowEditTool(const Value: TBooleanTool);
begin
   FAllowEditTool := Value;
end;

function TNodeEditor.CheckLine(aLine: TPolyLineElement): boolean;
begin
   result := true;
   if AllowEditTool <> nil then
   begin
      AllowEditTool.Input := aLine;
      AllowEditTool.DoIt;
      result := AllowEditTool.Result;
   end
end;

function TNodeEditor.CheckNode(aNode: TNode): boolean;
var
   NB: TNodeBranch;
   i: integer;
   e: TElement;
begin
   result := true;
   for i := 0 to aNode.LastIndex do
   begin
      e := aNode.Elements[i];
      if not (e is TNodeBranch) then continue;
      NB := TNodeBranch(e);
      result := CheckLine(NB.Line);
      if not result then
         exit;
   end;

end;

procedure TNodeEditor.SetVarningText(const Value: string);
begin
  FVarningText := Value;
end;

{ TLineBorderZip }

procedure TLineBorderZip.CHangeSelectCursor;
var
   d,dist,ddist: double;
   X1,Y1: double;
   CI: integer;
   aLine: TPolyLineElement;
   aNode: TNode;
   GE: TGeometryElement;
   e: TElement;
//   i: integer;

   function TryToFindNode(QT: TRectEntityQTree): boolean;
   begin
      QTFinder.Input := QT;
      QTFinder.X := X;
      QTFInder.Y := Y;
      QTFinder.Tolerance := d;
      QTFinder.DoIt;
      aNode := TNode(QTFinder.Output);
      SelN := aNode;
      SelEStatus := ness_NODE;
      result := aNode <> nil;
   end;

   function TryToFIndLinePoint(QT: TRectEntityQTree): boolean;
   var
      i: integer;
   begin
      result := false;
      QTFinder.Input := QT;
      QTFinder.X := X;
      QTFInder.Y := Y;
      QTFinder.Tolerance := d;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e <> nil then
      begin
         aLine := nil;
         if e is TPolyLineElement then
            aLine :=  TPolyLineElement(e)
         else
         begin
            if e is TGeometryElement then
            begin
               GE := TGeometryElement(e);
               if not GE.NoGeometry and (GE.Geometry is TPolyLineElement) then
                  aLine := TPolyLineElement(GE.Geometry);
            end;
         end;
         dist := d * 2;
         if (aLine <> nil) and (aLine.Count > 2) then // nejakou linii jsme nasli, tak jdem na body
         begin
            for i := 1 to aLine.LastIndex - 1 do
            begin
               X1 := aLine.Xi[i];
               Y1 := aLine.Yi[i];
               if InTolerance(X,Y,X1,Y1,d) then
               begin
                  ddist := distance(X,Y,X1,Y1);
                  if ddist < dist then
                  begin
                     SelL := aLine;
                     SelP := i;
                     dist := ddist;
                  end;
               end
               else
               if dist < d then
                  break;
            end;
            if SelL <> nil then
            begin
               SelEStatus := ness_POINT;
               result := true;
            end;
         end;
      end;
   end;

   procedure SetQTS1;
   begin
      SelNQT := NodeQT;
      SelNQT1 := Node1QT;
      SelLQT := LineQT;
      SelLQT1 := Line1QT;
   end;

   procedure SetQTS2;
   begin
      SelNQT := Node1QT;
      SelNQT1 := NodeQT;
      SelLQT := Line1QT;
      SelLQT1 := LineQT;
   end;

   function TestNode(RefN: TNode; RefL: TPolyLineElement): boolean;
   var
      i: integer;
      e: TElement;
//      Ow: TEntity;
      NB: TNodeBranch;
      NB1: TNodeBranch;
      NN: TNode;
      LL: TPolyLineElement;
//      Arc,Arcn: integer; // pocet ploch
      dl,dln: double;
   begin
      result := false;
      if RefN <> nil then
      begin
//         Arc := 1000;
         dl :=  MaxDouble;
         for i := 0 to SelN.LastIndex do
         begin
            e := SelN.Elements[i];
            if not (e is TNodeBranch) then continue;
            NB := TNodeBranch(e);
            NB.GiveLineSecondNodeBranch(NN,NB1);
            if NN = RefN then
            begin
//               Arcn := 0;
               LL := NB.Line;
               {
               for j := 0 to LL.OwnersCount - 1 do
               begin
                  if (LL.Owners[j] is TAreaElement) then
                     Inc(Arcn);
               end;
               }
               dln := LL.Length;
               //if Arcn < Arc then // vybereme linii s nejmensim poctem ploch - bejvavalo
               if dln < dl then                  // delka
               begin              // jeste by slo testovat delku - to tak ted asi je....
                  SelL := LL;
                  //Arc := Arcn;
                  dl := dln;
                  result := true;
               end;
            end;
         end;
         if result then
            exit;
      end
      else
      if RefL <> nil then
      begin
         for i := 0 to SelN.LastIndex do
         begin
            e := SelN.Elements[i];
            if not (e is TNodeBranch) then continue;
            NB := TNodeBranch(e);
            if NB.Line = RefL then
            begin
               result := true;
               exit;
            end;
         end;
      end;
      SelN := nil;
      SelEStatus := ness_NONE;
   end;

   function TestLine(RefN: TNode; RefL: TPolyLineElement; RefP: integer): boolean;
   var
//      i: integer;
//      e: TElement;
      NB: TNodeBranch;
   begin
      result := false;
      if RefN <> nil then
      begin
         NB := RefN.GetBranchOfLineSimp(SelL);
         if NB <> nil then
         begin
            result := true;
            exit;
         end;
      end
      else
      if RefL <> nil then
      begin
         if (SelL = RefL) and (SelP <> RefP)then
         begin
            result := true;
            exit;
         end;
      end;
      SelL := nil;
      SelP := -1;
      SelEStatus := ness_NONE;
   end;

begin
   if QTFinder.Busy then exit;
   d := Device.Projection.GetModelLength(PixelTolerance);
   CI := 0;
   SelEStatus := ness_NONE;
   SelN := nil;
   SelL := nil;
   SelP := -1;
   case SelStatus of
      1:  // Vyber prvniho bodu
      begin
         CI := 5;
         if TryToFindNode(NodeQT) then
            SetQTS1
         else
         if not OnlyLevel and TryToFindNode(Node1QT) then
            SetQTS2
         else
         begin
            CI := 1;
            if TryToFIndLinePoint(LineQT) then
               SetQTS1
            else
            if not OnlyLevel and TryToFIndLinePoint(Line1QT) then
               SetQTS2
            else
               CI := 0;
         end;
      end;
      2:
      begin
         if TryToFindNode(SelNQT1) then
            CI := 6
         else
         begin
            if TryToFIndLinePoint(SelLQT1) then
               CI := 2
            else
               CI := 0;
         end;
      end;
      3:
      begin
         if TryToFindNode(SelNQT) then
         begin
            if TestNode(N11,L1) then
               CI := 7
            else
               CI := 10;
         end
         else
         begin
            if TryToFIndLinePoint(SelLQT) then
            begin
               if TestLine(N11,L1,P11) then
                  CI := 3
               else
                  CI := 9;
            end
            else
               CI := 0;
         end;
      end;
      4:
      begin
         if TryToFindNode(SelNQT1) then
         begin
            if TestNode(N21,L2) then
               CI := 8
            else
               CI := 10;
         end
         else
         begin
            if TryToFIndLinePoint(SelLQT1) then
            begin
               if TestLine(N21,L2,P21) then
                  CI := 4
               else
                  CI := 9;
            end
            else
               CI := 0;
         end;
      end;
      5: CI := 11;
   end;
   CursorIndex := CI;
end;

procedure TLineBorderZip.ClearSelect;
begin
   N11 := nil;
   N12 := nil;
   N21 := nil;
   N22 := nil;
   L1 := nil;
   L2 := nil;
   P11 := -1;
   P12 := -1;
   P21 := -1;
   P22 := -1;
end;

function TLineBorderZip.GetR: TRectEntity;
begin
   if FR = nil then
   begin
      FR := TRectEntity.Create;
      AddElement(FR);
   end;
   result := FR;
end;

procedure TLineBorderZip.On_Activate;
begin
   EditorStatus := gesPrepare;
   HighActivity := true;
   if not Active then
   begin
      PrepareQTree;
      ClearSelect;
      SelStatus := 1;
   end;
   EditorStatus := gesSelect;
   QTFinder.Busy := false;
   if Color = 0 then Color := clSilver;
   inherited;
   HighActivity := false;
   //IsSelect := false;
end;

procedure TLineBorderZip.On_DeActivate;
begin
   inherited;
   XORDraw;
   //ClearSelect;
   NodeQT := nil;
   Node1QT := nil;
   LineQT := nil;
   Line1QT := nil;
end;

procedure TLineBorderZip.On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
  aX, aY: integer;
begin
   aX:= aMessage.x;
   aY:= aMessage.y;
   X:= Device.Projection.GetModelX(aX);
   Y:= Device.Projection.GetModelY(aY);
   //SelShift := ssShift in aMessage.Shift;
   if not Active then exit;
   if EditorStatus = gesSelect then
      ChangeSelectCursor;
end;

procedure TLineBorderZip.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
//var
//   StartSetup: boolean;

   procedure SwapSelQT;
   var
      aQT: TRectEntityQTree;
   begin
      aQT := SelNQT;
      SelNQT := SelNQT1;
      SelNQT1 := aQT;
      aQT := SelLQT;
      SelLQT := SelLQT1;
      SelLQT1 := aQT;
   end;
begin
   if (aMessage.Button = mbRight) then
   begin
      // tady asi bude dialog
      if (EditorStatus = gesSelect) then
      begin
         XORDraw;
         ClearSelect;
         SelEStatus := ness_NONE;
         SelStatus := 1;
      end;
      exit;
   end;
   if EditorStatus = gesSelect then
   begin
//      StartSetup := false;
      if (SelEStatus <> ness_None) or (SelStatus = 5) then // neco je k vyberu
      begin
         XORDraw;
         Case SelStatus of
            1:
            begin
               case SelEStatus of
                  ness_Node: N11 := SelN;
                  ness_Point:
                  begin
                     N11 := nil;
                     L1 := SelL;
                     P11 := SelP;
                  end;
               end;
               SelStatus := 2;
               SelEStatus := ness_NONE;
               //SwapSelQT;
            end;
            2:
            begin
               case SelEStatus of
                  ness_Node: N21 := SelN;
                  ness_Point:
                  begin
                     N21 := nil;
                     L2 := SelL;
                     P21 := SelP;
                  end;
               end;
               SelStatus := 3;
               SelEStatus := ness_NONE;
            end;
            3:
            begin
               case SelEStatus of
                  ness_Node:
                  begin
                     N12 := SelN;
                     if N11 <> nil then
                        L1 := SelL;   // pokud jsou 2 uzly tak je vybrana i linie (asi)
                  end;
                  ness_Point:
                  begin
                     N12 := nil;
                     L1 := SelL;
                     P12 := SelP;
                  end;
               end;
               SelStatus := 4;
               SelEStatus := ness_NONE;
            end;
            4:
            begin
               case SelEStatus of
                  ness_Node:
                  begin
                     N22 := SelN;
                     if N21 <> nil then
                        L2 := SelL;   // pokud jsou 2 uzly tak je vybrana i linie (asi)
                  end;
                  ness_Point:
                  begin
                     N22 := nil;
                     L2 := SelL;
                     P22 := SelP;
                  end;
               end;
               SelStatus := 5;
               SelEStatus := ness_NONE;
            end;
            5:
            begin
               ZipIt;
            end;
         end;
         XORDraw;
      end;
   end;
end;

procedure TLineBorderZip.On_tm_Review(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   XORDraw;
end;

procedure TLineBorderZip.PrepareQTree;
var
   TQ: TAddEntityToQTree;
   i: integer;
   FB: TFindBreakLineConnections;
   e: TElement;
   RE: TRectEntity;
   N: TNode;
   PomLevel: TRectEntity;

begin
   if AuxProjectsName <> '' then
   begin
      AuxProjects := FindLevel(AuxProjectsName);
      if AuxProjects = nil then exit;
   end;
   PomLevel := TRectEntity.Create;
   AddElement(PomLevel);
   Level := FindLevel(LevelName);
   if Level = nil then exit;
   PomLevel.AddAllFrom := Level;
   if LevelpName <> '' then
   begin
      Levelp := FindLevel(LevelpName);
      PomLevel.AddAllFrom := Levelp;
   end;
   PomLevel.Update;
   RE := TRectEntity.Create;
   FB := TFindBreakLineConnections.Create;
   FB.AllNodes := true;
   FB.Input := PomLevel;
   FB.Output := RE;
   FB.DoIt;
   FB.Free;
   if RE.NoElements then
   begin
      RE.Free;
      PomLevel.Free;
      exit;
   end;
   //RE.Update;
   for i := 0 to RE.LastIndex do
   begin
      e := RE.Elements[i];
      if not(e is TNode) then continue;
      N := TNode(e);
      N.MinX := N.X - 1;
      N.MaxX := N.X + 1;
      N.MinY := N.Y - 1;
      N.MaxY := N.Y + 1;
      N.AbsRefX := N.X;
      N.AbsRefY := N.Y;
      RE.UpdateRect(N);
   end;
   StartProgress('Pprava vrstev 1...',0,0);
   NodeQT :=TRectEntityQTree.Create;
   NodeQT.Name := 'Node QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := NodeQT;
   TQ.DoIt;
   TQ.Free;
   RE.Free;

   LineQT :=TRectEntityQTree.Create;
   LineQT.Name := 'Line QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := PomLevel;
   TQ.Output := LineQT;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
   PomLevel.Free;

   PomLevel := TRectEntity.Create;
   AddElement(PomLevel);
   Level1 := FindLevel(Level1Name);
   if Level1 = nil then exit;
   PomLevel.AddAllFrom := Level1;
   if Level1pName <> '' then
   begin
      Level1p := FindLevel(Level1pName);
      PomLevel.AddAllFrom := Level1p;
   end;
   PomLevel.Update;
   RE := TRectEntity.Create;
   FB := TFindBreakLineConnections.Create;
   FB.AllNodes := true;
   FB.Input := PomLevel;
   FB.Output := RE;
   FB.DoIt;
   FB.Free;
   if RE.NoElements then
   begin
      RE.Free;
      PomLevel.Free;
      exit;
   end;
   //RE.Update;
   for i := 0 to RE.LastIndex do
   begin
      e := RE.Elements[i];
      if not(e is TNode) then continue;
      N := TNode(e);
      N.MinX := N.X - 1;
      N.MaxX := N.X + 1;
      N.MinY := N.Y - 1;
      N.MaxY := N.Y + 1;
      N.AbsRefX := N.X;
      N.AbsRefY := N.Y;
      RE.UpdateRect(N);
   end;
   StartProgress('Pprava vrstev 2...',0,0);
   Node1QT :=TRectEntityQTree.Create;
   Node1QT.Name := 'Node1 QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := RE;
   TQ.Output := Node1QT;
   TQ.DoIt;
   TQ.Free;
   RE.Free;

   Line1QT :=TRectEntityQTree.Create;
   Line1QT.Name := 'Line1 QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := PomLevel;
   TQ.Output := Line1QT;
   TQ.DoIt;
   TQ.Free;
   EndProgress;
   PomLevel.Free;
end;

procedure TLineBorderZip.RegisterCursors;
begin
   NewCursorName('NOSELECT');     //0
   NewCursorName('POINTOLD1');    //1
   NewCursorName('POINTOLD2');    //2
   NewCursorName('POINTOLD3');    //3
   NewCursorName('POINTOLD4');    //4
   NewCursorName('NODEOLD1');     //5
   NewCursorName('NODEOLD2');     //6
   NewCursorName('NODEOLD3');     //7
   NewCursorName('NODEOLD4');     //8
   NewCursorName('POINTNOSELECT');//9
   NewCursorName('NODENOSELECT'); //10
   NewCursorName('OK');           //11
end;

procedure TLineBorderZip.SetAuxProjects(const Value: TEntity);
begin
  FAuxProjects := Value;
end;

procedure TLineBorderZip.SetAuxProjectsName(const Value: string);
begin
  FAuxProjectsName := Value;
end;

procedure TLineBorderZip.SetIsSelect(const Value: boolean);
begin
  FIsSelect := Value;
end;

procedure TLineBorderZip.SetL1(const Value: TPolyLineElement);
begin
  FL1 := Value;
end;

procedure TLineBorderZip.SetL2(const Value: TPolyLineElement);
begin
  FL2 := Value;
end;

procedure TLineBorderZip.SetLevel(const Value: TEntity);
begin
  FLevel := Value;
end;

procedure TLineBorderZip.SetLevel1(const Value: TEntity);
begin
  FLevel1 := Value;
end;

procedure TLineBorderZip.SetLevel1Name(const Value: string);
begin
  FLevel1Name := Value;
end;

procedure TLineBorderZip.SetLevel1p(const Value: TEntity);
begin
  FLevel1p := Value;
end;

procedure TLineBorderZip.SetLevel1pName(const Value: string);
begin
  FLevel1pName := Value;
end;

procedure TLineBorderZip.SetLevelName(const Value: string);
begin
  FLevelName := Value;
end;

procedure TLineBorderZip.SetLevelp(const Value: TEntity);
begin
  FLevelp := Value;
end;

procedure TLineBorderZip.SetLevelpName(const Value: string);
begin
  FLevelpName := Value;
end;

procedure TLineBorderZip.SetLine1QT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLine1QT), Value);
end;

procedure TLineBorderZip.SetLineQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FLineQT), Value);
end;

procedure TLineBorderZip.SetN11(const Value: TNode);
begin
  FN11 := Value;
end;

procedure TLineBorderZip.SetN12(const Value: TNode);
begin
  FN12 := Value;
end;

procedure TLineBorderZip.SetN21(const Value: TNode);
begin
  FN21 := Value;
end;

procedure TLineBorderZip.SetN22(const Value: TNode);
begin
  FN22 := Value;
end;

procedure TLineBorderZip.SetNode1QT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FNode1QT), Value);
end;

procedure TLineBorderZip.SetNodeQT(const Value: TRectEntityQTree);
begin
   ExchangeFieldElement(TElement(FNodeQT), Value);
end;

procedure TLineBorderZip.SetOnlyLevel(const Value: boolean);
begin
  FOnlyLevel := Value;
end;

procedure TLineBorderZip.SetP11(const Value: integer);
begin
  FP11 := Value;
end;

procedure TLineBorderZip.SetP12(const Value: integer);
begin
  FP12 := Value;
end;

procedure TLineBorderZip.SetP21(const Value: integer);
begin
  FP21 := Value;
end;

procedure TLineBorderZip.SetP22(const Value: integer);
begin
  FP22 := Value;
end;

procedure TLineBorderZip.SetPixelTolerance(const Value: integer);
begin
  FPixelTolerance := Value;
end;

procedure TLineBorderZip.SetSelEStatus(const Value: TNodeEditor_SelStatus);
begin
  FSelEStatus := Value;
end;

procedure TLineBorderZip.SetSelL(const Value: TPolyLineElement);
begin
  FSelL := Value;
end;

procedure TLineBorderZip.SetSelLQT(const Value: TRectEntityQTree);
begin
  FSelLQT := Value;
end;

procedure TLineBorderZip.SetSelLQT1(const Value: TRectEntityQTree);
begin
  FSelLQT1 := Value;
end;

procedure TLineBorderZip.SetSelN(const Value: TNode);
begin
  FSelN := Value;
end;

procedure TLineBorderZip.SetSelNQT(const Value: TRectEntityQTree);
begin
  FSelNQT := Value;
end;

procedure TLineBorderZip.SetSelNQT1(const Value: TRectEntityQTree);
begin
  FSelNQT1 := Value;
end;

procedure TLineBorderZip.SetSelP(const Value: integer);
begin
  FSelP := Value;
end;

procedure TLineBorderZip.SetSelStatus(const Value: integer);
begin
  FSelStatus := Value;
end;

procedure TLineBorderZip.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TLineBorderZip.SetY(const Value: double);
begin
  FY := Value;
end;

procedure TLineBorderZip.XORDraw;
var
   aR: double;
   ni: integer;
   iX,iY: double;
   aX1,aY1,aX2,aY2,aX3,aY3,aX4,aY4: double;
   L: TPolyLineElement;
   Is1: boolean;
   Is2: boolean;
   Is3: boolean;
   Is4: boolean;
   A,B,C: TDoubleCoord;
   Ang,dd: double;

   procedure DrawNode(aNode: TNode; var aXi,aYi: double; var Isi: boolean);
   begin
      if aNode <> nil then
      begin
         Isi := true;
         iX := aNode.X;
         iY := aNode.Y;
         aXi := iX;
         aYi := iY;
         Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
      end;
   end;

   procedure DrawLinePoints(aLine: TPolyLineElement; P1,P2: integer; var aXi1,aYi1,aXi2,aYi2: double; var Isi1,Isi2: boolean);
   begin
      if aLine <> nil then
      begin
         L := aLine;
         if P1 >= 0 then
         begin
            ni := P1;
            iX := L.Xi[ni];
            iY := L.Yi[ni];
            Isi1 := true;
            aXi1 := iX;
            aYi1 := iY;
            Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
         end;
         if P2 >= 0 then
         begin
            ni := P2;
            iX := L.Xi[ni];
            iY := L.Yi[ni];
            Isi2 := true;
            aXi2 := iX;
            aYi2 := iY;
            Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
         end;
      end;
   end;

   procedure DrawTextRom(aXi,aYi: double; aNum: integer);
   var
     Xi: integer;
     Yi: integer;
   begin
      Xi := Device.Projection.GetPageX(aXi);
      Yi := Device.Projection.GetPageY(aYi);
      Device.Pen.Width := 3;
      case aNum of
         1:
         begin
            Device.MoveToDevice(Xi,Yi + 10);
            Device.LineToDevice(Xi,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi-5,Yi - 12);
            Device.LineToDevice(Xi+5,Yi - 12);
            Device.MoveToDevice(Xi-5,Yi + 12);
            Device.LineToDevice(Xi+5,Yi + 12);
         end;
         2:
         begin
            Device.MoveToDevice(Xi-3,Yi + 10);
            Device.LineToDevice(Xi-3,Yi - 10);
            Device.MoveToDevice(Xi+3,Yi + 10);
            Device.LineToDevice(Xi+3,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi-8,Yi - 12);
            Device.LineToDevice(Xi+8,Yi - 12);
            Device.MoveToDevice(Xi-8,Yi + 12);
            Device.LineToDevice(Xi+8,Yi + 12);
         end;
         3:
         begin
            Device.MoveToDevice(Xi,Yi + 10);
            Device.LineToDevice(Xi,Yi - 10);
            Device.MoveToDevice(Xi-5,Yi + 10);
            Device.LineToDevice(Xi-5,Yi - 10);
            Device.MoveToDevice(Xi+5,Yi + 10);
            Device.LineToDevice(Xi+5,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi-10,Yi - 12);
            Device.LineToDevice(Xi+10,Yi - 12);
            Device.MoveToDevice(Xi-10,Yi + 12);
            Device.LineToDevice(Xi+10,Yi + 12);
         end;
         4:
         begin
            Device.MoveToDevice(Xi-5,Yi + 10);
            Device.LineToDevice(Xi-5,Yi - 10);
            Device.MoveToDevice(Xi,Yi - 10);
            Device.LineToDevice(Xi+3,Yi + 13);
            Device.LineToDevice(Xi+6,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi+3,Yi + 11);
            Device.LineToDevice(Xi+3,Yi + 2);
            Device.MoveToDevice(Xi-10,Yi - 12);
            Device.LineToDevice(Xi+11,Yi - 12);
            Device.MoveToDevice(Xi-10,Yi + 12);
            Device.LineToDevice(Xi+11,Yi + 12);
         end;
      end;
   end;

begin
   if not (EditorStatus = gesSelect) then exit;
   Is1 := false;
   Is2 := false;
   Is3 := false;
   Is4 := false;
   aX1 := 0;
   aY1 := 0;
   aX2 := 0;
   aY2 := 0;
   aX3 := 0;
   aY3 := 0;
   aX4 := 0;
   aY4 := 0;
   Device.Pen.Color:= Color;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 1;
   Device.Brush.Style:= bsSolid;
   aR:= Device.Projection.GetModelLength(7);
   DrawNode(N11,aX1,aY1,Is1);
   DrawNode(N21,aX2,aY2,Is2);
   DrawNode(N12,aX3,aY3,Is3);
   DrawNode(N22,aX4,aY4,Is4);
   aR:= Device.Projection.GetModelLength(5);
   DrawLinePoints(L1,P11,P12,aX1,aY1,aX3,aY3,Is1,Is3);
   DrawLinePoints(L2,P21,P22,aX2,aY2,aX4,aY4,Is2,Is4);

   aR:= Device.Projection.GetModelLength(25);
   Device.Font.Name := 'Arial';
   Device.Pen.Style := psSolid;
   Device.Pen.Width := 0;
   if Is1 then
   if Is2 then
   begin
      Device.Pen.Style := psDot;
      Device.MoveTo(aX1,aY1);
      Device.LineTo(aX2,aY2);
      A.X := aX1;
      A.Y := aY1;
      B.X := aX2;
      B.Y := aY2;
      XYToPolar(A,B,dd,Ang);
      VratPosunutyBod(B,Ang,aR,C);
      aX2 := C.X;
      aY2 := C.Y;
      VratPosunutyBod(A,Ang+Pi,aR,C);
      aX1 := C.X;
      aY1 := C.Y;
      Device.Pen.Style := psSolid;
      DrawTextRom(aX1,aY1,1);
      DrawTextRom(aX2,aY2,2);
   end
   else
      DrawTextRom(aX1+aR,aY1+aR,1);

   if Is3 then
   if Is4 then
   begin
      Device.Pen.Style := psDot;
      Device.MoveTo(aX3,aY3);
      Device.LineTo(aX4,aY4);
      A.X := aX3;
      A.Y := aY3;
      B.X := aX4;
      B.Y := aY4;
      XYToPolar(A,B,dd,Ang);
      VratPosunutyBod(B,Ang,aR,C);
      aX4 := C.X;
      aY4 := C.Y;
      VratPosunutyBod(A,Ang+Pi,aR,C);
      aX3 := C.X;
      aY3 := C.Y;
      Device.Pen.Style := psSolid;
      DrawTextRom(aX3,aY3,3);
      DrawTextRom(aX4,aY4,4);
   end
   else
      DrawTextRom(aX3+aR,aY3+aR,3);
   Application.processMessages;

end;

procedure TLineBorderZip.ZipIt;
var
   i,di: integer;
//   iMin,iMax: integer;
   C,C1,C2: T2DCoordinate;
   e: TElement;
   Areas: TOwnerEntity;
   A: TAreaElement;
   X1,Y1,X2,Y2: double;
   AP: TAreaPointPlacer;
   ProjE: TEntity;

   function IndexByNode(aNode: TNode; aLine: TPolyLineElement): integer;
   var
      NB: TNodeBranch;
   begin
      result := -1;
      NB := aNode.GetBranchOfLineSimp(aLine);
      if NB = nil then exit;
      result := 0;
      if not NB.LineBegin then
         result := aLine.LastIndex;
   end;

   procedure ProcessNode(aNode: TNode; const aX,aY: double);
   var
      NB: TNodeBranch;
      ii: integer;
      jj: integer;
      ee: TElement;
      L: TPolyLineElement;
   begin
      //RemoveElementOwners(aNode);
      if aNode = nil then exit;
      aNode.X := aX;
      aNode.Y := aY;
      aNode.MinX := aX - 1;
      aNode.MaxX := aX + 1;
      aNode.MinY := aY - 1;
      aNode.MaxY := aY + 1;
      aNode.AbsRefX := aX;
      aNode.AbsRefY := aY;
      for ii := 0 to aNode.LastIndex do
      begin
        ee := aNode.Elements[ii];
        if not (ee is TNodeBranch) then continue;
        NB := TNodeBranch(ee);
        L := NB.Line;
        if (L = nil) or (L = L1) then continue;
        C := L.GetOrientedCoordElement(0,NB.LineBegin);
        C.X := aX;
        C.Y := aY;
        L.CalculateLabelProps;
        L.RecalculateRect;
        R.UpdateRect(L);
        for jj := 0 to L.OwnersCount - 1 do
        begin
           ee := L.Owners[jj];
           if not (ee is TAreaElement) then continue;
           A := TAreaElement(ee);
           if not A.IsMyOwner(Areas) then
              Areas.AddElement(A);
        end;
      end;
   end;
begin
   if (L1 = nil) or (L2 = nil) then
   begin
      ShowMessage('Select Error: Line');
      ClearSelect;
      SelStatus := 1;
      SelEStatus := ness_NONE;
      exit;
   end;
   if N11 <> nil then
      P11 := IndexByNode(N11,L1);
   if N12 <> nil then
      P12 := IndexByNode(N12,L1);
   if N21 <> nil then
      P21 := IndexByNode(N21,L2);
   if N22 <> nil then
      P22 := IndexByNode(N22,L2);
   if (P11 < 0) or (P12 < 0) or (P21 < 0) or (P22 < 0) then
   begin
      ShowMessage('Select Error: Point');
      ClearSelect;
      SelStatus := 1;
      SelEStatus := ness_NONE;
      exit;
   end;
   R.Rect.Reset;
   Areas := TOwnerEntity.Create;
   AddElement(Areas);
   // Nastaveni hranicnich sour
   C1 := L1.GetOrientedCoordElement(P11,true);
   C2 := L2.GetOrientedCoordElement(P21,true);
   C1.X := C2.X;
   C1.Y := C2.Y;
   X1 := C2.X;
   Y1 := C2.Y;
   C1 := L1.GetOrientedCoordElement(P12,true);
   C2 := L2.GetOrientedCoordElement(P22,true);
   C1.X := C2.X;
   C1.Y := C2.Y;
   X2 := C2.X;
   Y2 := C2.Y;
   // Srovnani hranice
   if P11 > P12 then
   begin
      SwapInt(FP11, FP12);
      SwapInt(FP21, FP22);
   end;
   for i := P12 - 1  DownTo P11 + 1 do // zruseni sour v intervalu u L1
   begin
      //e := L1.Elements[i];
      //e.Free; crash
      L1.FreeElementIndex(i);
   end;
   i := P22;
   di := 1;
   if P22 > P21 then
      di := -1;
   i := i + di;
   while i <> P21 do
   begin
      C2 := L2.GetOrientedCoordElement(i,true);
      L1.InsertXY(P11 + 1,C2.X,C2.Y);
      i := i + di;
   end;
   L1.RecalculateRect;
   L1.CalculateLabelProps;
   R.UpdateRect(L1);
   // plochy - vytazeni do seznamu
   for i := 0 to L1.OwnersCount - 1 do
   begin
      e := L1.Owners[i];
      if not (e is TAreaElement) then continue;
      A := TAreaElement(e);
      if not A.IsMyOwner(Areas) then
         Areas.AddElement(A);
   end;
   ProcessNode(N11,X1,Y1);
   ProcessNode(N12,X2,Y2);
   // Uprava ploch
   for i := 0 to Areas.LastIndex do
   begin
      A := TAreaElement(Areas.Elements[i]);
      A.Update;
      AP := TAreaPointPlacer.Create;
      AP.Input := A;
      AP.DoIt;
      AP.Free;
   end;
   Areas.Free;

   ProjE := nil;
   if AuxProjects <> nil then
      ProjE := L1.FindClassWithOwnerInOwners(TEntity,AuxProjects);
   PostMsg(tm_ModelChanged,nil,ProjE,false);
   PostMsgToOwners(tm_ReviewByElement, R, false);

   ClearSelect;
   SelStatus := 1;
   SelEStatus := ness_NONE;
end;

INITIALIZATION
   RegisterOrbisClass(TNodeEditor);
   RegisterOrbisClass(TLineBorderZip);
END.
