unit t_PolyLineDivider;

interface
uses
   m_Intera,
   t_GraphicEditors;

type

TPolyLineDivider = class (TPolyLineEditorNoFrame)
  private
    FActualPointIndex: integer;
    FIgnoreMouseMove: boolean;
    procedure SetActualPointIndex(const Value: integer);
    procedure SetIgnoreMouseMove(const Value: boolean);
private
   property ActualPointIndex: integer read FActualPointIndex write SetActualPointIndex;
   property IgnoreMouseMove: boolean read FIgnoreMouseMove write SetIgnoreMouseMove;
public
   procedure XORDraw; override;
   procedure   On_tm_MouseMove(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure On_Activate; override;
   procedure RegisterCursors; override;
   procedure StoreObject; override;
end;


implementation
uses
   Controls,
   Forms,
   Graphics,
   Elements,
   e_Class,
   e_Geometry,
   e_GeometryElement,
   m_Orbis,
   MathLib;

{ TPolyLineDivider }

procedure TPolyLineDivider.On_Activate;
begin
   CanCreateNewObject := false;
   inherited;
end;

procedure TPolyLineDivider.On_tm_MouseDown(var aMessage: TOrbisMouseEvent;var Handled: boolean);
begin
   if not(EditorStatus = gesSetup) then
      inherited
   else
   if (aMessage.Button = mbRight) then
   begin
   end;
end;

procedure TPolyLineDivider.On_tm_MouseMove(var aMessage: TOrbisMouseEvent;var Handled: boolean);
var
   i: integer;
   d: double;
   aIdx: integer;
   v: double;
   L: TPolyElement;
begin
   if not(EditorStatus = gesSetup) then
     inherited
   else
   begin
      if PolyElement = nil then exit;
      if IgnoreMouseMove then exit;
      XORDraw;
      L := PolyElement;
      ActX:= Device.Projection.GetModelX(aMessage.X);
      ActY:= Device.Projection.GetModelY(aMessage.Y);
      v := 1E30;
      aIdx := ActualPointIndex;
      for i := 1 to L.LastIndex - 1 do
      begin
         d := Distance(ActX,ActY,L.Xi[i],L.Yi[i]);
         if d < v then
         begin
            v := d;
            aIdx := i;
         end;
      end;
      if aIdx <> ActualPointIndex then
      begin
         //XORDraw;
         ActualPointIndex := aIdx;
         //XORDraw;
      end;
      XORDraw;
   end;
end;

procedure TPolyLineDivider.On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean);
var
   aLevel,aProjE: TEntity;
begin
   if ZeroActivity then exit;
   case EditorStatus of
      gesSelect :
      begin
         if QTFinder.Output = nil then exit;
         HighActivity := True;
         Input := QTFinder.Output;
         aLevel := Level;
         FindLevel_ProjectEntity(Input,aLevel,aProjE);
         ActualLevel := aLevel;
         ActualProject := aProjE;
         SendProjectChange;
         //RemoveElementOwners(Input);
         if Input is TGeometryElement then
            GElement := TGeometryElement(Input)
         else
            PolyElement := TPolyElement(Input);
         PolyElementBackUp := TPolyElement(CreateObjectCopy); // ???
         //inherited On_Activate;
         StartEditing;
         Application.ProcessMessages;
         EditorStatus := gesSetup;
         ActualPointIndex := 1;
         XORDraw;
         {
         TestForDelete;
         TestForChangeGeometry;
         if not FreeToDelete or not FreeToChangeGeometry then
            aMessage.Button := mbRight;
         if aMessage.Button = mbRight then
            On_tm_MouseDown(aMessage,Handled);
         }
      end;
      gesSetup:
      begin
         if not (aMessage.Button = mbRight) then
            StoreObject
         else
         begin
            XORDraw;
            PolyElement := nil;
            GElement := nil;
            PolyElementBackUp := nil;
            Modified := false;
            Swap := false;
            On_Activate;
            Application.ProcessMessages;
            Application.ProcessMessages;
         end;
      end;
      //else
        //inherited;
   end;
end;

procedure TPolyLineDivider.RegisterCursors;
begin
  inherited;

end;

procedure TPolyLineDivider.SetActualPointIndex(const Value: integer);
begin
  FActualPointIndex := Value;
end;

procedure TPolyLineDivider.SetIgnoreMouseMove(const Value: boolean);
begin
  FIgnoreMouseMove := Value;
end;

procedure TPolyLineDivider.StoreObject;
var
   GC: TGeometryElementCollection;
   L,L1: TPolyLineElement;
   i: integer;
begin
   IgnoreMouseMove := true;
   XORDraw;
   if (PolyElement <> nil) and (PolyElement.Count > 2) and
      (ActualPointIndex > 0) and (ActualPointIndex < PolyElement.LastIndex) then
   begin
      L := TPolyLineElement(PolyElement);
      L1 := TPolyLineElement.Create;
      {
      for i := ActualPointIndex to L.LastIndex do
          L1.AddXY(L.Xi[i],L.Yi[i]);
      for i := L.LastIndex DownTo ActualPointIndex + 1 do
          L.FreeElement(L.Elements[i]);
      }
      for i := 0 to ActualPointIndex do
          L1.AddXY(L.Xi[i],L.Yi[i]);
      for i := ActualPointIndex - 1 DownTo 0 do
          L.FreeElement(L.Elements[i]);
      //L.RecalculateRect;
      //L.CalculateLabelProps;
      L.RecalculateRect;
      L.CalculateLabelProps;
      L.Angle := 0;
      L1.RecalculateRect;
      L1.CalculateLabelProps;
      PolyElement := nil;
      GElement := nil;
      PolyElementBackUp := nil;
      L.PostMsgToOwners(tm_DivideGeometry,L1,false);
      //Application.ProcessMessages;
      Application.ProcessMessages;
      EditorStatus := gesSelect;
      PostMsg(tm_ModelChanged,nil,ActualProject,false);
      GC := TGeometryElementCollection.Create;
      GC.UpdateRect(L);
      GC.UpdateRect(L1);
      GC.Name := '2';
      PostMsgToOwners(tm_ReviewByElement, GC, True);
   end;
   PolyElement := nil;
   GElement := nil;
   PolyElementBackUp := nil;
   Modified := false;
   On_Activate;
   IgnoreMouseMove := false;
   Application.ProcessMessages;
   Application.ProcessMessages;
   //CreateNewObject;
end;

procedure TPolyLineDivider.XORDraw;
var
   i: integer;
   L: TPolyElement;
   R: double;
begin
   L := PolyElement;
   if L = nil then exit;
   if L.CoordsCount < 3 then exit;
   Device.Pen.Color:= Color;
   Device.Pen.Style:= psSolid;
   Device.Pen.Mode:= pmXOR;
   Device.Pen.Width:= 1;
   Device.Brush.Style:= bsClear;
   if EditorStatus = gesSetup then
   begin
      Device.Pen.Style := psDot;
      Device.MoveTo(L.Xi[ActualPointIndex], L.Yi[ActualPointIndex]);
      Device.LineTo(ActX, ActY);
      Device.Pen.Style:= psSolid;
   end;
   //Device.Polyline(PolyElement)
   Device.MoveTo(L.Xi[0], L.Yi[0]);
   R := Device.Projection.GetModelLength(2);
   for i := 1 to L.LastIndex do
   begin
       Device.LineTo(L.Xi[i], L.Yi[i]);
   end;
   for i := 0 to L.LastIndex do
   begin
       DrawRect(L.Xi[i], L.Yi[i], R);
   end;
   R := R * 2;
   DrawRect(L.Xi[ActualPointIndex], L.Yi[ActualPointIndex], R);
end;

INITIALIZATION
   RegisterOrbisClass(TPolyLineDivider);
END.
