unit t_RasterManipulator;

interface
uses
  Elements,
  e_Device,
  m_Intera,
  t_GraphicEditors;

type

TRasterManipulator = class(TGraphicEditor)
  private
    FRasterLayerName: string;
    FDropRastersCount: integer;
    FDirectory: string;
    FSelStatus: integer;
    FX1: double;
    FY2: double;
    FX2: double;
    FY1: double;
    procedure SetRasterLayerName(const Value: string);
    procedure SetDropRastersCount(const Value: integer);
    procedure SetDirectory(const Value: string);
    procedure SetSelStatus(const Value: integer);
    procedure SetX1(const Value: double);
    procedure SetX2(const Value: double);
    procedure SetY1(const Value: double);
    procedure SetY2(const Value: double);
    procedure SetRastersForDevice(const Value: boolean);
    procedure SetRastersForDeviceDirectory(const Value: string);
    function GetAuxDevice: TDevice;
    procedure SetAuxDevice(const Value: TDevice);
public
   procedure   RegisterCursors; override;
   procedure   On_Activate; override;
   procedure   XORDraw; override;
   procedure   ReadRastersForDevice;
   function    AddRasterFile(const aRaster: string): boolean;
   function    FreeRaster(const aRaster: string): boolean;
   procedure   FreeAllRasters;
   procedure   DoShift;
   procedure   on_tm_DropFile(var aMessage: TOrbisTextMessage; var Handled: boolean); virtual;
   procedure   On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   On_tm_MouseUp(var aMessage: TOrbisMouseEvent; var Handled: boolean); override;
   procedure   HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   property    DropRastersCount: integer read FDropRastersCount write SetDropRastersCount;
   property    SelStatus: integer read FSelStatus write SetSelStatus;
   property    X1: double read FX1 write SetX1;
   property    Y1: double read FY1 write SetY1;
   property    X2: double read FX2 write SetX2;
   property    Y2: double read FY2 write SetY2;
published
   property    AuxDevice: TDevice read GetAuxDevice write SetAuxDevice stored false;
   property    RasterLayerName: string read FRasterLayerName write SetRasterLayerName;
   property    Directory: string read FDirectory write SetDirectory;
   property    RastersForDevice: boolean write SetRastersForDevice stored false;
   property    RastersForDeviceDirectory: string write SetRastersForDeviceDirectory stored false;
end;

implementation
uses
   Forms,
   Windows,
   Controls,
   DIalogs,
   Graphics,
   SysUtils,
   e_Class,
   e_Raster,
   Rasdata_TLB,
   UseLic,
   MathLib,
   e_Files,
   g_Elems,
   u_String,
   m_Orbis, Tools;

{ TRasterManipulator }

function TRasterManipulator.AddRasterFile(const aRaster: string): boolean;
begin
   result := true;
   PostTextMsg(tm_DropFile,aRaster,Self);
end;

procedure TRasterManipulator.DoShift;
var
   R,R1: e_Raster.TRaster;
   S,S1,S2,SPath,SName: string;
   SrvRas,SrvRas1: IRaster;
   hr: hresult;
   RasName1: string;
   dX,dY: double;
   RE: TRectEntity;
   i: integer;
const
   TLRasterLicKey: widestring = '=0FJA=0C4kVIB4AE0URF=T[:';
begin
   if Input = nil then exit;
   R := e_Raster.TRaster(Input);
   S := R.FileName;
   S1 := S;
   SPath := ExtractFilePath(S);
   CHeckBackSlash(SPath);
   SName := ExtractFileName(S);
   S2 := 'shcit_';
   if (pos('shcit1_',SName)=1) then
      SName := S2+copy(SName,8,Length(SName))
   else
   if (pos(S2,SName)=1) then
      SName := 'shcit1_'+copy(SName,7,Length(SName))
   else
      SName := 'shcit_'+SName;
   S := SPAth+SName;
   S := ChangeFileExt(S,'.cit');
   RE := TRectEntity.Create;
   RE.UpdateRect(R);
   R.Active := false;
   // ras 1
   try
   hr:= CreateInprocObjLic('TopoL.Raster', TLRasterLicKey,
                           IRaster, IUnknown(SrvRas));
   SrvRas.FileName := S1;
   SrvRas.OpenRas(S1, 1,2);
   except
      ShowMessage('S nejvt pravdpodobnost nemte nainstalovan rastrov knihovny, tak je probh nainstalujte!');
      //NoRasLib := true;
      RE.Free;
      exit;
   end;
   //ras 2
   try
   hr:= CreateInprocObjLic('TopoL.Raster', TLRasterLicKey,
                           IRaster, IUnknown(SrvRas1));
   SrvRas1.FileName := S;
   //SrvRas1.Compres := 3;
   SrvRas1.CopyFrom(SrvRas,0,0,SrvRas.Width,SrvRas.Height);
   Application.ProcessMessages;
   dX := Y1 - Y2;
   dY := X1 - X2;
   SrvRas1.X1 := SrvRas.X1 + dX;
   SrvRas1.Y1 := SrvRas.Y1 + dY;
   except
      ShowMessage('Chyba posunu rastru!');
      //NoRasLib := true;
      RE.Free;
      exit;
   end;
   Application.ProcessMessages;
   //SrvRas1.CloseRas(0);
   SrvRas.CloseRas(0);
   SrvRas := nil;
   SrvRas1 := nil;
   R.FileName := S;
   R.Active := true;
   //RE := TRectEntity.Create;
   RE.UpdateRect(R);
   PostMsg(tm_ModelChanged,nil,nil,false);
   PostMsgToOwners(tm_ReviewByElement,RE,true);
end;
{
var
   R,R1: e_Raster.TRaster;
   S,SPath,SName: string;
   hr: hresult;
   SrvRas1: IRaster;
   dX,dY: double;
   RE: TRectEntity;
const
   TLRasterLicKey: widestring = '=0FJA=0C4kVIB4AE0URF=T[:';
begin
   if Input = nil then exit;
   R := e_Raster.TRaster(Input);
   S := R.FileName;
   SPath := ExtractFilePath(S);
   CHeckBackSlash(SPath);
   SName := ExtractFileName(S);
   SName := 'sh_'+SName;
   S := SPAth+SName;
   if R.RasTyp = 0 then
      S := ChangeFileExt(S,'.cit');
   try
   hr:= CreateInprocObjLic('TopoL.Raster', TLRasterLicKey,
                           IRaster, IUnknown(SrvRas1));
   SrvRas1.FileName := S;
   //SrvRas1.Compres := 3;
   SrvRas1.CopyFrom(R.SrvRas,0,0,R.SrvRas.Width,R.SrvRas.Height);
   Application.ProcessMessages;
   dX := Y1 - Y2;
   dY := X1 - X2;
   SrvRas1.X1 := R.SrvRas.X1 + dX;
   SrvRas1.Y1 := R.SrvRas.Y1 + dY;
   Application.ProcessMessages;
   SrvRas1 := nil;
   except
      ShowMessage('Error: ');
   end;
   RE := TRectEntity.Create;
   RE.UpdateRect(R);
   R.FileName := S;
   RE.UpdateRect(R);
   PostMsg(tm_ModelChanged,nil,nil,false);
   PostMsgToOwners(tm_ReviewByElement,RE,true);
end;
}

procedure TRasterManipulator.FreeAllRasters;
var
   E: TEntity;
begin
   E := FindLevel(RasterLayerName);
   if E = nil then exit;
   E.FreeAll;
end;

function TRasterManipulator.FreeRaster(const aRaster: string): boolean;
var
   E: TEntity;
   R: TElement;
begin
   result := false;
   E := FindLevel(RasterLayerName);
   if E = nil then exit;
   R := E.FindElementName(aRaster);
   if R = nil then exit;
   E.FreeElement(R);
   result := true;
end;

function TRasterManipulator.GetAuxDevice: TDevice;
begin
   result := Device;
end;

procedure TRasterManipulator.HandleMessage(var aMessage: TOrbisMessage;var Handled: boolean);
begin
   inherited;
   case aMessage.Message of
      tm_DropFile: on_tm_DropFile(TOrbisTextMessage(aMessage),Handled);
   end;
end;

procedure TRasterManipulator.On_Activate;
begin
   inherited;
   EditorStatus := gesSelect;
end;

procedure TRasterManipulator.on_tm_DropFile(var aMessage: TOrbisTextMessage; var Handled: boolean);
var
   E: TEntity;
   R: e_Raster.TRaster;
   S: string;
begin
   Handled := true;
   Error := 0;
   S := aMessage.Text;
   if S = '' then
   begin
      DropRastersCount := 0;
      PostMsgToOwners(tm_BreakAndReview,nil,false);
      exit;
   end;
   E := FindLevel(RasterLayerName);
   if E = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Raster Manipulator: Entity '+RasterLayerName +' not found!',nil);
      exit;
   end;
   if E.FindElementName(S) <> nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Raster: '+S +' just exists in model!',nil);
      exit;
   end;
   R := e_Raster.TRaster.Create;
   try
      R.FileName := S;
      R.Name := S;
      R.Active := true;
   except
      Error := 1;
   end;
   if (Error > 0) or (R.Error > 1) or not R.GeoRaster then
   begin
      PostTextMsg(tm_AddToProtocol,'Raster Manipulator: '+S +' is not valid raster! '+R.ErrorText,nil);
      R.Free;
      exit;
   end;
   if E is TRectEntity then
      TRectEntity(E).AddAndUpdateRect(R)
   else
      E.AddElement(R);

   DropRastersCount := DropRastersCount + 1;
   PostMsg(tm_ModelCHanged,nil,nil,false);
end;

procedure TRasterManipulator.On_tm_MouseDown(var aMessage: TOrbisMouseEvent; var Handled: boolean);
begin
   //MessageBeep($FFFFFFFF);
end;

procedure TRasterManipulator.On_tm_MouseUp(var aMessage: TOrbisMouseEvent;var Handled: boolean);
var
   E: TEntity;
   El: TElement;
   R: e_Raster.TRaster;
   i: integer;
   RE: TRectEntity;
   aX,aY: double;
   DoNext: boolean;
   ENext: TEntity;
begin
   aX := Device.Projection.GetModelX(aMessage.X);
   aY := Device.Projection.GetModelY(aMessage.Y);
   if EditorStatus = gesEdit then
   begin
      if aMessage.Button = mbRight then
      begin
         XORDraw;
         SelStatus := 0;
         EditorStatus := gesSelect;
         exit;
      end
      else
      begin
         XORDraw;
         Case SelStatus of
            1:
            begin
               X1 := aX;
               Y1 := aY;
               SelStatus := 2;
               XORDraw;
            end;
            2:
            begin
               X2 := aX;
               Y2 := aY;
               SelStatus := 3;
               XORDraw;
            end;
            3:
            begin
               DoShift;
               SelStatus := 0;
               EditorStatus := gesSelect;
            end;
         end;
         exit;
      end;
   end;
   Input := nil;
   R := nil;
   Handled := true;
   E := FindLevel(RasterLayerName);
   if E = nil then exit;
   ENext := TOwnerEntity.Create;
   AddElement(ENext);
   if aMessage.Button = mbLeft then
   begin
      for i := 0 to E.LastIndex do
      begin
         El := E.Elements[i];
         if not (El is e_Raster.TRaster) then continue;
         R := e_Raster.TRaster(El);
         if not R.PointIsInside (aX,aY) then
            continue;
         Input := R;
         ENext.AddElement(R);
         //break;
      end;
      XORDraw;
   end;
   R := e_Raster.TRaster(Input);
   if Dialog <> nil then
   begin
      DoNext  := true;
      While DoNext do
      begin
         Dialog.SetPropertyStringByName('Directory',Directory);
         Dialog.SetPropertyBooleanByName('Next',(Input <> nil) and (ENext.IndexOf(Input)> 0));
         Dialog.Input := Input;
         Dialog.Output := E;
         Dialog.DoIt;
         if Dialog.Result = 8 then
         begin
            i := ENext.IndexOf(Input);
            if i > 0 then
            begin
               XORDraw;
               Input := ENext.Elements[i-1];
               R := e_Raster.TRaster(Input);
               XORDraw;
            end
            else
               DoNext := false;
         end
         else
            DoNext := false;
      end;
   end;
   XORDraw;
   ENext.Free;
   if Dialog = nil then exit;
   Directory := Dialog.GetPropertyStringByName('Directory');
   if Dialog.Result = 1 then
   begin
      if Dialog.Input = nil then
         PostMsgToOwners(tm_Review,nil,false)
      else
         PostMsgToOwners(tm_ReviewByElement,R,false);
      PostMsg(tm_ModelChanged,nil,nil,false);
   end
   else
   if Dialog.Result = 3 then
   begin
      if Dialog.Input = nil then
      begin
         if E <> nil then
            E.FreeAll;
         PostMsgToOwners(tm_Review,nil,false)
      end
      else
      begin
         if R <> nil then
         begin
            RE := TRectEntity.Create;
            RE.MinX := R.Minx;
            RE.MinY := R.Miny;
            RE.MaxX := R.MaxX;
            RE.MaxY := R.MaxY;
            R.Free;
            PostMsgToOwners(tm_ReviewByElement,RE,true);
         end;
      end;
      PostMsg(tm_ModelChanged,nil,nil,false);
   end
   else
   if Dialog.Result = 4 then
   begin
      ReadRastersForDevice;
   end
   else
   if Dialog.Result = 6 then
   begin
      SelStatus := 1;
      EditorStatus := gesEdit;
   end
end;

procedure TRasterManipulator.ReadRastersForDevice;
var
   En: TEntity;
   e: TElement;
   D: TDirectoryElement;
   F: TFileElement;
   i: integer;
   S: string;
   R: e_Raster.TRaster;
   DR: TDoubleRect;
   col: integer;
begin
   En := FindLevel(RasterLayerName);
   if En = nil then exit;
   col := 0;
   if Dialog <> nil then
      Col := Dialog.GetPropertyLongIntByName('AllColor');
   if not DirectoryExists(Directory) then
   begin
      ShowMessage('Adres rastr nelze najt');
      exit;
   end;
   En.FreeAll;
   DR := Device.Projection.ModelView;
   D := TDirectoryElement.Create;
   D.FileName := Directory;
   StartProgress('Rastry z '+ Directory,0,D.LastIndex);
   for i := 0 to D.LastIndex do
   begin
      e := D.Elements[i];
      if e is TDirectoryElement then continue;
      if not (e is TFileElement) then continue;
      F := TFileElement(e);
      Progress := i;
      ProgressText := F.Name;
      Application.ProcessMessages;
      S := UpperCase(F.FileExt);
      if not ((S = 'RAS') or (S = 'RAK') or (S = 'CIT') or (S = 'TIF') or (S = 'BMP') or (S = 'JPG')) then continue;
      try
         R := e_Raster.TRaster.Create;
         R.Color := Col;
         R.FileName := F.FileName;
         R.Name := F.FileName;
         R.Active := true;
      except
         R.Free;
         Continue;
      end;
      if R.Error > 0 then
      begin
         R.Free;
         Continue;
      end;
      if not R.GeoRaster then
      begin
         R.Free;
         continue;
      end;
      if not R.IsOverRegion(DR.MinX,DR.MinY,DR.MaxX,DR.MaxY) then
      begin
         R.Active := false;
         //R.Free;
         //continue;
      end;
      En.AddElement(R);
   end;
   EndProgress;
   if En is TRectEntity then
      TRectEntity(En).Update;
   PostMsgToOwners(tm_BreakAndReview,nil,false);
end;

procedure TRasterManipulator.RegisterCursors;
begin
   NewCursorName('RAS');          //0
   NewCursorName('POINTOLD1');    //1
   NewCursorName('POINTOLD2');    //2
   NewCursorName('OK');           //3
end;

procedure TRasterManipulator.SetAuxDevice(const Value: TDevice);
begin
   Device := Value;
end;

procedure TRasterManipulator.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TRasterManipulator.SetDropRastersCount(const Value: integer);
begin
  FDropRastersCount := Value;
end;

procedure TRasterManipulator.SetRasterLayerName(const Value: string);
begin
   FRasterLayerName := Value;
end;

procedure TRasterManipulator.SetRastersForDevice(const Value: boolean);
begin
   if not Value then exit;
   ReadRastersForDevice;
end;

procedure TRasterManipulator.SetRastersForDeviceDirectory(const Value: string);
begin
   Directory := Value;
   ReadRastersForDevice;
end;

procedure TRasterManipulator.SetSelStatus(const Value: integer);
begin
   FSelStatus := Value;
   CursorIndex := Value;
end;

procedure TRasterManipulator.SetX1(const Value: double);
begin
  FX1 := Value;
end;

procedure TRasterManipulator.SetX2(const Value: double);
begin
  FX2 := Value;
end;

procedure TRasterManipulator.SetY1(const Value: double);
begin
  FY1 := Value;
end;

procedure TRasterManipulator.SetY2(const Value: double);
begin
  FY2 := Value;
end;

procedure TRasterManipulator.XORDraw;
var
   R: TRectEntity;
   aR: double;
   A,B,C: TDoubleCoord;
   Ang,dd: double;
   aX1,aY1,aX2,aY2: double;

   procedure DrawPoint(IX,IY: double);
   begin
      Device.MoveTo(IX,IY-aR);
      Device.LineTo(IX,IY+aR);
      Device.MoveTo(IX-aR,IY);
      Device.LineTo(IX+aR,IY);
      Device.Rectangle(IX-aR,IY-aR,IX+aR,IY+aR);
   end;

   procedure DrawTextRom(aXi,aYi: double; aNum: integer);
   var
     Xi: integer;
     Yi: integer;
   begin
      Xi := Device.Projection.GetPageX(aXi);
      Yi := Device.Projection.GetPageY(aYi);
      Device.Pen.Width := 3;
      case aNum of
         1:
         begin
            Device.MoveToDevice(Xi,Yi + 10);
            Device.LineToDevice(Xi,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi-5,Yi - 12);
            Device.LineToDevice(Xi+5,Yi - 12);
            Device.MoveToDevice(Xi-5,Yi + 12);
            Device.LineToDevice(Xi+5,Yi + 12);
         end;
         2:
         begin
            Device.MoveToDevice(Xi-3,Yi + 10);
            Device.LineToDevice(Xi-3,Yi - 10);
            Device.MoveToDevice(Xi+3,Yi + 10);
            Device.LineToDevice(Xi+3,Yi - 10);
            //
            Device.Pen.Width := 0;
            Device.MoveToDevice(Xi-8,Yi - 12);
            Device.LineToDevice(Xi+8,Yi - 12);
            Device.MoveToDevice(Xi-8,Yi + 12);
            Device.LineToDevice(Xi+8,Yi + 12);
         end;
      end;
   end;

begin
   if Input = nil then
      Exit;
   if EditorStatus = gesEdit then
   begin
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 1;
      Device.Brush.Style:= bsSolid;
      aR:= Device.Projection.GetModelLength(10);
      if SelStatus > 1 then
         DrawPoint(X1,Y1);
      if SelStatus > 2 then
         DrawPoint(X2,Y2);

      aR:= Device.Projection.GetModelLength(25);
      Device.Font.Name := 'Arial';
      Device.Pen.Style := psSolid;
      Device.Pen.Width := 0;
      if SelStatus > 1 then
      if SelStatus > 2 then
      begin
         Device.Pen.Style := psDot;
         Device.MoveTo(X1,Y1);
         Device.LineTo(X2,Y2);
         A.X := X1;
         A.Y := Y1;
         B.X := X2;
         B.Y := Y2;
         XYToPolar(A,B,dd,Ang);
         VratPosunutyBod(B,Ang,aR,C);
         aX2 := C.X;
         aY2 := C.Y;
         VratPosunutyBod(A,Ang+Pi,aR,C);
         aX1 := C.X;
         aY1 := C.Y;
         Device.Pen.Style := psSolid;
         DrawTextRom(aX1,aY1,1);
         DrawTextRom(aX2,aY2,2);
      end
      else
         DrawTextRom(X1+aR,Y1+aR,1);

      Application.processMessages;
      exit;
   end;
   if EditorStatus = gesSelect then
   begin
      if Color = 0 then Color := $AAAAAA;
      Device.Pen.Color:= Color;
      Device.Pen.Style:= psSolid;
      Device.Pen.Mode:= pmXOR;
      Device.Pen.Width:= 3;
      Device.Brush.Style:= bsbDiagonal;
      R := TRectEntity(Input);
      Device.Rectangle(R.MinX,R.MinY,R.MaxX,R.MaxY );
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TRasterManipulator);
END.
