unit t_ProgressMonitor;

interface
uses
   Elements,
   Tools,
   f_ProgressMonitor,
   f_ProcessMonitor;

type

TProcessMonitor = class (TTool)
  private
    FProcess: TElement;
    FForm: TProcessMonitorForm;
    procedure SetForm(const Value: TProcessMonitorForm);
    procedure SetMaxProgress(const Value: integer);
    procedure SetMinProgress(const Value: integer);
    procedure SetProcess(const Value: TElement);
    procedure SetProcessName(const Value: string);
    procedure SetProgress(const Value: integer);
    procedure SetText(const Value: string);
    function GetMaxProgress: integer;
    function GetMinProgress: integer;
    function GetProcessName: string;
    function GetProgress: integer;
    function GetText: string;
public
   constructor Create; override;
   destructor  Destroy; override;
   property Form: TProcessMonitorForm read FForm write SetForm;
published
   property Process: TElement read FProcess write SetProcess;
   property ProcessName: string read GetProcessName write SetProcessName;
   property Text: string read GetText write SetText;
   property MinProgress: integer read GetMinProgress write SetMinProgress;
   property MaxProgress: integer read GetMaxProgress write SetMaxProgress;
   property Progress: integer read GetProgress write SetProgress;
end;

TProgressMonitor = class(TTechnology)
  private
    FForm: TProgressMonitorForm;
    procedure SetForm(const Value: TProgressMonitorForm);
    function GetProcessMonitor(Process: TElement): TProcessMonitor;
public
   constructor Create; override;
   destructor  Destroy; override;
   procedure on_tm_StartProgress(var aMessage: TProgressMessage; var Handled: boolean);
   procedure on_tm_EndProgress(var aMessage: TOrbisMessage; var Handled: boolean);
   procedure on_tm_Progress(var aMessage: TProgressMessage; var Handled: boolean);
   procedure on_tm_MaxProgress(var aMessage: TProgressMessage; var Handled: boolean);
   procedure on_tm_ProgressText(var aMessage: TOrbisTextMessage; var Handled: boolean);
   procedure HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   procedure UpdateSize;
   property ProcessMonitor[Process: TElement]: TProcessMonitor read GetProcessMonitor;
   property Form: TProgressMonitorForm read FForm write SetForm;
published
   //ActualProcess: TProcessMonitor;
end;


implementation
uses
   Controls,
   Forms,
   Windows,
   m_Orbis;

{ TProcessMonitor }

constructor TProcessMonitor.Create;
begin
   inherited;
   FForm := TProcessMonitorForm.Create(Application);
end;

destructor TProcessMonitor.Destroy;
begin
   if Form <> nil then
   begin
      Form.IsPossibleToClose := true;
      Form.Free;
   end;
   inherited;
end;

function TProcessMonitor.GetMaxProgress: integer;
begin
   //result := Form.ProgressBar.Max;
   result := Form.Gauge.MaxValue;
end;

function TProcessMonitor.GetMinProgress: integer;
begin
   //result := Form.ProgressBar.Min;
   result := Form.Gauge.MinValue;
end;

function TProcessMonitor.GetProcessName: string;
begin
   result := Form.l_ProcessName.Caption;
end;

function TProcessMonitor.GetProgress: integer;
begin
   //result := Form.ProgressBar.Position;
   result := Form.Gauge.Progress;
end;

function TProcessMonitor.GetText: string;
begin
   result := Form.l_Text.Caption;
end;

procedure TProcessMonitor.SetForm(const Value: TProcessMonitorForm);
begin
  FForm := Value;
end;

procedure TProcessMonitor.SetMaxProgress(const Value: integer);
begin
   //Form.ProgressBar.Max := Value;
   Form.Gauge.MaxValue := Value;
end;

procedure TProcessMonitor.SetMinProgress(const Value: integer);
begin
   //Form.ProgressBar.Min := Value;
   Form.Gauge.MinValue := Value;
end;

procedure TProcessMonitor.SetProcess(const Value: TElement);
begin
  FProcess := Value;
end;

procedure TProcessMonitor.SetProcessName(const Value: string);
begin
  Form.l_ProcessName.Caption := Value;
end;

procedure TProcessMonitor.SetProgress(const Value: integer);
begin
   //Form.ProgressBar.Position := Value;
   Form.Gauge.Progress := Value;
end;

procedure TProcessMonitor.SetText(const Value: string);
begin
  Form.l_Text.Caption := Value;
end;


{ TProgressMonitor }

constructor TProgressMonitor.Create;
begin
   inherited;
   FForm := TProgressMonitorForm.Create(Application);
end;

destructor TProgressMonitor.Destroy;
begin
   if Form <> nil then Form.Free;
   inherited;
end;

function TProgressMonitor.GetProcessMonitor(
  Process: TElement): TProcessMonitor;
var
   i: integer;
   T: TTool;
   TT: TProcessMonitor;
begin
   result := nil;
   if Tools.NoElements then exit;
   for i := 0 to Tools.LastIndex do
   begin
      T := Tool[i];
      if not (T is TProcessMonitor) then continue;
      TT := TProcessMonitor(T);
      if TT.Process = Process then
      begin
         result := TT;
         break;
      end;
   end;
end;

procedure TProgressMonitor.HandleMessage(var aMessage: TOrbisMessage;
          var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_StartProgress: on_tm_StartProgress(TProgressMessage(aMessage), Handled);
      tm_EndProgress: on_tm_EndProgress(aMessage, Handled);
      tm_Progress: on_tm_Progress(TProgressMessage(aMessage), Handled);
      tm_MaxProgress: on_tm_MaxProgress(TProgressMessage(aMessage), Handled);
      tm_ProgressText: on_tm_ProgressText(TOrbisTextMessage(aMessage), Handled);
   end;
end;

procedure TProgressMonitor.on_tm_EndProgress(var aMessage: TOrbisMessage;
  var Handled: boolean);
begin
   FreeTool(ProcessMonitor[aMessage.Sender]);
   if Tools.Count < 1 then
   begin
      Form.Hide;
      PostMsgToOwners(tm_Done, nil, false);
      Application.ProcessMessages;
      PostMsg(tm_QuitTask, Self, nil, false);
   end
   else
   begin
      UpdateSize;
   end;
end;

procedure TProgressMonitor.on_tm_MaxProgress(
  var aMessage: TProgressMessage; var Handled: boolean);
var
   PM: TProcessMonitor;
begin
   PM := ProcessMonitor[aMessage.Sender];
   if PM <> nil then
   begin
      if aMessage.MaxProgress < 0 then
         PM.MaxProgress := 0
      else
         PM.MaxProgress := aMessage.MaxProgress;
      if aMessage.MaxProgress > 0 then
         //PM.Form.ProgressBar.Visible := true
         PM.Form.Gauge.Visible := true
      else
         //PM.Form.ProgressBar.Visible := false;
         PM.Form.Gauge.Visible := false;
   end;
end;

procedure TProgressMonitor.on_tm_Progress(var aMessage: TProgressMessage;
  var Handled: boolean);
var
   PM: TProcessMonitor;
begin
   PM := ProcessMonitor[aMessage.Sender];
   if PM <> nil then PM.Progress := aMessage.Progress;
end;

procedure TProgressMonitor.on_tm_ProgressText(
  var aMessage: TOrbisTextMessage; var Handled: boolean);
var
   PM: TProcessMonitor;
begin
   PM := ProcessMonitor[aMessage.Sender];
   if PM <> nil then PM.Text := aMessage.Text;
end;

procedure TProgressMonitor.on_tm_StartProgress(
  var aMessage: TProgressMessage; var Handled: boolean);
var
   PM: TProcessMonitor;
begin
   PM := TProcessMonitor.Create;
   AddTool(PM);
   UpdateSize;
   if not Form.Visible then Form.Visible := true;
   PM.Form.Left := Form.Left;
   PM.Form.Top := Form.Top + Form.Height - PM.Form.Height;
   Application.ProcessMessages;
   if (aMessage.MinProgress = 0) and (aMessage.MaxProgress = 0) then
      //PM.Form.ProgressBar.Visible := false;
      PM.Form.Gauge.Visible := false;
   PM.Form.Visible := true;
   PM.Form.ManualDock(Form,nil,alBottom);
   //DrawAnimatedRects(0,0, PM.Form.BoundsRect, Form.BoundsRect);
   Application.ProcessMessages;
   PM.Process := aMessage.Sender;
   PM.ProcessName := aMessage.ProcessName;
   PM.MinProgress := aMessage.MinProgress;
   PM.MaxProgress := aMessage.MaxProgress;
   PM.Progress := aMessage.MinProgress;
end;

procedure TProgressMonitor.SetForm(const Value: TProgressMonitorForm);
begin
  FForm := Value;
end;


procedure TProgressMonitor.UpdateSize;
begin
   Form.ClientHeight := Tools.Count * 105;
end;

END.
