unit e_Memory;

interface
uses
   windows,
   Elements;

type
{
var
    MS: TMemoryStatus;
begin
   GlobalMemoryStatus(MS);
   l_Length.Caption        := IntToStr((MS.DWTotalPhys-MS.DWAvailPhys)+(MS.DWTotalPageFile-MS.DWAvailPageFile));
   l_MemoryLoad.Caption    := IntToStr(MS.DWMemoryLoad);
   l_TotalPhys.Caption     := IntToStr(MS.DWTotalPhys);
   l_AvailPhys.Caption     := IntToStr(MS.DWAvailPhys);
   l_TotalPageFile.Caption := IntToStr(MS.DWTotalPageFile);
   l_AvailPageFile.Caption := IntToStr(MS.DWAvailPageFile);
   l_TotalVirtual.Caption  := IntToStr(MS.DWTotalVirtual);
   l_AvailVirtual.Caption  := IntToStr(MS.DWAvailVirtual);
end;
}
TMemoryElement = class(TOwnerElement)
private
    FMS: TMemoryStatus;
    function GetAlocatedMemory: longWord;
    function GetAvailMemory: longWord;
    function GetAvailPageFile: longWord;
    function GetAvailPhys: longWord;
    function GetAvailVirtual: longWord;
    function GetMemoryLoad: longWord;
    function GetTotalPageFile: longWord;
    function GetTotalPhys: longWord;
    function GetTotalVirtual: longWord;
public
   constructor Create; override;
   procedure Refresh;
published
   property AlocatedMemory: longWord read GetAlocatedMemory stored false; // Celkov alokovan pam
   property AvailMemory: longWord read GetAvailMemory stored false;       // Voln pam vypoten na zklad procenta vyuit pamti (MemoryLoad)
   property MemoryLoad: longWord read GetMemoryLoad stored false;       // percent of memory in use
   property TotalPhys: longWord read GetTotalPhys stored false;         // bytes of physical memory
   property AvailPhys: longWord read GetAvailPhys stored false;         // free physical memory bytes
   property TotalPageFile: longWord read GetTotalPageFile stored false; // bytes of paging file
   property AvailPageFile: longWord read GetAvailPageFile stored false; // free bytes of paging file
   property TotalVirtual: longWord read GetTotalVirtual stored false;   // user bytes of address space
   property AvailVirtual: longWord read GetAvailVirtual stored false;   // free user bytes
end;

implementation
uses
   e_Class;

{ TMemoryElement }

constructor TMemoryElement.Create;
begin
   inherited;
   Name := 'Memory status';
   Refresh;
end;

function TMemoryElement.GetAlocatedMemory: longWord;
begin
   result := FMS.dwTotalVirtual - FMS.dwAvailVirtual;
   //result := (FMS.DWTotalPhys-FMS.DWAvailPhys)+(FMS.DWTotalPageFile-FMS.DWAvailPageFile);
end;

function TMemoryElement.GetAvailMemory: longWord;
begin
   result := FMS.dwAvailVirtual;
   //result := ((FMS.DWTotalPhys-FMS.DWAvailPhys)+(FMS.DWTotalPageFile-FMS.DWAvailPageFile)) div 100 * (100-FMS.DWMemoryLoad);
end;

function TMemoryElement.GetAvailPageFile: longWord;
begin
   result := FMS.dwAvailPageFile;
end;

function TMemoryElement.GetAvailPhys: longWord;
begin
   result := FMS.dwAvailPhys;
end;

function TMemoryElement.GetAvailVirtual: longWord;
begin
   result := FMS.dwAvailVirtual;
end;

function TMemoryElement.GetMemoryLoad: longWord;
begin
   result := FMS.dwMemoryLoad;
end;

function TMemoryElement.GetTotalPageFile: longWord;
begin
   result := FMS.dwTotalPageFile;
end;

function TMemoryElement.GetTotalPhys: longWord;
begin
   result := FMS.dwTotalPhys;
end;

function TMemoryElement.GetTotalVirtual: longWord;
begin
   result := FMS.dwTotalVirtual;
end;

procedure TMemoryElement.Refresh;
begin
   GlobalMemoryStatus(FMS);
end;

INITIALIZATION
   RegisterOrbisClass(TMemoryElement);
END.
