unit e_Sort;

interface
uses
   Elements;

type

{ TODO -oPPK -cDodlat : SortEntity zobecnit a vubec dodelat }
// v prvnim priblizeni jednoduchy sort a vyhledavani podle Name (jako classes)

TCustomSortedEntity = class(TOwnerEntity)
public
    function    FindIndex(const aValue: string; var NearestRight: integer): integer; virtual;
    function    Compare(const aValue: string; aElement: TElement): integer; virtual;
end;

TSortedEntity = class(TOwnerEntity)  // predelat na potomka TCustomSortedEntity
private
    FIndex: integer;
    FNoDuplicates: boolean;
    procedure SetNoDuplicates(const Value: boolean);
protected
    function    FindIndex(const aName: string; var NearestRight: integer): integer;
public
    function    FindElementName(aName: string): TElement; override;
    procedure   AddElement(aElement: TElement); override;
published
   property NoDuplicates: boolean read FNoDuplicates write SetNoDuplicates;
end;

TSortedByPropertyEntity = class(TCustomSortedEntity)
  private
    FPropertyName: string;
    procedure SetPropertyName(const Value: string);
public
    function    Compare(const aValue: string; aElement: TElement): integer; override;
    function    FindElementByProperty(aValue: string): TElement;  override;
    procedure   AddElement(aElement: TElement); override;
published
    property    PropertyName: string read FPropertyName write SetPropertyName;
end;

implementation
uses
   SysUtils,
   e_Class;

{ TSortedEntity }

procedure TSortedEntity.AddElement(aElement: TElement);
var
   i: integer;
   S: string;
begin
   if aElement = nil then exit;
   if NoDuplicates then
   begin
      i := IndexOf(aElement);
      if i > -1 then exit;    // uz existuje
   end;
   S := aElement.Name;
   FindIndex(S, i);
   if i > -1 then InsertElement(i, aElement)
             else inherited;
   aElement.ID := FIndex;
   inc(FIndex);
end;

function TSortedEntity.FindElementName(aName: string): TElement;
var
   i, ri: longInt;
begin
   result := nil;
   i := FindIndex(aName, ri);
   if i > -1 then
   begin
      result := Elements[i];
   end;
end;

function TSortedEntity.FindIndex(const aName: string; var NearestRight: integer): integer;
var
   L,R,C: integer;
   LResult, RResult, CResult: integer;
begin
   result := -1;
   NearestRight := -1;
   if NoElements then exit;
   L := 0;
   R := Count - 1;
   RResult := AnsiCompareText(aName, Elements[R].Name);
   if (RResult > 0) then exit;
   if RResult = 0 then
   begin
      result := R;
      exit;
   end;
   if R > L then
   begin
      LResult := AnsiCompareText(aName, Elements[L].Name);
      if (LResult < 0) then
      begin
         NearestRight := L;
         exit;
      end;
      if LResult = 0 then
      begin
         result := L;
         NearestRight := L + 1;
         exit;
      end;
   end
   else
   begin
      NearestRight := L;
      exit;
   end;
   repeat
      NearestRight := R;
      if R - L <= 1 then exit;
      C := ((R - L) div 2) + L;
      CResult := AnsiCompareText(aName, Elements[C].Name);
      if CResult < 0 then R := C else
      if CResult > 0 then L := C else
      begin
         result := C;
         NearestRight := C + 1;
         exit;
      end;
   until L >= R;
end;

procedure TSortedEntity.SetNoDuplicates(const Value: boolean);
begin
   FNoDuplicates := Value;
end;

{ TCustomSortedEntity }

function TCustomSortedEntity.Compare(const aValue: string; aElement: TElement): integer;
begin
   result := AnsiCompareText(aValue, aElement.Name);
end;

function TCustomSortedEntity.FindIndex(const aValue: string; var NearestRight: integer): integer;
var
   L,R,C: integer;
   LResult, RResult, CResult: integer;
begin
   result := -1;
   NearestRight := -1;
   if NoElements then exit;
   L := 0;
   R := Count - 1;
   RResult := Compare(aValue, Elements[R]);
   if (RResult > 0) then exit;
   if RResult = 0 then
   begin
      result := R;
      exit;
   end;
   if R > L then
   begin
      LResult := Compare(aValue, Elements[L]);
      if (LResult < 0) then
      begin
         NearestRight := L;
         exit;
      end;
      if LResult = 0 then
      begin
         result := L;
         NearestRight := L + 1;
         exit;
      end;
   end
   else
   begin
      NearestRight := L;
      exit;
   end;
   repeat
      NearestRight := R;
      if R - L <= 1 then exit;
      C := ((R - L) div 2) + L;
      CResult := Compare(aValue, Elements[C]);
      if CResult < 0 then R := C else
      if CResult > 0 then L := C else
      begin
         result := C;
         NearestRight := C + 1;
         exit;
      end;
   until L >= R;
end;

{ TSortedByPropertyEntity }

procedure TSortedByPropertyEntity.AddElement(aElement: TElement);
var
   i: integer;
   S: string;
begin
   if aElement = nil then exit;
   S := aElement.GetPropertyStringByName(PropertyName);
   FindIndex(S, i);
   if i > -1 then InsertElement(i, aElement)
             else inherited;
//   aElement.ID := FIndex;
//   inc(FIndex);
end;

function TSortedByPropertyEntity.Compare(const aValue: string; aElement: TElement): integer;
begin
   result := AnsiCompareText(aValue, aElement.GetPropertyStringByName(PropertyName));
end;

function TSortedByPropertyEntity.FindElementByProperty(aValue: string): TElement;
var
   i, ri: longInt;
begin
   result := nil;
   i := FindIndex(aValue, ri);
   if i > -1 then
   begin
      result := Elements[i];
   end;
end;

procedure TSortedByPropertyEntity.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TSortedEntity);
   RegisterOrbisClass(TSortedByPropertyEntity);
END.
