unit E_time;
{
[Unit_Description]
Description=asovac element
comment=zatm pouit v interpretaci
Admin=Petr
TechTopic=Elements
Topic=system
Version=1.0
}

interface

uses
   SysUtils,
   // KERNEL
   Elements;

type


TTickElement = class(TOwnerElement)
private
  FInterval, FLastTime, FStartTime: LongWord;
  FPause: boolean;
  function  GetAlarm: boolean;
public
  procedure Reset;
published
  property Interval: longWord read FInterval write FInterval;
  property Alarm: boolean read GetAlarm stored false;
  property Pause: boolean read FPause write FPause;
end;



TTimeElement = class(TOwnerElement)
private
   FStartTime: TDateTime;
   FLastTime: TDateTime;
   FInterval: TDateTime;
   FAlarmCount: longInt;
   FMultiAlarm: boolean;
   function  GetInterval: string;
   function  GetAlarm: boolean;
   procedure SetInterval(aValue: string);
    function GetTimeStr: string;
    procedure SetStart(const Value: boolean);
    procedure SetStop(const Value: boolean);
public
   procedure Reset;
   property  StartTime: TDateTime read FStartTime write FStartTime;
   property  LastTime: TDateTime read FLastTime write FLastTime;
published
   property  Interval: string read GetInterval write SetInterval;
   property  Alarm: boolean read GetAlarm stored false;
   property  AlarmCount: longInt read FAlarmCount stored false;
   property  MultiAlarm: boolean read FMultiAlarm write FMultiAlarm stored false;
   property  Start: boolean write SetStart stored false;
   property  Stop: boolean write SetStop stored false;
   property  TimeStr: string read GetTimeStr stored false;
end;


implementation

uses
   Windows,
   // KERNEL
   e_Class,
   u_String;



        { TTickElement }


procedure TTickElement.Reset;
begin
  FStartTime:= GetTickCount;
  FLastTime:= FStartTime;
  Pause:= false;
end;


function  TTickElement.GetAlarm: boolean;
var
  aNow: LongWord;
begin
  result:= false;
  if Pause then exit;
  aNow:= GetTickCount;
  if aNow > FLastTime + FInterval  then
  begin
    result:= true;
    FLastTime:= aNow;
  end;
end;


{ TTimeElement }

procedure TTimeElement.Reset;
begin
   StartTime  := Time;
   LastTime   := StartTime;
   FAlarmCount := 0;
end;



function  TTimeElement.GetInterval: string;
var
   Hour, Min, Sec, MSec: Word;
begin
   DeCodeTime(FInterval, Hour, Min, Sec, MSec);
   result := IntToStr(Hour)+':'+IntToStr(Min)+':'+IntToStr(Sec)+'.'+IntToStr(MSec);
end;

function  TTimeElement.GetAlarm: boolean;
var
   Tm: double;
   Diference: double;
   AC: longInt;
begin
   result := false;
   Tm := Time;
   Diference := Tm - LastTime;
   LastTime  := Tm;
   if Diference > FInterval then
   begin
      if MultiAlarm then
      begin
         AC := round (Diference / FInterval);
         if FAlarmCount < (MaxLongInt - AC) then inc(FAlarmCount, AC);
         dec(FAlarmCount);
         result := true;
      end
      else
      begin
         result := true;
      end;
   end;
end;

procedure TTimeElement.SetInterval(aValue: string);
var
   Hour, Min, Sec, MSec: Word;
   Params: array[0..3]of string;
   S: string;
   i: byte;
begin
   S := '';
   for i := 0 to 3 do Params[i] := '0';
   while StringAsTwoParams(aValue, ':', S, aValue) do
   begin
      for i := 0 to 2 do Params[i] := Params[i+1];
      Params[3] := S;
   end;
   for i := 0 to 1 do Params[i] := Params[i+2];
   if StringAsTwoParams(S, '.', S, aValue) then
   begin
      Params[2] := S;
      Params[3] := aValue;
   end
   else
   begin
      Params[2] := S;
      Params[3] := '0';
   end;

   Hour := 0;
   Min  := 0;
   Sec  := 0;
   MSec := 0;
   try
      Hour := StrToInt(Params[0]);
      Min  := StrToInt(Params[1]);
      Sec  := StrToInt(Params[2]);
      MSec := StrToInt(Params[3]);
   except
   end;
   try
      FInterval := EncodeTime(Hour,Min,Sec,MSec);
   except
      FInterval := 0;
   end;
end;

function TTimeElement.GetTimeStr: string;
var
   Tm: TDateTime;
   Hour, Min, Sec, MSec: Word;
begin
   Tm := LastTime - StartTime;
   DeCodeTime(Tm, Hour, Min, Sec, MSec);
   result := IntToStr(Hour)+':'+IntToStr(Min)+':'+IntToStr(Sec)+'.'+IntToStr(MSec);
end;

procedure TTimeElement.SetStart(const Value: boolean);
begin
   if not Value then exit;
   Reset;
end;

procedure TTimeElement.SetStop(const Value: boolean);
begin
   if not Value then exit;
   LastTime := Time;
end;

INITIALIZATION
   RegisterOrbisClass(TTimeElement);
END.
