unit T_CreateModelFromLayer;

interface
uses
   Elements,
   e_Type,
   Tools;

type

TCreateModelFromLayer = class(TTool)
private
    FDirectory: string;
    FDataModel: TStringListElement;
    FTransferModel: TStringListElement;
    FAreaTransferModel: TStringListElement;
    FLineTransferModel: TStringListElement;
    FPointTransferModel: TStringListElement;
    FAreaGroupTransferModel: TEntity;
    FLineGroupTransferModel: TEntity;
    FPointGroupTransferModel: TEntity;
    FData: TEntity;
    FModelScript: TEntity;
    FComment: string;
    FAreasLayer: TEntity;
    FLinesLayer: TEntity;
    FPointsLayer: TEntity;
    FFreeRegisterIfPossible: boolean;
    FTextsLayer: TEntity;
    FFilterMinX: double;
    FFilterMaxY: double;
    FFilterMaxX: double;
    FFilterMinY: double;
    FUseRectFilter: boolean;
    FErrs: TOwnerEntity;
    procedure SetDataModel(const Value: TStringListElement);
    procedure SetDirectory(const Value: string);
    function GetTransferModel: TStringListElement;
    procedure SetData(const Value: TEntity);
    procedure SetModelScript(const Value: TEntity);
    procedure SetComment(const Value: string);
    function GetNoTransferModel: boolean;
    procedure SetAreasLayer(const Value: TEntity);
    procedure SetLinesLayer(const Value: TEntity);
    procedure SetPointsLayer(const Value: TEntity);
    function GetAreaTransferModel: TStringListElement;
    function GetLineTransferModel: TStringListElement;
    function GetPointTransferModel: TStringListElement;
    function GetNoAreaTransferModel: boolean;
    function GetNoLineTransferModel: boolean;
    function GetNoPointTransferModel: boolean;
    procedure SetFreeRegisterIfPossible(const Value: boolean);
    procedure DeleteRegister;
    procedure SetTextsLayer(const Value: TEntity);
    procedure SetFilterMaxX(const Value: double);
    procedure SetFilterMaxY(const Value: double);
    procedure SetFilterMinX(const Value: double);
    procedure SetFilterMinY(const Value: double);
    procedure SetUseRectFilter(const Value: boolean);
    function GetAreaGroupTransferModel: TEntity;
    function GetLineGroupTransferModel: TEntity;
    function GetPointGroupTransferModel: TEntity;
    function GetNoAreaGroupTransferModel: boolean;
    function GetNoLineGroupTransferModel: boolean;
    function GetNoPointGroupTransferModel: boolean;
    function GetErrs: TOwnerEntity;
public
   procedure DoIt; override;
   procedure CreateModelScript; virtual;
   procedure ReadData; virtual;
   procedure CreateModel; virtual;
   procedure CreateSimpleModel(EFrom,ETo: TEntity; NoTransfer: boolean; Transfer: TStringListElement; NoGTransfer: boolean; GTransfer: TEntity);
   function  GetGroupTransferModel(aTransfer: TEntity; const aValue: string): TStringListElement;
   property ModelScript: TEntity read FModelScript write SetModelScript;
   property Data: TEntity read FData write SetData;
published
   property Comment: string read FComment write SetComment;
   property Directory: string read FDirectory write SetDirectory;
   property LinesLayer: TEntity read FLinesLayer write SetLinesLayer;
   property AreasLayer: TEntity read FAreasLayer write SetAreasLayer;
   property PointsLayer: TEntity read FPointsLayer write SetPointsLayer;
   property TextsLayer: TEntity read FTextsLayer write SetTextsLayer;
   property DataModel: TStringListElement read FDataModel write SetDataModel;
   property TransferModel: TStringListElement read GetTransferModel;
   property AreaTransferModel: TStringListElement read GetAreaTransferModel;
   property LineTransferModel: TStringListElement read GetLineTransferModel;
   property PointTransferModel: TStringListElement read GetPointTransferModel;
   property AreaGroupTransferModel: TEntity read GetAreaGroupTransferModel;
   property LineGroupTransferModel: TEntity read GetLineGroupTransferModel;
   property PointGroupTransferModel: TEntity read GetPointGroupTransferModel;
   property NoTransferModel: boolean read GetNoTransferModel;
   property NoAreaTransferModel: boolean read GetNoAreaTransferModel;
   property NoLineTransferModel: boolean read GetNoLineTransferModel;
   property NoPointTransferModel: boolean read GetNoPointTransferModel;
   property NoAreaGroupTransferModel: boolean read GetNoAreaGroupTransferModel;
   property NoLineGroupTransferModel: boolean read GetNoLineGroupTransferModel;
   property NoPointGroupTransferModel: boolean read GetNoPointGroupTransferModel;
   property FreeRegisterIfPossible: boolean read FFreeRegisterIfPossible write SetFreeRegisterIfPossible;
   property FilterMinX: double read FFilterMinX write SetFilterMinX;
   property FilterMinY: double read FFilterMinY write SetFilterMinY;
   property FilterMaxX: double read FFilterMaxX write SetFilterMaxX;
   property FilterMaxY: double read FFilterMaxY write SetFilterMaxY;
   property UseRectFilter: boolean read FUseRectFilter write SetUseRectFilter;
   property Errs: TOwnerEntity read GetErrs;
end;


implementation
uses
   Forms,
   Dialogs,
   e_Class,
   e_GeometryElement,
   e_Props,
   m_Orbis,
   g_Elems,
   t_LayScr,
   t_LMCre,
   Worlds;

{ TCreateModelFromLayer }

procedure TCreateModelFromLayer.CreateModel;
var
   E: TElement;
   En: TEntity;
   Trans: TStringListElement;
   GTrans: TEntity;
   NoTrans: boolean;
   NoGTrans: boolean;
begin
   Error := 1;
   // linie
   if Data.NoElements then
   begin
      //ShowMessage(Comment+': Nebyla nalezena dn data kter by stla za import');
      PostTextMsg(tm_AddToProtocol,Comment+': Nebyla nalezena dn data kter by stla za import',nil);
      exit;
   end;
   // Linie
   E := Data.FindElementName('LIN');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TPolyLineCollection) then
      begin
         En := TEntity(E);
         if LinesLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro linie');
         end
         else
         begin
            NoTrans := NoLineTransfermodel;
            if NoTrans then Trans := nil else Trans := LineTransferModel;
            NoGTrans := NoLineGroupTransferModel;
            if NoGTrans then GTrans := nil else GTrans := LineGroupTransferModel;
            CreateSimpleModel(En, LinesLayer, NoTrans, Trans, NoGtrans, GTrans);
         end;
      end;
   end;
   // Plochy
   E := Data.FindElementName('ARE');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TAreaCollection) then
      begin
         En := TEntity(E);
         if AreasLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro plochy');
         end
         else
         begin
            NoTrans := NoAreaTransfermodel;
            if NoTrans then Trans := nil else Trans := AreaTransferModel;
            NoGTrans := NoAreaGroupTransferModel;
            if NoGTrans then GTrans := nil else GTrans := AreaGroupTransferModel;
            CreateSimpleModel(En, AreasLayer, NoTrans, Trans, NoGtrans, GTrans);
         end;
      end;
   end;
   // Body
   E := Data.FindElementName('PNT');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TPointCollection) then
      begin
         En := TEntity(E);
         if PointsLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro body');
         end
         else
         begin
            NoTrans := NoPointTransfermodel;
            if NoTrans then Trans := nil else Trans := PointTransferModel;
            NoGTrans := NoPointGroupTransferModel;
            if NoGTrans then GTrans := nil else GTrans := PointGroupTransferModel;
            CreateSimpleModel(En, PointsLayer, NoTrans, Trans, NoGtrans, GTrans);
         end;
      end;
   end;
   // Texty
   E := Data.FindElementName('TXT');
   if (E <> nil) and (E is TEntity) and not TEntity(E).NoElements  then
   begin
      E := TEntity(E).Elements[0];
      if (E is TTextCollection) then
      begin
         if TextsLayer = nil then
         begin
            ShowMessage(Comment+': Nen nastavena entita pro texty');
         end
         else
         begin
            TextsLayer.AddAllFrom := E;
            if TextsLayer is TRectEntity then
               TRectEntity(TextsLayer).ResetAndUpdate;
         end;
      end;
   end;
end;

procedure TCreateModelFromLayer.CreateModelScript;
var
  aMSC: TModelScriptCreator;
begin
  Error:= 1;
  if DataModel = nil then exit;
  aMSC:= TModelScriptCreator.Create;
  aMSC.input:= DataModel;
  aMSC.DataDirectory:= Directory;
  aMSC.DoIt;
  ModelScript:= TEntity(aMSC.Output);
  aMSC.Free;
  Error:= 0;
end;

procedure TCreateModelFromLayer.DoIt;
begin
   StartProgress(Comment,0,0);
   ProgressText := 'ten dat...';
   Application.ProcessMessages;
   CreateModelScript;
   if Error > 0 then exit;
   ReadData;
   if Error > 0 then exit;
   ProgressText := 'penos do modelu...';
   Application.ProcessMessages;
   CreateModel;
   DeleteRegister;
end;


function TCreateModelFromLayer.GetAreaTransferModel: TStringListElement;
begin
   if FAreaTransferModel = nil then
   begin
      FAreaTransferModel := TStringListElement.Create;
      FAreaTransferModel.Name := 'AreaTransferModel';
      AddElement(FAreaTransferModel);
   end;
   result := FAreaTransferModel;
end;

function TCreateModelFromLayer.GetLineTransferModel: TStringListElement;
begin
   if FLineTransferModel = nil then
   begin
      FLineTransferModel := TStringListElement.Create;
      FLineTransferModel.Name := 'LineTransferModel';
      AddElement(FLineTransferModel);
   end;
   result := FLineTransferModel;
end;

function TCreateModelFromLayer.GetPointTransferModel: TStringListElement;
begin
   if FPointTransferModel = nil then
   begin
      FPointTransferModel := TStringListElement.Create;
      FPointTransferModel.Name := 'PointTransferModel';
      AddElement(FPointTransferModel);
   end;
   result := FPointTransferModel;
end;

function TCreateModelFromLayer.GetNoAreaTransferModel: boolean;
begin
   result := (FAreaTransferModel = nil) or FAreaTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetNoLineTransferModel: boolean;
begin
   result := (FLineTransferModel = nil) or FLineTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetNoPointTransferModel: boolean;
begin
   result := (FPointTransferModel = nil) or FPointTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetNoTransferModel: boolean;
begin
   result := (FTransferModel = nil) or FTransferModel.NoElements;
end;


function TCreateModelFromLayer.GetTransferModel: TStringListElement;
begin
   if FTransferModel = nil then
   begin
      FTransferModel := TStringListElement.Create;
      AddElement(FTransferModel);
   end;
   result := FTransferModel;
end;

procedure TCreateModelFromLayer.ReadData;
var
  aMBSC: TModelByScriptCreator;
  i: integer;
  E: TEntity;
begin
  Error := 1;

  if ModelScript = nil then exit;
  if Data <> nil then
     Data.FreeAll
  else
     Data:= TRectEntity.Create;
  if Data = nil then exit;

  aMBSC:= TModelByScriptCreator.Create;
  aMBSC.Input:= ModelScript;
  aMBSC.Output:= Data;
  if UseRectFilter then
  begin
     aMBSC.MinX := FilterMinX;
     aMBSC.MinY := FIlterMinY;
     aMBSC.MaxX := FilterMaxX;
     aMBSC.MaxY := FIlterMaxY;
     aMBSC.CutLayer := true;
  end;
  aMBSC.DoIt;
  aMBSC.Free;
  for i := 0 to Data.LastIndex do
  begin
     E := TEntity(Data.Elements[i]);
     if not E.NoElements then
     begin
        if E.Elements[0] is TAreaCollection then
           E.Name := 'ARE'
        else
        if E.Elements[0] is TPolyLineCollection then
           E.Name := 'LIN'
        else
        if E.Elements[0] is TPointCollection then
           E.Name := 'PNT'
        else
        if E.Elements[0] is TTextCollection then
           E.Name := 'TXT';
     end;
  end;
  Error := 0;
end;

procedure TCreateModelFromLayer.SetAreasLayer(const Value: TEntity);
begin
  FAreasLayer := Value;
end;

procedure TCreateModelFromLayer.SetComment(const Value: string);
begin
   FComment := Value;
end;

procedure TCreateModelFromLayer.SetData(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FData),Value);
end;

procedure TCreateModelFromLayer.SetDataModel(const Value: TStringListElement);
begin
   FDataModel := Value;
end;

procedure TCreateModelFromLayer.SetDirectory(const Value: string);
begin
  FDirectory := Value;
end;

procedure TCreateModelFromLayer.SetLinesLayer(const Value: TEntity);
begin
  FLinesLayer := Value;
end;

procedure TCreateModelFromLayer.SetModelScript(const Value: TEntity);
begin
   ExchangeFieldElement(TElement(FModelScript),Value);
end;

procedure TCreateModelFromLayer.SetPointsLayer(const Value: TEntity);
begin
  FPointsLayer := Value;
end;


procedure TCreateModelFromLayer.SetFreeRegisterIfPossible(
  const Value: boolean);
begin
  FFreeRegisterIfPossible := Value;
end;

procedure TCreateModelFromLayer.DeleteRegister;
var
   E: TElement;
begin
   if not FreeRegisterIfPossible then exit;
   E := World.ObjectModel.FindElementName('TRegister');
   if E <> nil then
      E.Free;
end;

procedure TCreateModelFromLayer.SetTextsLayer(const Value: TEntity);
begin
  FTextsLayer := Value;
end;

procedure TCreateModelFromLayer.SetFilterMaxX(const Value: double);
begin
  FFilterMaxX := Value;
end;

procedure TCreateModelFromLayer.SetFilterMaxY(const Value: double);
begin
  FFilterMaxY := Value;
end;

procedure TCreateModelFromLayer.SetFilterMinX(const Value: double);
begin
  FFilterMinX := Value;
end;

procedure TCreateModelFromLayer.SetFilterMinY(const Value: double);
begin
  FFilterMinY := Value;
end;

procedure TCreateModelFromLayer.SetUseRectFilter(const Value: boolean);
begin
  FUseRectFilter := Value;
end;

procedure TCreateModelFromLayer.CreateSimpleModel(EFrom, ETo: TEntity;
          NoTransfer: boolean; Transfer: TStringListElement; NoGTransfer: boolean; GTransfer: TEntity);
var
   E: TElement;
   i,j: integer;
   S: TStringElement;
   PGE: TPropertyGeometryElement;
   aN: string;
   Trans: TStringListElement;
begin
   if (NoTransfer and NoGTransfer) then
   begin
      ETo.AddAllFrom := EFrom;
      if ETo is TRectEntity then
         TRectEntity(ETo).ResetAndUpdate;
   end
   else
   begin
      if EFrom.Count > 50 then
      begin
         MaxProgress := EFrom.LastIndex;
         Application.ProcessMessages;
      end;
      for i := 0 to EFrom.LastIndex do
      begin
         if i mod 50 = 0 then
         begin
            Progress := i;
            Application.ProcessMessages;
         end;
         E := EFrom.Elements[i];
         if NoGTransfer then
            Trans := Transfer
         else
         begin
            Trans := GetGroupTransferModel(GTransfer,e.GetPropertyStringByName('Group'));
         end;
         if Trans <> nil then
         for j := 0 to Trans.LastIndex do
         begin
            S := Trans.StringElements[j];

            if (S.Name='DYNAMIC') or (S.Name[1]='@') then
            begin
               if S.Name[1]='@' then
                  aN := copy(S.Name,2,Length(S.Name))
               else
                  aN := S.Value;
               if E is TPropertyGeometryElement then
               begin
                  PGE := TPropertyGeometryElement(E);
                  if PGE.NoProperties then
                     PGE.Properties := TDynamicProperties.Create;
                  PGE.Properties.SetPropertyStringByName(S.Value,E.GetPropertyStringByName(aN));
               end;
            end
            else
               E.SetPropertyStringByName(S.Value,E.GetPropertyStringByName(S.Name));
         end;
         if ETo is TRectEntity then
            TRectEntity(ETo).AddAndUpdateRect(E)
         else
            ETo.AddElement(E);
      end;
   end;
end;

function TCreateModelFromLayer.GetAreaGroupTransferModel: TEntity;
begin
   if FAreaGroupTransferModel = nil then
   begin
      FAreaGroupTransferModel := TStringListElement.Create;
      FAreaGroupTransferModel.Name := 'AreaGroupTransferModel';
      AddElement(FAreaGroupTransferModel);
   end;
   result := FAreaGroupTransferModel;
end;

function TCreateModelFromLayer.GetGroupTransferModel(aTransfer: TEntity;
  const aValue: string): TStringListElement;
var
   e: TElement;
begin
   result := nil;
   if aTransfer = nil then exit;
   e := aTransfer.FindElementName(aValue);
   if e = nil then exit;
   if not (e is TStringListElement) then exit;
   result := TStringListElement(e);
end;

function TCreateModelFromLayer.GetLineGroupTransferModel: TEntity;
begin
   if FLineGroupTransferModel = nil then
   begin
      FLineGroupTransferModel := TStringListElement.Create;
      FLineGroupTransferModel.Name := 'LineGroupTransferModel';
      AddElement(FLineGroupTransferModel);
   end;
   result := FLineGroupTransferModel;
end;

function TCreateModelFromLayer.GetPointGroupTransferModel: TEntity;
begin
   if FPointGroupTransferModel = nil then
   begin
      FPointGroupTransferModel := TStringListElement.Create;
      FPointGroupTransferModel.Name := 'PointGroupTransferModel';
      AddElement(FPointGroupTransferModel);
   end;
   result := FPointGroupTransferModel;
end;

function TCreateModelFromLayer.GetNoAreaGroupTransferModel: boolean;
begin
   result := (FAreaGroupTransferModel = nil) or FAreaGroupTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetNoLineGroupTransferModel: boolean;
begin
   result := (FLineGroupTransferModel = nil) or FLineGroupTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetNoPointGroupTransferModel: boolean;
begin
   result := (FPointGroupTransferModel = nil) or FPointGroupTransferModel.NoElements;
end;

function TCreateModelFromLayer.GetErrs: TOwnerEntity;
begin
   if FErrs = nil then
   begin
      FErrs := TOwnerEntity.Create;
      FErrs.Name := 'Errors';
      AddElement(FErrs);
   end;
   result := FErrs;
end;

INITIALIZATION
   RegisterOrbisClass(TCreateModelFromLayer);
END.
