unit e_Annotation;

interface
uses
   e_GeometryElement,
   Elements,
   g_Elems;

type

TGeometryAnnotationCollection = class(TGeometryElementCollection)
  private
    FStyle: integer;
    FElementClassName: string;
    FPropertyName: string;
    FElementClass: TElementClass;
    FError: integer;
    FNoGeometry: boolean;
    FByText: boolean;
    FAltPropertyName: string;
    FGProperty: boolean;
    procedure SetElementClass(const Value: TElementClass);
    procedure SetElementClassName(const Value: string);
    procedure SetPropertyName(const Value: string);
    procedure SetStyle(const Value: integer);
    procedure SetError(const Value: integer);
    procedure SetNoGeometry(const Value: boolean);
    procedure SetByText(const Value: boolean);
    procedure SetAltPropertyName(const Value: string);
    procedure SetGProperty(const Value: boolean);
public
   property ElementClass: TElementClass read FElementClass write SetElementClass;
published
   property ElementClassName: string read FElementClassName write SetElementClassName;
   property GProperty: boolean read FGProperty write SetGProperty;
   property PropertyName: string read FPropertyName write SetPropertyName;
   property AltPropertyName: string read FAltPropertyName write SetAltPropertyName;
   property Style: integer read FStyle write SetStyle;
   property NoGeometry: boolean read FNoGeometry write SetNoGeometry;
   property ByText: boolean read FByText write SetByText;
   property Error: integer read FError write SetError;
end;

TGeometryAnnotation = class(TRectEntity);


TAnnotationAbscissa = class(TRectEntity)
  private
    FY1: double;
    FY2: double;
    FX1: double;
    FX2: double;
    procedure SetX1(const Value: double);
    procedure SetX2(const Value: double);
    procedure SetY1(const Value: double);
    procedure SetY2(const Value: double);
public
   procedure   Update; override;
published
   property X1: double read FX1 write SetX1;
   property Y1: double read FY1 write SetY1;
   property X2: double read FX2 write SetX2;
   property Y2: double read FY2 write SetY2;
end;

TAnnotationLostAnchor = class(TOwnerElement)
  private
    FY: double;
    FX: double;
    FFous: boolean;
    procedure SetX(const Value: double);
    procedure SetY(const Value: double);
    procedure SetFous(const Value: boolean);
published
   property X: double read FX write SetX;
   property Y: double read FY write SetY;
   property Fous: boolean read FFous write SetFous;
end;

TGeometryTextAnnotation = class(TGeometryAnnotation)
  private
    FVisible: boolean;
    FStyle: integer;
    FNoFous: boolean;
    FNoAnchor: boolean;
    FAngle: double;
    FDestroying: boolean;
    FAltText: boolean;
    function GetGACollection: TGeometryAnnotationCollection;
    function GetText: string;
    procedure SetVisible(const Value: boolean);
    procedure SetStyle(const Value: integer);
    function GetIsNumber: boolean;
    function GetIsUpperCase: boolean;
    procedure SetNoFous(const Value: boolean);
    function GetLinked: boolean;
    function GetHasLink: boolean;
    procedure SetUnLink(const Value: boolean);
    procedure SetAnchor(const Value: boolean);
    procedure SetNoAnchor(const Value: boolean);
    function GetMishMash: boolean;
    procedure SetAngle(const Value: double);
    function GetTextElement: TElement;
    procedure SetDestroying(const Value: boolean);
    function GetByText: boolean;
    function GetMulti: boolean;
    procedure SetAltText(const Value: boolean);
public
   destructor Destroy; override;
   function  CanFreeElement(aElement: TElement): boolean; override;
   procedure RemoveElement(aElement: TElement); override;
   property  GACollection: TGeometryAnnotationCollection read GetGACollection;
   property  TextElement: TElement read GetTextElement;
   property  Destroying: boolean read FDestroying write SetDestroying;
published
   property Text: string read GetText stored false;
   property Style: integer read FStyle write SetStyle;
   property Visible: boolean read FVisible write SetVisible;
   property IsNumber: boolean read GetIsNumber stored false;
   property IsUpperCase: boolean read GetIsUpperCase stored false;
   property NoFous: boolean read FNoFous write SetNoFous;
   property Linked: boolean read GetLinked stored false;
   property HasLink: boolean read GetHasLink stored false;
   property UnLink: boolean write SetUnLink stored false;
   property Anchor: boolean write SetAnchor stored false;
   property NoAnchor: boolean read FNoAnchor write SetNoAnchor;
   property MishMash: boolean read GetMishMash stored false;
   property Angle: double read FAngle write SetAngle;
   property ByText: boolean read GetByText stored false;
   property Multi: boolean read GetMulti stored false;
   property AltText: boolean read FAltText write SetAltText;
end;


implementation
uses
   SysUtils,
   e_Class,
   e_Geometry;


{ TGeometryAnnotationCollection }

procedure TGeometryAnnotationCollection.SetAltPropertyName(
  const Value: string);
begin
  FAltPropertyName := Value;
end;

procedure TGeometryAnnotationCollection.SetByText(const Value: boolean);
begin
  FByText := Value;
end;

procedure TGeometryAnnotationCollection.SetElementClass(const Value: TElementClass);
begin
  FElementClass := Value;
end;

procedure TGeometryAnnotationCollection.SetElementClassName(const Value: string);
begin
   Error := 0;
   FElementClassName := Value;
   ElementClass := GetOrbisClass(Value);
   if ElementClass = nil then
      Error := 1;
end;

procedure TGeometryAnnotationCollection.SetError(const Value: integer);
begin
  FError := Value;
end;

procedure TGeometryAnnotationCollection.SetGProperty(const Value: boolean);
begin
  FGProperty := Value;
end;

procedure TGeometryAnnotationCollection.SetNoGeometry(
  const Value: boolean);
begin
  FNoGeometry := Value;
end;

procedure TGeometryAnnotationCollection.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;

procedure TGeometryAnnotationCollection.SetStyle(const Value: integer);
begin
  FStyle := Value;
end;

{ TGeometryTextAnnotation }

function TGeometryTextAnnotation.CanFreeElement(aElement: TElement): boolean;
begin
   result := not Destroying and (aElement is TGeometry);
end;

destructor TGeometryTextAnnotation.Destroy;
begin
   FNoAnchor := true;
   FDestroying := true;
   inherited;
end;

function TGeometryTextAnnotation.GetByText: boolean;
var
   GAC: TGeometryAnnotationCollection;
begin
   result := false;
   GAC := GACollection;
   if GAC <> nil then
      result := GAC.ByText;
end;

function TGeometryTextAnnotation.GetGACollection: TGeometryAnnotationCollection;
begin
   // tady se to pvodn hledalo pomoc findClassInOwners, ale to blblo u linku
   result := TGeometryAnnotationCollection(FindOwnerClass(TGeometryAnnotationCollection));
end;

function TGeometryTextAnnotation.GetHasLink: boolean;
begin
   result := FindElementClass(TGeometryTextAnnotation) <> nil;
end;

function TGeometryTextAnnotation.GetIsNumber: boolean;
var
   S: string;
   i: integer;
begin
   S := Text;
   result := true;
   for i := 1 to Length(S) do
   begin
      if not (S[i] in ['0'..'9']) then
      begin
         result := false;
         break;
      end;
   end;
end;

function TGeometryTextAnnotation.GetIsUpperCase: boolean;
var
   S: string;
begin
   S := AnsiUpperCase(Text);
   result := AnsiCompareStr(S,Text) = 0;
end;

function TGeometryTextAnnotation.GetLinked: boolean;
begin
   result := FindOwnerClass(TGeometryTextAnnotation) <> nil;
end;

function TGeometryTextAnnotation.GetMishMash: boolean;
var
   GAC: TGeometryAnnotationCollection;
   EC: TElementClass;
   E: TElement;
   EE: TElement;
   i: integer;
   _ByText: boolean;
   S1,S2: string;
begin
   result := false;
   GAC := GACollection;
   if GAC = nil then exit;
   if GAC.NoGeometry then exit;
   _ByText := GAC.ByText;
   EC := GAC.ElementClass;
   if EC = nil then exit;
   EE := nil;
   S1 := '';
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not(E is TGeometry) then continue;
      E := E.FindClassInOwners(EC);
      if E = nil then continue;
      if EE = nil then
      begin
         EE := E;
         if _ByText then
            S1 := E.GetPropertyStringByName(GAC.PropertyName);
      end
      else
      if _ByText then
      begin
         S2 := E.GetPropertyStringByName(GAC.PropertyName);
         if not SameText(S1,S2) then
         begin
            result := true;
            exit;
         end;
      end
      else
      if E <> EE then
      begin
         result := true;
         exit;
      end;
   end;
end;

function TGeometryTextAnnotation.GetMulti: boolean;
var
   E: TElement;
   EE: TElement;
   i: integer;
begin
   result := false;
   EE := nil;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not(E is TGeometry) then continue;
      if EE = nil then
         EE := E
      else
      begin
         result := true;
         exit;
      end;
   end;
end;

function TGeometryTextAnnotation.GetText: string;
var
   GAC: TGeometryAnnotationCollection;
   PS: string;
   EC: TElementClass;
   E: TElement;
begin
   result := '';
   GAC := GACollection;
   if GAC = nil then exit;
   if GAC.NoGeometry then
   begin
      E := nil;
      if not NoElements then
         E := Elements[0];
   end
   else
   begin
      E := FindElementClass(TGeometry);
      if E = nil then exit;
      if not GAC.GProperty then  // vlastnost se ziska z geometry ... jinak...
      begin
         EC := GAC.ElementClass;
         if EC = nil then exit;
         E := E.FindClassInOwners(EC);
      end;
   end;
   if E = nil then exit;
   if AltText then
   begin
      if GAC.AltPropertyName = '' then
         result := '???'
      else
         result := E.GetPropertyStringByName(GAC.AltPropertyName)
   end
   else
   if GAC.PropertyName = '' then
      result := '???'
   else
      result := E.GetPropertyStringByName(GAC.PropertyName);
end;


function TGeometryTextAnnotation.GetTextElement: TElement;
var
   GAC: TGeometryAnnotationCollection;
   EC: TElementClass;
   E: TElement;
begin
   result := nil;
   GAC := GACollection;
   if GAC = nil then exit;
   if GAC.NoGeometry then
   begin
      if not NoElements then
         result := Elements[0];
   end
   else
   begin
      E := FindElementClass(TGeometry);
      if E = nil then exit;
      if not GAC.GProperty then
      begin
         EC := GAC.ElementClass;
         if EC = nil then exit;
         result := E.FindClassInOwners(EC);
      end
      else
         result := E;
   end;
end;

procedure TGeometryTextAnnotation.RemoveElement(aElement: TElement);
var
   i: integer;
   e: TElement;
   G: TGeometry;
   LA: TAnnotationLostAnchor;
begin
   if Destroying then
   begin
      inherited RemoveElement(aElement);  // odstraneni
      exit;
   end;
   i := IndexOf(aElement);
   LA := nil;
   G := nil;
   if aElement is TGeometry then
      G := TGeometry(aElement);
   // zanechn ztracen kotvy
   if not NoAnchor then   // pokud neni kotveni zakazano....
   if G <> nil then   // a je to geometry tak....
   begin
      LA := TAnnotationLostAnchor.Create;
      //LA.X := CenterX;
      //LA.Y := CenterY;
      LA.X := G.X;
      LA.Y := G.Y;
      if i + 1 > LastIndex then
         AddElement(LA)
      else
         InsertElement(i + 1, LA);
   end;
   // konec kotvy
   inherited RemoveElement(aElement);  // odstraneni
   if G <> nil then     // pokud jsme odstranovali geometry
   if i > 0 then        // a nebylo to na zacatku...
   begin
      e := Elements[i-1];
      if e.ClassType = TAnnotationAbscissa then  // a element pred byl fous ...
      begin
         if LA <> nil then
         begin
            LA.X := TAnnotationAbscissa(e).X1;   // nastavime kotvu podle fousu
            LA.Y := TAnnotationAbscissa(e).Y1;
            LA.Fous := true;
         end;
         e.Free;                                 // a fous depilujeme
      end;
   end;
end;

procedure TGeometryTextAnnotation.SetAltText(const Value: boolean);
begin
  FAltText := Value;
end;

procedure TGeometryTextAnnotation.SetAnchor(const Value: boolean);
var
   e: TElement;
begin
   if not Value then exit;
   e := FindElementClass(TGeometry);
   while e <> nil do
   begin
      RemoveElement(e);
      e := FindElementClass(TGeometry);
   end;
end;

procedure TGeometryTextAnnotation.SetAngle(const Value: double);
begin
  FAngle := Value;
end;

procedure TGeometryTextAnnotation.SetDestroying(const Value: boolean);
begin
  FDestroying := Value;
end;

procedure TGeometryTextAnnotation.SetNoAnchor(const Value: boolean);
begin
  FNoAnchor := Value;
end;

procedure TGeometryTextAnnotation.SetNoFous(const Value: boolean);
begin
  FNoFous := Value;
end;

procedure TGeometryTextAnnotation.SetStyle(const Value: integer);
begin
  FStyle := Value;
end;

procedure TGeometryTextAnnotation.SetUnLink(const Value: boolean);
var
   LinkGTA: TGeometryTextAnnotation;
begin
   LinkGTA := nil;
   if Linked then
   begin
      LinkGTA := TGeometryTextAnnotation(FindOwnerClass(TGeometryTextAnnotation));
      LinkGTA.RemoveElement(Self);
   end
   else
   if HasLink then
   begin
      LinkGTA := TGeometryTextAnnotation(FindElementClass(TGeometryTextAnnotation));
      RemoveElement(LinkGTA);
   end;
end;

procedure TGeometryTextAnnotation.SetVisible(const Value: boolean);
begin
  FVisible := Value;
end;

{ TAnnotationAbscissa }

procedure TAnnotationAbscissa.SetX1(const Value: double);
begin
  FX1 := Value;
end;

procedure TAnnotationAbscissa.SetX2(const Value: double);
begin
  FX2 := Value;
end;

procedure TAnnotationAbscissa.SetY1(const Value: double);
begin
  FY1 := Value;
end;

procedure TAnnotationAbscissa.SetY2(const Value: double);
begin
  FY2 := Value;
end;

procedure TAnnotationAbscissa.Update;
begin
   Rect.UpdateByXY(X1,Y1);
   Rect.UpdateByXY(X2,Y2);
end;

{ TAnnotationLostAnchor }

procedure TAnnotationLostAnchor.SetFous(const Value: boolean);
begin
  FFous := Value;
end;

procedure TAnnotationLostAnchor.SetX(const Value: double);
begin
  FX := Value;
end;

procedure TAnnotationLostAnchor.SetY(const Value: double);
begin
  FY := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometryAnnotationCollection);
   RegisterOrbisClass(TGeometryAnnotation);
   RegisterOrbisClass(TGeometryTextAnnotation);
   RegisterOrbisClass(TAnnotationAbscissa);
   RegisterOrbisClass(TAnnotationLostAnchor);
END.
