unit t_Create1VSLayer;

interface
uses
   Elements,
   e_QTree,
   t_QTreeTools,
   Tools;

type

TCreate1VSLayer = class(TTool)
  private
    FDETLayer: TEntity;
    FLayer: TEntity;
    FSRFLayer: Tentity;
    FL1: TEntity;
    FL12: TEntity;
    FComment: string;
    FL1_SRF: TEntity;
    FL12_SRF: TEntity;
    FL1QT: TRectEntityQTree;
    FTab1: TResultTool;
    FTab2: TResultTool;
    FQTFinder: TFindRectEntityInQTreeByXY;
    FL1_SRFQT: TRectEntityQTree;
    FLines: TEntity;
    FAreas: TEntity;
    FOtherSRFYellow: boolean;
    FBaseValue: integer;
    procedure SetDETLayer(const Value: TEntity);
    procedure SetLayer(const Value: TEntity);
    procedure SetSRFLayer(const Value: Tentity);
    procedure SetL1(const Value: TEntity);
    procedure SetL12(const Value: TEntity);
    procedure SetComment(const Value: string);
    procedure SetL12_SRF(const Value: TEntity);
    procedure SetL1_SRF(const Value: TEntity);
    procedure SetL1QT(const Value: TRectEntityQTree);
    procedure SetTab1(const Value: TResultTool);
    procedure SetTab2(const Value: TResultTool);
    function GetQTFinder: TFindRectEntityInQTreeByXY;
    procedure SetL1_SRFQT(const Value: TRectEntityQTree);
    procedure SetLines(const Value: TEntity);
    procedure SetAreas(const Value: TEntity);
    procedure SetOtherSRFYellow(const Value: boolean);
    procedure SetBaseValue(const Value: integer);
public
   procedure DoIt; override;
   procedure Prepare;
   procedure Find1VS;
   procedure PrepareQTree;
   procedure Find1VS_SRF;
   procedure PrepareSRFQTree;
   procedure FindLines;
   procedure MakeAreas;
   function  VS1(const aStr: string; var VS2: boolean): boolean;
   property  L1: TEntity read FL1 write SetL1;
   property  L12: TEntity read FL12 write SetL12;
   property  L1_SRF: TEntity read FL1_SRF write SetL1_SRF;
   property  L12_SRF: TEntity read FL12_SRF write SetL12_SRF;
   property  L1QT: TRectEntityQTree read FL1QT write SetL1QT;
   property  L1_SRFQT: TRectEntityQTree read FL1_SRFQT write SetL1_SRFQT;
   property  QTFinder: TFindRectEntityInQTreeByXY read GetQTFinder;
   property  Lines: TEntity read FLines write SetLines;
   property  Areas: TEntity read FAreas write SetAreas;
published
   property  Layer: TEntity read FLayer write SetLayer;
   property  DETLayer: TEntity read FDETLayer write SetDETLayer;
   property  SRFLayer: TEntity read FSRFLayer write SetSRFLayer;
   property  Comment: string read FComment write SetComment;
   property  Tab1: TResultTool read FTab1 write SetTab1;
   property  Tab2: TResultTool read FTab2 write SetTab2;
   property  OtherSRFYellow: boolean read FOtherSRFYellow write SetOtherSRFYellow;
   property  BaseValue: integer read FBaseValue write SetBaseValue;
end;


implementation
uses
   Forms,
   SysUtils,
   e_Geometry,
   e_GeometryElement,
   e_Class,
   e_LHCElements,
   t_AreMak,
   t_LineExtractor,
   u_String;

{ TCreate1VSLayer }

procedure TCreate1VSLayer.DoIt;
begin
   if DETLayer = nil  then exit;
   if DETLayer.NoElements then exit;
   if Layer = nil then exit;
   StartProgress(Comment,0,0);
   ProgressText := 'hledn 1 VS...';
   Application.ProcessMessages;
   Prepare;
   Find1VS;
   PrepareQTree;
   Find1VS_SRF;
   PrepareSRFQTree;
   FindLines;
   MakeAreas;
end;

procedure TCreate1VSLayer.Find1VS;
var
   i: integer;
   e: TElement;
   VS2: boolean;
begin
   Progress := 0;
   MaxProgress := DetLayer.LastIndex;
   for i := 0 to DetLayer.LastIndex do
   begin
      if i mod 50 = 0 then
      begin
         Progress := i;
         Application.ProcessMessages;
      end;
      e := DetLayer.Elements[i];
      if VS1(e.Name,VS2) then
      begin
         L1.AddElement(e);
         if VS2 then
            L12.AddElement(e);
      end;
   end;
end;

procedure TCreate1VSLayer.Find1VS_SRF;
var
   i: integer;
   e: TElement;
   GE: TGeometryElement;
   Znacka: integer;
begin
   if (SRFLayer = nil) or SRFLayer.NoElements then exit;
   for i := 0 to SRFLayer.LastIndex do
   begin
      e := SRFLayer.Elements[i];
      if not (e is TGeometryElement) then continue;
      GE := TGeometryElement(e);
      // ma zlutou?
      TAB1.Input := GE;
      TAB1.DoIt;
      Znacka := TAB1.GetPropertyLongIntByName('result');
      if Znacka = 0 then continue; // nema!
      // ma...pokracujeme
      QTFinder.Input := L1QT;
      QTFinder.X := GE.X;
      QTFinder.Y := GE.Y;
      QTFinder.DoIt;
      e := QTFinder.Output;
      if e = nil then
      begin
         // srafa nalezi nad 1vs, pokud je zapnuto a ma zlutou, pridame ji
         if OtherSRFYellow then
         begin
            L1_SRF.AddElement(GE);
         end;
         continue;
      end;
      L1_SRF.AddElement(GE);
      if e.IsMyOwner(L12) then
         L12_SRF.AddElement(GE);
   end;
end;

procedure TCreate1VSLayer.FindLines;
var
   LEX: TLineExtractor;
begin
   ProgressText := 'Hledn lini..';
   Application.ProcessMessages;
   Lines := TPolyLineCollection.Create;
   Lines.Name := 'Lines';
   AddElement(Lines);
   //DET
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := L1;
   LEX.Output := Lines;
   LEX.DoIt;
   LEX.Free;
   //SRF
   LEX := TLineExtractor.Create;
   LEX.CopyLines := true;
   LEX.Input := L1_SRF;
   LEX.Output := Lines;
   LEX.DoIt;
   LEX.Free;
   TPolyLineCollection(Lines).Update;
end;

function TCreate1VSLayer.GetQTFinder: TFindRectEntityInQTreeByXY;
begin
   if FQTFinder = nil then
   begin
      FQTFinder := TFindRectEntityInQTreeByXY.Create;
      AddElement(FQTFinder);
   end;
   result := FQTFinder;
end;

procedure TCreate1VSLayer.MakeAreas;
var
   AM: TCreateAreasFromLinesTool;
   A: TAreaElement;
   SA: TSRFPlocha;
   E: TElement;
   i: integer;
   Znacka: integer;
   Barva: integer;
begin
   if Lines.NoElements then
   begin
      //PostTextMsg(tm_AddToProtocol,Name+': No lines for Area Found! ',nil);
      exit;
   end;
   ProgressText := 'Sofistikovan tvorba ploch...';
   Application.ProcessMessages;
   AM := TCreateAreasFromLinesTool.Create;
   AM.Input := Lines;
   AM.DoIt;
   Areas := TEntity(AM.Output);

   ProgressText := 'Sofistikovan tvorba ploch ... FINLE';
   MaxProgress := Areas.LastIndex;
   Application.ProcessMessages;

   for i := 0 to Areas.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      E := Areas.Elements[i];
      if not (E is TAreaElement) then Continue;
      A := TAreaElement(E);
      // hledame SRF plochu
      E := nil;
      if L1_SRFQT <> nil then
      begin
         QTFinder.Input := L1_SRFQT;
         QTFinder.X := A.X;
         QTFinder.Y := A.Y;
         QTFinder.DoIt;
         E := QTFinder.Output;
      end;
      if E <> nil then
      begin
         TAB1.Input := E;
         TAB1.DoIt;
         Znacka := TAB1.GetPropertyLongIntByName('result');
         if Znacka <> 0 then
         begin
            SA := TSRFPlocha.Create;
            SA.Barva := 14;
            SA.Znacka := Znacka;
            if E.IsMyOwner(L12_SRF) then
            begin
               TAB2.Input := SA;
               TAB2.DoIt;
               Znacka := TAB2.GetPropertyLongIntByName('result');
               SA.Znacka := Znacka;
            end;
            SA.Geometry := A;
            Layer.AddElement(SA);
         end;
         continue;
      end;
      // hledame DET plochu
      QTFinder.Input := L1QT;
      QTFinder.X := A.X;
      QTFinder.Y := A.Y;
      QTFinder.DoIt;
      E := QTFinder.Output;
      if E <> nil then
      begin
         Znacka := E.GetPropertyLongIntByName('Znacka');
         if Znacka = 1 then
         begin
            Barva := E.GetPropertyLongIntByName('Barva');
            if Barva <> 14 then continue;
         end;
         TAB1.Input := E;
         TAB1.DoIt;
         Znacka := TAB1.GetPropertyLongIntByName('result');
         if Znacka <> 0 then
         begin
            SA := TSRFPlocha.Create;
            SA.Barva := 14;
            SA.Znacka := Znacka;
            if E.IsMyOwner(L12) then
            begin
               TAB2.Input := SA;
               TAB2.DoIt;
               Znacka := TAB2.GetPropertyLongIntByName('result');
               SA.Znacka := Znacka;
            end;
            SA.Geometry := A;
            Layer.AddElement(SA);
         end;
      end;
   end;
   //Areas := nil;
   AM.Free;
end;

procedure TCreate1VSLayer.Prepare;
begin
   L1 := TAreaCollection.Create;
   L1.Name := 'VS 1';
   AddElement(L1);
   L12 := TAreaCollection.Create;
   L12.Name := 'VS 1/2';
   AddElement(L12);
   L1_SRF := TAreaCollection.Create;
   L1_SRF.Name := 'VS 1 SRF';
   AddElement(L1_SRF);
   L12_SRF := TAreaCollection.Create;
   L12_SRF.Name := 'VS 1/2 SRF';
   AddElement(L12_SRF);
end;

procedure TCreate1VSLayer.PrepareQTree;
var
   TQ: TAddEntityToQTree;
begin
   if L1 = nil then exit;
   Progress := 0;
   ProgressText := 'Pprava QTree...';
   L1QT :=TRectEntityQTree.Create;
   L1QT.Name := 'L1 QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := L1;
   TQ.Output := L1QT;
   TQ.DoIt;
   TQ.Free;
end;

procedure TCreate1VSLayer.PrepareSRFQTree;
var
   TQ: TAddEntityToQTree;
begin
   if L1_SRF = nil then exit;
   if L1_SRF.NoElements then exit;
   Progress := 0;
   ProgressText := 'Pprava QTree pro SRF...';
   L1_SRFQT :=TRectEntityQTree.Create;
   L1_SRFQT.Name := 'L1_SRF QT';
   TQ := TAddEntityToQTree.Create;
   TQ.Input := L1_SRF;
   TQ.Output := L1_SRFQT;
   TQ.DoIt;
   TQ.Free;
end;

procedure TCreate1VSLayer.SetAreas(const Value: TEntity);
begin
  ExchangeFieldElement(TElement(FAreas), Value);
end;

procedure TCreate1VSLayer.SetBaseValue(const Value: integer);
begin
  FBaseValue := Value;
end;

procedure TCreate1VSLayer.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCreate1VSLayer.SetDETLayer(const Value: TEntity);
begin
  FDETLayer := Value;
end;

procedure TCreate1VSLayer.SetL1(const Value: TEntity);
begin
  FL1 := Value;
end;

procedure TCreate1VSLayer.SetL12(const Value: TEntity);
begin
  FL12 := Value;
end;

procedure TCreate1VSLayer.SetL12_SRF(const Value: TEntity);
begin
  FL12_SRF := Value;
end;

procedure TCreate1VSLayer.SetL1QT(const Value: TRectEntityQTree);
begin
  ExchangeFieldElement(TElement(FL1QT), Value);
end;

procedure TCreate1VSLayer.SetL1_SRF(const Value: TEntity);
begin
  FL1_SRF := Value;
end;

procedure TCreate1VSLayer.SetL1_SRFQT(const Value: TRectEntityQTree);
begin
  ExchangeFieldElement(TElement(FL1_SRFQT), Value);
end;

procedure TCreate1VSLayer.SetLayer(const Value: TEntity);
begin
  FLayer := Value;
end;

procedure TCreate1VSLayer.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

procedure TCreate1VSLayer.SetOtherSRFYellow(const Value: boolean);
begin
  FOtherSRFYellow := Value;
end;

procedure TCreate1VSLayer.SetSRFLayer(const Value: Tentity);
begin
  FSRFLayer := Value;
end;

procedure TCreate1VSLayer.SetTab1(const Value: TResultTool);
begin
  ExchangeFieldElement(TElement(FTab1), Value);
end;

procedure TCreate1VSLayer.SetTab2(const Value: TResultTool);
begin
  ExchangeFieldElement(TElement(FTab2), Value);
end;

function TCreate1VSLayer.VS1(const aStr: string; var VS2: boolean): boolean;
type
   TEtazRec =  record
      VS: integer;
      bla: string
   end;
var
   S,S1,ErrS,VSS,blaS: string;
   Etaz: Array [0..3]of TEtazRec;
   i,LastEt: integer;
   Delimiter: char;
   Position: integer;
   Numbers: boolean;
   PomI,PomII: integer;
begin
   result := false;
   VS2 := false;
   S := aStr;
   LastEt := -1;
   Delimiter := '/';
   Position := 1;
   ErrS := '';
   while Delimiter <> #0 do
   begin
      Inc(LastEt);
      Etaz[LastEt].VS := 0;
      Etaz[LastEt].bla := '';
      S1 := ReadToDelimiter(S,['/'],Position,Delimiter,Position);
      Numbers := true;
      VSS := '';
      blaS := '';
      // ezen do et, kontrola mezer a poad slo, psmeno
      for i := 1 to Length(S1) do
      begin
         if (S1[i]=' ')then
            continue;
         if S1[i] in ['0'..'9'] then
         begin
            if not Numbers then
               continue;
            VSS := VSS + S1[i];
         end
         else
         if S1[i] in ['a'..'z'] then
         begin
            if Numbers then
               Numbers := false;
            blaS := blas + S1[i];
         end
         else
         begin
            continue;
         end;
      end;
      if VSS = '' then
      begin
         VSS := '0';
      end;
      PomI := StrToIntDef(VSS,0);
      if (BaseValue > 0) and (PomI > 0) then
      begin
         PomII := PomI - BaseValue;
         if PomII >= 0 then
            PomI := PomII;
      end;
      Etaz[LastEt].VS := PomI;
      Etaz[LastEt].bla := blaS;
   end;
   for i := 0 to LastEt do
   begin
      if Etaz[i].VS = 1 then
         result := true;
      if Etaz[i].VS = 2 then
         VS2 := true;
   end;
end;

INITIALIZATION
   RegisterOrbisClass(TCreate1VSLayer);
END.

