unit t_DivideLHC;

interface
uses
   Elements,
   e_Geometry,
   Tools;

type

TLHCDivider = class(TTool)
private
    FSourceDirectory: string;
    FDestinationDirectory: string;
    FFrameAreas: TEntity;
    FWorkEntity: TEntity;
    FSourceEntity: TEntity;
    FExportScript: string;
    FActualName: string;
    FBlockName: string;
    FBrzda: integer;
    FARE_DET_By_LS_KOD: boolean;
    procedure SetDestinationDirectory(const Value: string);
    procedure SetSourceDirectory(const Value: string);
    procedure SetFrameAreas(const Value: TEntity);
    procedure SetWorkEntity(const Value: TEntity);
    procedure SetSourceEntity(const Value: TEntity);
    procedure SetExportScript(const Value: string);
    procedure SetActualName(const Value: string);
    procedure SetBlockName(const Value: string);
    function GetFullBlockName: string;
    procedure SetBrzda(const Value: integer);
    procedure SetARE_DET_By_LS_KOD(const Value: boolean);
public
   procedure DoIt; override;
   procedure CreateWorkModel;
   procedure SelectByArea(anArea: TAreaElement);
   procedure FreeWorkModel;
   procedure RunExportScript;
   property ActualName: string read FActualName write SetActualName;
published
   property SourceDirectory: string read FSourceDirectory write SetSourceDirectory;
   property DestinationDirectory: string read FDestinationDirectory write SetDestinationDirectory;
   property ExportScript: string read FExportScript write SetExportScript;
   property FrameAreas: TEntity read FFrameAreas write SetFrameAreas;
   property SourceEntity: TEntity read FSourceEntity write SetSourceEntity;
   property WorkEntity: TEntity read FWorkEntity write SetWorkEntity;
   property BlockName: string read FBlockName write SetBlockName;
   property FullBlockName: string read GetFullBlockName stored false;
   property Brzda: integer read FBrzda write SetBrzda;
   property ARE_DET_By_LS_KOD: boolean read FARE_DET_By_LS_KOD write SetARE_DET_By_LS_KOD;

end;

implementation
uses
   Dialogs,
   e_Class,
   e_GeometryElement,
   e_TopolElements,
   Forms,
   FileCtrl,
   g_Elems,
   m_Orbis,
   m_Proj,
   t_SpatialAnalysis,
   u_String;

{ TLHCDivider }

procedure TLHCDivider.CreateWorkModel;
var
   i: integer;
   e: TElement;
begin
   if WorkEntity = nil then exit;
   if SourceEntity = nil then exit;
   WorkEntity.FreeAll;
   for i := 0 to SourceEntity.LastIndex do
   begin
      //e := TGeometryElementCollection.Create;
      e := TElement(SourceEntity.Elements[i].ClassType.Create);
      e.Name := SourceEntity.Elements[i].Name;
      WorkEntity.AddElement(e);
   end;
end;

procedure TLHCDivider.DoIt;
var
   i: integer;
   G: TGeometryElement;
   A: TAreaElement;
   e: TElement;
begin
   //CreateWorkModel;
   //exit;
   if SourceDirectory = '' then
      exit;
   if not DirectoryExists(SourceDirectory) then
   begin
      ShowMessage('Zdrojov sloka: '+ SourceDirectory+' nenalezena!!!');
      exit;
   end;
   if DestinationDirectory = '' then
      exit;
   if not DirectoryExists(DestinationDirectory) then
   begin
      ShowMessage('Zdrojov sloka: '+ DestinationDirectory+' nenalezena!!!');
      exit;
   end;
   if FrameAreas = nil then exit;
   if WorkEntity = nil then exit;
   //if ExportScript = '' then exit;
   // Dal kontroly .........
   StartProgress('Dlen LHC po revrech',0,0);
   for i := 0 to FrameAreas.LastIndex do
   begin
      if (Brzda > 0) and (I >= Brzda) then
         exit;
      CreateWorkModel;
      e := FrameAreas.Elements[i];
      if not (e is TGeometryElement) then continue;
      G := TGeometryElement(e);
      if G.NoGeometry or not (G.Geometry is TAreaElement) then continue;
      ActualName := G.Name;
      A := TAreaElement(G.Geometry);
      SelectByArea(A);
      Progress := 0;
      ProgressText :=' Zpis rev.. '+ActualName+' na disk';
      Application.ProcessMessages;
      //exit;
      RunExportScript;
      Application.ProcessMessages;
      Application.ProcessMessages;
      //exit;
      FreeWorkModel;
   end;
end;

procedure TLHCDivider.FreeWorkModel;
begin
   if WorkEntity = nil then exit;
   WorkEntity.FreeAll;
end;

function TLHCDivider.GetFullBlockName: string;
begin
   result := DestinationDirectory;
   CheckBackSlash(result);
   result := result + ActualName;
   CheckBackSlash(result);
   result := result+BlockName;
end;

procedure TLHCDivider.RunExportScript;
begin
   PostTextMsg(tm_Run,ExportScript,nil);
   Application.ProcessMessages;
end;

procedure TLHCDivider.SelectByArea(anArea: TAreaElement);
var
   i,j,ii: integer;
   E1,E2: TEntity;
   e: TElement;
   GE: TGeometryElement;
   SA: TSpatialAnalysisTool;
   Are: TAreaElement;
   MA: TMultiAreaElement;
   Lin: TPolyLineElement;
   Pnt: TPointElement;
   Txt: TTextElement;
   FindByArea: boolean;
   AREDET: TEntity;
   LSS: string;
   By_LS: boolean;
begin
   AREDET := nil;
   SA := TSpatialAnalysisTool.Create;
   for j := 0 to SourceEntity.LastIndex do
   begin
      By_LS := false;
      E1 := TEntity(SourceEntity.Elements[j]);
      if E1.NoElements then continue;
      ProgressText :='Rev.. '+ActualName+' Vrstva '+E1.Name;
      MaxProgress := E1.LastIndex;
      Progress := 0;
      Application.ProcessMessages;
      if (E1.Name = 'LIN_REV') or (E1.Name = 'ARE_REV') then continue;// revry jsou jen pro referenci
      E2 := TEntity(WorkEntity.FindElementName(E1.Name));
      if (E1.Name = 'ARE_DET') then
      begin
         AREDET := E2;
         if ARE_DET_By_LS_KOD then
            By_LS := true;
      end;
      FindByArea := (E1.Name = 'LIN_DET') and (AREDET <> nil);
      for i := 0 to E1.LastIndex do
      begin
         e := E1.Elements[i];
         if not (e is TGeometryElement) then continue;
         GE := TGeometryElement(e);
         if not By_LS then
            if not GE.IsOverRegion(anArea.MinX,anArea.MinY,anArea.MaxX,anArea.MaxY) then continue;
         if i mod 100 = 0 then
         begin
            Progress := i;
            Application.ProcessMessages;
         end;
         if By_LS then
         begin
            LSS := GE.GetPropertyStringByName('LS_KOD');
            if LSS = ActualName then
            begin
               E2.AddElement(GE);
            end;
            continue;
         end;
         e := GE.Geometry;
         if e = nil then continue;
         if e is TAreaElement then
         begin
            Are := TAreaElement(e);
            if Are is TMultiAreaElement then
            begin
               MA := TMultiAreaElement(Are);
               for ii := 0 to MA.LastIndex do
               begin
                  Are := MA.AreaElement[ii];
                  if anArea.IsHitByXY(Are.X,Are.Y,0) then
                  begin
                     E2.AddElement(GE);
                     break;
                  end;
               end;
            end
            else
            if anArea.IsHitByXY(Are.X,Are.Y,0) then
               E2.AddElement(GE);
         end
         else
         if e is TPolyLineElement then
         begin
            Lin := TPolyLineElement(e);
            if FindByArea then
            begin
               if Lin.FindClassWithOwnerInOwners(TTopolAreaElement,AREDET) <> nil then
                  E2.AddElement(GE);
            end
            else
               if SA.Relate_Line_Area(Lin,anArea) then
                  E2.AddElement(GE);
         end
         else
         if e is TPointElement then
         begin
            Pnt := TPointElement(e);
            if anArea.IsHitByXY(Pnt.X,Pnt.Y,0) then
               E2.AddElement(GE);
         end
         else
         if e is TTextElement then
         begin
            Txt := TTextElement(e);
            if anArea.IsHitByXY(Txt.X,Txt.Y,0) then
               E2.AddElement(GE);
         end;
      end;
      if E2 is TRectEntity then
         TRectEntity(E2).ResetAndUpdate;
   end;
   SA.Free;
end;

procedure TLHCDivider.SetActualName(const Value: string);
begin
  FActualName := Value;
end;

procedure TLHCDivider.SetARE_DET_By_LS_KOD(const Value: boolean);
begin
  FARE_DET_By_LS_KOD := Value;
end;

procedure TLHCDivider.SetBlockName(const Value: string);
begin
  FBlockName := Value;
end;

procedure TLHCDivider.SetBrzda(const Value: integer);
begin
  FBrzda := Value;
end;

procedure TLHCDivider.SetDestinationDirectory(const Value: string);
begin
  FDestinationDirectory := Value;
end;

procedure TLHCDivider.SetExportScript(const Value: string);
begin
  FExportScript := Value;
end;

procedure TLHCDivider.SetFrameAreas(const Value: TEntity);
begin
  FFrameAreas := Value;
end;

procedure TLHCDivider.SetSourceDirectory(const Value: string);
begin
  FSourceDirectory := Value;
end;

procedure TLHCDivider.SetSourceEntity(const Value: TEntity);
begin
  FSourceEntity := Value;
end;

procedure TLHCDivider.SetWorkEntity(const Value: TEntity);
begin
  FWorkEntity := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TLHCDivider);
END.
