unit t_LHCSelect;

interface
uses
   Elements,
   e_Space,
   t_Recurs,
   T_SpatialAnalysis;

type

TMultiSelectSkupina = class(TRecursor)
private
   FOModel: TElement;
protected
   procedure   FindInput; virtual;
   procedure   SetOutput(aOutput: TElement); override;
   procedure   CheckObjectModel;
public
   constructor Create; override;
   procedure   DoIt; override;
   procedure   Reset; virtual;
end;

TMultiSelectSkupinaByRect = class(TMultiSelectSkupina)
private
    FMinY: double;
    FMinX: double;
    FMaxX: double;
    FMaxY: double;
    procedure SetMaxX(const Value: double);
    procedure SetMaxY(const Value: double);
    procedure SetMinX(const Value: double);
    procedure SetMinY(const Value: double);
protected
   function  QGoInside(Element: TElement): boolean; override;
   function  QProcessBefore(Element: TElement): boolean; override;
   function  QProcessAfter(Element: TElement): boolean; override;
   procedure Process(Element: TElement); override;
   function  TestArea(anArea: TAreaElement): boolean; virtual;
published
   property  MinX: double read FMinX write SetMinX;
   property  MinY: double read FMinY write SetMinY;
   property  MaxX: double read FMaxX write SetMaxX;
   property  MaxY: double read FMaxY write SetMaxY;
end;

TMultiSelectSkupinaByLine = Class(TMultiSelectSkupinaByRect)
  private
    FLine: TPolyLineElement;
    FSATool: TSpatialAnalysisTool;
    procedure SetLine(const Value: TPolyLineElement);
protected
   function  TestArea(anArea: TAreaElement): boolean; override;
   property  SATool: TSpatialAnalysisTool read FSATool;
public
   constructor Create; override;
published
   property Line: TPolyLineElement read FLine write SetLine;
end;

implementation
uses
   e_LHCModel,
   g_Elems,
   m_Orbis,
   Worlds;

{ TMultiSelectSkupina }

procedure TMultiSelectSkupina.CheckObjectModel;
var
   E: TElement;
begin
   E := World.ObjectModel;
   if E <> FOModel then
      FindInput;
end;

constructor TMultiSelectSkupina.Create;
begin
   inherited;
   FindInput;
end;

procedure TMultiSelectSkupina.DoIt;
var
   E: TOwnerEntity;
begin
   CheckObjectModel;
   if Output = nil then
   begin
      E := TOwnerEntity.Create;
      Output := E;
   end
   else
   begin
      if not (Output is TEntity) then
      begin
         PostTextMsg(tm_AddToprotocol,'ERROR: TMultiSelectSkupina required TEntity class at Output!',nil);
         exit;
      end;
   end;
   inherited;
end;

procedure TMultiSelectSkupina.FindInput;
var
   aData,
   aHier: TEntity;
begin
   FOModel := World.ObjectModel;
   Input := nil;
   aData:= TEntity(World.ObjectModel.FindElementName('DATA'));
   if aData = nil then exit;
   aHier:= TEntity(aData.FindElementName('HIERARCHY'));
   if aHier = nil then exit;
   Input := aHier;
end;

procedure TMultiSelectSkupina.Reset;
begin
   if Output <> nil then
   begin
      if Output is TEntity then
         TEntity(Output).FreeAll;
   end;
end;

procedure TMultiSelectSkupina.SetOutput(aOutput: TElement);
begin
   ExchangeFieldElement(FOutput, aOutput);
end;

{ TMultiSelectSkupinaByRect }

procedure TMultiSelectSkupinaByRect.Process(Element: TElement);
var
   PSK: TPorostniSkupina;
   R: TRectEntity;
   i: integer;
   E: TElement;
   A: TAreaElement;
begin
   if not (Element is TPorostniSkupina) then exit;
   PSK := TPorostniSkupina(Element);
   if PSK.NoGeometry then exit;
   R := PSK.Geometry;
   for i := 0 to R.LastIndex do
   begin
      E := R.Elements[i];
      if not (E is TAreaElement) then Continue;
      A := TAreaElement(E);
      if TestArea(A) then
      begin
         TEntity(Output).AddElement(PSK);
         Break;
      end;
   end;
end;

function TMultiSelectSkupinaByRect.QGoInside(Element: TElement): boolean;
begin
   result := false;
   if not (Element is TEntity) then exit;
   if not (Element is TRectEntity) then
   begin
      result := true;
      exit;
   end;
   if Element is TPorostniSkupina then exit;
   result := TRectEntity(Element).IsOverRegion(Minx,Miny,Maxx,Maxy);
end;

function TMultiSelectSkupinaByRect.QProcessAfter(
  Element: TElement): boolean;
begin
   result := false;
end;

function TMultiSelectSkupinaByRect.QProcessBefore(
  Element: TElement): boolean;
begin
   result := Element is TPorostniSkupina;
   if result then
   begin
      result := TPorostniSkupina(Element).IsOverRegion(Minx,Miny,Maxx,Maxy);
   end;
end;

procedure TMultiSelectSkupinaByRect.SetMaxX(const Value: double);
begin
  FMaxX := Value;
end;

procedure TMultiSelectSkupinaByRect.SetMaxY(const Value: double);
begin
  FMaxY := Value;
end;

procedure TMultiSelectSkupinaByRect.SetMinX(const Value: double);
begin
  FMinX := Value;
end;

procedure TMultiSelectSkupinaByRect.SetMinY(const Value: double);
begin
  FMinY := Value;
end;

function TMultiSelectSkupinaByRect.TestArea(anArea: TAreaElement): boolean;
begin
   result := anArea.IsOverRegion(Minx,Miny,Maxx,Maxy);
end;

{ TMultiSelectSkupinaByLine }


constructor TMultiSelectSkupinaByLine.Create;
begin
   inherited;
   ExchangeFieldElement(TElement(FSATool), TSpatialAnalysisTool.Create);
end;


procedure TMultiSelectSkupinaByLine.SetLine(const Value: TPolyLineElement);
begin
   FLine := Value;
   if Value = nil then
   begin
      MinX:= 1E30;
      MinY:= 1E30;
      MaxX:= -1E30;
      MaxY:= -1E30;
      exit;
   end;
   Minx := FLine.Minx;
   Miny := FLine.Miny;
   Maxx := FLine.Maxx;
   Maxy := FLine.Maxy;
end;


function TMultiSelectSkupinaByLine.TestArea(anArea: TAreaElement): boolean;
begin
   result := SATool.Relate_Line_Area(Line, anArea);
end;

end.
