unit t_RasterDialog;

interface
uses
   Elements,
   f_RasterDialog,
   t_OrbisDialog;

type

TRasterDialog = class(TOrbisDialog)
 private
    FNext: boolean;
    function GetForm: TRasterDialog_Form;
    function GetRasterLayer: TEntity;
    function GetAllColor: integer;
    function GetColor: integer;
    function GetAllRasters: boolean;
    procedure SetAllRasters(const Value: boolean);
    procedure SetDirectory(const Value: string);
    function GetDirectory: string;
    procedure SetNext(const Value: boolean);
    procedure SetAllColor(const Value: integer);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TRasterDialog_Form read GetForm;
published
   property RasterLayer: TEntity read GetRasterLayer;
   property Color: integer read GetColor;
   property AllColor: integer read GetAllColor write SetAllColor;
   property AllRasters: boolean read GetAllRasters write SetAllRasters;
   property Directory: string read GetDirectory write SetDirectory;
   property Next: boolean read FNext write SetNext;
end;

implementation
uses
   Forms,
   Graphics,
   SysUtils,
   e_Raster,
   e_Class;

{ TRasterDialog }

procedure TRasterDialog.CreateForm;
begin
   aForm := TRasterDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TRasterDialog.DoIt;
var
   R: TRaster;
   S: string;
   i: integer;
begin
   if Input = nil then
   begin
      AllRasters := true;
      Form.Raster.TabVisible := false;
   end
   else
   begin
      Form.Raster.TabVisible := true;
      Form.b_Next.Enabled := Next;
      AllRasters := false;
      R := TRaster(input);
      if R.RasTyp <> 0 then
      begin
         Form.ColorPanel.Color := clSilver;
         Form.ColorPanel.Enabled := false;
      end
      else
      begin
         Form.ColorPanel.Color := R.Color;
         Form.ColorPanel.Enabled := True;
      end;
      Form.l_FileName.Caption := R.FileName;
      Form.l_FileName.Hint := R.FileName;
      Form.l_FileSize.Caption := IntToStr(R.FileSize);
      Case R.RasTyp of
         0: S := 'Binrn rastr';
         1: S := '8 bitov edotnov rastr';
         2: S := '8 bitov barevn rastr';
         3: S := '4 bitov edotnov rastr';
         4: S := '4 bitov barevn rastr';
         5: S := '24 bitov barevn rastr';
      else
            S := 'Neznm';
      end;
      Form.l_RasTyp.Caption := S;
      Case R.Compres of
         0: S := 'Nekomprimovan';
         1: S := 'Packbits';
         2: S := 'CCITT Group 3';
         3: S := 'CCITT Group 4';
         4: S := 'LZW';
         5: S := 'CCITT Group 3 modifikovan';
      else
            S := 'Neznm';
      end;
      Form.l_Compress.Caption := S;
      if R.Tiled then S := 'Ano' else S := 'Ne';
      Form.l_Tiled.Caption := S;
      Form.l_PixelWidth.Caption := IntToStr(R.PixelWidth);
      Form.l_PixelHeight.Caption := IntToStr(R.PixelHeight);
      Form.l_Horizres.Caption := FloatToStr(R.HorizRes);
      Form.l_Vertres.Caption := FloatToStr(R.VertRes);
   end;
   inherited;
   if result = 1 then
   begin
      if not AllRasters then
         R.Color := Form.ColorPanel.Color
      else
      begin
         Input := nil;
         if RasterLayer = nil then exit;
         for i := 0 to RasterLayer.LastIndex do
         begin
            R := TRaster(RasterLayer.Elements[i]);
            if not (TElement(R) is TRaster) then
               continue;
            R.Color := AllColor;
         end;
      end;
   end
   else
   if result = 3 then
   begin
      if AllRasters then
         Input := nil;
   end;
end;

function TRasterDialog.GetAllColor: integer;
begin
   result := Form.AllColorPanel.Color;
end;

function TRasterDialog.GetAllRasters: boolean;
begin
   result := Form.PageControl.ActivePage = Form.Rasters;
end;

function TRasterDialog.GetColor: integer;
begin
   result := Form.ColorPanel.Color;
end;

function TRasterDialog.GetDirectory: string;
begin
   result := Form.ed_Directory.Text;
end;

function TRasterDialog.GetForm: TRasterDialog_Form;
begin
   result := TRasterDialog_Form(aForm);
end;

function TRasterDialog.GetRasterLayer: TEntity;
begin
   result := TEntity(Output);
end;

procedure TRasterDialog.SetAllColor(const Value: integer);
begin
   Form.AllColorPanel.Color := Value;
end;

procedure TRasterDialog.SetAllRasters(const Value: boolean);
begin
   if Value then
      Form.PageControl.ActivePage := Form.Rasters
   else
      Form.PageControl.ActivePage := Form.Raster;
   Application.ProcessMessages;
   Form.PageControlChange(nil);
end;

procedure TRasterDialog.SetDirectory(const Value: string);
begin
  Form.ed_Directory.Text := Value;
end;

procedure TRasterDialog.SetNext(const Value: boolean);
begin
  FNext := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TRasterDialog);
END.
