unit t_VodPDialog;

interface
uses
   Elements,
   e_GeometryElement,
   f_VodPDialog,
   t_OrbisDialog;


type
TVodPDialog = class(TOrbisDialog)
private
    FFrame: TElement;
    FBigFrame: TElement;
    FGroup: integer;
    FColor: integer;
    FSymbol: integer;
    FDynamicProp: string;
    FComment: string;
    FOldObjectText: string;
    FNewObjectText: string;
    function GetForm: TVodPDialog_Form;
    procedure SetFrame(const Value: TElement);
    procedure SetBigFrame(const Value: TElement);
    procedure SetColor(const Value: integer);
    procedure SetGroup(const Value: integer);
    procedure SetSymbol(const Value: integer);
    procedure SetDynamicProp(const Value: string);
    procedure SetComment(const Value: string);
    procedure SetNewObjectText(const Value: string);
    procedure SetOldObjectText(const Value: string);
public
   procedure DoIt; override;
   procedure CreateForm; override;
   property  Form: TVodPDialog_Form read GetForm;
published
   property  BigFrame: TElement read FBigFrame write SetBigFrame stored false;
   property  Frame: TElement read FFrame write SetFrame stored false;
   property  Group: integer read FGroup write SetGroup;
   property  Color: integer read FColor write SetColor;
   property  Symbol: integer read FSymbol write SetSymbol;
   property  DynamicProp: string read FDynamicProp write SetDynamicProp;
   property  Comment: string read FComment write SetComment;
   property  NewObjectText: string read FNewObjectText write SetNewObjectText;
   property  OldObjectText: string read FOldObjectText write SetOldObjectText;
end;

implementation
uses
   Forms,
   SysUtils,
   e_Class,
   e_Props,
   e_TopolElements,
   e_Geometry;

{ TVodPDialog }

procedure TVodPDialog.CreateForm;
begin
   aForm := TVodPDialog_Form.Create(Application);
   aForm.Tool := Self;
   aForm.IsPossibleToClose := false;
end;

procedure TVodPDialog.DoIt;
var
   Are: TAreaElement;
   TP: TTopolAreaElement;
   GAre: TGeometryElement;
   S: string;
begin
   Are := nil;
   GAre := nil;
   TP := nil;
   if input = nil then exit;
   if Input is TAreaElement then Are := TAreaElement(Input);
   if Input = nil then exit;
   if Output <> nil then
      if Output is TGeometryElement then GAre := TGeometryElement(Output);

   Form.l_Area.Caption := FormatFloat('0.####', Are.Area/10000);
   if GAre <> nil then
   begin
      Caption := OldObjectText+Comment;
      if not (GAre is TTopolAreaElement) then exit;
      TP := TTopolAreaElement(GAre);
      S := '';
      if TP.Properties <> nil then
         S := TP.Properties.GetPropertyStringByName(DynamicProp);
      Form.ed_Name.Text := S;
      Form.b_Delete.Enabled := true;
   end
   else
   begin
      Form.b_Delete.Enabled := false;
      Caption := NewObjectText+Comment;
   end;
   inherited;
   if Result = 1 then
   begin
      if GAre = nil then
      begin
         GAre := TTopolAreaElement.Create;
         GAre.Geometry := Are;
         TP := TTopolAreaElement(GAre);
         TP.Druh := Group;
         TP.Barva := Color;
         TP.Znacka := Symbol;
         //TP.Properties := TDynamicProperties.Create;
      end;
      if TP.NoProperties then
         TP.Properties := TDynamicProperties.Create;
      Output := GAre;
      TP.Properties.SetPropertyStringByName(DynamicProp, Form.ed_Name.Text);
   end;
end;

function TVodPDialog.GetForm: TVodPDialog_Form;
begin
   result := TVodPDialog_Form(aForm);
end;

procedure TVodPDialog.SetBigFrame(const Value: TElement);
begin
   FBigFrame := Value;
end;

procedure TVodPDialog.SetColor(const Value: integer);
begin
  FColor := Value;
end;

procedure TVodPDialog.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TVodPDialog.SetDynamicProp(const Value: string);
begin
  FDynamicProp := Value;
end;

procedure TVodPDialog.SetFrame(const Value: TElement);
begin
   FFrame := Value;
end;

procedure TVodPDialog.SetGroup(const Value: integer);
begin
  FGroup := Value;
end;

procedure TVodPDialog.SetNewObjectText(const Value: string);
begin
  FNewObjectText := Value;
end;

procedure TVodPDialog.SetOldObjectText(const Value: string);
begin
  FOldObjectText := Value;
end;

procedure TVodPDialog.SetSymbol(const Value: integer);
begin
  FSymbol := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TVodPDialog);
END.
