unit I_gpoint;
{
********************************************************
**                                                    **
**                    ORBIS                           **
**     Skupina interpretr Topolovskch bod          **
**                                                    **
**                 Topol Pro 1996                    **
********************************************************
}

interface
uses
   Elements,
   e_Device,
   e_PSymb,
   g_Elems,
   i_Gener,
   Tools;

type

TGeneralPointInterpret = class(TGeneralXYInterpret)
 protected
    Symbols: TTopoLSymbols;
    //Rect: TDoubleRect;
    function  IsItsClass(AObject: TObject): boolean; override;
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
 public
   procedure  Reset(aDevice: TDevice); override;
   procedure  Close; override;
end;



implementation

uses
   WinProcs,
   Controls,
   Forms,
   Graphics,
   SysUtils,
   e_Geometry,
   e_GeometryELement,
   e_TopolElements,
   e_Class,
   e_Color,
   u_Symbol;

function  TGeneralPointInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (AObject is TTopolPointElement) or (AObject is TPointCollection);
end;


function  TGeneralPointInterpret.QGoInside(Element: TElement): boolean;
begin
   Result:= Element is TPointCollection;
end;

function  TGeneralPointInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= Element is TTopolPointElement;
end;



procedure TGeneralPointInterpret.Process(Element: TElement);
var
  IX, IY: double;
  iiX, iiY: integer;
  I: longInt;
  Barva: longInt;
  Bod: TPointElement;
  BP: TTopolPointElement;
  aPoint: TPointElement;
  G: TGeometry;
  R: TDoubleRect;
  IsInside: boolean;
begin
  IsInside := false;
  if (Element is TTopolPointElement) then
  begin
     BP := TTopolPointElement(Element);
     G := BP.Geometry;
     if G = nil then exit;
     if StrictRects and (Symbols <> nil) and (BP.Znacka > 999 )then
     begin
        Symbols.OpsanyObdelnik(G,BP.Znacka,R);
        iX := G.X;
        iY := G.Y;
        G.MinX := R.MinX;
        G.MinY := R.MinY;
        G.MaxX := R.MaxX;
        G.MaxY := R.MaxY;
        G.AbsRefX := iX;
        G.AbsRefY := iY;
        IsInside := NoRect or G.IsOverRegionEx(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY);
     end
     else
        IsInside := TPointElement(Element).IsInsideRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY);
     if IsInside then
     begin
      BP := TTopolPointElement(Element);
      Bod:= TPointElement(BP.Geometry);
      if Bod = nil then exit;
      //if not (BP is TTopolProps) then exit;
      try
         if UseColor then
            Barva:= Color
         else
         if ToolColor <> nil then
         begin
            ToolColor.Input := BP;
            ToolColor.DoIt;
            Barva := StrToInt(ToolColor.Result);
         end
         else
         if PropColor = '' then Barva := Color
                           else Barva  := StrToInt(Element.GetPropertyStringByName(PropColor));

         if ColorByPalette then Device.Pen.Color := Palette.Color[Barva]
         else
         if ColorByTopol then
         begin
            Device.Pen.Color := DejBarvicku(Barva)
         end
         else Device.Pen.Color := Barva;

         Device.Pen.Style := psSolid;
         if (Symbols <> nil) and (BP.Znacka > 999) then
         begin
            Symbols.InterpretSymbol(Bod, BP.Znacka);
            exit;
         end;
         IX:= Bod.X;
         IY:= Bod.Y;
         iiX := Device.Projection.GetPageX(IX);
         iiY := Device.Projection.GetPageY(IY);
         Device.MoveToDevice(iiX-2, iiY);
         Device.LineToDevice(iiX+2, iiY);
         Device.MoveToDevice(iiX, iiY-2);
         Device.LineToDevice(iiX, iiY+2);
      except
         {
         MessageDlg(IntToStr(TTopoLLin(Element).Count) + ' Chyba interpretace linie . '+ IntToStr(TTopoLLin(Element).Cislo),
                             mtInformation, [mbOK], 0);
         }
//         MessageBeep(word(-1));
      end;

     end;
  end
  else inherited process(Element);
end;


procedure   TGeneralPointInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   //Rect := Device.Projection.ModelView;

   if SymbolsFile = '' then
      SymbolsFile:= PathOfLibraries+'\Fonty.txt';

   if fileExists(SymbolsFile) then
   begin
     if Symbols <> nil then
        Symbols.Free;

     Symbols := TTopoLSymbols.CreateAll(SymbolsFile);

     try
       Symbols.Device := Device
     except
       if Symbols <> nil then
          Symbols.Free;
       Symbols:= nil
     end;
   end
end;

procedure   TGeneralPointInterpret.Close;
begin
  if Symbols <> nil then
  begin
     Symbols.Free;
     Symbols:= nil
  end;
  inherited Close
end;






BEGIN
   RegisterOrbisClass(TGeneralPointInterpret);
END.
