unit T_mscip;

interface
uses
   Classes,
   Dialogs,
   Elements,
   Tools;

const
   ce_QueryTextDlg = 'QUERYTEXTDLG';

type

TMSCInterpret = class(TTool)
private
   FLines: TStringList;
   function GetEntity: TEntity;
public
   constructor CreateAll(aName: string);
   procedure   DoIt; override;
   procedure   InterpretLines;
   procedure   QueryTextDialog;
   property    Lines: TStringList read FLines write FLines;
   property    Entity: TEntity read GetEntity ;
end;

implementation

uses
   SysUtils,
   T_VarTxt,
   g_Elems,
   u_Props,
   u_String,
   D_TxtBlk;

constructor TMSCInterpret.CreateAll(aName: string);
begin
   inherited Create;
   Name := aName;
end;

procedure   TMSCInterpret.DoIt;
begin
   if Entity = nil then exit;
   Output := Entity;
   if not FileExists(Name) then exit;
   Lines := TStringList.Create;
   try
      Lines.LoadFromFile(Name);
      if Lines.Count > 0 then
      begin
         InterpretLines;
      end;
   except
   end;
   Lines.Free;
end;

function    TMSCInterpret.GetEntity: TEntity;
begin
   result := TEntity(Input);
end;


procedure   TMSCInterpret.InterpretLines;
var
   i: longInt;
   L, R: string;
   Delimiters : TDelimiters;
   Delimiter: char;
   Position: longInt;
   S: string;
   E: TElement;
   Query: boolean;
   List: boolean;
   Command: boolean;
begin
    Delimiters := ['.','(',')',':'];
    for i := 0 to Lines.Count - 1 do
    begin
       Query := false;
       List  := false;
       Command := false;
       try
          if StringAsParameter(Lines.Strings[i], L, R) then
          begin
             { Zpracovn prav strany - hledn Query, Command a List }
             S := ReadToDelimiterWithoutWS(R,  Delimiters, 1, Delimiter, Position);
             while  (Delimiter = ':') do
             begin
                if S = '?' then Query := true else
                if S = '!' then Command := true else
                if UpperCase(S) = ce_List then List := true;
                R := Copy(R, Position, (Length(R) - Position)+1);
                S := ReadToDelimiterWithoutWS(R,  Delimiters, 1, Delimiter, Position);
             end;

             { Zpracovn lev strany - Nstroj, properta, metoda }
             S := ReadToDelimiterWithoutWS(L,  Delimiters, 1, Delimiter, Position);
             if  Delimiter = '.' then
             begin
             { nalezen objekt na desktopu ?, pak L je jeho vlastnost }
                E := Entity.FindElementName(S);
                L := Copy(L, Position, (Length(L) - Position)+1);
                if E = nil then
                begin
                  ShowMessage('ten souboru ' + Name + #13+
                              'Chyba na dku '+ IntToStr(i+1)+ ': '+
                              'neznm objekt ('+S+')' );
                  continue
                end
             end
             else
             begin
                E := Entity;
                if GetPropIndex(E, S) < 0 then
                { S nen vlastnost Desktopu, oekvm, e lev strana je nzev
                  nstroje, ktermu se na vstup vlo vsledek vrazu na prav stran }
                begin
                  E:= Entity.ExecuteCommandLineToElement(R);
                  if E = nil then
                  begin
                     ShowMessage('ten souboru ' + Name + #13+
                                 'Chyba na dku '+ IntToStr(i+1)+ ': '+
                                 'neznm objekt ('+R+')' );
                     continue
                  end;

                  if not CreateAndExecuteTool(S, E) then
                     ShowMessage('ten souboru ' + Name + #13+
                                 'Chyba na dku '+ IntToStr(i+1)+ ': '+
                                 'neznm vlastnost ('+S+')' );
                  continue
                end
             end;

             if Command then
             begin
                if List then
                   SetObjectPropByName(E, L, Entity.ExecuteCommandLineToList(R))
                else
                begin
                   R := Entity.ExecuteCommandLine(R);
                   if Query then R := '?:'+R;
                   SetPropStringByName(E, L, R);
                end;
             end
             else
             begin
                R:= TakeFromApos(R);
                if Query then R := '?:'+R;
                SetPropStringByName(E, L, R);
             end;
          end
          else
          begin
             if UpperCase(L) = ce_QueryTextDlg then
             begin
                QueryTextDialog;
             end;
          end;
       except
          on EE: Exception do
          ShowMessage('ten souboru ' + Name + #13+
                      'Chyba na dku '+ IntToStr(i+1)+ ': '+EE.Message);
       end;
    end;
end;


procedure   TMSCInterpret.QueryTextDialog;
var
   Delimiters : TDelimiters;
   Delimiter: char;
   Position: longInt;
   S: string;
   i: longInt;
   T: Ttext;
   VTD: TVarTextDialog;
   E, DTP: TEntity;
begin
   Delimiters := ['.','(',')',':'];
   E := TEntity.Create;
   VTD := TVarTextDialog.Create;
   for i := 0 to Entity.Count-1 do
   begin
      if Entity.Elements[i] is Ttext then
      begin
         T := TText(Entity.Elements[i]);
         S := ReadToDelimiterWithoutWS(T.Text,  Delimiters, 1, Delimiter, Position);
         if (Delimiter = ':') and (S = '?') then
         begin
           T.Text := Copy(T.Text, Position, (Length(T.Text) - Position)+1);
           E.AddElement(T);
         end;
      end;
   end;
   if E.Count > 0 then
   begin
     VTD.Input := E;
     VTD.DoIt;
   end;
   E.Free;
   VTD.Free;
end;



END.
