unit I_gtext;
{****************************************************************************}
{                                                                            }
{          ORBIS ... Interpret obecnho True Type Textu                      }
{                    (c) 1996                                                }
{                                                                            }
{****************************************************************************}

interface

uses
   Elements,
   e_Text,
   i_Gener;

type
  TGeneralTextInterpret = class(TGeneralXYInterpret)
  private
     FPropText: string;
     FPropFont: string;
     FValFont: string;
     FAbsSize: boolean;
    FDontCenter: boolean;
     procedure SetAbsSize(const Value: boolean);
    procedure SetDontCenter(const Value: boolean);
  protected
     procedure Process(Element: TElement); override;
  published
     property PropText: string   read FPropText   write FPropText;
     property PropFont: string   read FPropFont   write FPropFont;
     property ValFont: string    read FValFont    write FValFont;
     property AbsSize: boolean read FAbsSize write SetAbsSize;
     property DontCenter: boolean read FDontCenter write SetDontCenter;
  end;


  TGeneralFormatedTextBlockInterpret = class(TGeneralXYInterpret)
  private
     FTextBlock: TFormatedTextBlock;
    FAbsSize: boolean;
     function  GetTextBlock: TFormatedTextBlock;
     procedure SetTextBlock(aValue: TFormatedTextBlock);
    procedure SetAbsSize(const Value: boolean);
  protected
     procedure Process(Element: TElement); override;
  published
     property TextBlock: TFormatedTextBlock read GetTextBlock write SetTextBlock ;
     property AbsSize: boolean read FAbsSize write SetAbsSize;
  end;



implementation

uses
   SysUtils,
   e_GeometryElement,
   e_Geometry,
   e_Class,
   e_Device,
   u_String;

procedure TGeneralTextInterpret.Process(Element: TElement);
var
  IX, IY: double;
  aFont: string;
  aColor: longInt;
  aHeight: double;
  aSize: integer;
  GE: TGeometryElement;
  G: TGeometry;
  i: integer;
  MA: TMultiAreaElement;
  A: TAreaElement;
  S,S1,S2: string;
begin
   try
      if Device is TCanvasDevice then
      begin
         if PropHeight <> '' then aHeight := StrToFloat(Element.GetPropertyStringByName(PropHeight))
                             else aHeight := ValHeight;
         if aHeight <= 0 then exit;
         if AbsSize then aSize := Device.Projection.GetPageLengthVert1(aHeight * Device.Projection.Scale)
                    else aSize := Device.Projection.GetPageLengthVert1(aHeight * Device.Projection.VirtualScale);
         Device.Font.Height := -aSize;
         if PropFont <> '' then aFont := Element .GetPropertyStringByName(PropFont)
                           else aFont := ValFont;
         Device.Font.Name:= aFont;
         if PropColor <> '' then aColor := StrToInt(Element.GetPropertyStringByName(PropColor))
                            else aColor := Color;
         Device.Font.Color:= aColor;
         Device.Font.Pitch
      end;
      GE := nil;
      S1 := PropText;
      S := '';
      while StringAsTwoParams(S1,',',S1,S2) do
      begin
         if (S1 <> '') and (S1[1]='''') then
            S := S + AnsiDequotedStr( S1,'''') + ' '
         else
            S := S + Element.GetPropertyStringByName(S1) + ' ';
         S1 := S2;
      end;
      if (S1 <> '') and (S1[1]='''') then
         S := AnsiDequotedStr( S1,'''') + ' '
      else
         S := S + Element.GetPropertyStringByName(S1) + ' ';
  { TODO -oPPK -cBerlika : General text samozejm nem co ourat v geometry }
      if (Element is TGeometryElement) then
      begin
         GE := TGeometryElement(Element);
         if not GE.NoGeometry and (GE.Geometry is TMultiAreaElement) then
         begin
            MA := TMultiAreaElement(GE.Geometry);
            for i := 0 to MA.LastIndex do
            begin
               A := TAreaElement(MA.Elements[i]);
               IX := A.X;
               IY := A.Y;
               if not DontCenter then
               begin
                  IX := IX - Device.TextModelWidth(S)/2;
                  IY := IY + Device.TextModelHeight(S)/2;;
               end;
               Device.TextOut(IX, IY, S);
            end;
         end
         else
            GE := nil;
      end;
      if GE = nil then
      begin
         IX := StrToFloat(Element.GetPropertyStringByName(PropX));
         IY := StrToFloat(Element.GetPropertyStringByName(PropY));
         if not DontCenter then
         begin
            IX := IX - Device.TextModelWidth(S)/2;
            IY := IY + Device.TextModelHeight(S)/2;;
         end;
         Device.TextOut(IX, IY, S);
      end;
   except
   end;
end;

{ TGeneralFormatedTextBlockInterpret }

function  TGeneralFormatedTextBlockInterpret.GetTextBlock: TFormatedTextBlock;
begin
   if FTextBlock = nil then
   begin
      FTextBlock := TFormatedTextBlock.Create;
      AddElement(FTextBlock);
   end;
   result := FTextBlock;
end;

procedure TGeneralFormatedTextBlockInterpret.SetTextBlock(aValue: TFormatedTextBlock);
begin
   if FTextBlock <> nil then
   begin
      FreeElement(FTextBlock);
      FTextBlock := nil;
   end;
   FTextBlock := aValue;
end;

procedure TGeneralFormatedTextBlockInterpret.Process(Element: TElement);
var
  DX, DY, DXX, DLine: double;
  iX, iY, iXX, iLine: integer;
  i,j: longInt;
  UseSize: boolean;
  P: TParagraph;
  T: TFormatedText;
  XYbyProp: boolean;
  E: TElement;
  aSize: integer;
begin
   try
      if Device is TCanvasDevice then
      begin
         //PostTextMsg(tm_AddToProtocol,'Here',nil);
         if TextBlock.ParagraphStyle.TextHeight = 0 then UseSize := true
                                                    else UseSize := false;
         if UseSize and (TextBlock.ParagraphStyle.TextSize = 0) then exit;
         if (PropX = '') or (PropY='') then
         begin
            DX := ValX;
            DY := ValY;
            XYbyProp := false;
         end
         else
         begin
            DX := StrToFloat(Element.GetPropertyStringByName(PropX));
            DY := StrToFloat(Element.GetPropertyStringByName(PropY));
            XYbyProp := true;
         end;
         if UseSize then
         begin
            Device.Font.Height := -TextBlock.ParagraphStyle.TextSize;
            iLine := TextBlock.ParagraphStyle.LineSize;
            if XYbyProp then
            begin
               ix := Device.Projection.GetPageX(DX);
               iy := Device.Projection.GetPageY(DY);
            end
            else
            begin
               iX := round(DX);
               iY := round(DY);
            end;
         end
         else
         begin
            if AbsSize then
            begin
               aSize := Device.Projection.GetPageLengthVert1(TextBlock.ParagraphStyle.TextHeight * Device.Projection.Scale);
               DLine := TextBlock.ParagraphStyle.LineHeight* Device.Projection.Scale;
            end
            else
            begin
               aSize := Device.Projection.GetPageLengthVert1(TextBlock.ParagraphStyle.TextHeight * Device.Projection.VirtualScale);
               DLine := TextBlock.ParagraphStyle.LineHeight* Device.Projection.VirtualScale;
            end;
            Device.Font.Height := -aSize;
            if (DX =0) and (DY = 0) then
            begin
               DX := Device.Projection.GetModelX(0);
               DY := Device.Projection.GetModelY(0);
            end;
         end;
         Device.Font.Name:= TextBlock.ParagraphStyle.FontName;
         Device.Font.Color := TextBlock.ParagraphStyle.TextColor;

         for i := 0 to TextBlock.Count - 1 do
         begin
            E := TextBlock.Elements[i];
            if not (E is TParagraph) then Continue;
            P := TParagraph(E);
            if UseSize then iXX := iX
                       else DXX := DX;
            if P.Text <> '' then
            begin
               if UseSize then
               begin
                  Device.TextOutDevice(ix,iy, P.Text);
                  ixx := ixx + Device.TextWidth(P.Text);
               end
               else
               begin
                  Device.TextOut(DX, Dy, P.Text);
                  DXX := DXX + Device.TextModelWidth(P.Text);
               end;
            end;
            if not P.NoElements then
            for j := 0 to P.Count -1 do
            begin
               E := P.Elements[j];
               if not (E is TFormatedText) then Continue;
               T := TFormatedText(E);
               if T is TVariableText then
               begin
                  T.Text := Element.GetPropertyStringByName(T.Name);
               end;
               if T.Text <> '' then
               begin
                  if UseSize then
                  begin
                     Device.TextOutDevice(ixx,iy, T.Text);
                     ixx := ixx + Device.TextWidth(T.Text);
                  end
                  else
                  begin
                     Device.TextOut(DXX, Dy, T.Text);
                     DXX := DXX + Device.TextModelWidth(T.Text);
                  end;
               end;
            end;
            if UseSize then iY := iY + iLine
                       else DY := DY - DLine;
         end;
      end;
   except
   end;
end;





procedure TGeneralTextInterpret.SetAbsSize(const Value: boolean);
begin
  FAbsSize := Value;
end;

procedure TGeneralTextInterpret.SetDontCenter(const Value: boolean);
begin
  FDontCenter := Value;
end;

procedure TGeneralFormatedTextBlockInterpret.SetAbsSize(
  const Value: boolean);
begin
  FAbsSize := Value;
end;


INITIALIZATION
   RegisterOrbisClass(TGeneralTextInterpret);
   RegisterOrbisClass(TGeneralFormatedTextBlockInterpret);
END.
