unit t_MultiPageDesktop;

interface
uses
   Graphics,
   e_Desktp,
   e_View,
   G_Elems,
   Tools;

type

TMultiDesktopTool = class(TTool)
private
   FPageViewName: string;
   FSourceViewName: string;
   FTemplateFile: string;
   FMap: TDesktop;
   procedure SetPageViewName(const Value: string);
   procedure SetSourceViewName(const Value: string);
   procedure SetTemplateFile(const Value: string);
   function GetPageView: TView;
   procedure SetMap(const Value: TDesktop);
public
   property  Map: TDesktop read FMap write SetMap;
   property  PageView: TView read GetPageView;
published
   property TemplateFile: string read FTemplateFile write SetTemplateFile;
   property SourceViewName: string read FSourceViewName write SetSourceViewName;
   property PageViewName: string read FPageViewName write SetPageViewName;
end;

TMultiPageDesktopCreator = class(TMultiDesktopTool)
  private
    FNumberTextName: string;
    FRightNumberTextName: string;
    FTopNumberTextName: string;
    FLeftNumberTextName: string;
    FBottomNumberTextName: string;
    FNoNumberText: string;
    FOverlap: double;
    FSpace: double;
    FReferSpace: TRectEntity;
    procedure SetBottomNumberTextName(const Value: string);
    procedure SetLeftNumberTextName(const Value: string);
    procedure SetNoNumberText(const Value: string);
    procedure SetNumberTextName(const Value: string);
    procedure SetRightNumberTextName(const Value: string);
    procedure SetTopNumberTextName(const Value: string);
    procedure SetOverlap(const Value: double);
    procedure SetSpace(const Value: double);
    procedure SetReferSpace(const Value: TRectEntity);
public
   procedure DoIt; override;
   procedure CreateMap(aX,aY: double);
published
   property NumberTextName: string read FNumberTextName write SetNumberTextName;
   property LeftNumberTextName: string read FLeftNumberTextName write SetLeftNumberTextName;
   property RightNumberTextName: string read FRightNumberTextName write SetRightNumberTextName;
   property TopNumberTextName: string read FTopNumberTextName write SetTopNumberTextName;
   property BottomNumberTextName: string read FBottomNumberTextName write SetBottomNumberTextName;
   property NoNumberText: string read FNoNumberText write SetNoNumberText;
   property Overlap: double read FOverlap write SetOverlap;
   property Space: double read FSpace write SetSpace;
   property ReferSpace: TRectEntity read FReferSpace write SetReferSpace;
end;


TMultiPageDesktopPreview = class(TMultiDesktopTool)
private
   FOverlap: double;
   FPenWidth: double;
   FBrushStyle: TBrushStyle;
   FPenColor: TColor;
   FBrushColor: TColor;
   FPenStyle: TPenStyle;
   function GetResultMap: TDesktop;
   function GetResultView: TView;
   function GetMPDesktop: TMultiPageDesktop;
    procedure SetOverlap(const Value: double);
public
   Procedure DoIt; override;
   procedure CreateResultMap;
   property  ResultMap: TDesktop read GetResultMap;
   property  ResultView: TView read GetResultView;
   property  MPDesktop: TMultiPageDesktop read GetMPDesktop;
   property  Overlap: double read FOverlap write SetOverlap;
published
   property PenColor: TColor read FPenColor write FPenColor;
   property PenStyle: TPenStyle read FPenStyle write FPenStyle;
   property PenWidth:  double read FPenWidth write FPenWidth;
   property BrushColor: TColor read FBrushColor write FBrushColor;
   property BrushStyle: TBrushStyle read FBrushStyle write FBrushStyle;
end;


implementation
uses
   SysUtils,
   Elements,
   e_Class,
   m_Orbis,
   t_MapSh;

{ TMultiPageDesktopCreator }

procedure TMultiPageDesktopCreator.CreateMap(aX,aY: double);
var
   MTG: TMSTemplateGenerator;
begin
   MTG := TMSTemplateGenerator.Create;
   MTG.TemplateFileName := TemplateFile;
   MTG.DeltaX := aX;
   MTG.DeltaY := aY;
   MTG.DoIt;
   Map := TDesktop(MTG.Output);
   MTG.Free;
end;

procedure TMultiPageDesktopCreator.DoIt;
var
   D: TDesktop;
   SV,PV: TView;             // SourceView, pageView
   PageX,PageY: double;
   PageW,PageH: double;
   xPages,yPages: integer;
   DD: Double;
   R,Rt: TDoubleRect;
   PageD: TDoubleCoord;
   i,j: integer;
   E: TElement;
   MPD: TMultiPageDesktop;
   OverlapD: double;
begin
// Nalezen view ve zdrojovm desktopu **********
   if input = nil then exit;
   if not (Input is TDesktop) then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage - Input is not Desktop!', nil);
      exit;
   end;
   D := TDesktop(Input);
   E := D.FindElementName(SourceViewName);
   if (E=nil)or not (E is TView) then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage - SourceView not found!: '+SourceViewName, nil);
      exit;
   end;
   SV := TView(E);
// Vytvoen Template map*************************
   CreateMap(0,0);
   if Map=nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage - Cannot create template: '+TemplateFile, nil);
      exit;
   end;
// Poty Pages **********************************
   if PageView = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage - Template View not found: '+PageViewName, nil);
      exit;
   end;
   PageW := Map.Width;
   PageH := Map.Height;
   PageX := PageView.Width-Overlap;
   PageY := PageView.Height-Overlap;

   DD := (SV.Width+Overlap)/PageX;
   xPages := round(int(DD));
   if Frac(DD)<> 0 then Inc(xPages);
   DD := (SV.Height+Overlap)/PageY;
   yPages := round(int(DD));
   if Frac(DD)<> 0 then Inc(yPages);
// Nastaven a naten Viewu *********************
   SV.Width  := (PageX * xPages)+Overlap;
   SV.Height := (PageY * yPages)+Overlap;
   if ReferSpace <> nil then
   begin
      SV.ModelScript.Free;
      SV.ReadModel := true;
      ReferSpace.SetByDoubleRect(SV.Projection.ModelView);
      exit;
   end;
   SV.ReadModel := true;
   R := SV.Projection.ModelView;
// Vytvoen Multipage ********************
   MPD := TMultiPageDesktop.Create;
   Output := MPD;
   MPD.NumberTextName := NumberTextName;
   MPD.LeftNumberTextName := LeftNumberTextName;
   MPD.RightNumberTextName := RightNumberTextName;
   MPD.TopNumberTextName := TopNumberTextName;
   MPD.BottomNumberTextName := BottomNumberTextName;
   MPD.NoNumberText := NoNumberText;
   MPD.Overlap := Overlap;
   Map := nil;
   for j := YPages-1 downto 0 do
   begin
      for i := 0 to XPages-1 do
      begin
         CreateMap(i*(PageW+Space)+Space,j*(PageH+Space)+Space);
         Map.Name := 'PAGE';
         PV := PageView;
         PV.Scale := SV.Scale;
         PV.Model := SV.Model;
         if (i = 0) and (j = YPages-1) then
         begin
            Rt := PV.Projection.ModelView;
            PageD.x := Rt.MaxX - Rt.MinX;
            PageD.Y := Rt.MaxY - Rt.MinY;
            OverlapD := Overlap * PV.Scale;
            MPD.RealOverlap := OverlapD;
         end;
         PV.ModelCenterx := R.MinX+PageD.x/2+i*(PageD.x-OverlapD);
         PV.ModelCentery := R.MinY+PageD.y/2+j*(PageD.y-OverlapD);
         MPD.AddElement(Map);
         Map := nil;
      end;
   end;
   MPD.ResetAndUpdate;
   MPD.MaxX := MPD.MaxX+Space;
   MPD.MaxY := MPD.MaxY+Space;
   MPD.FillNumbers := true;
end;



procedure TMultiPageDesktopCreator.SetBottomNumberTextName(
  const Value: string);
begin
  FBottomNumberTextName := Value;
end;

procedure TMultiPageDesktopCreator.SetLeftNumberTextName(
  const Value: string);
begin
  FLeftNumberTextName := Value;
end;


procedure TMultiPageDesktopCreator.SetNoNumberText(const Value: string);
begin
  FNoNumberText := Value;
end;

procedure TMultiPageDesktopCreator.SetNumberTextName(const Value: string);
begin
  FNumberTextName := Value;
end;

procedure TMultiPageDesktopCreator.SetOverlap(const Value: double);
begin
  FOverlap := Value;
end;

procedure TMultiPageDesktopCreator.SetReferSpace(const Value: TRectEntity);
begin
  FReferSpace := Value;
end;

procedure TMultiPageDesktopCreator.SetRightNumberTextName(
  const Value: string);
begin
  FRightNumberTextName := Value;
end;

procedure TMultiPageDesktopCreator.SetSpace(const Value: double);
begin
  FSpace := Value;
end;


procedure TMultiPageDesktopCreator.SetTopNumberTextName(
  const Value: string);
begin
  FTopNumberTextName := Value;
end;

{ TMultiPageDesktopPreview }

procedure TMultiPageDesktopPreview.CreateResultMap;
var
   MTG: TMSTemplateGenerator;
begin
   MTG := TMSTemplateGenerator.Create;
   MTG.TemplateFileName := TemplateFile;
   MTG.DoIt;
   Output := TDesktop(MTG.Output);
   AddElement(Output);
   MTG.Free;
end;

procedure TMultiPageDesktopPreview.DoIt;
var
   R: TDrawRectangle;
   i: integer;
   DOverlap: double;
begin
   if MPDesktop = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage PreView - Input not found or is not TMultipageDesktop!', nil);
      exit;
   end;
   CreateResultMap;
   if ResultView = nil then
   begin
      PostTextMsg(tm_AddToProtocol,'Multipage PreView - SourceView not found!: '+SourceViewName, nil);
      exit;
   end;
   ResultView.Model := TRectEntity.Create;
   ResultView.Model.Name := 'MODELVIEW';
   DOverlap := MPDesktop.RealOverlap/2;
   for i := 0 to MPDesktop.LastIndex do
   begin
      Map := MPDesktop.DesktopPage[i];
      MPDesktop.ActualPage := Map;
      if i = 0 then
      begin
         TEntity(ResultView.Model).AddElement(PageView.Model);
      end;
      R := TDrawRectangle.Create;
      R.Name := IntToStr(MPDesktop.ActualNumber);
      R.MinX := PageView.ModelMinX;
      R.MinY := PageView.ModelMinY;
      R.MaxX := PageView.ModelMaxX;
      R.MaxY := PageView.ModelMaxY;

      if MPDesktop.ActualLeftNumber > 0 then
         R.MinX := R.Minx + DOverlap;
      if MPDesktop.ActualRightNumber > 0 then
         R.MaxX := R.MaxX - DOverlap;
      if MPDesktop.ActualBottomNumber > 0 then
         R.MinY := R.MinY + DOverlap;
      if MPDesktop.ActualTopNumber > 0 then
         R.MaxY := R.MaxY - DOverlap;
      R.PenColor   := PenColor;
      R.PenStyle   := PenStyle;
      R.PenWidth   := PenWidth;
      R.BrushColor := BrushColor;
      R.BrushStyle := BrushStyle;
      ResultView.AddToModel(R);
   end;
   ResultView.Projection.ModelView := TRectEntity(ResultView.Model).Rect.AsDoubleRect;
end;

function TMultiPageDesktopPreview.GetMPDesktop: TMultiPageDesktop;
begin
   result := nil;
   if Input = nil then exit;
   if not(Input is TMultiPageDesktop) then exit;
   result := TMultiPageDesktop(Input);
end;

function TMultiPageDesktopPreview.GetResultMap: TDesktop;
begin
   result := TDesktop(Output);
end;

function TMultiPageDesktopPreview.GetResultView: TView;
begin
   result := nil;
   if ResultMap = nil then exit;
   result := TView(ResultMap.FindElementName(SourceViewName));
end;



procedure TMultiPageDesktopPreview.SetOverlap(const Value: double);
begin
   FOverlap := Value;
end;

{ TMultiDesktopTool }

function TMultiDesktopTool.GetPageView: TView;
begin
   result := nil;
   if Map = nil then exit;
   result := TView(Map.FindElementName(PageViewName));
end;

procedure TMultiDesktopTool.SetMap(const Value: TDesktop);
begin
   ExchangeFieldElement(TElement(FMap),Value);
end;

procedure TMultiDesktopTool.SetPageViewName(const Value: string);
begin
  FPageViewName := Value;
end;

procedure TMultiDesktopTool.SetSourceViewName(const Value: string);
begin
  FSourceViewName := Value;
end;

procedure TMultiDesktopTool.SetTemplateFile(const Value: string);
begin
  FTemplateFile := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TMultiPageDesktopCreator);
   RegisterOrbisClass(TMultiPageDesktopPreView);
END.
