unit i_RASTER;
{
********************************************************
**                                                    **
**                    ORBIS                           **
**     interpret rastru TopoL Software                **
**                                                    **
**                 Topol Pro 1999                    **
********************************************************
}

interface
uses
   Graphics,
   // KERNEL
   Elements,
   Worlds,
   // GLib
   G_Elems,
   // DEVICE
   e_Device,
   // INTERPRET
   i_Basic,
   // RASTER
   TLViewTrf_TLB,
   TLRastersMan_TLB;

type
TRasDoubleRect = record
   Minx, MinY, Width, Height: double;
end;

TRasterInterpret = class(TInterpret)
 private
    procedure SetProtocol(const Value: string);
 protected
    Rect: TDoubleRect;
    RasStore: IRasStore;
    ViewTrf: IViewTrfObj;
    function  IsItsClass(AObject: TObject): boolean; override;
    function  QGoInside(Element: TElement): boolean; override;
    function  QProcessBefore(Element: TElement): boolean; override;
    procedure Process(Element: TElement); override;
    procedure DrawTransparentBinaryBitmap(ax,ay,BMx: integer; const Bitmap: Graphics.TBitmap; aColor: integer);
    procedure InterpretGeoRaster(Element: TElement);
    procedure InterpretPicture(Element: TElement);
 public
    constructor Create; override;
    procedure   Reset(aDevice: TDevice); override;
    property    Protocol: string write SetProtocol;
end;

implementation

uses
   //{!}T_Intera,
   {!}Dialogs,
   Windows,
   Classes,
   Forms,
   SysUtils,
   // KERNEL
   e_Class,
   e_Type,
   e_Memory,
   m_Orbis,
   // DEVICE
   d_StdImg,
   // RASTER
   UseLic,
   e_Raster;


constructor TRasterInterpret.Create;
var
  hr: hresult;
  hh: integer;
const
  TLRasStoreLicKey: widestring      = '=0Y9A=0C4kVIB4BF0URF=T[:';
  TLViewTrfLicKey: widestring       = '=0BXA=0C4c@FB4c^0URF>0D=';
begin
   inherited;
{
//  RasStore := CoRasStore.Create;
  hr:= CreateInprocObjLic(CLASS_RasStore, TLRasStoreLicKey,
                          IRasStore, IUnknown(RasStore));

//  ViewTrf := CoViewTrfObj.Create;
  hr := CreateInprocObjLic(CLASS_ViewTrfObj, TLViewTrfLicKey,
                           IViewTrfObj, IUnknown(ViewTrf));

 !!!NEWRAS }
//  RasStore := CoRasStore.Create;
  hr:= CreateInprocObjLic('TLRastersMan.RasStore', TLRasStoreLicKey,
                          IRasStore, IUnknown(RasStore));

//  ViewTrf := CoViewTrfObj.Create;
  hr := CreateInprocObjLic('TLViewTrf.ViewTrfObj', TLViewTrfLicKey,
                           IViewTrfObj, IUnknown(ViewTrf));

  hh:= 123;

  RasStore.PresentationStoreConnect(hh,
      integer(nil), integer(nil), integer(nil), integer(nil),
      integer(nil), integer(nil), integer(nil),
      ViewTRF);

end;


function  TRasterInterpret.IsItsClass(AObject: TObject): boolean;
begin
   result := (AObject is TRaster);
end;


function  TRasterInterpret.QGoInside(Element: TElement): boolean;
begin
   Result:= false;
end;

function  TRasterInterpret.QProcessBefore(Element: TElement): boolean;
begin
  Result:= Element is TRaster;
end;

procedure TRasterInterpret.DrawTransparentBinaryBitmap(ax,ay,BMx: integer; const Bitmap: Graphics.TBitmap; aColor: integer);
var
   IsLine: Bytebool;
   PB: PByteArray;
   Dy,DDx: integer;
   BlackIndex: integer;
   i: integer;
   H: integer;
begin
   TCanvasDevice(Device).Pen.color := aColor;
   TCanvasDevice(Device).Pen.Style := psSolid;
   TCanvasDevice(Device).Pen.Mode :=  pmCopy;
   TCanvasDevice(Device).Pen.Width := 0;
   Dy := 0;
   PB := Bitmap.ScanLine[0];
   BlackIndex := PB[0];
   H := Bitmap.Height - 1;
   BMx := BMx - 1;
   while Dy <= H do
   begin
      PB := Bitmap.ScanLine[dy];        // pixelov dek je v poli PB
      isLine := false;
      DDx := 0;
      while DDx <= BMx do
      begin
         if PB[DDx] <> BlackIndex then
         begin
            if not IsLine then
            begin
               MoveToEx(TCanvasDevice(Device).Canvas.Handle, ax+DDx, ay+Dy, nil);
               IsLine := true;
            end;
         end
         else
         begin
            if IsLine then
               LineTo(TCanvasDevice(Device).Canvas.Handle, ax+DDx, ay+Dy);
            IsLine := false;
         end;
         inc(DDx);
      end;
      if IsLine then
      begin
         LineTo(TCanvasDevice(Device).Canvas.Handle, ax+DDx, ay+Dy);
         IsLine := false;
      end;
      inc(Dy);
      if Dy mod 100 = 0 then Application.ProcessMessages;
   end;
   //PostTextMsg(tm_AddToProtocol,'Real Black:'+IntToStr(BlackIndex),nil);
   //Protocol := 'B:'+IntToStr(BlackIndex)+' W:'+IntToStr(WhiteIndex);
   Application.ProcessMessages;
end;


procedure TRasterInterpret.interpretGeoRaster(Element: TElement);
var
   k,l: integer;
   RAS: TRaster;
   VertMin, HorMin, DVert, DHor: double;
   hh: integer;
   a: integer;
   aByte: integer;
   i: integer;
   BDC,BH: integer;
   MemoryDC,PomDC: HDC;
   PomBitmap,BM: HBitmap;
   Bitmap,B1: Graphics.TBitmap;
   CDevice: TCanvasDevice;
   Rmodel: TDoubleRect;                   //vez v modelovch a rastrovch souednicch
   Rras: TRasDoubleRect;
   Rtras: TRasDoubleRect;                 //Tile v ras coordinates
   BMx,BMy: integer;                      //rozmr vezu strnky (Bitmapy)
   TilesCount: integer;
   TileY, dTileY: integer;
   DevStartX, DevStartY: integer;
   BottomY: integer;
   aCol: integer;
   pcx,pcy: double;
   BinaryRaster: boolean;
   S: string;
   ME: TMemoryElement;
   MA: longWord;
   pt: TPoint;
   RasInfo: TStringListElement;
   BasicMem: integer;
   CX,CY,aScale: double;
   //**** only test
   BitmapInfo: ^TBitmapInfo;
   Bits: Pointer;
   InfoHeaderSize: Cardinal;
   ImageSize: Cardinal;
   TX,TY,dTX,dTY: double;
begin
   CDevice := TCanvasDevice(Device);
   Ras := TRaster(Element);
   if RAS.Error > 0 then
   begin
      PostTextMsg(tm_AddToProtocol,'Raster' + Ras.Name + ' : '+ Ras.ErrorText,nil);
      exit;
   end;
   S := Ras.FileName;
   S := ExtractFileName(S);
   if not Ras.IsOverRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY)then exit;

   //*{!!!!!}PostTextMsg(tm_AddToProtocol,'****** Rastr '+ S,nil);

   Rmodel := Ras.InterSectByRect(Rect);
   Bitmap := Graphics.TBitmap.Create;
   BMx := CDevice.Projection.GetPageLengthHoriz1(Rmodel.MaxX-Rmodel.Minx);
   BMy := CDevice.Projection.GetPageLengthVert1(Rmodel.MaxY-Rmodel.MinY);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Bmx '+ IntToStr(Bmx),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Bmy '+ IntToStr(Bmy),nil);
   if (BMx < 1) or (BMy < 1) then exit;
   hh := 123;//handle;

   { TODO -oPPK -cRASTER : Testk vykreslen obrysu rastru obdlnkem ... OK }
   //************************
   //    Test only
   //************************
    (*
    Device.Pen.Color := clRed;
    Device.Pen.Mode  := pmCopy;
    Device.Brush.Style := bsClear;
    Device.Rectangle(RModel.Minx,RModel.Miny,RModel.Maxx,RModel.Maxy);
    *)
    //exit;

   //************************

   DevStartX := CDevice.Projection.GetPageX(Rmodel.Minx);
   DevStartY := CDevice.Projection.GetPageY(Rmodel.MaxY);
   // Korekce transformace do integer soustavy
   // ----------------------------------------
   while Device.Projection.GetModelX(DevStartX+bmX) < Rmodel.MaxX do Inc(bmX);
   while Device.Projection.GetModelY(DevStartY+bmY) > Rmodel.MinY do Inc(bmY);
   // Korekce END
   // --------------

    { TODO -oPPK -cRASTER : BM se zvtuje na nsobek 8, reln vez je teba taky nafouknout }
   a := BMx;
   if (Bmx mod 8) > 0 then a := BMx + 8 - (BMx mod 8);
   if a > Bmx then
   begin
      CX := CDevice.Projection.GetModelLengthHoriz(a);
      RModel.MaxX := RModel.MinX + CX;
   end;
   {
   if (Bmy mod 8) > 0 then b := BMy + 8 - (BMy mod 8);
   if b > Bmy then
   begin
      CY := CDevice.Projection.GetModelLengthVert(b);
      RModel.MaxY := RModel.MinY + CY;
   end;
   }
{
   RasStore.PresentationStoreConnect(hh,
      integer(nil), integer(nil), integer(nil), integer(nil),
      integer(nil), integer(nil), integer(nil),
      ViewTRF);
}
   RasStore.ConnectDataStore(hh, Ras.ViewEntry);
   Application.ProcessMessages;{!!!!!!!!!!!!!!!!}
   RasStore.GetRectForHandle(hh, VertMin, HorMin, DVert, DHor);
   (*
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' VertMin '+ FloatToStr(VertMin),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' HorMin  '+ FloatToStr(HorMin),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' DVert   '+ FloatToStr(DVert),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' DHor    '+ FloatToStr(DHor),nil);
   *)
   //Width:= (Width div 8) * 8;
   {
   ViewTrf.SetViewRect(VertMin, HorMin, Dvert, DHor,
                       hh, Width, Height,
                       0, 0, Canvas.Handle, true);
   }

   //******
   if not Ras.GeoRaster then
   begin        { TODO -oPPK -cRASTER : Laborovn s mtkem }
      //aScale := Device.Scale;
      //CX := RModel.MinX + (RModel.MaxX - RModel.MinX)/2;
      pcx := DHor/Ras.Width;
      pcy := DVert/Ras.Height;
      aScale := Device.Projection.Scale*pcx;
      Rras.Miny   := VertMin+(Rmodel.MinY-Ras.Miny)*pcy;
      Rras.Minx   := HorMin+(Rmodel.MinX-Ras.Minx)*pcx;
      Rras.Width  := (Rmodel.MaxX-Rmodel.MinX)*pcx;
      Rras.Height := (Rmodel.Maxy-Rmodel.Miny)*pcy;
   end
   else
   begin
      aScale := Device.Projection.Scale;
      Rras.Miny   := Rmodel.MinY;
      Rras.Minx   := Rmodel.MinX;
      Rras.Width  := Rmodel.MaxX-Rmodel.MinX;
      Rras.Height := Rmodel.Maxy-Rmodel.Miny;
   end;


   //************************
   //    Test only 2
   //************************
   {
    Device.Pen.Color := Ras.Color;
    Device.Pen.Mode  := pmCopy;
    Device.Brush.Style := bsClear;
    CDevice.Canvas.Rectangle(DevStartX,DevStartY,DevStartX+bmX,DevStartY+bmY);
    }
    (*
   {!!!!!}PostTextMsg(tm_AddToProtocol,S,nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' MinY   '+FloatToStr(Ras.MinY),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' MinX   '+FloatToStr(Ras.MinX),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' MaxY   '+FloatToStr(Ras.MaxY),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' MaxX   '+FloatToStr(Ras.MaxX),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' Height '+FloatToStr(Ras.Height),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,' Width  '+FloatToStr(Ras.Width),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,'***************',nil);
   *)
   (*
    Bitmap.Free;
    exit;
    *)
    (**)
   { TODO -oPPK -cRASTER : Test 2  Vykreslen obrysu dladice obdlnkem }


   BinaryRaster := Ras.RasTyp = 0;
   TilesCount := 1;
   TileY := BMy;
   dTileY := BMY;
   Bitmap.Width := 1;
   Bitmap.Height := 1;
   // binrn rastry jedou pes 8bitovou bitmapu, ostatn pes truecolor
   if BinaryRaster then
   begin
      Bitmap.Pixelformat := pf8Bit;
      aByte := a;
   end
   else
   begin
      Bitmap.Pixelformat := pf24Bit;
      aByte := a * 3;
   end;
   MemoryDC := CreateCompatibleDC(Bitmap.Canvas.Handle);
   // Kontrola memory
   BasicMem := 8000000;
   ME := TMemoryElement.Create;
   RasInfo := nil;
   if World <> nil then
      RasInfo := TStringListElement(World.SystemRegister.FindElementName('RASTER'));
   if RasInfo = nil then MA := BasicMem
   else
   begin
      if BinaryRaster then
      begin
         MA := RasInfo.GetIntegerByName('MEMORY_BW');
         if MA = 0 then MA := BasicMem;
      end
      else
      begin
         MA := RasInfo.GetIntegerByName('MEMORY_C');
         if MA = 0 then MA := BasicMem;
      end;
   end;
   if ME.AvailMemory < MA then MA := ME.AvailMemory;
   if MA < 2000000 then
   begin
      //PostTextMsg(tm_AddToProtocol,S+'... kritick stav pamti: '+IntToStr(MA),nil);
      //if ME.AvailPageFile > 2000000 then
      //begin
      //   MA := 2000000;
      //end
      //else
      begin
         PostTextMsg(tm_AddToProtocol,'... Zoufale kritick stav pamti ... uvolnte msto na swapovacm disku!',nil);
         MA := 0;
      end;
   end;
   ME.Free;
   if MA > 0 then
   begin
      while (aByte * TileY) > MA do
      //while TilesCount < 5 do
      begin
         TilesCount := TilesCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
   end;
   BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY);
   while BM = 0 do
   begin
      TilesCount := TilesCount * 2;
      TileY := (TileY div 2) + (TileY mod 2);
      BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY);
      {
      if BinaryRaster then BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY)
                      else BM := CreateCompatibleBitmap(CDevice.Canvas.Handle,a,TileY);
      }
   end;
   // prava potu dladic
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic: '+ IntToStr(TilesCount),nil);
   while not(((TileY * TilesCount) - TileY) < BMy) do Dec(TilesCount);
   dTileY := TileY;
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic: '+ IntToStr(TilesCount),nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' TileY        : '+ IntToStr(TileY),nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' BMY          : '+ IntToStr(BMy),nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' BM           : '+ IntToStr(BM),nil);

   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Velikost BM:   '+ IntToStr(TileY*a),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic: '+ IntToStr(TilesCount),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' BM    '+ IntToStr(BM),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic '+ IntToStr(TilesCount),nil);
   PostTextMsg(tm_ShowText,S + ' Poet dladic '+ IntToStr(TilesCount),nil);
   Application.ProcessMessages;
   SelectObject(MemoryDC,BM);
   Bitmap.Handle := BM;
   Bitmap.Canvas.Handle := MemoryDC;
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' Bitmap Y     : '+ IntToStr(Bitmap.Height),nil);
   Application.ProcessMessages;
   BottomY := TileY - 1;
   RtRas.Minx := RRas.Minx;
   RtRas.Miny := RRas.MinY + (Rras.Height/BMy)*(BMy-1-BottomY);
   RtRas.Width := RRas.Width;
   //RtRas.Height := RRas.Height/BMy * TileY;
   RtRas.Height := RtRas.Width * TileY/a;
   //RtRas.Height := RtRas.Height * 0.9;
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' bmx/bmy  '+ FloatToStr(a/TileY),nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,' W/H      '+ FloatToStr(RtRas.Width/RtRas.Height),nil);
   if BinaryRaster then
   begin
      //DeleteObject(Bitmap.ReleasePalette);
     //{!!!!!!!!!!!}ResizePalette(Bitmap.Palette,256);
     aCol := Ras.Color;
     //Bitmap.PaletteModified := false;
     //Ras.Color := clBlack;
   end;

   for i := 1 to TilesCount do
   begin
      PostTextMsg(tm_ShowText,S + ' interpretace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
      //PostTextMsg(tm_ShowText,S + ' pprava '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
      //Application.ProcessMessages;
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Transformace...',nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' MinY   '+FloatToStr(Rtras.MinY),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' MinX   '+FloatToStr(Rtras.MinX),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Height '+FloatToStr(Rtras.Height),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Width  '+FloatToStr(Rtras.Width),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' a      '+IntToStr(a),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' TileY  '+IntToStr(TileY),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Handle '+IntToStr(Bitmap.Canvas.Handle),nil);
      Application.ProcessMessages;
      try
      if TileY >1 then
      begin
         //CX := HorMin + DHor/2;
         //CY := VertMin + DVert/2;
         CX := RtRas.MinX;// + RtRas.Width/2;
         CY := RtRas.MinY;// + RtRas.Height/2;
         {$RANGECHECKS OFF}
         { TODO -oPPK -cRASTER : Poznatek ... rastrov knihovny totln serou na nastaven mtkem }
         //while true do
         begin
            if RtRas.Width >= RtRas.Height then
            begin
               ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height * 0.5, Rtras.width,
                                   hh, a, dTiley,
                                   0, 0, Bitmap.Canvas.Handle, true);
               RtRas.Height := ViewTrf.DVert;
            end
            else
            begin
               ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height, Rtras.width * 0.5,
                                   hh, a, dTiley,
                                   0, 0, Bitmap.Canvas.Handle, true);
               RtRas.Width := ViewTrf.DHor;
            end;
            ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, ViewTrf.DVert, Rtras.width,
                                hh, a, dTiley,
                                0, 0, Bitmap.Canvas.Handle, true);
            //Napasovn prvn dladice :-|
            if (i = 1) and Ras.GeoRaster then
            begin
               // Oprava sirky
               ViewTrf.TrfScrGeo(a-bmx,0,TY,TX);
               dTX := 0;
               if TX > Ras.MaxX then
               begin
                  dTX := TX - Ras.MaxX;
                  Rtras.Width := RtRas.Width - dTX;
                  ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height *0.5, Rtras.width,
                                      hh, a, dTiley,
                                      0, 0, Bitmap.Canvas.Handle, true);
                  RtRas.Height := ViewTrf.DVert;
                  ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height, Rtras.width,
                                      hh, a, dTiley,
                                      0, 0, Bitmap.Canvas.Handle, true);
               end;
               {
               ViewTrf.TrfScrGeo(0,0,TY,TX);
               ViewTrf.ChangeGeoToBasic(TY,TX);
               PostTextMsg(tm_AddToProtocol, S,nil);
               PostTextMsg(tm_AddToProtocol, ' X :'+FloatToStr(TX),nil);
               PostTextMsg(tm_AddToProtocol, ' Y :'+FloatToStr(TY),nil);
               ViewTrf.TrfScrGeo(a,dTileY,TY,TX);
               ViewTrf.ChangeGeoToBasic(TY,TX);
               PostTextMsg(tm_AddToProtocol, ' X1:'+FloatToStr(TX),nil);
               PostTextMsg(tm_AddToProtocol, ' Y1:'+FloatToStr(TY),nil);
               }
               // Oprava Vysky
               ViewTrf.TrfScrGeo(0,dTileY-1,TY,TX);
               dTY := 0;
               if TY > Ras.MaxY then
               begin
                  dTY := TY - Ras.MaxY;
                  ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height - dtY, Rtras.width*0.5,
                                      hh, a, dTiley,
                                      0, 0, Bitmap.Canvas.Handle, true);
                  RtRas.Width := ViewTrf.DHor;
                  ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, RtRas.Height - dtY, Rtras.width,
                                      hh, a, dTiley,
                                      0, 0, Bitmap.Canvas.Handle, true);
               end;
            end;
            if (i = TilesCount) and Ras.GeoRaster then
            begin
               if ViewTrf.VertMin < Ras.MinY then
               begin
                  RtRas.MinY := Ras.MinY;
                  ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, ViewTrf.DVert, Rtras.width,
                                      hh, a, dTiley,
                                      0, 0, Bitmap.Canvas.Handle, true);
               end;
            end;

            {
            ViewTrf.TrfGeoScr(Rtras.MinY, Rtras.MinX,k,l);
            if l <=0 then break;
            Rtras.Height := Rtras.Height + Ras.VertRes;
            }
         end;
         (*
        {!!!!!}PostTextMsg(tm_AddToProtocol,' R.H   '+FloatToStr(Rtras.Height),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' T.H   '+FloatToStr(ViewTrf.DVert),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' R.W   '+FloatToStr(Rtras.Width),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' T.W   '+FloatToStr(ViewTrf.DHor),nil);
        *)
         (*
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinY   '+FloatToStr(Rtras.MinY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinY__ '+IntToStr(k),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' TileY  '+IntToStr(TileY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinX   '+FloatToStr(Rtras.MinX),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinX___'+IntToStr(l),nil);
         *)

         if not BinaryRaster and (Device is TBitmapDevice) then
         begin
            //Bitmap.Canvas.Draw(-DevStartX+1,-DevStartY+1,TBitmapDevice(Device).Bitmap);
            Bitmap.Canvas.Draw(-DevStartX,-DevStartY,TBitmapDevice(Device).Bitmap);
         end
         else

         begin

            Bitmap.Canvas.Brush.Color := clWhite;
            //Bitmap.Canvas.Brush.Color := clBlack;
            Bitmap.Canvas.Brush.Style := bsSolid;
            Bitmap.Canvas.Pen.Style := psClear;
            Bitmap.Canvas.Rectangle(0,0,a+1,Tiley+1);
            //Bitmap.Canvas.Rectangle(0,0,a-1,Tiley-1);
         end;
         if BinaryRaster then Bitmap.Canvas.Pixels[0,0] := 0;   // cinknut rastru
         //PostTextMsg(tm_ShowText,S + ' interpretace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
        (*
        {!!!!!}PostTextMsg(tm_AddToProtocol,' CX ......'+FloatToStr(CX),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' CY ......'+FloatToStr(CY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' Scale ...'+FloatToStr(aScale),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+' T VertMin '+ FloatToStr(ViewTrf.VertMin),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+' T HorMin  '+ FloatToStr(ViewTrf.HorMin),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+' T DVert   '+ FloatToStr(ViewTrf.DVert),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+' T DHor    '+ FloatToStr(ViewTrf.DHor),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinY   '+FloatToStr(Rtras.MinY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' MinX   '+FloatToStr(Rtras.MinX),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' Height '+FloatToStr(Rtras.Height),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' Width  '+FloatToStr(Rtras.Width),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,' *****',nil);
        *)
         Application.ProcessMessages;
        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Display ...',nil);
         Bitmap.Canvas.Lock;
         //BM := Bitmap.Handle;
         RasStore.DisplayAllDataStores(hh,0,Bitmap.Canvas.Handle,integer(BM),false);
         Bitmap.Canvas.UnLock;
         //**************
         Application.ProcessMessages;

        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Display ...OK',nil);
         //PostTextMsg(tm_ShowText,S + ' finalizace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
         //Application.ProcessMessages;
        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Draw Transparent ...',nil);
         if BinaryRaster then
         begin
            Bitmap.Canvas.Pixels[0,0] := clWhite;   // cinknut rastru
            DrawTransparentBinaryBitmap(DevStartX+1,DevStartY{+1},BMx,Bitmap,aCol)
         end
         else
         begin
            if (CDevice is TPrintDevice) then
            begin
               //if (i > 10) and (i < 20) or (i > 50) then
               //   Bitmap.SaveToFile('e:\kos\0\aa'+IntToStr(i)+'.bmp')
               //else
                  CDevice.StretchDrawDevice(DevStartX,DevStartY,BMx,dTileY,0,0,BMx,dTileY,Bitmap);
                  Application.ProcessMessages;
            end
            else
            begin
               B1 := nil;
               if (GetDeviceCaps(CDevice.Canvas.Handle, BITSPIXEL) *
                   GetDeviceCaps(CDevice.Canvas.Handle, PLANES)) <= 8 then
               begin
                  B1 := Graphics.TBitmap.Create;
                  B1.Width := Bitmap.Width;
                  B1.Height := Bitmap.Height;
                  Application.ProcessMessages;
                  B1.Canvas.Draw(0,0,Bitmap);
                  CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                                       B1.Canvas,Classes.Rect(0,0,BMx,dTileY));
                  B1.Free;
               end
               else
                  {
                  CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                                       Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
                  }
                  // Test obrys Bitmapy
                  (*
                  Bitmap.Canvas.Pen.Color := clYellow;
                  Bitmap.Canvas.Pen.Mode  := pmCopy;
                  Bitmap.Canvas.Brush.Style := bsClear;
                  Bitmap.Canvas.Rectangle(0,0,bmX,dTileY);
                  *)
                  CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                                       Bitmap.Canvas,Classes.Rect(0,0,BMx,dTileY));
    //*************
    (*
    Device.Pen.Color := clLime;
    Device.Pen.Mode  := pmCopy;
    Device.Brush.Style := bsClear;
    CDevice.Canvas.Rectangle(DevStartX,DevStartY,DevStartX+bmX,DevStartY+dTileY);
    *)
   //{!!!!!}PostTextMsg(tm_AddToProtocol,'dTileY        : '+ IntToStr(dTileY),nil);
            end;
         end;
         //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Draw Transparent ...OK',nil);
         Application.ProcessMessages;
      end;
      except
         on Ex: Exception do
         begin
         PostTextMsg(tm_AddToProtocol,Ex.Message,nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+'... Chyba v transformaci nebo kde !!!!!!!',nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   Minx: '+FloatToStr(Rtras.Minx),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   Miny: '+FloatToStr(Rtras.MinY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   H   : '+FloatToStr(Rtras.Height),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   W   : '+FloatToStr(Rtras.Width),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   x   : '+IntToStr(a),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   y   : '+IntToStr(TileY),nil);
         end;
      end;
      if i < TilesCount then
      begin
         DevStartY := DevStartY + TileY;
         if BottomY >= Bmy then
         begin
            dTileY := dTileY-(Bottomy-(BmY-1));
         end;
         BottomY := BottomY + dTileY;
         //RtRas.Height := RRas.Height/BMy * dTileY;
         RtRas.Height := RtRas.Width * dTileY/a;
         RtRas.Miny := RRas.MinY + (Rras.Height/BMy)*(BMy-1-BottomY);
         //RtRas.Height := RtRas.Height * 0.9;
      end;
   end;
   //if BinaryRaster then Ras.Color := aCol;

   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Finish...',nil);
   RasStore.DisConnectDataStore(hh, Ras.ViewEntry);
   //RasStore.PresentationStoreDisConnect(hh);
   Bitmap.Free;
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Finish...OK',nil);
   PostTextMsg(tm_ShowText,S + ' Interpretace skonena ',nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,'******************************',nil);
   Application.ProcessMessages;
//     FSrvRas.CloseRas(false);
end;


(*
procedure TRasterInterpret.interpretGeoRaster(Element: TElement);
var
   RAS: TRaster;
   VertMin, HorMin, DVert, DHor: double;
   hh: integer;
   a: integer;
   aByte: integer;
   i: integer;
   BDC,BH: integer;
   MemoryDC,PomDC: HDC;
   PomBitmap,BM: HBitmap;
   Bitmap,B1: Graphics.TBitmap;
   CDevice: TCanvasDevice;
   Rmodel: TDoubleRect;                   //vez v modelovch a rastrovch souednicch
   Rras: TRasDoubleRect;
   Rtras: TRasDoubleRect;                 //Tile v ras coordinates
   BMx,BMy: integer;                      //rozmr vezu strnky (Bitmapy)
   TilesCount: integer;
   TileY, dTileY: integer;
   DevStartX, DevStartY: integer;
   BottomY: integer;
   aCol: integer;
   pcx,pcy: double;
   BinaryRaster: boolean;
   S: string;
   ME: TMemoryElement;
   MA: longWord;
   pt: TPoint;
   RasInfo: TStringListElement;
   BasicMem: integer;
begin
   CDevice := TCanvasDevice(Device);
   Ras := TRaster(Element);
   S := Ras.FileName;
   S := ExtractFileName(S);
   if not Ras.IsOverRegion(Rect.MinX,Rect.MinY,Rect.MaxX,Rect.MaxY)then exit;

   //*{!!!!!}PostTextMsg(tm_AddToProtocol,'****** Rastr '+ S,nil);

   Rmodel := Ras.InterSectByRect(Rect);
   Bitmap := Graphics.TBitmap.Create;
   BMx := CDevice.Projection.GetPageLengthHoriz1(Rmodel.MaxX-Rmodel.Minx);
   BMy := CDevice.Projection.GetPageLengthVert1(Rmodel.MaxY-Rmodel.MinY);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Bmx '+ IntToStr(Bmx),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Bmy '+ IntToStr(Bmy),nil);
   if (BMx < 1) or (BMy < 1) then exit;
   hh := 123;//handle;

{
   RasStore.PresentationStoreConnect(hh,
      integer(nil), integer(nil), integer(nil), integer(nil),
      integer(nil), integer(nil), integer(nil),
      ViewTRF);
}
   RasStore.ConnectDataStore(hh, Ras.ViewEntry);
   Application.ProcessMessages;{!!!!!!!!!!!!!!!!}
   RasStore.GetRectForHandle(hh, VertMin, HorMin, DVert, DHor);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' VertMin '+ FloatToStr(VertMin),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' HorMin  '+ FloatToStr(HorMin),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' DVert   '+ FloatToStr(DVert),nil);
   {!!!!!}PostTextMsg(tm_AddToProtocol,S+' DHor    '+ FloatToStr(DHor),nil);

   //Width:= (Width div 8) * 8;
   {
   ViewTrf.SetViewRect(VertMin, HorMin, Dvert, DHor,
                       hh, Width, Height,
                       0, 0, Canvas.Handle, true);
   }

   //******
   if not Ras.GeoRaster then
   begin
      pcx := DHor/Ras.Width;
      pcy := DVert/Ras.Height;
      Rras.Miny   := VertMin+(Rmodel.MinY-Ras.Miny)*pcy;
      Rras.Minx   := HorMin+(Rmodel.MinX-Ras.Minx)*pcx;
      Rras.Width  := (Rmodel.MaxX-Rmodel.MinX)*pcx;
      Rras.Height := (Rmodel.Maxy-Rmodel.Miny)*pcy;
   end
   else
   begin
      Rras.Miny   := Rmodel.MinY;
      Rras.Minx   := Rmodel.MinX;
      Rras.Width  := Rmodel.MaxX-Rmodel.MinX;
      Rras.Height := Rmodel.Maxy-Rmodel.Miny;
   end;
   //a := (BMx div 8) * 8;
   a := BMx;
   if (Bmx mod 8) > 0 then a := BMx + 8 - (BMx mod 8);
   DevStartX := CDevice.Projection.GetPageX(Rmodel.Minx);
   DevStartY := CDevice.Projection.GetPageY(Rmodel.MaxY);

   BinaryRaster := Ras.RasTyp = 0;
   TilesCount := 1;
   TileY := BMy;
   dTileY := BMY;
   Bitmap.Width := 1;
   Bitmap.Height := 1;
   // binrn rastry jedou pes 8bitovou bitmapu, ostatn pes truecolor
   if BinaryRaster then
   begin
      Bitmap.Pixelformat := pf8Bit;
      aByte := a;
   end
   else
   begin
      Bitmap.Pixelformat := pf24Bit;
      aByte := a * 3;
   end;
   MemoryDC := CreateCompatibleDC(Bitmap.Canvas.Handle);
   // Kontrola memory
   BasicMem := 8000000;
   ME := TMemoryElement.Create;
   RasInfo := nil;
   if World <> nil then
      RasInfo := TStringListElement(World.SystemRegister.FindElementName('RASTER'));
   if RasInfo = nil then MA := BasicMem
   else
   begin
      if BinaryRaster then
      begin
         MA := RasInfo.GetIntegerByName('MEMORY_BW');
         if MA = 0 then MA := BasicMem;
      end
      else
      begin
         MA := RasInfo.GetIntegerByName('MEMORY_C');
         if MA = 0 then MA := BasicMem;
      end;
   end;
   if ME.AvailMemory < MA then MA := ME.AvailMemory;
   if MA < 2000000 then
   begin
      PostTextMsg(tm_AddToProtocol,S+'... kritick stav pamti: '+IntToStr(MA),nil);
      if ME.AvailPageFile > 2000000 then
      begin
         MA := 2000000;
      end
      else
      begin
         PostTextMsg(tm_AddToProtocol,'... Zoufale kritick stav pamti ... uvolnte msto na swapovacm disku!',nil);
         MA := 0;
      end;
   end;
   ME.Free;
   if MA > 0 then
   begin
      while (aByte * TileY) > MA do
      //while TilesCount < 5 do
      begin
         TilesCount := TilesCount * 2;
         TileY := (TileY div 2) + (TileY mod 2);
      end;
   end;
   BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY);
   while BM = 0 do
   begin
      TilesCount := TilesCount * 2;
      TileY := (TileY div 2) + (TileY mod 2);
      BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY);
      {
      if BinaryRaster then BM := CreateCompatibleBitmap(Bitmap.Canvas.Handle,a,TileY)
                      else BM := CreateCompatibleBitmap(CDevice.Canvas.Handle,a,TileY);
      }
   end;
   // prava potu dladic
   while not(((TileY * TilesCount) - TileY) < BMy) do Dec(TilesCount);
   dTileY := TileY;
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Velikost BM:   '+ IntToStr(TileY*a),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic: '+ IntToStr(TilesCount),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' BM    '+ IntToStr(BM),nil);
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Poet dladic '+ IntToStr(TilesCount),nil);
   PostTextMsg(tm_ShowText,S + ' Poet dladic '+ IntToStr(TilesCount),nil);
   Application.ProcessMessages;
   SelectObject(MemoryDC,BM);
   Bitmap.Handle := BM;
   Bitmap.Canvas.Handle := MemoryDC;
   Application.ProcessMessages;
   BottomY := TileY - 1;
   RtRas.Minx := RRas.Minx;
   RtRas.Miny := RRas.MinY + (Rras.Height/BMy)*(BMy-1-BottomY);
   RtRas.Width := RRas.Width;
   RtRas.Height := RRas.Height/BMy * TileY;
   if BinaryRaster then
   begin
      //DeleteObject(Bitmap.ReleasePalette);
     //{!!!!!!!!!!!}ResizePalette(Bitmap.Palette,256);
     aCol := Ras.Color;
     //Bitmap.PaletteModified := false;
     //Ras.Color := clBlack;
   end;
   for i := 1 to TilesCount do
   begin
      PostTextMsg(tm_ShowText,S + ' interpretace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
      //PostTextMsg(tm_ShowText,S + ' pprava '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
      //Application.ProcessMessages;
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Transformace...',nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' MinY   '+FloatToStr(Rtras.MinY),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' MinX   '+FloatToStr(Rtras.MinX),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Height '+FloatToStr(Rtras.Height),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Width  '+FloatToStr(Rtras.Width),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' a      '+IntToStr(a),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' TileY  '+IntToStr(TileY),nil);
     //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Handle '+IntToStr(Bitmap.Canvas.Handle),nil);
      Application.ProcessMessages;
      try
      if TileY >1 then
      begin
         {$RANGECHECKS OFF}
         ViewTrf.SetViewRect(Rtras.MinY, Rtras.MinX, Rtras.Height, Rtras.width,
                              {VertMin, HorMin, DVert, DHor,}
                             hh, a, Tiley,
                             0, 0, Bitmap.Canvas.Handle, true);
        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Transformace...OK',nil);
         Bitmap.Canvas.Brush.Color := clWhite;
         Bitmap.Canvas.Brush.Style := bsSolid;
         Bitmap.Canvas.Pen.Style := psClear;
         Bitmap.Canvas.Rectangle(0,0,a+1,Tiley+1);
         if BinaryRaster then Bitmap.Canvas.Pixels[0,0] := 0;   // cinknut rastru
         //PostTextMsg(tm_ShowText,S + ' interpretace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
         Application.ProcessMessages;
        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Display ...',nil);
         Bitmap.Canvas.Lock;
         //BM := Bitmap.Handle;
         RasStore.DisplayAllDataStores(hh,0,Bitmap.Canvas.Handle,integer(BM),false);
         Bitmap.Canvas.UnLock;

        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Display ...OK',nil);
         //PostTextMsg(tm_ShowText,S + ' finalizace '+ IntToStr(i)+'. dladice z '+ IntToStr(TilesCount),nil);
         //Application.ProcessMessages;
        //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Draw Transparent ...',nil);
         if BinaryRaster then
            DrawTransparentBinaryBitmap(DevStartX+1,DevStartY+1,BMx,Bitmap,aCol)
         else
         begin
            if (CDevice is TPrintDevice) then
            begin
               CDevice.StretchDrawDevice(DevStartX,DevStartY,BMx,dTileY,0,0,BMx-1,dTileY-1,Bitmap);
            end
            else
            begin
               B1 := nil;
               if (GetDeviceCaps(CDevice.Canvas.Handle, BITSPIXEL) *
                   GetDeviceCaps(CDevice.Canvas.Handle, PLANES)) <= 8 then
               begin
                  B1 := Graphics.TBitmap.Create;
                  B1.Width := Bitmap.Width;
                  B1.Height := Bitmap.Height;
                  Application.ProcessMessages;
                  B1.Canvas.Draw(0,0,Bitmap);
                  CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                                       B1.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
                  B1.Free;
               end
               else
                  CDevice.Canvas.CopyRect(Classes.Rect(DevStartX,DevStartY,DevStartX+BMx,DevStartY+dTileY),
                                       Bitmap.Canvas,Classes.Rect(0,0,BMx-1,dTileY-1));
            end;
         end;
         //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Draw Transparent ...OK',nil);
         Application.ProcessMessages;
      end;
      except
         on Ex: Exception do
         begin
         PostTextMsg(tm_AddToProtocol,Ex.Message,nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,S+'... Chyba v transformaci nebo kde !!!!!!!',nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   Minx: '+FloatToStr(Rtras.Minx),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   Miny: '+FloatToStr(Rtras.MinY),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   H   : '+FloatToStr(Rtras.Height),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   W   : '+FloatToStr(Rtras.Width),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   x   : '+IntToStr(a),nil);
        {!!!!!}PostTextMsg(tm_AddToProtocol,'   y   : '+IntToStr(TileY),nil);
         end;
      end;
      if i < TilesCount then
      begin
         DevStartY := DevStartY + TileY;
         BottomY := BottomY + TileY;
         if BottomY >= Bmy then
         begin
            dTileY := dTileY-(Bottomy-(BmY-1));
         end;
         RtRas.Miny := RRas.MinY + (Rras.Height/BMy)*(BMy-1-BottomY);
         RtRas.Height := RRas.Height/BMy * TileY;
      end;
   end;
   //if BinaryRaster then Ras.Color := aCol;

   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Finish...',nil);
   RasStore.DisConnectDataStore(hh, Ras.ViewEntry);
   //RasStore.PresentationStoreDisConnect(hh);
   Bitmap.Free;
   //*{!!!!!}PostTextMsg(tm_AddToProtocol,' Finish...OK',nil);
   PostTextMsg(tm_ShowText,S + ' Interpretace skonena ',nil);
   //{!!!!!}PostTextMsg(tm_AddToProtocol,'******************************',nil);
   Application.ProcessMessages;
//     FSrvRas.CloseRas(false);
end;
*)


procedure TRasterInterpret.InterpretPicture(Element: TElement);
begin
end;

procedure TRasterInterpret.Process(Element: TElement);
begin
   if (Element is TRaster) then
   begin
       if not (Device is TCanvasDevice) then exit;
       InterpretGeoRaster(Element);
   end;
end;


procedure   TRasterInterpret.Reset(aDevice: TDevice);
begin
   inherited Reset(aDevice);
   Rect := Device.Projection.CLipModelView;
   ViewTRF.CoordOrientation:= 0;
   {
   ViewTRF.HorSize:= Device.Page.Width;
   ViewTRF.VertSize:= Device.Page.Height;
   ViewTRF.TileHorSize:= Device.Page.Width;
   ViewTRF.TileVertSize:= Device.Page.Height;
   }
   {
   ViewTrf.SetViewRectFromScale(Device.Projection.Scale,
           Device.Projection.Center_y,Device.Projection.Center_X,1,
   Device.Page.Width,Device.Page.Height,0,0,Device.Handle,true);
   }
end;



procedure TRasterInterpret.SetProtocol(const Value: string);
begin
   PostTextMsg(tm_AddToProtocol,Value,nil);
end;

INITIALIZATION
   RegisterOrbisClass(TRasterInterpret);
END.
