unit T_ChkSp;


interface
uses
  E_QTLin,
  E_QTree,
  //E_Space,
  e_Geometry,
  e_GeometryElement,
  //F_Info,
  e_Chk,
  T_Chk,
  T_PrpFlt,
  T_Recurs,
  Elements,
  Tools;


const
  ch_PolyAngle    = 5000; { + 1,2,3}
  ch_PolyDistance = 5010; { + 1,2,3}
  ch_PolyLineOverlay  = 5020; { + 1,2,3}
  ch_AreaPointInside = 6000;
  ch_AreaInFather = 6010; { +1 (Pater neex.), 2 (nejsem uvnitr) }
  ch_PointPointDistance = 6020;
  ch_PointLineDistance = 6020;


type



TCheckTextCodePage = class(TCheckTool)
  private
    FCodePage: integer;
    FErrElTXT: string;
    FAutoCodePage: boolean;
    FRealCPOnly: boolean;
    FMultiCp: boolean;
    FErrCodePage: integer;
    FIgnoreGeometry: boolean;
    procedure SetCodePage(const Value: integer);
    procedure SetErrElTXT(const Value: string);
    procedure SetAutoCodePage(const Value: boolean);
    procedure SetRealCPOnly(const Value: boolean);
    procedure SetMultiCp(const Value: boolean);
    procedure SetErrCodePage(const Value: integer);
    procedure SetIgnoreGeometry(const Value: boolean);
protected
   function ElementCondition: boolean; override;
   function GetErrorMsg: string; override;
   property ErrElTXT: string read FErrElTXT write SetErrElTXT;
published
   property CodePage: integer read FCodePage write SetCodePage;
   property ErrCodePage: integer read FErrCodePage write SetErrCodePage;
   Property MultiCp: boolean read FMultiCp write SetMultiCp;
   property AutoCodePage: boolean read FAutoCodePage write SetAutoCodePage;
   property RealCPOnly: boolean read FRealCPOnly write SetRealCPOnly;
   property IgnoreGeometry: boolean read FIgnoreGeometry write SetIgnoreGeometry;
end;

TCheckGeometryElementCodePage = class(TCheckTextCodePage)
  private
    FPropertyName: string;
    procedure SetPropertyName(const Value: string);
protected
   function ElementCondition: boolean; override;
published
   property PropertyName: string read FPropertyName write SetPropertyName;
end;

TCheckSpaceEntity = class(TCheckTool)
protected
  FX: double;
  FY: double;
  FState: integer;
  function CoordsInfo: string; virtual;
public
  procedure Reset; override;
  function CreateErrorElement: TErrorGeometryElement; override;
  function  CHeckErrorElement(const aErr: TErrorGeometryElement;var aSkipKnown: boolean): boolean; override;
published
  property State: integer read FState write FState;
  property X: double read FX write FX;
  property Y: double read FY write FY;
end;


TCheckSpaceEntitySingleValue = class(TCheckSpaceEntity)
private
  FMinVal, FMaxVal, FValue: double;
  FOutSide: boolean;
protected
  function ValueCondition: integer;
public
  function CreateErrorElement: TErrorGeometryElement; override;

published
  property OutSide: boolean read FOutSide write FOutSide;
  property MinValue: double read FMinVal write FMinVal;
  property MaxValue: double read FMaxVal write FMaxVal;
  property Value: double read FValue write FValue stored false;
end;



{ test hlu lomovch bod }
TCheckPolyAngle = class(TCheckSpaceEntitySingleValue)
protected
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
  function ElementCondition: boolean; override;
end;


TCheckPolyDistance = class(TCheckSpaceEntitySingleValue)
protected
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
  function ElementCondition: boolean; override;
end;



TCheckPolyLineOverlay = class(TCheckSpaceEntitySingleValue)
private
  FQTree: TQTreeTopLin;
  FTolerance: double;
  //FInfoForm: TF_InfoForm;

  procedure AddToQTree(aPoly: TPolyLineElement);
  function Coincidence(aPol: TPolyLineElement): integer;
protected
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
  function ElementCondition: boolean; override;
public
  constructor Create; override;
  destructor  Destroy; override;
published
  property QTree: TQTreeTopLin read FQTree;
  property Tolerance: double read FTolerance write FTolerance;
end;





TCheckAreaPointInside = class(TCheckSpaceEntity)
protected
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
  function ElementCondition: boolean; override;
end;





TCheckPointPointDistance = class(TCheckSpaceEntitySingleValue)
private
  FQTree: TRectEntityQTree;
  FTolerance: double;
  //FInfoForm: TF_InfoForm;

  procedure AddToQTree(aPnt: TGeometry);
  function Coincidence(aPnt: TGeometry): integer;
protected
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
  function ElementCondition: boolean; override;
public
  constructor Create; override;
  destructor  Destroy; override;
published
  property QTree: TRectEntityQTree read FQTree;
  property Tolerance: double read FTolerance write FTolerance;
end;


TCheckPointLineDistance = class(TCheckSpaceEntitySingleValue)
private
   FQTree: TQTreeTopLin;
   FTolerance: double;
   FInTolerance: boolean;
   //FInfoForm: TF_InfoForm;
   FFilter: TIntegerPropertyFilter;
    FLines: TEntity;
   procedure AddToQTree(aLine: TPolyLineElement);
   function Coincidence(aPnt: TPointElement): integer;
   procedure CreateLineQTree(aString: TGeometryElementCollection);
   function GetFilter: TIntegerPropertyFilter;
   function GetValuesString: string;
   function GetExclude: boolean;
   function GetPropertyName: string;
   procedure SetValuesString(aValue: string);
   procedure SetExclude(aValue: boolean);
   procedure SetPropertyName(aValue: string);
    procedure SetLines(const Value: TEntity);
protected
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function ElementCondition: boolean; override;
public
   constructor Create; override;
   destructor  Destroy; override;
   property Filter: TIntegerPropertyFilter read GetFilter;
published
   property QTree: TQTreeTopLin read FQTree;
   property Tolerance: double read FTolerance write FTolerance;
   property InTolerance: boolean read FInTolerance write FInTolerance;
   property Exclude: boolean read GetExclude write SetExclude;
   property PropertyName: string read GetPropertyName write SetPropertyName;
   property ValuesString: string read GetValuesString write SetValuesString;
   property Lines: TEntity read FLines write SetLines;
end;





implementation
uses
  {!!!!}Dialogs,
  cpConvrt,
  e_InterpretContainer,
  SysUtils, Forms,
  E_Class,
  e_TopolElements,
  MathLib,
  G_Elems,
  u_Symbol,
  T_LinJoi;

                         { TCheckSpaceEntity }

function TCheckSpaceEntity.CHeckErrorElement(const aErr: TErrorGeometryElement;var aSkipKnown: boolean): boolean;
begin
   result := (aErr.ErrorCode = ErrorCode) and (aErr.ErrX = X) and (aErr.ErrY = Y);
   if result then
   begin
      aSkipKnown := (KnownErrors <> nil) and aErr.IsMyOwner(KnownErrors);
      if not aSkipKnown then
         result := aErr.IsMyOwner(Errors);
   end;
end;

function TCheckSpaceEntity.CoordsInfo: string;
begin
  result:= '[' + FloatToStr(X) + ',' + FloatToStr(Y) + ']'
end;



function TCheckSpaceEntity.CreateErrorElement: TErrorGeometryElement;
begin
   result := inherited CreateErrorElement;
   result.ErrX := X;
   result.ErrY := Y;
end;

procedure TCheckSpaceEntity.Reset;
begin
  inherited Reset;
  FX:= 0;
  FY:= 0;
  State:= 0;
end;



               { TCheckSpaceEntitySingleValue }


function TCheckSpaceEntitySingleValue.CreateErrorElement: TErrorGeometryElement;
begin
   result := inherited CreateErrorElement;
   result.Name := GetPropertyStringByName('Value');
end;

function TCheckSpaceEntitySingleValue.ValueCondition: integer;
{ 0 - OK, 1 - pod, 2 - nad, 3 - uvnitr pro outside }
begin
  result:= 0;
  if not Outside then
  begin
     if (Value < MinValue) then
        result:= 1
     else
     if (Value > MaxValue) then
        result:= 2
  end
  else
    if (Value >= MinValue) and (Value <= MaxValue) then
       result:= 3;
end;




                 { TCheckPolyAngle }

function TCheckPolyAngle.GetError: longInt;
begin
  result:= ch_PolyAngle + State;
end;



function TCheckPolyAngle.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:= { '<' + FloatToStr(MinValue) + '; ' + FloatToStr(MaxValue) + '> ' + }
               'Hodnota:' + FloatToStr(Value) + ' sour:' + CoordsInfo;
end;





function TCheckPolyAngle.ElementCondition: boolean;
{ projde vechny hly a zjiuje, kter se vymykaj min maxu}
var
   PL: TPolyElement;
   G: TGeometry;
   GE: TGeometryElement;
   MA: TMultiAreaElement;
   Holes: TEntity;
   Are: TAreaElement;
   Crd0, Crd1, Crd2: T2DCoordinate;
   I,j: longInt;
   procedure processPoly;
   var
      i: integer;
      IsAre: boolean;
      cc: integer;
   begin
      cc := PL.CoordsCount;
      if cc < 3 then exit;
      IsAre := PL is TAreaElement;
      if IsAre then
         Inc(cc) // u plochy jedeme pres okraj
      else
         Dec(cc);// u linie do count -1
      Crd1:= PL.GetOrientedCoordElement(0, true);
      Crd2:= PL.GetOrientedCoordElement(1, true);
      for I:= 2 to cc do
      begin
        Crd0:= Crd1;
        Crd1:= Crd2;
        if IsAre then
        begin
           if i = cc - 1 then // pridavame nultou souradnici
           begin
              Crd2:= PL.GetOrientedCoordElement(0, true);
           end
           else
           if i = cc then // perhoupneme se pres okraj ... +1.sour.
           begin
              Crd2:= PL.GetOrientedCoordElement(1, true);
           end
           else
              Crd2:= PL.GetOrientedCoordElement(I, true);
        end
        else
           Crd2:= PL.GetOrientedCoordElement(I, true);
        {
        if i = cc then
        begin
           if not IsAre then
              break
           else
              Crd2:= PL.GetOrientedCoordElement(0, true);
        end
        else
           Crd2:= PL.GetOrientedCoordElement(I, true);
        }
        FValue:= CoordUhel2Usecek(Crd0, Crd1, Crd2);

        State:= ValueCondition;

        if State > 0 then
        begin
          X:= Crd1.X;
          Y:= Crd1.Y;
          result:= false;
          Error:= GetError;
          AddErrorElement;
          if not FullCheck then
             break;
        end
      end;

   end;
begin
   result:= true;
   if not (Input is TGeometryElement) then exit;
   GE := TGeometryElement(Input);
   if GE.NoGeometry or not (GE.Geometry is TPolyElement) then exit;
   G := GE.Geometry;
   Are := nil;
   if G is TMultiAreaElement then
   begin
      MA := TMultiAreaElement(G);
      for i := 0 to MA.LastIndex do
      begin
         Are := MA.AreaElement[i];
         PL := MA.AreaElement[i];
         ProcessPoly;
         if Are.Holes <> nil then
         begin
            Holes := Are.Holes;
            for j := 0 to Holes.LastIndex do
            begin
               PL := TAreaElement(Holes.Elements[j]);
               ProcessPoly;
            end;
         end;
      end;
   end
   else
   begin
      PL := TPolyElement(G);
      ProcessPoly;
      if not (G is TAreaElement) then exit;
      Are := TAreaElement(G);
      if Are.Holes <> nil then
      begin
         Holes := Are.Holes;
         for j := 0 to Holes.LastIndex do
         begin
            PL := TAreaElement(Holes.Elements[j]);
            ProcessPoly;
         end;
      end;
   end;
end;



                 { TCheckPolyDistance }

function TCheckPolyDistance.GetError: longInt;
begin
  result:= ch_PolyDistance + State;
end;



function TCheckPolyDistance.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:=  {'<' + FloatToStr(MinValue) + ';' + FloatToStr(MaxValue) + '> ' +}
               'Hodnota: ' + FloatToStr(Value) + ' sour:' + CoordsInfo;
end;


function TCheckPolyDistance.ElementCondition: boolean;
{ projde vechny hly a zjiuje, kter se vymykaj min maxu}
var
  PL: TPolyElement;
  G: TGeometryElement;
  I: longInt;
begin
  result:= true;
  if not (Input is TGeometryElement) then exit;
  G := TGeometryElement(Input);
  if G.NoGeometry or not (G.Geometry is TPolyElement) then exit;
  PL := TPolyElement(G.Geometry);
  //PL:= TPolyElement(Input);
  //if not (PL is TPolyElement) then exit;
  if PL.CoordsCount < 2 then exit;

  for I:= 0 to PL.CoordsCount-2 do
  begin
    FValue:= Distance(Pl.Xi[I], Pl.Yi[I], Pl.Xi[I+1], Pl.Yi[I+1]);

    State:= ValueCondition;

    if State > 0 then
    begin
      X:= Pl.Xi[I];
      Y:= Pl.Yi[I];
      result:= false;
      Error:= GetError;
      AddErrorElement;
      if not FullCheck then
         break;
    end
  end;

end;



                 { TCheckPolyLineOverlay }

function TCheckPolyLineOverlay.GetError: longInt;
begin
  result:= ch_PolyLineOverlay + State;
end;



function TCheckPolyLineOverlay.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:= {'<' + FloatToStr(Tolerance) + '>'}' sour:' + CoordsInfo;
end;




procedure TCheckPolyLineOverlay.AddToQTree(aPoly: TPolyLineElement);
begin
  if FQtree = nil then
     FQTree:= TQTreeTopLin.Create;

  if FQTree = nil then exit;

  FQTree.NextRectEntity:= aPoly
end;


function TCheckPolyLineOverlay.Coincidence(aPol: TPolyLineElement): integer;
var
  LC: TLinComparator;
  LAE: TLinesAbscEntity;
{ 0 - dobr vzdlenosti, 1 - nco je blzko, 2 - linie se stratila, 3 - jako 2, jene solo }
begin
  result:= 0;
  LC:= TLinComparator.Create;
  {
  LC.Protocol :='e:\orbis32\exe\protocol.txt';
  LC.UseProtocol := true;
  LC.ProtocoledLine := 1415;
  }
  LC.Input:= aPol;
  LC.SelfTest := true;
  LC.QTree:= FQTree;
  LC.SnapDistance:= FTolerance;
  {
  LC.Protocol:= 'e:\orbis\prot.ptc';
  LC.UseProtocol:= true;
  LC.ProtocoledLine:= 5;
  }
  LC.DoIt;
  LAE:= TLinesAbscEntity(LC.Output);
  LC.Free;

  if LAE.Count = 0 then
     result:= 3
  else
  if LAE.GetLine(0) = {New}aPol then
  begin
    if LAE.IsChanged then
       result:= 1;
  end
  else
    result:= 2
end;




function TCheckPolyLineOverlay.ElementCondition: boolean;
{ projde vechny hly a zjiuje, kter se vymykaj min maxu}
var
  PL, PL2: TPolyLineElement;
  I: longInt;
  G: TGeometryElement;
begin
  result:= true;
  Error:= 0;
  if not (Input is TGeometryElement) then exit;
  G := TGeometryElement(Input);
  if G.NoGeometry or not (G.Geometry is TPolyLineElement) then exit;
  PL:= TPolyLineElement(G.Geometry);
  //PL:= TPolyLineElement(Input);
  //if not (PL is TPolyLineElement) then exit;

{ 1. pro dan PL hledat v QTree vechny pekrvajc se elementy }
  //FInfoForm.InfoNo:= PL.SPaceId;
  (*
  {!!!!}if {(PL.SpaceID = 118) or }(PL.SpaceID = 1415) then
           ShowMessage('Now!');
  *)
  if FQTree <> nil then
  begin
    State:= Coincidence(PL);
    if State > 0 then
    begin
      X:= Pl.X;
      Y:= Pl.Y;
      Error:= GetError;
      result:= false;
    end
  end;
{  3. pidat PL do QTree }
  AddToQTree(PL);
end;



constructor TCheckPolyLineOverlay.Create;
begin
  inherited Create;
  //Application.CreateForm(TF_InfoForm, FInfoForm);
  //FInfoForm.Show
end;



destructor  TCheckPolyLineOverlay.Destroy;
begin
  //FInfoForm.Free;
  inherited Destroy
end;




                 { TCheckPointPointDistance }

function TCheckPointPointDistance.GetError: longInt;
begin
  result:= ch_PointPointDistance + State;
end;



function TCheckPointPointDistance.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:= {'<' + FloatToStr(Tolerance) + '>'} ' sour:' + CoordsInfo;
end;




procedure TCheckPointPointDistance.AddToQTree(aPnt: TGeometry);
begin
  if FQtree = nil then
     FQTree:= TRectEntityQTree.Create;

  if FQTree = nil then exit;

  FQTree.NextRectEntity:= aPnt
end;


function TCheckPointPointDistance.Coincidence(aPnt: TGeometry): integer;
var
  QTW: TQtreeWalker;
  aPnt2: TGeometry;
begin
  result:= 0;
  QTW:= TQTreeWalker.Create;

  aPnt2:= TGeometry(QTW.GetFirstElementForRectangle
                        (FQTree, aPnt.X, aPnt.Y, aPnt.X, aPnt.Y, Tolerance));

  while aPnt2 is TGeometry do
  begin
     FValue := Distance(aPnt.X, aPnt.Y, aPnt2.X, aPnt2.Y);
    if FValue < Tolerance then
    begin
      result:= 1;
      break;
    end;

    aPnt2:= TGeometry(QTW.GetNextElement)
  end;

  QTW.Free;
end;




function TCheckPointPointDistance.ElementCondition: boolean;
var
  aPnt: TGeometry;
  I: longInt;
  G: TGeometryElement;
  IsText: boolean;
begin
  result:= true;
  Error:= 0;
  if not (Input is TGeometryElement) then exit;
  G := TGeometryElement(Input);
  if G.NoGeometry then exit;
  aPnt:= G.Geometry;
  if not ((aPnt is TPointElement)or (aPnt is TTextElement)) then exit;
{ 1. pro dan aPnt hledat v QTree vechny pekrvajc se elementy }
  //FInfoForm.InfoNo:= aPnt.SPaceId;

  if FQTree <> nil then
  begin
    State:= Coincidence(aPnt);
    if State > 0 then
    begin
      X:= aPnt.X;
      Y:= aPnt.Y;
      Error:= GetError;
      AddErrorElement;
      result:= false;
    end
  end;
{  3. pidat PL do QTree }
  AddToQTree(aPnt);
end;




constructor TCheckPointPointDistance.Create;
begin
  inherited Create;
  //Application.CreateForm(TF_InfoForm, FInfoForm);
  //FInfoForm.Show
end;



destructor  TCheckPointPointDistance.Destroy;
begin
  //FInfoForm.Free;
  inherited Destroy
end;





                 { TCheckPointLineDistance }

function TCheckPointLineDistance.GetError: longInt;
begin
  result:= ch_PointLineDistance + State;
end;



function TCheckPointLineDistance.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:= {'<' + FloatToStr(Tolerance) + '>'} ' sour:' + CoordsInfo;
end;




procedure TCheckPointLineDistance.AddToQTree(aLine: TPolyLineElement);
begin
  if FQtree = nil then
     FQTree:= TQTreeTopLin.Create;

  if FQTree = nil then exit;

  FQTree.NextRectEntity:= aLine
end;




procedure TCheckPointLineDistance.CreateLineQTree(aString: TGeometryElementCollection);
var
  I: longInt;
  aLine: TPolyLineElement;
  TP: TTopolLineElement;
begin
  for I:= 0 to aString.Count-1 do
  begin
    TP:= TTopolLineElement(aString.Elements[I]);
    if not (TElement(TP) is TTopolLineElement) then
       continue;
    aLine := TPolyLineElement(TP.Geometry);
    if aLine = nil then continue;
    AddToQTree(aLine)
  end;
end;




function TCheckPointLineDistance.Coincidence(aPnt: TPointElement): integer;
var
  QTW: TQtreeWalker;
  aLine: TPolyLineElement;
begin
  result:= byte(InTolerance);
  QTW:= TQTreeWalker.Create;

  aLine:= TPolyLineElement(QTW.GetFirstElementForRectangle
                        (FQTree, aPnt.X, aPnt.Y, aPnt.X, aPnt.Y, Tolerance));

  while aLine <> nil do
  begin
    if aLine.IsHitByXY(aPnt.X,aPnt.Y, Tolerance) then
    begin
      result:= byte(not InTolerance);
      break;
    end;

    aLine:= TPolyLineElement(QTW.GetNextElement)
  end;

  QTW.Free;
end;




function TCheckPointLineDistance.ElementCondition: boolean;
var
  aPnt: TPointElement;
  TP: TTopolPointElement;
  i: longInt;
  Owners: TEntity;
  IL: TInterpretLayer;
  e: TElement;
begin
  result:= true;
  Error:= 0;
  if not (Input is TTopolPointElement) then exit;
  TP := TTopolPointElement(Input);
  aPnt:= TPointElement(TP.Geometry);
  if aPnt = nil then exit;
  if not (aPnt is TPointElement) then exit;
  if not Filter.Values.NoValue then
  begin
     Filter.Input := TP;
     Filter.DoIt;
     if not Filter.Result then exit;
  end;
{ 1. pro dan aPnt hledat v QTree vechny pekrvajc se elementy }
  //FInfoForm.InfoNo:= aPnt.SPaceId;


  if FQTree = nil then
  begin
     if Lines <> nil then
     begin
        CreateLineQTree(TGeometryElementCollection(Lines));
     end
     else
     begin
        Owners:= TP.FindOwnerClass(TPointCollection);
        if Owners = nil then exit;
        Owners:= Owners.FindOwnerClass(TInterpretLayer);
        if Owners = nil then exit;
        Owners:= Owners.FindOwnerClass(TRectEntity);
        if Owners = nil then exit;
        IL := nil;
        for i := 0 to Owners.LastIndex do
        begin
           e := Owners.Elements[i];
           if not (e is TInterpretLayer) then continue;
           IL := TInterpretLayer(e);
           if IL.LayerType='LIN' then break;
        end;
        if IL = nil then exit;
        if IL.NoElements then exit;
        for i := 0 to IL.LastIndex do
        begin
           e := IL.Elements[i];
           if not (e is TPolyLineCollection) then continue;
           Owners:= TEntity(e);
           CreateLineQTree(TGeometryElementCollection(Owners));
        end;
     end;
  end;

  if FQTree = nil then exit;


  State:= Coincidence(aPnt);
  if State > 0 then
  begin
    X:= aPnt.X;
    Y:= aPnt.Y;
    Error:= GetError;
    AddErrorElement;
    result:= false;
  end
{  3. pidat PL do QTree }
end;




constructor TCheckPointLineDistance.Create;
begin
  inherited Create;
  //Application.CreateForm(TF_InfoForm, FInfoForm);
  //FInfoForm.Show
end;



destructor  TCheckPointLineDistance.Destroy;
begin
  //FInfoForm.Free;
  inherited Destroy
end;


function TCheckPointLineDistance.GetFilter: TIntegerPropertyFilter;
begin
   if FFilter = nil then
   begin
      FFilter := TIntegerPropertyFilter.Create;
      AddElement(FFilter);
   end;
   result := FFilter;
end;

function TCheckPointLineDistance.GetValuesString: string;
begin
   result := Filter.ValuesString;
end;

function TCheckPointLineDistance.GetExclude: boolean;
begin
   result := Filter.Exclude;
end;

function TCheckPointLineDistance.GetPropertyName: string;
begin
   result := Filter.PropertyName;
end;

procedure TCheckPointLineDistance.SetValuesString(aValue: string);
begin
   Filter.ValuesString := aValue;
end;

procedure TCheckPointLineDistance.SetExclude(aValue: boolean);
begin
   Filter.Exclude := aValue;
end;

procedure TCheckPointLineDistance.SetPropertyName(aValue: string);
begin
   Filter.PropertyName := aValue;
end;








procedure TCheckPointLineDistance.SetLines(const Value: TEntity);
begin
  FLines := Value;
end;

{ TCheckAreaPointInside }

function TCheckAreaPointInside.GetError: longInt;
begin
  result:= ch_AreaPointInside + State;
end;



function TCheckAreaPointInside.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:=  ' X: ' + FloatToStr(FX) + ' Y: ' + FloatToStr(FY)
end;



function TCheckAreaPointInside.ElementCondition: boolean;
var
  PL: TAreaElement;
  G: TGeometryElement;
begin
  result:= true;
  State:= 0;
  G := TGeometryElement(Input);
  if G.NoGeometry or not (G.Geometry is TAreaElement) then exit;
  PL:= TAreaElement(G.Geometry);
  //if not (PL is TAreaElement) then exit;
  FX:= PL.X;
  FY:= PL.Y;
  result:= PL.XYIsInside(PL.X, PL.Y);

  if not result then
  begin
    State:= 1;
    Error:= GetError;
    AddErrorElement;
  end;
end;





{ TCheckTextCodePage }

function TCheckTextCodePage.ElementCondition: boolean;
var
   T: TTopolTextElement;
   TX: TTextElement;
   S: string;
   i: integer;
begin
   result := true;
   if not (Input is TTopolTextElement) then exit;
   T := TTopolTextElement(Input);
   if not IgnoreGeometry then
      if T.NoGeometry then exit;
   i := -1;
   case T.CodePage of
      cpIBM         : i := 0;
      cpLatin2      : i := 852;
      cpKamenickych : i := 1;
      cpWindows     : i := 1250;
   end;
   if AutoCodePage then
   begin
      if (CodePage >= 0) and (i <> CodePage) then
         MultiCP := true;
      CodePage := i;
   end
   else
   begin
      if i <> CodePage then
      begin
         MultiCP := true;
         ErrCodePage := i;
      end;
   end;
   TX := TTextElement(T.Geometry);
   case CodePage of
      1250: result := cp_IsWin(TX.Text);
      852 : result := cp_Is852(TX.Text);
      1   : result := cp_IsKam(TX.Text);
      0   : result := cp_IsIBM(TX.Text);
   end;
   if not Result then
   begin
      if not RealCPOnly then
      begin
         S := 'M bt ';
         case CodePage of
            1250: S := S + '[1250]';
            852 : S := S + '[852]';
            1   : S := S + '[Kam]';
            0   : S := S + '[IBM]';
         end;
         S := S +' je ';
      end
      else
         S := 'cp ';
      if cp_IsWin(TX.Text) then
         S := S+'[1250]';
      if cp_Is852(TX.Text) then
         S := S+'[852]';
      if cp_IsKam(TX.Text) then
         S := S+'[Kam]';
      ErrElTXT := S;
      AddErrorElement;
   end;
end;

function TCheckTextCodePage.GetErrorMsg: string;
begin
   result := ErrElTXT;
end;

procedure TCheckTextCodePage.SetAutoCodePage(const Value: boolean);
begin
  FAutoCodePage := Value;
  CodePage := -1;
  MultiCP := false;
end;

procedure TCheckTextCodePage.SetCodePage(const Value: integer);
begin
  FCodePage := Value;
end;

procedure TCheckTextCodePage.SetErrCodePage(const Value: integer);
begin
  FErrCodePage := Value;
end;

procedure TCheckTextCodePage.SetErrElTXT(const Value: string);
begin
  FErrElTXT := Value;
end;

procedure TCheckTextCodePage.SetIgnoreGeometry(const Value: boolean);
begin
  FIgnoreGeometry := Value;
end;

procedure TCheckTextCodePage.SetMultiCp(const Value: boolean);
begin
  FMultiCp := Value;
end;

procedure TCheckTextCodePage.SetRealCPOnly(const Value: boolean);
begin
  FRealCPOnly := Value;
end;

{ TCheckGeometryElementCodePage }

function TCheckGeometryElementCodePage.ElementCondition: boolean;
var
   GE: TGeometryElement;
   PS: string;
   S: string;
   i: integer;
begin
   result := true;
   if not (Input is TGeometryElement) then exit;
   GE := TGeometryElement(Input);
   if not IgnoreGeometry then
      if GE.NoGeometry then exit;
   if PropertyName = '' then exit;
   PS := GE.GetPropertyStringByName(PropertyName);
   if PS = '' then exit;
   case CodePage of
      1250: result := cp_IsWin(PS);
      852 : result := cp_Is852(PS);
      1   : result := cp_IsKam(PS);
      0   : result := cp_IsIBM(PS);
   end;
   if not Result then
   begin
      if not RealCPOnly then
      begin
         S := 'M bt ';
         case CodePage of
            1250: S := S + '[1250]';
            852 : S := S + '[852]';
            1   : S := S + '[Kam]';
            0   : S := S + '[IBM]';
         end;
         S := S +' je ';
      end
      else
         S := 'cp ';
      if cp_IsWin(PS) then
      begin
         ErrCodePage := 1250;
         S := S+'[1250]';
      end
      else
      if cp_Is852(PS) then
      begin
         ErrCodePage := 852;
         S := S+'[852]';
      end
      else
      if cp_IsKam(PS) then
      begin
         ErrCodePage := 1;
         S := S+'[Kam]';
      end;
      ErrElTXT := S + ': '+PS;
      AddErrorElement;
   end;
end;

procedure TCheckGeometryElementCodePage.SetPropertyName(const Value: string);
begin
  FPropertyName := Value;
end;


initialization
   RegisterOrbisClass(TCheckPolyAngle);
   RegisterOrbisClass(TCheckPolyDistance);
   RegisterOrbisClass(TCheckPolyLineOverlay);
   RegisterOrbisClass(TCheckAreaPointInside);
   RegisterOrbisClass(TCheckPointPointDistance);
   RegisterOrbisClass(TCheckPointLineDistance);
   RegisterOrbisClass(TCheckTextCodePage);
   RegisterOrbisClass(TCheckGeometryElementCodePage);
end.

