unit T_Chk;

interface
uses
  e_Chk,
  e_Type,
  I_TXTDev,
  t_PrpFlt,
  T_Recurs,
  Elements,
  Tools;

const
  ch_Props   = 1000;
  ch_PropsF  = 1100;
  ch_PropSum = 1110;


type

TCheckTool = class(TTool)
private
  FError: longInt;
  FErrorTxt: string;
  FErrorCount: longInt;
  FDontSendErrors: boolean;
  FCheckGeometry: boolean;
    FErrorCode: integer;
    FFullCheck: boolean;
    FCreateErrorElements: boolean;
    FKnownErrors: TEntity;
    FErrors: TErrorElementCollection;
    FSkippedErrors: TErrorElementCollection;
  procedure SetCheckGeometry(const Value: boolean);
    procedure SetErrorCode(const Value: integer);
    procedure SetFullCheck(const Value: boolean);
    procedure SetCreateErrorElements(const Value: boolean);
    procedure SetKnownErrors(const Value: TEntity);
    function GetErrors: TErrorElementCollection;
    function GetHasErrors: boolean;
    function GetHasSkippedErrors: boolean;
    function GetSkippedErrors: TErrorElementCollection;
protected
  function GetError: longInt; virtual;
  function GetErrorMsg: string; virtual;
  function GetResultMsg: string; virtual;
  function ElementCondition: boolean; virtual;
  function GetErrorMessage: string; virtual;
  function GetErrorTxt: string; virtual;
public
  procedure DoIt; override;
  procedure Reset; virtual;
  function CreateErrorElement: TErrorGeometryElement; virtual;
  procedure AddErrorElement;
  function  CHeckErrorElement(const aErr: TErrorGeometryElement;var aSkipKnown: boolean): boolean; virtual;
  property  ErrorCount: longInt read FErrorCount write FErrorCount;
published
  property ErrorTxt: string read GetErrorTxt write FErrorTxt;
  property Error: longInt read FError write FError;
  property ErrorMsg: string read GetErrorMsg;
  property ResultMsg: string read GetResultMsg;
  property DontSendErrors: boolean read FDontSendErrors write FDontSendErrors;
  property CheckGeometry: boolean read FCheckGeometry write SetCheckGeometry;
  property ErrorCode: integer read FErrorCode write SetErrorCode;
  property FullCheck: boolean read FFullCheck write SetFullCheck;
  property CreateErrorElements: boolean read FCreateErrorElements write SetCreateErrorElements;
  property KnownErrors: TEntity read FKnownErrors write SetKnownErrors stored false;
  property HasErrors: boolean read GetHasErrors stored false;
  property Errors: TErrorElementCollection read GetErrors stored false;
  property HasSkippedErrors: boolean read GetHasSkippedErrors stored false;
  property SkippedErrors: TErrorElementCollection read GetSkippedErrors stored false;
end;



TCheckPropsTool = class(TCheckTool)
private
  FTestCondition: string;
  FMsgProp: string;
protected
  function ElementCondition: boolean; override;
  function GetError: longInt; override;
  function GetErrorMsg: string; override;
published
  property TestCondition: string read FTestCondition write FTestCondition;
  property MsgProp: string read FMsgProp write FMsgProp;
end;

TCheckPropsByFilter = class(TCheckTool)
private
   FFilter: TIntegerPropertyFilter;
   FErrorValues: TIntegerListElement;
   function GetFilter: TIntegerPropertyFilter;
   function GetErrorValues: TIntegerListElement;
   function GetValuesString: string;
   function GetExclude: boolean;
   function GetPropertyName: string;
   procedure SetValuesString(aValue: string);
   procedure SetExclude(aValue: boolean);
   procedure SetPropertyName(aValue: string);
protected
   function ElementCondition: boolean; override;
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function GetResultMsg: string; override;
public
   property Filter: TIntegerPropertyFilter read GetFilter;
   property ErrorValues: TIntegerListElement read GetErrorValues;
published
   property Exclude: boolean read GetExclude write SetExclude;
   property PropertyName: string read GetPropertyName write SetPropertyName;
   property ValuesString: string read GetValuesString write SetValuesString;
end;

TCheckAndCreateErrorElements = class (TTechnology)
  private
    FKnownErrors: TEntity;
    FComment: string;
    procedure SetKnownErrors(const Value: TEntity);
    procedure SetComment(const Value: string);
public
   procedure DoIt; override;
published
   property  KnownErrors: TEntity read FKnownErrors write SetKnownErrors;
   property  Comment: string read FComment write SetComment;
end;

(*
TPropertySumTool = class(TCheckPropsByFilter)
private
   FSumPropertyName: string;
   FSum: extended;
protected
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function GetResultMsg: string; override;
public
   procedure Reset; override;
   procedure DoIt; override;
published
   property SumPropertyName: string read FSumPropertyName write FSumPropertyName;
   property Sum: extended read FSum write FSum;
end;
*)

implementation
uses
  Forms,
  Dialogs,
  SysUtils,
  E_Class,
  e_Geometry,
  e_GeometryElement,
  d_Txt,
  CreaCond,
  T_Oper,
  u_String;

{ TCheckTool }

procedure  TCheckTool.Reset;
begin
  FError:= 0;
end;



procedure TCheckTool.DoIt;
begin
  if ElementCondition then
     FError:= 0
  else
  begin
    FError:= GetError
  end;
end;

function TCheckTool.GetResultMsg: string;
begin
   result := '';
end;


procedure TCheckTool.SetCheckGeometry(const Value: boolean);
begin
  FCheckGeometry := Value;
end;

function TCheckTool.GetErrorTxt: string;
begin
   result := FErrorTxt;
end;

procedure TCheckTool.SetErrorCode(const Value: integer);
begin
  FErrorCode := Value;
end;

procedure TCheckTool.SetFullCheck(const Value: boolean);
begin
  FFullCheck := Value;
end;

procedure TCheckTool.SetCreateErrorElements(const Value: boolean);
begin
  FCreateErrorElements := Value;
end;

procedure TCheckTool.SetKnownErrors(const Value: TEntity);
begin
  FKnownErrors := Value;
end;


function TCheckTool.GetErrors: TErrorElementCollection;
begin
   if FErrors = nil then
   begin
      FErrors := TErrorElementCollection.Create;
      FErrors.Name := Name;
      AddElement(FErrors);
      Output := FErrors;
   end;
   result := FErrors;
end;

function TCheckTool.CreateErrorElement: TErrorGeometryElement;
begin
   result := TErrorGeometryElement.Create;
   result.ErrorCode := ErrorCode;
   result.Name := ErrorMsg;
end;

procedure TCheckTool.AddErrorElement;
var
   GE: TGeometryElement;
   G: TGeometry;
   i: integer;
   O: TEntity;
   EE: TErrorGeometryElement;
   DontCreate: boolean;
   aSkipKnown: boolean;
begin
   if not CreateErrorElements then exit;
   if Input = nil then exit;
   if not (Input is TGeometryElement) then exit;
   GE := TGeometryElement(Input);
   if GE.NoGeometry then
      G := nil
   else
      G := GE.Geometry;
   if G = nil then exit;
   DontCreate := false;
   //if KnownErrors <> nil then
   begin
      for i := 0 to G.OwnersCount - 1 do
      begin
         O := G.Owners[i];
         if not (O is TErrorGeometryElement) then continue;
         EE := TErrorGeometryElement(O);
         if CHeckErrorElement(EE,aSkipKnown) then
         begin
            if aSkipKnown then
               SkippedErrors.AddAndUpdateRect(EE);  // preskakujeme znamou chybu
            DontCreate := true;                     // a kazdopadne se nevytvari nova chyba
         end;
      end;
   end;
   if DontCreate then  // uz je v chybach nebo se preskakuje
      Exit;
   EE := CreateErrorElement;
   EE.Geometry := G;
   Errors.AddAndUpdateRect(EE);
end;

function TCheckTool.CHeckErrorElement(const aErr: TErrorGeometryElement;var aSkipKnown: boolean): boolean;
begin
   result := (aErr.ErrorCode = ErrorCode);
   if result then
   begin
      aSkipKnown := (KnownErrors <> nil) and aErr.IsMyOwner(KnownErrors);
      if not aSkipKnown then
         result := aErr.IsMyOwner(Errors);
   end;
end;

function TCheckTool.GetHasErrors: boolean;
begin
   result := (FErrors <> nil) and (FErrors.Count > 0);
end;

function TCheckTool.ElementCondition: boolean;
begin
   result := true;
end;

function TCheckTool.GetError: longInt;
begin
   result := ErrorCode;
end;

function TCheckTool.GetErrorMessage: string;
begin
   result := '';
end;

function TCheckTool.GetErrorMsg: string;
begin
   result := '';
end;

function TCheckTool.GetHasSkippedErrors: boolean;
begin
   result := (FSkippedErrors <> nil) and (FSkippedErrors.Count > 0);
end;

function TCheckTool.GetSkippedErrors: TErrorElementCollection;
begin
   if FSkippedErrors = nil then
   begin
      FSkippedErrors := TErrorElementCollection.Create;
      FSkippedErrors.Name := 'SKIP-'+Name;
      AddElement(FSkippedErrors);
   end;
   result := FSkippedErrors;
end;

{ TCheckPropsTool }

function TCheckPropsTool.GetError: longInt;
begin
  result:= ch_Props;
end;



function TCheckPropsTool.GetErrorMsg: string;
var
   S,S1: string;

function GetPropertySValue: string;
var
   E: TElement;
   GE: TGeometryElement;
begin
   E := Input;
   result := '?';
   if E.HasPropertyByName(S) then
      result := E.GetPropertyStringByName(S)
   else
   begin
      if E is TGeometryElement then
      begin
         GE := TGeometryElement(E);
         if not GE.NoGeometry then
            result := GE.Geometry.GetPropertyStringByName(S);
      end;
   end;
end;

begin
  result:= ': ';
  S := MsgProp;
  if Error > 0 then
  begin
     while StringAsTwoParams(S,',',S,S1)do
     begin
        result:= result + GetPropertySValue + ' * ';
        S := S1;
     end;
     result:= result + GetPropertySValue;
  end;
end;



function TCheckPropsTool.ElementCondition: boolean;
var
  CC: TCondCreator;
  aCondition: TCondition;
  El: TElement;
begin
  result:= true;
  if TestCondition = '' then exit;
  if Input = nil then exit;

  try
    CC := TCondCreator.Create;
    CC.Command:=TestCondition;
    CC.DoIt;
    aCondition := TCondition(CC.Output);
    CC.Free;

    if aCondition = nil then
       exit;
    El := Input;
    if CheckGeometry then
    begin
       if Input is TGeometryElement then
          El := TGeometryElement(Input).Geometry;
    end;
    aCondition.ParamItem[0]:= El;

    result:= aCondition.IsTrue;

    aCondition.Free;
    Error := 1;
    if not Result then
       AddErrorElement;
  except
    ShowMessage('Condition processing Error' + #13+#10+ TestCondition)
  end
end;


{ TCheckPropsByFilter }

function TCheckPropsByFilter.ElementCondition: boolean;
var
   El: TElement;
begin
   result := false;
   El := Input;
   if CheckGeometry then
   begin
      if Input is TGeometryElement then
         El := TGeometryElement(Input).Geometry;
   end;
   Filter.Input := El;
   Filter.DoIt;
   result := Filter.Result;
   if not result then
   begin
      ErrorValues.AddInteger(Filter.LastValue);
      AddErrorElement;
   end;
end;

function TCheckPropsByFilter.GetFilter: TIntegerPropertyFilter;
begin
   if FFilter = nil then
   begin
      FFilter := TIntegerPropertyFilter.Create;
      AddElement(FFilter);
   end;
   result := FFilter;
end;

function TCheckPropsByFilter.GetErrorValues: TIntegerListElement;
begin
   if FErrorValues = nil then
   begin
      FErrorValues := TIntegerListElement.Create;
      AddElement(FErrorValues);
   end;
   result := FErrorValues;
   result.Sorted := true;
   result.Duplicates := false;
end;

function TCheckPropsByFilter.GetValuesString: string;
begin
   result := Filter.ValuesString;
end;

function TCheckPropsByFilter.GetExclude: boolean;
begin
   result := Filter.Exclude;
end;

function TCheckPropsByFilter.GetPropertyName: string;
begin
   result := Filter.PropertyName;
end;

procedure TCheckPropsByFilter.SetValuesString(aValue: string);
begin
   Filter.ValuesString := aValue;
end;

procedure TCheckPropsByFilter.SetExclude(aValue: boolean);
begin
   Filter.Exclude := aValue;
end;

procedure TCheckPropsByFilter.SetPropertyName(aValue: string);
begin
   Filter.PropertyName := aValue;
end;

function TCheckPropsByFilter.GetError: longInt;
begin
  result:= ch_PropsF;
end;



function TCheckPropsByFilter.GetErrorMsg: string;
begin
  result:= '';
  if Error > 0 then
     result:= Input.GetPropertyStringByName(PropertyName);
end;

function TCheckPropsByFilter.GetResultMsg: string;
var
   i: longInt;
begin
   result := '';
   for i := 0 to ErrorValues.LastIndex do
   begin
      if i > 0 then result := result + ',';
      result := result + IntToStr(ErrorValues.Integers[i]);
   end;
end;


{ TPropertySumTool }

(*

procedure TPropertySumTool.DoIt;
begin

end;

function TPropertySumTool.GetError: longInt;
begin
end;

function TPropertySumTool.GetErrorMsg: string;
begin
end;

function TCheckTool.GetResultMsg: string;
begin
   result := '';
end;


procedure TPropertySumTool.Reset;
begin
end;

*)

{ TCheckAndCreateErrorElements }

procedure TCheckAndCreateErrorElements.DoIt;
var
   GC: TGeometryElementCollection;
   En: TEntity;
   E: TElement;
   GE: TGeometryElement;
   i: integer;
   it: integer;
   T: TTool;
   CHT: TCheckTool;
begin
   GC := TGeometryElementCollection.Create;
   GC.Name := Comment;
   Output := GC;
   AddElement(GC);
   if Input = nil then exit;
   if not (Input is TEntity) then exit;
   En := TEntity(Input);
   if En.NoElements then exit;
   if NoTools then exit;
   StartProgress(Comment,0,En.LastIndex);
   for it := 0 to Tools.Count - 1 do
   begin
      T := Tool[it];
      if not (T is TCheckTool) then continue;
      CHT := TCheckTool(T);
      CHT.FullCheck := true;
      CHT.CreateErrorElements := true;
      CHT.KnownErrors := KnownErrors;
   end;
   for i := 0 to En.LastIndex do
   begin
      Progress := i;
      Application.ProcessMessages;
      e := En.Elements[i];
      if not (e is TGeometryElement) then continue;
      for it := 0 to Tools.Count - 1 do
      begin
         T := Tool[it];
         T.Input := e;
         T.DoIt;
      end;
   end;
   for it := 0 to Tools.Count - 1 do
   begin
      T := Tool[it];
      if not (T is TCheckTool) then continue;
      CHT := TCheckTool(T);
      if not CHT.HasErrors then continue;
      GC.AddAndUpdateRect(CHT.Errors);
   end;
end;

procedure TCheckAndCreateErrorElements.SetComment(const Value: string);
begin
  FComment := Value;
end;

procedure TCheckAndCreateErrorElements.SetKnownErrors(const Value: TEntity);
begin
  FKnownErrors := Value;
end;

initialization
   RegisterOrbisClass(TCheckTool);
   RegisterOrbisClass(TCheckPropsTool);
   RegisterOrbisClass(TCheckPropsByFilter);
   RegisterOrbisClass(TCheckAndCreateErrorElements);
end.

