unit t_ChkDB;

interface
{$I ..\_conditions.inc}
uses
   DB,
   e_Type,
   t_Chk;

const
   ch_DB               = 7000;
   ch_DBFieldNotExists = 7001;
   ch_DBFieldTypeError = 7002;
   ch_DBFieldSizeError = 7003;
   ch_DBFieldDecError  = 7004;
   ch_DBFieldNameError = 7005;

type

TCheckDBFieldDef = class(TCheckTool)
  private
    FFieldType: string;
    FFieldSize: integer;
    FFieldDec: integer;
    FFieldName: string;
    FFieldSizeErrorText: string;
    FFieldTypeErrorText: string;
    FFieldNameErrorText: string;
    FFieldDecErrorText: string;
    FTableErrorText: string;
    FRealType: string;
    FRealSize: integer;
    FRealDec: integer;
    FSubErrorCount: integer;
    procedure SetFieldDec(const Value: integer);
    procedure SetFieldName(const Value: string);
    procedure SetFieldSize(const Value: integer);
    procedure SetFieldType(const Value: string);
    procedure SetFieldDecErrorText(const Value: string);
    procedure SetFieldNameErrorText(const Value: string);
    procedure SetFieldSizeErrorText(const Value: string);
    procedure SetFieldTypeErrorText(const Value: string);
    procedure SetTableErrorText(const Value: string);
    procedure SetRealDec(const Value: integer);
    procedure SetRealSize(const Value: integer);
    procedure SetRealType(const Value: string);
    procedure SetSubErrorCount(const Value: integer);
protected
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function ElementCondition: boolean; override;
   function GetErrorTxt: string; override;
public
   procedure Reset; override;
   property RealType: string read FRealType write SetRealType;
   property RealSize: integer read FRealSize write SetRealSize;
   property RealDec: integer read FRealDec write SetRealDec;
   property SubErrorCount: integer read FSubErrorCount write SetSubErrorCount;
published
   property FieldName: string read FFieldName write SetFieldName;
   property FieldType: string read FFieldType write SetFieldType;
   property FieldSize: integer read FFieldSize write SetFieldSize;
   property FieldDec: integer read FFieldDec write SetFieldDec;
   property TableErrorText: string read FTableErrorText write SetTableErrorText;
   property FieldNameErrorText: string read FFieldNameErrorText write SetFieldNameErrorText;
   property FieldTypeErrorText: string read FFieldTypeErrorText write SetFieldTypeErrorText;
   property FieldSizeErrorText: string read FFieldSizeErrorText write SetFieldSizeErrorText;
   property FieldDecErrorText: string read FFieldDecErrorText write SetFieldDecErrorText;
end;

TCheckDBFieldList = class(TCheckTool)
private
    FSubErrorCount: integer;
    FFieldName: string;
    FTableErrorText: string;
    FErrorText: string;
    FErrorNames: string;
    procedure SetFieldName(const Value: string);
    procedure SetSubErrorCount(const Value: integer);
    procedure SetTableErrorText(const Value: string);
    procedure SetErrorText(const Value: string);
    procedure SetNextErrorText(const Value: string);
    procedure SetErrorNames(const Value: string);
protected
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function ElementCondition: boolean; override;
   function GetResultMsg: string; override;
   function GetErrorTxt: string; override;
public
   procedure Reset; override;
   property ErrorText: string read FErrorText write SetErrorText;
   property NextErrorText: string write SetNextErrorText;
   property SubErrorCount: integer read FSubErrorCount write SetSubErrorCount;
   property ErrorNames: string read FErrorNames write SetErrorNames;
published
   property FieldNames: string read FFieldName write SetFieldName;
   property TableErrorText: string read FTableErrorText write SetTableErrorText;
end;

TCheckDBFields = class(TCheckTool)
  private
    FFieldName: string;
    FErrorText: string;
    FExclude: boolean;
    FControlFieldName: string;
    FControlFieldValue: string;
    FSubErrorCount: integer;
    procedure SetFieldName(const Value: string);
    procedure SetErrorText(const Value: string);
    procedure SetNextErrorText(const Value: string);
    procedure SetExclude(const Value: boolean);
    procedure SetControlFieldName(const Value: string);
    procedure SetControlFieldValue(const Value: string);
    procedure SetSubErrorCount(const Value: integer);
protected
   function GetError: longInt; override;
   function GetErrorMsg: string; override;
   function ElementCondition: boolean; override;
   function GetResultMsg: string; override;
   function GetErrorTxt: string; override;
public
   procedure Reset; override;
   procedure ProcessField(const aField: TField); virtual;
   function CheckField(const aField: TField): boolean; virtual;
   property ErrorText: string read FErrorText write SetErrorText;
   property NextErrorText: string write SetNextErrorText;
   property ControlFieldValue: string read FControlFieldValue write SetControlFieldValue;
   property SubErrorCount: integer read FSubErrorCount write SetSubErrorCount;
published
   property FieldName: string read FFieldName write SetFieldName;
   property ControlFieldName: string read FControlFieldName write SetControlFieldName;
   property Exclude: boolean read FExclude write SetExclude;
end;


TCheckDBFieldCodePage = class(TCheckDBFields)
  private
    FCodePage: integer;
    procedure SetCodePage(const Value: integer);
public
   function  CheckField(const aField: TField): boolean; override;
   procedure ProcessField(const aField: TField); override;
published
   property  CodePage: integer read FCodePage write SetCodePage;
end;


TCheckDBFieldValue = class(TCheckDBFields)
  private
    FFieldValue: string;
    FCaseSensitive: boolean;
    procedure SetFieldValue(const Value: string);
    procedure SetCaseSensitive(const Value: boolean);
public
   procedure ProcessField(const aField: TField); override;
published
   property  FieldValue: string read FFieldValue write SetFieldValue;
   property  CaseSensitive: boolean read FCaseSensitive write SetCaseSensitive;
end;

TCheckDBFieldPorostniSkupina = class(TCheckDBFields)
  private
    FCheckWhiteSpaces: boolean;
    procedure SetCheckWhiteSpaces(const Value: boolean);
public
   procedure ProcessField(const aField: TField); override;
published
   property CheckWhiteSpaces: boolean read FCheckWhiteSpaces write SetCheckWhiteSpaces;
end;

TCheckDBFieldValues = class(TCheckDBFields)
private
    FCaseSensitive: boolean;
    FFieldValues: string;
    FValues: TStringListElement;
    FCanBeEmpty: boolean;
    procedure SetCaseSensitive(const Value: boolean);
    procedure SetFieldValues(const Value: string);
    function GetValues: TStringListElement;
    procedure SetValues(const Value: TStringListElement);
    procedure SetCanBeEmpty(const Value: boolean);
public
   procedure ProcessField(const aField: TField); override;
   property  Values: TStringListElement read GetValues write SetValues;
published
   property  FieldValuesList: string read FFieldValues write SetFieldValues;
   property  CaseSensitive: boolean read FCaseSensitive write SetCaseSensitive;
   property  CanBeEmpty: boolean read FCanBeEmpty write SetCanBeEmpty;
end;

TCheckDBFieldValuesByTable = class(TCheckDBFieldValues)
  private
    FLookUpTableName: string;
    FLookUpFieldName: string;
    procedure SetLookUpTableName(const Value: string);
    procedure SetLookUpFieldName(const Value: string);
public
   function CheckField(const aField: TField): boolean; override;
published
   property LookUpTableName: string read FLookUpTableName write SetLookUpTableName;
   property LookUpFieldName: string read FLookUpFieldName write SetLookUpFieldName;
end;

TCheckDBFieldValueRange = class(TCheckDBFields)
private
    FMaxValue: double;
    FMinValue: double;
    FMaxValueString: string;
    FMinValueString: string;
    FChars: boolean;
    FMinChar: char;
    FMaxChar: char;
    procedure SetMaxValue(const Value: double);
    procedure SetMaxValueString(const Value: string);
    procedure SetMinValue(const Value: double);
    procedure SetMinValueString(const Value: string);
    procedure SetChars(const Value: boolean);
    procedure SetMaxChar(const Value: char);
    procedure SetMinChar(const Value: char);
public
   procedure ProcessField(const aField: TField); override;
   function  CheckField(const aField: TField): boolean; override;
   property MinValue: double read FMinValue write SetMinValue;
   property MaxValue: double read FMaxValue write SetMaxValue;
   property MinChar: char read FMinChar write SetMinChar;
   property MaxChar: char read FMaxChar write SetMaxChar;
published
   property MinValueString: string read FMinValueString write SetMinValueString;
   property MaxValueString: string read FMaxValueString write SetMaxValueString;
   property Chars: boolean read FChars write SetChars;
end;

implementation
uses
{$IFNDEF NOBDE}
   BDE,
{$ENDIF}
   dbf,
   dbf_Fields,
   Classes,
   DBTables,
   Forms,
   SysUtils,
   CpConvrt,
   Elements,
   e_Class,
   e_DBTable,
   u_String;


{ TCheckDBFieldDef }

function FieldTypeToCHar(aValue: TFieldType): Char;
begin
   case aValue of
      ftString	: result := 'C';
      ftSmallint,ftInteger,ftWord,ftFloat: result := 'N';
      ftBoolean	: result := 'L';
      ftDate,ftDateTime: result := 'D';
   else
      result := '?';
   end;
end;

{$IFNDEF NOBDE}
function GetFieldSizeAndDec(const aValue: TField; var aSize,aDec: integer): boolean;
var
   curProp: CURProps;
   pfldDes, pCurFld: pFLDDesc;
   // pfldDes is a pointer to a list of field descriptors.
   // It must be allocated with (iFields * sizeof(FLDDesc))
   // where iFields is a field in the structure curProps
   // from DbiGetCursorProps
   // pCurFld is a pointer the description of one field in the list.
   MemSize: Integer;
begin
   result := true;
   aSize := 0;
   aDec := 0;
   Check(DbiGetCursorProps(TTable(aValue.DataSet).Handle, curProp));

   // Get enough memory for one field desc times the # of fields
   MemSize := curProp.iFields * SizeOf(FLDDesc);
   pfldDes := AllocMem(MemSize);
   try
     pCurFld := pfldDes;
     Check(DbiGetFieldDescs(TTable(aValue.DataSet).Handle, pfldDes));
     inc(pCurFld,aValue.FieldNo-1);
     aSize := pCurFld^.iUnits1;
     aDec  := pCurFld^.iUnits2;
   finally
     //ShowMessage(FieldList);
     FreeMem(pfldDes, MemSize);
   end;
end;

{$ELSE}
function GetFieldSizeAndDec(const aValue: TField; var aSize,aDec: integer): boolean;
var
   FD: TdbfFieldDef;
   adbf: TDbf;
   i: integer;
begin
   result := true;
   aSize := 0;
   aDec := 0;
   adbf := TDbf(aValue.Dataset);
   i := adbf.Fields.IndexOf(aValue);
   FD :=TDbf(aValue.Dataset).DbfFieldDefs.Items[i];
   if FD = nil then
   begin
      result := false;
      exit;
   end;
   aSize := FD.Size;
   aDec  := FD.Precision;
end;

{$ENDIF}

function TCheckDBFieldDef.ElementCondition: boolean;
var
//   T: TTable;
   T: TDataset;
   F: TField;
begin
   result := true;
   if not (Input is TDBTableElement) then exit;
   T := TDBTableElement(Input).Tab;
   if not T.Active then
      T.Active := true;
   if not T.Active then
   begin
      Error := ch_DB;
      result := false;
      exit;
   end;
   T.First;
   F := T.FindField(FieldName);
   if F = nil then
   begin
      Error := ch_DBFieldNotExists;
      result := false;
      exit;
   end;
   RealType := FieldTypeToChar(F.DataType);
   GetFieldSizeAndDec(F,FRealSize,FRealDec);
   if RealType <> FieldType then
   begin
      Error := ch_DBFieldTypeError;
      result := false;
      exit;
   end;
   if RealSize <> FieldSize then
   begin
      Error := ch_DBFieldSizeError;
      result := false;
      exit;
   end;
   if RealDec <> FieldDec then
   begin
      Error := ch_DBFieldDecError;
      result := false;
   end;
   exit;
end;

function TCheckDBFieldDef.GetError: longInt;
begin
   result := Error;
end;

function TCheckDBFieldDef.GetErrorMsg: string;
var
   S: string;
begin
   result := 'Neznm chyba';
   S := RealType + ' ' + IntToStr(RealSize)+' '+IntToStr(RealDec)+' ('+
        FieldType + ' ' + IntToStr(FieldSize)+' '+IntToStr(FieldDec)+')';
   case Error of
      ch_DB              : result := TableErrorText;
      ch_DBFieldNotExists: result := FieldName+'  '+FieldNameErrorText;
      ch_DBFieldTypeError: result := FieldName+'  '+FieldTypeErrorText + #13+#10+'  '+S;
      ch_DBFieldSizeError: result := FieldName+'  '+FieldSizeErrorText + #13+#10+'  '+S;
      ch_DBFieldDecError : result := FieldName+'  '+FieldDecErrorText + #13+#10+'  '+S;
   end;
end;

function TCheckDBFieldDef.GetErrorTxt: string;
begin
   result := inherited GetErrorTxt{+'  '+FieldName};
end;

procedure TCheckDBFieldDef.Reset;
begin
   inherited;
   //SubErrorCount := 0;
end;

procedure TCheckDBFieldDef.SetFieldDec(const Value: integer);
begin
  FFieldDec := Value;
end;

procedure TCheckDBFieldDef.SetFieldDecErrorText(const Value: string);
begin
  FFieldDecErrorText := Value;
end;

procedure TCheckDBFieldDef.SetFieldName(const Value: string);
begin
  FFieldName := Value;
end;

procedure TCheckDBFieldDef.SetFieldNameErrorText(const Value: string);
begin
  FFieldNameErrorText := Value;
end;

procedure TCheckDBFieldDef.SetFieldSize(const Value: integer);
begin
  FFieldSize := Value;
end;

procedure TCheckDBFieldDef.SetFieldSizeErrorText(const Value: string);
begin
  FFieldSizeErrorText := Value;
end;

procedure TCheckDBFieldDef.SetFieldType(const Value: string);
begin
  FFieldType := Value;
end;

procedure TCheckDBFieldDef.SetFieldTypeErrorText(const Value: string);
begin
  FFieldTypeErrorText := Value;
end;

procedure TCheckDBFieldDef.SetRealDec(const Value: integer);
begin
  FRealDec := Value;
end;

procedure TCheckDBFieldDef.SetRealSize(const Value: integer);
begin
  FRealSize := Value;
end;

procedure TCheckDBFieldDef.SetRealType(const Value: string);
begin
  FRealType := Value;
end;

procedure TCheckDBFieldDef.SetSubErrorCount(const Value: integer);
begin
  FSubErrorCount := Value;
end;

procedure TCheckDBFieldDef.SetTableErrorText(const Value: string);
begin
  FTableErrorText := Value;
end;

{ TCheckDBFields }

function TCheckDBFields.CheckField(const aField: TField): boolean;
begin
   result := true;
   if aField = nil then
   begin
      Error := 1;
      NextErrorText := '   !!! Poloka '+FieldName+' neexistuje!';
      result := false;
   end;
end;

function TCheckDBFields.ElementCondition: boolean;
var
//   T: TTable;
   T: TDataset;
   F: TField;
begin
   T := TDBTableElement(Input).Tab;
   if not T.Active then
      T.Active := true;
   if not T.Active then
   begin
      Error := ch_DB;
      ErrorText := '   Ble neuren chyba DB tabulky';
      result := false;
      exit;
   end;
   T.First;
   ErrorText := '';
   Error := 0;
   F := T.FindField(FieldName);
   if not CheckField(F) then
   begin
      result := false;
      exit;
   end;
   While not T.Eof do
   begin
      ControlFieldValue := '?';
      if ControlFieldName = '' then
      begin
         ControlFieldValue := IntToStr(T.RecNo);
      end
      else
      begin
         F := T.FindField(ControlFieldName);
         if F <> nil then
            ControlFieldValue := F.AsString;
      end;
      F := T.FindField(FieldName);
      ProcessField(F);
      T.Next;
   end;
   result := Error = 0;
end;

function TCheckDBFields.GetError: longInt;
begin
   result := error;
end;

function TCheckDBFields.GetErrorMsg: string;
begin
   result := ErrorText;
end;

function TCheckDBFields.GetErrorTxt: string;
begin
   result := inherited GetErrorTxt+'  '+FieldName;
end;

function TCheckDBFields.GetResultMsg: string;
begin
   result := ' ('+IntToStr(SubErrorCount)+' zznam)';
end;

procedure TCheckDBFields.ProcessField(const aField: TField);
begin
   Error := 1;
   NextErrorText := aField.AsString;
   SubErrorCount := SubErrorCount+1;
end;

procedure TCheckDBFields.Reset;
begin
   inherited;
   //SubErrorCount := 0;
end;

procedure TCheckDBFields.SetControlFieldName(const Value: string);
begin
  FControlFieldName := Value;
end;

procedure TCheckDBFields.SetControlFieldValue(const Value: string);
begin
  FControlFieldValue := Value;
end;

procedure TCheckDBFields.SetErrorText(const Value: string);
begin
  FErrorText := Value;
end;

procedure TCheckDBFields.SetExclude(const Value: boolean);
begin
  FExclude := Value;
end;

procedure TCheckDBFields.SetFieldName(const Value: string);
begin
  FFieldName := Value;
end;

procedure TCheckDBFields.SetNextErrorText(const Value: string);
begin
   FErrorText := FErrortext + #13+#10+Value;
end;

procedure TCheckDBFields.SetSubErrorCount(const Value: integer);
begin
  FSubErrorCount := Value;
end;

{ TCheckDBFieldValue }

procedure TCheckDBFieldValue.ProcessField(const aField: TField);
var
   S: string;
begin
   S := aField.AsString;
   if (CaseSensitive and (AnsiCompareStr(S,FieldValue) <> 0)) or
      (not CaseSensitive and (AnsiCompareText(S,FieldValue) <> 0)) then
   begin
      NextErrorText := '  '+ControlFieldValue+': '+S;
      Error := 1;
      SubErrorCount := SubErrorCount+1;
   end;
end;

procedure TCheckDBFieldValue.SetCaseSensitive(const Value: boolean);
begin
   FCaseSensitive := Value;
end;

procedure TCheckDBFieldValue.SetFieldValue(const Value: string);
begin
   FFieldValue := Value;
end;

{ TCheckDBFieldValues }

function TCheckDBFieldValues.GetValues: TStringListElement;
begin
   if FValues = nil then
   begin
      FValues := TStringListElement.Create;
      AddElement(FValues);
   end;
   result := FValues;
end;

procedure TCheckDBFieldValues.ProcessField(const aField: TField);
var
   S,SS: string;
   SE: TStringElement;
   SF: TStringField;
   procedure Err;
   begin
      NextErrorText := '  '+ControlFieldValue+': '+S;
      Error := 1;
      SubErrorCount := SubErrorCount+1;
   end;
begin
   if (aField is TStringField) then
   begin
      SF := TStringField(aField);
      SF.Transliterate := false;
      S := SF.Value;
   end
   else
      S := aField.AsString;
   if (S = '') and CanBeEmpty then exit;
   SE := Values.GetStringElementByName(S);
   if SE = nil then
   begin
      if not Exclude then
         Err;
      exit;
   end;
   SS := SE.Name;
   if (CaseSensitive and (AnsiCompareStr(S,SS) <> 0)) then
   begin
      if not Exclude then
         Err;
   end
   else
   if Exclude then
      Err;
end;

procedure TCheckDBFieldValues.SetCanBeEmpty(const Value: boolean);
begin
  FCanBeEmpty := Value;
end;

procedure TCheckDBFieldValues.SetCaseSensitive(const Value: boolean);
begin
   FCaseSensitive := Value;
end;

procedure TCheckDBFieldValues.SetFieldValues(const Value: string);
var
   L: TStringList;
begin
   FFieldValues := Value;
   Values.FreeAll;
   L := TStringList.Create;
   L.CommaText := Value;
   Values.AddStringList(L);
   L.Free;
end;

procedure TCheckDBFieldValues.SetValues(const Value: TStringListElement);
begin
   ExchangeFieldElement(TElement(FValues),Value);
end;

{ TCheckDBFieldValueRange }

function TCheckDBFieldValueRange.CheckField(const aField: TField): boolean;
var
   aType: string;
   aSize,aDec: integer;
   S,S1: string;
begin
   result := inherited CheckField(aField);
   S := '   !!! Chybn nastaven kontroly';
   S1 := '   !!! Nesoulad nastaven kontroly a formtu poloky';
   if not result then exit;
   aType := FieldTypeToChar(aField.DataType);
   GetFieldSizeAndDec(aField,aSize,aDec);
   if Chars then
   begin
      if not((aType = 'C')and(aSize=1)) then
      begin
         result := false;
         Error := 1;
         NextErrorText := S1;
         NextErrorText := '   !!! Poloka '+FieldName+' nem formt C1, nelze kontrolovat znakov interval';
         exit;
      end;
      if (Length(MinValueString) <> 1) or (Length(MaxValueString)<>1) then
      begin
         result := false;
         Error := 1;
         NextErrorText := S;
         NextErrorText := '   !!! Minimln("'+MinValueString+'") nebo maximln("'+MaxValueString+'") hodnota kontroly nen znak';
         exit;
      end;
      MinChar := MinValueString[1];
      MaxChar := MaxValueString[1];
   end
   else
   begin
      if not(aType = 'N') then
      begin
         result := false;
         Error := 1;
         NextErrorText := S1;
         NextErrorText := '   !!! Poloka '+FieldName+' nem seln formt(N), nelze kontrolovat rozsah';
         exit;
      end;
      if not StringIsFloat(MinValueString) or not StringIsFloat(MaxValueString) then
      begin
         result := false;
         Error := 1;
         NextErrorText := S;
         NextErrorText := '   !!! Minimln("'+MinValueString+'") nebo maximln("'+MaxValueString+'") hodnota kontroly nen slo';
         exit;
      end;
      MinValue := StrToExtended(MinValueString);
      MaxValue := StrToExtended(MaxValueString);
   end;
end;

procedure TCheckDBFieldValueRange.ProcessField(const aField: TField);
var
   S: string;
   CH: char;
   D: double;
   procedure Err;
   begin
      NextErrorText := '  '+ControlFieldValue+': '+S;
      Error := 1;
      SubErrorCount := SubErrorCount+1;
   end;
begin
   S := aField.AsString;
   if Chars then
   begin
      if S = '' then
      begin
         Err;
         exit;
      end;
      CH := S[1];
      if Exclude then
      begin
         if ((CH >= MinChar) and (CH <= MaxChar)) then
            Err;
      end
      else
      begin
         if not ((CH >= MinChar) and (CH <= MaxChar)) then
            Err;
      end;
   end
   else
   begin
      D := aField.AsFloat;
      if Exclude then
      begin
         if ((D >= MinValue) and (D <= MaxValue)) then
            Err;
      end
      else
      begin
         if not ((D >= MinValue) and (D <= MaxValue)) then
            Err;
      end;
   end;
end;

procedure TCheckDBFieldValueRange.SetChars(const Value: boolean);
begin
  FChars := Value;
end;

procedure TCheckDBFieldValueRange.SetMaxChar(const Value: char);
begin
  FMaxChar := Value;
end;

procedure TCheckDBFieldValueRange.SetMaxValue(const Value: double);
begin
  FMaxValue := Value;
end;

procedure TCheckDBFieldValueRange.SetMaxValueString(const Value: string);
begin
  FMaxValueString := Value;
end;

procedure TCheckDBFieldValueRange.SetMinChar(const Value: char);
begin
  FMinChar := Value;
end;

procedure TCheckDBFieldValueRange.SetMinValue(const Value: double);
begin
  FMinValue := Value;
end;

procedure TCheckDBFieldValueRange.SetMinValueString(const Value: string);
begin
  FMinValueString := Value;
end;

{ TCheckDBFieldValuesByTable }

function TCheckDBFieldValuesByTable.CheckField(const aField: TField): boolean;
var
   DBE: TDbTableElement;
   LT: TDataset;
   F: TField;
   SF: TStringField;
   S,S1,SS: string;
   FF: boolean;
begin
   //result := true;
   S1 := '   !!! Chyba selnku';
   if LookUpTableName = '' then
   begin
      result := false;
      Error := 1;
      NextErrorText := S1;
      NextErrorText := '   !!! Nen definovna tabulka selnku ';
      exit;
   end;
   if LookUpFieldName = '' then
   begin
      result := false;
      Error := 1;
      NextErrorText := S1;
      NextErrorText := '   !!! Nen definovna poloka selnku ';
      exit;
   end;
   S := GetAbsolutePath(LookUpTableName);
   if not FileExists(S) then
   begin
      result := false;
      Error := 1;
      NextErrorText := S1;
      NextErrorText := '   !!! selnk '+S+' neexistuje(asi) ';
      exit;
   end;
   DBE := TDbTableElement.Create;
   DBE.FileName := S;
   DBE.Active := true;
   LT := DBE.Tab;
//   LT.TableName := S;
//   LT.Active := true;
   LT.First;
   F := LT.FindField(LookUpFieldName);
   if F = nil then
   begin
      result := false;
      Error := 1;
      NextErrorText := S1;
      NextErrorText := '   !!! selnk '+S;
      NextErrorText := '   !!! poloka '+LookUpFieldName+' neexistuje';
      LT.Active := false;
      LT.Free;
      exit;
   end;
   FF := true;
   S := '';
   While not LT.Eof do
   begin
      F := LT.FindField(LookUpFieldName);
      if not FF then
         S := S + ','
      else
         S := '';
      FF := false;
      if (F is TStringField) then
      begin
         SF := TStringField(F);
         SF.Transliterate := false;
         SS := SF.Value;
      end
      else
         SS := F.AsString;
      if SS = '' then CanBeEmpty := true;
      SS := '"'+SS+'"';
      S := S+SS;
      LT.Next;
   end;
//   LT.Active := false;
//   LT.Free;
   DBE.Free;
   FieldValuesList := S;
   result := inherited CheckField(aField);
end;

procedure TCheckDBFieldValuesByTable.SetLookUpFieldName(const Value: string);
begin
  FLookUpFieldName := Value;
end;

procedure TCheckDBFieldValuesByTable.SetLookUpTableName(const Value: string);
begin
  FLookUpTableName := Value;
end;

{ TCheckDBFieldPorostniSkupina }

procedure TCheckDBFieldPorostniSkupina.ProcessField(const aField: TField);
type
   TEtazRec =  record
      VS: integer;
      bla: string
   end;
var
   S,S1,ErrS,VSS,blaS: string;
   Etaz: Array [0..3]of TEtazRec;
   i,LastEt: integer;
   Delimiter: char;
   Position: integer;
   Numbers: boolean;
   WSFixed: boolean;
   procedure Err;
   begin
      NextErrorText := '  '+ControlFieldValue+': '+S+': '+ErrS;
      Error := 1;
      SubErrorCount := SubErrorCount+1;
   end;
begin
   S := aField.AsString;
   LastEt := -1;
   Delimiter := '/';
   Position := 1;
   ErrS := '';
   WSFixed := false;
   while Delimiter <> #0 do
   begin
      Inc(LastEt);
      Etaz[LastEt].VS := 0;
      Etaz[LastEt].bla := '';
      S1 := ReadToDelimiter(S,['/'],Position,Delimiter,Position);
      Numbers := true;
      VSS := '';
      blaS := '';
      // ezen do et, kontrola mezer a poad slo, psmeno
      for i := 1 to Length(S1) do
      begin
         if (S1[i]=' ')then
         begin
            if CheckWhiteSpaces and not WSFixed then
            begin
               ErrS := ErrS+',mezery';
               Error := 1;
               WSFixed := true;
            end;
            continue;
         end;
         if S1[i] in ['0'..'9'] then
         begin
            if not Numbers then
            begin
               ErrS := ErrS+',poad sla a textu';
               Err;
               exit;
            end;
            VSS := VSS + S1[i];
         end
         else
         if S1[i] in ['a'..'z'] then
         begin
            if Numbers then
               Numbers := false;
            blaS := blas + S1[i];
         end
         else
         begin
            ErrS := ErrS+',nepovolen znak';
            Err;
            exit;
         end;
      end;
      if VSS = '' then
      begin
         ErrS := ErrS+',chyb slo ete';
         Err;
         exit;
      end;
      Etaz[LastEt].VS := StrToInt(VSS);
      Etaz[LastEt].bla := blaS;
   end;
   for i := 0 to LastEt do
   begin
      if (i < LastEt) and (Etaz[i].VS < Etaz[i+1].VS) then
      begin
         ErrS := ErrS+',poad et';
         Err;
         exit;
      end;
   end;
   if WSFixed then
   begin
      Err;
      exit;
   end;
end;

procedure TCheckDBFieldPorostniSkupina.SetCheckWhiteSpaces(const Value: boolean);
begin
  FCheckWhiteSpaces := Value;
end;

{ TCheckDBFieldCodePage }

function TCheckDBFieldCodePage.CheckField(const aField: TField): boolean;
var
   aType: string;
   aSize,aDec: integer;
   S,S1: string;
begin
   result := inherited CheckField(aField);
   S := '   !!! Chybn nastaven kontroly';
   S1 := '   !!! Nesoulad nastaven kontroly a formtu poloky';
   if not result then exit;
   if not ((CodePage = 1) or (CodePage = 852) or (CodePage = 1250)) then
   begin
      result := false;
      Error := 1;
      NextErrorText := S;
      NextErrorText := '   !!! kdov strnka '+IntToStr(CodePage)+' je blbost, smysl maj hodnoty 1250,852 nebo 1(Kamenick brothers)';
      exit;
   end;
   aType := FieldTypeToChar(aField.DataType);
   GetFieldSizeAndDec(aField,aSize,aDec);
   if not(aType = 'C') then
   begin
      result := false;
      Error := 1;
      NextErrorText := S1;
      NextErrorText := '   !!! Poloka '+FieldName+' nem formt C, nelze kontrolovat kdovou strnku';
      exit;
   end;
end;

procedure TCheckDBFieldCodePage.ProcessField(const aField: TField);
var
   S: string;
   b: boolean;
   SF: TStringField;
begin
   if not (aField is TStringField) then exit;
   SF := TStringField(aField);
   SF.Transliterate := false;
   S := SF.Value;
   b := true;
   case CodePage of
      1   : b := cp_IsKam(S);
      852 : b := cp_Is852(S);
      1250: b := cp_IsWin(S);
   end;
   if not b then
   begin
      NextErrorText := '  '+ControlFieldValue+': '+S;
      Error := 1;
      SubErrorCount := SubErrorCount+1;
   end;
end;

procedure TCheckDBFieldCodePage.SetCodePage(const Value: integer);
begin
  FCodePage := Value;
end;

{ TCheckDBFieldList }

function TCheckDBFieldList.ElementCondition: boolean;
var
//   T: TTable;
   T: TDataSet;
   F: TField;
   L: TStringList;
   SLE: TStringListElement;
   S: string;
   i: integer;
   RealType: char;
   RealSize: integer;
   RealDec: integer;
begin
   result := true;
   if not (Input is TDBTableElement) then exit;
   T := TDBTableElement(Input).Tab;
   if not T.Active then
      T.Active := true;
   if not T.Active then
   begin
      Error := ch_DB;
      result := false;
      exit;
   end;
   L := TStringList.Create;
   L.CommaText := FieldNames;
   SLE := TStringListElement.CreateFromStringList(L);
   L.Free;
   T.First;
   for i := 0 to T.FieldCount - 1 do
   begin
      F := T.Fields[i];
      S := F.FieldName;
      if SLE.FindElementName(S) = nil then
      begin
         result := false;
         Error := ch_DBFieldNameError;
         RealType := FieldTypeToChar(F.DataType);
         GetFieldSizeAndDec(F,RealSize,RealDec);
         NextErrorText := '  '+S+'  '+RealType+' '+ IntToStr(RealSize)+' '+IntToStr(RealDec);
         SubErrorCount := SubErrorCount + 1;
         if ErrorNames <> '' then ErrorNames := ErrorNames + ',';
         ErrorNames := ErrorNames + S;
      end;
   end;
end;

function TCheckDBFieldList.GetError: longInt;
begin
   result := Error;
end;

function TCheckDBFieldList.GetErrorMsg: string;
begin
   result := ErrorText;
end;

function TCheckDBFieldList.GetErrorTxt: string;
begin
   result := inherited GetErrorTxt{+'  '+FieldName};
end;

function TCheckDBFieldList.GetResultMsg: string;
begin
   result := ' ('+IntToStr(SubErrorCount)+' poloek) '+ErrorNames;
end;

procedure TCheckDBFieldList.Reset;
begin
  inherited;

end;

procedure TCheckDBFieldList.SetErrorNames(const Value: string);
begin
  FErrorNames := Value;
end;

procedure TCheckDBFieldList.SetErrorText(const Value: string);
begin
   FErrorText := Value;
end;

procedure TCheckDBFieldList.SetFieldName(const Value: string);
begin
  FFieldName := Value;
end;

procedure TCheckDBFieldList.SetNextErrorText(const Value: string);
begin
   FErrorText := FErrortext + #13+#10+Value;
end;

procedure TCheckDBFieldList.SetSubErrorCount(const Value: integer);
begin
  FSubErrorCount := Value;
end;

procedure TCheckDBFieldList.SetTableErrorText(const Value: string);
begin
  FTableErrorText := Value;
end;

INITIALIZATION
   RegisterOrbisClass(TCheckDBFieldDef);
   RegisterOrbisClass(TCheckDBFieldList);
   RegisterOrbisClass(TCheckDBFields);
   RegisterOrbisClass(TCheckDBFieldValue);
   RegisterOrbisClass(TCheckDBFieldCodePage);
   RegisterOrbisClass(TCheckDBFieldPorostniSkupina);
   RegisterOrbisClass(TCheckDBFieldValues);
   RegisterOrbisClass(TCheckDBFieldValuesByTable);
   RegisterOrbisClass(TCheckDBFieldValueRange);
END.

