unit C_SHP;

interface
uses
  classes,
  U_BigEnd;


const
  sht_point      =  1;
  sht_arc        =  3;
  sht_polygon    =  5;
  sht_multipoint =  8;
  sht_PolyLineZ  = 13;
  sht_polygonZ   = 15;
  sht_PolyLineM  = 23;

type

TShpPoint = record
              X, Y: double;
            end;

TShpIndexRec = class(TPersistent)
public
  FOffset: TBigEndian;
  FContentLength: TBigEndian;
  function GetOffset: longInt;
  function GetContentLength: longInt;
  procedure SetOffset(aVal: longInt);
  procedure SetContentLength(aVal: longInt);
published
  property Offset: longInt read GetOffset write SetOffset;
  property ContentLength: longInt read GetContentLength write SetContentLength;
end;


TShpFileHeader = class(TPersistent)
  private
    function GetDivne1: longInt;
    function GetDivne2: longInt;
    procedure SetDivne1(const Value: longInt);
    procedure SetDivne2(const Value: longInt);
public
  FFileCode: TBigEndian;
  Unused1  : array [0..4] of TBigEndian;
  FFileLength: TBigEndian;
  FVersion  : longInt;
  FShapeType: longInt;
  FXMin: double;
  FYmin: double;
  FXmax: double;
  FYmax: double;
  FDivne1: TBigEndian;
  FDivne2: TBigEndian;
  Unused2 : array[70..100] of byte;
  function GetFileCode: longInt;
  procedure SetFileCode(aVal: longInt);
  function GetFileLength: longInt;
  procedure SetFileLength(aVal: longInt);
published
  property FileCode: longInt read GetFileCode write SetFileCode;
  property FileLength: longInt read GetFileLength write SetFileLength;
  property Version: longInt read FVersion write FVersion;
  property ShapeType: longInt read FShapeType write FShapeType;
  property XMin: double read FXMin write FXMin;
  property YMin: double read FYMin write FYMin;
  property XMax: double read FXMax write FXMax;
  property YMax: double read FYMax write FYMax;
  property Divne1: longInt read GetDivne1 write SetDivne1;
  property Divne2: longInt read GetDivne2 write SetDivne2;
end;



TShpFileRecHeader = class(TPersistent)
public
  FRecNo: TBigEndian;
  FRecLength: TBigEndian;
  FShapeType: longInt;
  function GetRecNo: longInt;
  procedure SetRecNo(aVal: longInt);
  function GetRecLength: longInt;
  procedure SetRecLength(aVal: longInt);
published
  property RecNo: longInt read GetRecNo write SetRecNo;
  property RecLength: longInt read GetRecLength write SetRecLength;
  property ShapeType: longInt read FShapeType write FShapeType;
end;



TShpArcRec = class(TPersistent)
public
  FXmin, FYmin, FXmax, FYmax: double;
  FNumParts: longInt;
  FNumPoints: longInt;
  function GetXMin: double;
  function GetYMin: double;
  function GetXMax: double;
  function GetYMax: double;
  function GetNumParts: longInt;
  function GetNumPoints: longInt;
published
  property Xmin: double read GetXmin write FXMin;
  property Ymin: double read GetYmin write FYMin;
  property Xmax: double read GetXmax write FXMax;
  property Ymax: double read GetYmax write FYMax;
  property NumParts: longInt read GetNumParts write FNumParts;
  property NumPoints: longInt read GetNumPoints write FNumPoints;
end;

TShpPolygonRec = class(TShpArcRec);


TShpPointRec = class(TPersistent)
public
  FX, FY: double;
published
  property X: double read FX write FX;
  property Y: double read FY write FY;
end;



implementation



             { TShpIndexec }


function TShpIndexRec.GetContentLength: longInt;
begin
  result:= GetLongIntFromBigEnd(FOffset)
end;

function TShpIndexRec.GetOffset: longInt;
begin
  result:= GetLongIntFromBigEnd(FContentLength)
end;


procedure TShpIndexRec.SetOffset(aVal: longInt);
begin
  FOffset:= GetBigEndFromLongInt(aVal);
end;

procedure TShpIndexRec.SetContentLength(aVal: longInt);
begin
  FContentLength:= GetBigEndFromLongInt(aVal);
end;


function TShpFileHeader.GetFileCode: longInt;
begin
  result:= GetLongIntFromBigEnd(FFileCode)
end;

procedure TShpFileHeader.SetFileCode(aVal: longInt);
begin
  FFileCode:= GetBigEndFromLongInt(aVal);
end;

function TShpFileHeader.GetFileLength: longInt;
begin
  result:= GetLongIntFromBigEnd(FFileLength)*2
end;

procedure TShpFileHeader.SetFileLength(aVal: longInt);
begin
  FFileLength:= GetBigEndFromLongInt(aVal);
end;


               { TShpFileRecHeader }

function TShpFileRecHeader.GetRecNo: longInt;
begin
  result:= GetLongIntFromBigEnd(FRecNo)
end;

procedure TShpFileRecHeader.SetRecNo(aVal: longInt);
begin
  FRecNo:= GetBigEndFromLongInt(aVal);
end;


function TShpFileRecHeader.GetRecLength: longInt;
begin
  result:= GetLongIntFromBigEnd(FRecLength)*2
end;

procedure TShpFileRecHeader.SetRecLength(aVal: longInt);
begin
  FRecLength:= GetBigEndFromLongInt(aVal);
end;



function TShpArcRec.GetXMin: double;
begin
  result:= FXmin
end;

function TShpArcRec.GetYMin: double;
begin
  result:= FYmin
end;

function TShpArcRec.GetXMax: double;
begin
  result:= FXmax
end;

function TShpArcRec.GetYMax: double;
begin
  result:= FYmax
end;

function TShpArcRec.GetNumParts: longInt;
begin
  result:= FNumParts
end;

function TShpArcRec.GetNumPoints: longInt;
begin
  result:= FNumPoints
end;

function TShpFileHeader.GetDivne1: longInt;
begin
  result:= GetLongIntFromBigEnd(FDivne1)
end;

function TShpFileHeader.GetDivne2: longInt;
begin
  result:= GetLongIntFromBigEnd(FDivne2)
end;

procedure TShpFileHeader.SetDivne1(const Value: longInt);
begin
  FDivne1:= GetBigEndFromLongInt(Value);
end;

procedure TShpFileHeader.SetDivne2(const Value: longInt);
begin
  FDivne2:= GetBigEndFromLongInt(Value);
end;

end.
