unit E_QTLin;

interface
uses
// KERNEL
  Elements,
// SPACETOOL
  E_QTree,
  e_Geometry,
  //E_Space,
  T_ToTool;


const
  MaxCountInCellForTopLin    = 128;
  MaxCountInCellForTopLinAbsc=  32;


type

TTopLinAbscissa = class(TOwnerElement)
private
  FLin: TPolyLineElement;
  FNewAbscissa: TTopLinAbscissa;
  FAbscIndex: longInt;
  FSwap: boolean;
  FIsNewEnd : boolean;
  FIsNew    : boolean;
  FIsDeleted: boolean;
  FFixedBegin: boolean;
  FFixedEnd  : boolean;

  function GetX1: double;
  function GetY1: double;
  function GetX2: double;
  function GetY2: double;
  procedure PutX1(aCoord: double);
  procedure PutY1(aCoord: double);
  procedure PutX2(aCoord: double);
  procedure PutY2(aCoord: double);
  function  GetXmin: double;
  function  GetYmin: double;
  function  GetXmax: double;
  function  GetYmax: double;
  function  GetLength: double;
  function  FIsMargin: boolean;
  function  FIsLast  : boolean;
  function  FIsFirst : boolean;
  function  GetLineNo: longInt;
public
  function  JoinToPoint(aBegin: boolean;
                        aX, aY: double;
                        aSnapDistance: double): boolean;
  function IsJoinedPoint(aBegin: boolean;
                         aAbsc: TTopLinAbscissa;
                         aAbscBegin: boolean;
                         aSnapDistance: double;
                         var aDistance: double;
                         var aX, aY: double): boolean;
  function  JoinToAbscPoint(aBegin: boolean;
                            aAbsc: TTopLinAbscissa;
                            aAbscBegin: boolean;
                            aSnapDistance: double): boolean;
  function  IsJoinToAbsc(aBegin: boolean;
                         aAbsc: TTopLinAbscissa;
                         aSnapDistance: double;
                         var aDist: double;
                         var PX, PY: double): boolean;
  function  JoinToAbsc(aBegin: boolean;
                        aAbsc: TTopLinAbscissa;
                        aSnapDistance: double;
                        var PX, PY: double): boolean;
  procedure RemoveFromLine;
  procedure CreateDividedWith(X, Y: double; var aAbsc: TTopLinAbscissa);
  function  IsOverRectangle(aXmin, aYmin, aXmax, aYmax, Tolerance: double): boolean;
  function  IsIdenticalTo(aAbsc: TTopLinAbscissa): boolean;
  procedure IdentTest(aAbsc: TTopLinAbscissa; var Id11, Id22, Id12, Id21: boolean);
  procedure RelationToPoint(aAbsc: TTopLinAbscissa; aBegin: boolean; aSnapDistance: double;
                            var NearPoint, Danger: boolean; var PX, PY, PataX, PataY: double);
published
  property Line: TPolyLineElement read FLin write FLin;
  property LineNo: longInt read GetLineNo;
  property Index: longInt read FAbscIndex write FAbscIndex;
  property NewAbscissa: TTopLinAbscissa read FNewAbscissa write FNewAbscissa;
  property Swap: boolean read FSwap write FSwap;
  property X1: double read GetX1 write PutX1;
  property Y1: double read GetY1 write PutY1;
  property X2: double read GetX2 write PutX2;
  property Y2: double read GetY2 write PutY2;
  property Xmin: double read GetXMin;
  property Ymin: double read GetYMin;
  property Xmax: double read GetXMax;
  property Ymax: double read GetYMax;
  property Length: double read GetLength;
  property IsMargin: boolean read FIsMargin;
  property IsLast  : boolean read FIsLast;
  property IsFirst : boolean read FIsFirst;
  property IsNewEnd: boolean read FIsNewEnd write FIsNewEnd;
  property FixedBegin: boolean read FFixedBegin write FFixedBegin;
  property FixedEnd: boolean read FFixedEnd write FFixedEnd;
  { pznak, e po.bod seky je novm koncem sublinie }
  property IsNew   : boolean read FIsNew    write FIsNew;
  { pznak, e po.bod seky je novm vloenm bodem linie }
  property IsDeleted: boolean read FIsDeleted write FIsDeleted;
end;



TQTreeTopLinAbsc=class(TQTree)
{ QTree seek lini }
protected
  function  MaxCountInCell: longInt; override;
  procedure GiveElementRectangle(Element: TElement; var aXmin, aYmin, aXmax, aYmax: double); override;
public
  function  ElementIsOverRectangle
            (Element: TElement; Xmin, Ymin, Xmax, Ymax: double): boolean; override;
  procedure on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;


TQTreeTopLin=class(TRectEntityQTree)
{QTree lini }
protected
  function MaxCountInCell: longInt; override;
public
  procedure on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;




TCreateQTreeTopLinAbsc = class(TProchazecLiniiGauge)
private
 FQTree: TQTreeTopLinAbsc;
public
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
end;


TCreateQTreeTopLin = class(TProchazecLiniiGauge)
protected
  FQTree: TQTreeTopLin;
  FContainerQTree: TEntity;
public
  procedure Process(Element: TElement); override;
  procedure DoIt; override;
end;



implementation
uses
  Dialogs,
  SysUtils,
// KERNEL
  U_String,
// GLIB
  AnalInt,
  G_Elems,
  MathLib;

                    { TTopLinAbscissa }


function TTopLinAbscissa.GetX1: double;
var
  X, Y: double;
begin
  FLin.GetOrientedCoord(FAbscIndex, true, X, Y);
  result:= X
end;

function TTopLinAbscissa.GetY1: double;
var
  X, Y: double;
begin
  FLin.GetOrientedCoord(FAbscIndex, true, X, Y);
  result:= Y
end;

function TTopLinAbscissa.GetX2: double;
var
  X, Y: double;
begin
  FLin.GetOrientedCoord(FAbscIndex+1, true, X, Y);
  result:= X
end;

function TTopLinAbscissa.GetY2: double;
var
  X, Y: double;
begin
  FLin.GetOrientedCoord(FAbscIndex+1, true, X, Y);
  result:= Y
end;


function TTopLinAbscissa.GetXmin: double;
begin
  result:= X1;
  if X2 < X1 then result:= X2
end;


function TTopLinAbscissa.GetYmin: double;
begin
  result:= Y1;
  if Y2 < Y1 then result:= Y2
end;


function TTopLinAbscissa.GetXmax: double;
begin
  result:= X1;
  if X2 > X1 then result:= X2
end;


function TTopLinAbscissa.GetYmax: double;
begin
  result:= Y1;
  if Y2 > Y1 then result:= Y2
end;


procedure TTopLinAbscissa.PutX1(aCoord: double);
begin
  FLin.Xi[FAbscIndex]:= aCoord
end;

procedure TTopLinAbscissa.PutY1(aCoord: double);
begin
  FLin.Yi[FAbscIndex]:= aCoord
end;

procedure TTopLinAbscissa.PutX2(aCoord: double);
begin
  FLin.Xi[FAbscIndex+1]:= aCoord
end;

procedure TTopLinAbscissa.PutY2(aCoord: double);
begin
  FLin.Yi[FAbscIndex+1]:= aCoord
end;

function TTopLinAbscissa.GetLength: double;
begin
  result:= sqrt(sqr(X2-X1)+sqr(Y2-Y1))
end;


procedure TTopLinAbscissa.RemoveFromLine;
begin
  FLin.FreeElement(FLin.Elements[FAbscIndex]);
end;

function TTopLinAbscissa.FIsMargin;
begin
  result:= (FAbscIndex = 0) or (FAbscIndex >= FLin.Count-2)
end;

function TTopLinAbscissa.FIsLast;
begin
  result:= (FAbscIndex >= FLin.Count-2)
end;

function TTopLinAbscissa.FIsFirst;
begin
  result:= (FAbscIndex = 0)
end;




function  TTopLinAbscissa.GetLineNo: longInt;
begin
  result:= -1;
  if FLin <> nil then
     result:= FLin.GetPropertyLongIntByName('Cislo')
end;


function TTopLinAbscissa.JoinToPoint(aBegin: boolean;
                                     aX, aY: double;
                                     aSnapDistance: double): boolean;
begin
  result:= false;
  if aBegin then
  begin
    if InTolerance(aX, aY, X1, Y1, aSnapDistance) then
    begin
      X1:= aX; Y1:= aY;
      result:= true;
    end
  end
  else
  begin
    if InTolerance(aX, aY, X2, Y2, aSnapDistance) then
    begin
      X2:= aX; Y2:= aY;
      result:= true;
    end
  end
end;




function TTopLinAbscissa.IsJoinedPoint(aBegin: boolean;
                                       aAbsc: TTopLinAbscissa;
                                       aAbscBegin: boolean;
                                       aSnapDistance: double;
                                       var aDistance: double;
                                       var aX, aY: double): boolean;
begin
  result:= false;
  if aAbscBegin then
  begin
    aX:= aAbsc.X1; aY:= aAbsc.Y1
  end
  else
  begin
    aX:= aAbsc.X2; aY:= aAbsc.Y2
  end;

  if aBegin then
  begin
    if not FixedBegin and InTolerance(aX, aY, X1, Y1, aSnapDistance) then
    begin
      result:= true;
      aDistance:= Distance(aX, aY, X1, Y1);
{      if (Index > 0) then IsNewEnd:= true; }
    end
  end
  else
  begin
    if not FixedEnd and InTolerance(aX, aY, X2, Y2, aSnapDistance) then
    begin
      result:= true;
      aDistance:= Distance(aX, aY, X2, Y2);
    end
  end;
end;




function TTopLinAbscissa.JoinToAbscPoint(aBegin: boolean;
                                         aAbsc: TTopLinAbscissa;
                                         aAbscBegin: boolean;
                                         aSnapDistance: double): boolean;
var
  aX, aY: double;
begin
  result:= false;
  if aAbscBegin then
  begin
    aX:= aAbsc.X1; aY:= aAbsc.Y1
  end
  else
  begin
    aX:= aAbsc.X2; aY:= aAbsc.Y2
  end;

  if aBegin then
  begin
    if not FixedBegin and InTolerance(aX, aY, X1, Y1, aSnapDistance) then
    begin
      X1:= aX; Y1:= aY;
      result:= true;
      FixedBegin:= true;
{      if (Index > 0) then IsNewEnd:= true; }
    end
  end
  else
  begin
    if not FixedEnd and InTolerance(aX, aY, X2, Y2, aSnapDistance) then
    begin
      X2:= aX; Y2:= aY;
      result:= true;
      FixedEnd:= true
    end
  end;
(*
  if result then
  { oznaovn seky, pokud byla dotena a nen koncov }
     aAbsc.IsNewEnd:= not aAbsc.IsLast and not aAbscBegin
*)
end;





function  TTopLinAbscissa.IsJoinToAbsc(aBegin: boolean;
                                       aAbsc: TTopLinAbscissa;
                                       aSnapDistance: double;
                                       var aDist: double;
                                       var PX, PY: double): boolean;
var
  PataUvnitr: boolean;

begin
  result:= false;

  if RectanglesOverlay(X1, Y1, X2, Y2, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2, aSnapDistance) then
  begin
    if aBegin and not FixedBegin then
    begin
      if not EqualPoints(X1, Y1, aAbsc.X1, aAbsc.Y1)
         and
         not EqualPoints(X1, Y1, aAbsc.X2, aAbsc.Y2)then
      begin
        DejVztahyBod_Usecka(X1, Y1, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2,
                           PataUvnitr, PX, PY, aDist);
        if PataUvnitr and (aDist < aSnapDistance) then
        begin
      {    if Index > 0 then IsNewEnd:= true; }
          result:= true;
        end
      end
    end
    else
    if not FixedEnd then
    begin
      if not EqualPoints(X2, Y2, aAbsc.X1, aAbsc.Y1)
         and
         not EqualPoints(X2, Y2, aAbsc.X2, aAbsc.Y2)then
      begin
        DejVztahyBod_Usecka(X2, Y2, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2,
                           PataUvnitr, PX, PY, aDist);
        if PataUvnitr and (aDist < aSnapDistance) then
        begin
          result:= true;
        end
      end
    end
  end
end;





function  TTopLinAbscissa.JoinToAbsc(aBegin: boolean;
                                     aAbsc: TTopLinAbscissa;
                                     aSnapDistance: double;
                                     var PX, PY: double): boolean;
var
  PataUvnitr: boolean;
  Vzdalenost: double;

begin
  result:= false;

  if RectanglesOverlay(X1, Y1, X2, Y2, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2, aSnapDistance) then
  begin
    if aBegin and not FixedBegin then
    begin
      if not EqualPoints(X1, Y1, aAbsc.X1, aAbsc.Y1)
         and
         not EqualPoints(X1, Y1, aAbsc.X2, aAbsc.Y2)then
      begin
        DejVztahyBod_Usecka(X1, Y1, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2,
                           PataUvnitr, PX, PY, Vzdalenost);
        if PataUvnitr and (Vzdalenost < aSnapDistance) then
        begin
      {    if Index > 0 then IsNewEnd:= true; }
          X1:= PX; Y1:= PY;
          FixedBegin:= true;
          result:= true;
        end
      end
    end
    else
    if not FixedEnd then
    begin
      if not EqualPoints(X2, Y2, aAbsc.X1, aAbsc.Y1)
         and
         not EqualPoints(X2, Y2, aAbsc.X2, aAbsc.Y2)then
      begin
        DejVztahyBod_Usecka(X2, Y2, aAbsc.X1, aAbsc.Y1, aAbsc.X2, aAbsc.Y2,
                           PataUvnitr, PX, PY, Vzdalenost);
        if PataUvnitr and (Vzdalenost < aSnapDistance) then
        begin
          X2:= PX; Y2:= PY;
          FixedEnd:= true;
          result:= true;
        end
      end
    end
  end
end;



procedure TTopLinAbscissa.RelationToPoint(aAbsc: TTopLinAbscissa; aBegin: boolean; aSnapDistance: double;
                                          var NearPoint, Danger: boolean; var PX, PY, PataX, PataY: double);
var
  PataUvnitr: boolean;
  Vzdalenost, Dist1, Dist2: double;
  Id11, Id22, Id12, Id21: boolean;
begin
  NearPoint:= false;
  Danger:= false;
  IdentTest(aAbsc, Id11, Id22, Id12, Id21);

  {nesm bt zcela identick}
  if (Id11 and Id22) or (Id12 and Id21) then exit;
  { nesm bt identick ve zkoumanm bod }
  if (aBegin and (Id11 or Id21)) or (not aBegin and (Id12 or Id22)) then exit;
  { nesm bt nkde identick a pitom bt sousednmi sekami tyt linie }
  if (Id11 or Id22 or Id12 or Id21) and (Index = aAbsc.Index-1) and
     (Line = aAbsc.Line) then exit;

  if aBegin then
  begin
    PX:= aAbsc.X1;
    PY:= aAbsc.Y1
  end
  else
  begin
    PX:= aAbsc.X2;
    PY:= aAbsc.Y2
  end;

 { nesm se napojovat na bod identick s pedchozm bodem linie tto seky }
  if (Index > 0) and (Line.Xi[Index-1]=PX) and (Line.Yi[Index-1]=PY) then exit;

  DejVztahyBod_Usecka(PX, PY, X1, Y1, X2, Y2,
                     PataUvnitr, PataX, PataY, Vzdalenost);
  if PataUvnitr and (Vzdalenost < aSnapDistance) then
  begin
    NearPoint:= true;
    Dist1:= Distance(pataX, pataY, X1, Y1);
    Dist2:= Distance(pataX, pataY, X2, Y2);
    {!!!!!!!!!!!!G
    if aSnapDistance < VektorNula then
       aSnapDistance:= VektorNula;
     }
    Danger:= (Dist1 < Vzdalenost) or (Dist2 < Vzdalenost);
  end
end;






procedure TTopLinAbscissa.CreateDividedWith(X, Y: double; var aAbsc: TTopLinAbscissa);
var
  Coord: T2DCoordinate;
begin
  Coord:= T2DCoordinate.Create;
  Coord.X:= X;
  Coord.Y:= Y;
  FLin.InsertElement(Index+1, Coord);
  aAbsc:= TTopLinAbscissa.Create;
  aAbsc.IsNew:= true;
  aAbsc.Line:= FLin;
      // PK Test !!!!!!!!!!!
      //aAbsc.Name := FLin.Name;
  aAbsc.Index:= Index + 1;
end;



function TTopLinAbscissa.IsOverRectangle(aXmin, aYmin, aXmax, aYmax, Tolerance: double): boolean;
begin
  result:= false;
  if (aXmin > Xmax+Tolerance) or (aXmax < Xmin-Tolerance) or
      (aYmin > Ymax+Tolerance) or (aYmax < Ymin-Tolerance) then exit;
  result := true;
end;



function  TTopLinAbscissa.IsIdenticalTo(aAbsc: TTopLinAbscissa): boolean;
begin
  result:= ((X1 = aAbsc.X1) and (Y1 = aAbsc.Y1) and
            (X2 = aAbsc.X2) and (Y2 = aAbsc.Y2))
            or
           ((X1 = aAbsc.X2) and (Y1 = aAbsc.Y2) and
            (X2 = aAbsc.X1) and (Y2 = aAbsc.Y1))
end;


procedure TTopLinAbscissa.IdentTest(aAbsc: TTopLinAbscissa; var Id11, Id22, Id12, Id21: boolean);
begin
  Id11:= (X1 = aAbsc.X1) and (Y1 = aAbsc.Y1);
  Id22:= (X2 = aAbsc.X2) and (Y2 = aAbsc.Y2);
  Id12:= (X1 = aAbsc.X2) and (Y1 = aAbsc.Y2);
  Id21:= (X2 = aAbsc.X1) and (Y2 = aAbsc.Y1)
end;



                  { TQTreeTopLin }

function TQTreeTopLin.MaxCountInCell: longInt;
begin
  result:= MaxCountInCellForTopLin
end;




procedure TQTreeTopLin.on_tm_DivideGeometry(var aMessage: TOrbisMessage;var Handled: boolean);
begin

end;

{ TQTreeTopLinAbsc }

function TQTreeTopLinAbsc.MaxCountInCell: longInt;
begin
  result:= MaxCountInCellForTopLinAbsc
end;

procedure TQTreeTopLinAbsc.GiveElementRectangle(Element: TElement; var aXmin, aYmin, aXmax, aYmax: double);
begin
  with TTopLinAbscissa(Element) do
  begin
    aXmin:= Xmin; aYmin:= Ymin; aXmax:= Xmax; aYmax:= Ymax
  end
end;


function TQTreeTopLinAbsc.ElementIsOverRectangle
            (Element: TElement; Xmin, Ymin, Xmax, Ymax: double): boolean;
begin
  result:= TTopLinAbscissa(Element).IsOverRectangle(Xmin, Ymin, Xmax, Ymax, 0);
end;



procedure TQTreeTopLinAbsc.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
begin

end;

{ TCreateQTreeTopLin }


procedure TCreateQTreeTopLin.Process(Element: TElement);
//var
  //Lin: TPolyLineElement;
  //I: longInt;
begin
  if Element is TPolyLineElement then
  begin
    inc(FPocetLinii);
    GaugeSet(FPocetLinii);

{$IFDEF TestProtocol}
    ProtocolName:= IntToStrZ(FPocetLinii, 4, true) +
                   IntToStrZ(TPolyLineElement(Element).Cislo,4,true) + '.PTC';
    Uroven:= 0;
{$ENDIF}

    FQTree.Add(TPolyLineElement(Element));
  end
end;





procedure TCreateQTreeTopLin.DoIt;
var
  RE: TRectEntity;
  QX, QY, QW: longInt;
begin
  try
    if NoParameters then
       FContainerQTree:= TEntity(Input)
    else
       FContainerQTree:= TEntity(Parameter[0]);

    if TElement(FContainerQTree) is TEntity then
    begin
      FQTree:= TQTreeTopLin.Create;
      if FQTree <> nil then
      begin
        if TElement(FContainerQTree) is TRectEntity then
           TRectEntity(FContainerQTree).AddAndUpdateRect(FQTree)
        else
          FContainerQTree.AddElement(FQTree);
        if Input is TRectEntity then
        begin
          RE:= TRectEntity(Input);
          VratNormCtverec(RE.minX, RE.MinY, RE.MaxX, RE.MaxY,
                          QX, QY, QW);
          FQTree.QXMin := QX;
          FQTree.QYMin := QY;
          FQTree.QWidth:= QW;
        end;
        inherited DoIt
      end
    end;
  except
    on E:Exception do
       ShowMessage('Chyba pi vytven QTree pro linie: ' + E.Message);
{    MessageDlg('Chyba pi vytven QTree pro linie', mtError, [mbOK], 0) }
  end;
end;



                 { TCreateQTreeTopLinAbsc }


procedure TCreateQTreeTopLinAbsc.Process(Element: TElement);
var
  Lin: TPolyLineElement;
  Absc: TTopLinAbscissa;
  I: longInt;
begin
  if Element is TPolyLineElement then
  begin
    inc(FPocetLinii);
    GaugeSet(FPocetLinii);

    Lin:= TPolyLineElement(Element);
    for I:= 0 to Lin.Count-2 do
    begin
      Absc:= TTopLinAbscissa.Create;
      Absc.Line:= Lin;
         // PK Test !!!!!!!!!!!
         //Absc.Name := Lin.Name;
      Absc.Index:= I;
      FQTree.Add(Absc)
    end;
  end
end;



procedure TCreateQTreeTopLinAbsc.DoIt;
var
  DrzakQTree: TEntity;
begin
  if NoParameters then
     DrzakQTree:= TEntity(Input)
  else
     DrzakQTree:= TEntity(Parameter[0]);

  if TElement(DrzakQTree) is TEntity then
  begin
    FQTree:= TQTreeTopLinAbsc.Create;
    if FQTree <> nil then
       inherited DoIt
  end;
end;






end.
