unit e_Geometry;
(*
ORBIS 2001
Zkladn geometrick elementy, dve SpaceEntities
!!! nemaj dynamick properties, budou vlastnny tmatickmi elementy
*)
interface
uses
  //Classes,
// KERNEL
  Elements,
// GLIB
  G_Elems;



type

TGeometry = class(TRectEntity)
private
   FAngle: double;
    FGIndex: integer;
    FTimeStamp: TDateTime;
    FUUID: string;
    FUID: string;
   function  GetIsDone: boolean;
   procedure SetIsDone(const Value: boolean);
    procedure SetGIndex(const Value: integer);
    procedure SetTimeStamp(const Value: TDateTime);
    procedure SetUID(const Value: string);
    procedure SetUUID(const Value: string);
    procedure SetNewUUID(const Value: boolean);
    function GetTimeStampString: string;
public
   procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); virtual;
   procedure  HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean); override;
   function   IsIsolatedFrom(aBiggerGeometry: TGeometry): boolean;
   function   IsHit(CoordEl: TCoordinate; Tolerance: double): boolean; override;
   function   IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
   function   GetSimpleGeometryByXY(aX,aY,Tolerance: double): TGeometry; virtual;
   procedure  CreateUUID;
published
   property UUID: string read FUUID write SetUUID;
   property TimeStamp: TDateTime read FTimeStamp write SetTimeStamp;
   property TimeStampString: string read GetTimeStampString stored false;
   property UID: string read FUID write SetUID;
   property NewUUID: boolean write SetNewUUID stored false;
   property IsDone: boolean read GetIsDone write SetIsDone stored false;
   property Angle: double read FAngle write FAngle stored true;
   property GIndex: integer read FGIndex write SetGIndex;
end;

TGeometryString = class(TGeometry)
public
   procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); override;
end;


TPointElement = class(TGeometry)
protected
   procedure SetMinX(aValue: double); override;
   procedure SetMinY(aValue: double); override;
end;


TTextElement = class(TGeometry)
private
  FText: string;
  procedure SetText(aVal: string);
  function  GetText: string;
public
   function   IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
published
  property Text: string read GetText write SetText stored true;
end;

TPolyLineElement = class;

TPolyElement = class(TGeometry)
private
  function  GetCoordsCount: longInt; virtual;
  function  GetCoordElement(aIndex: longInt): T2DCoordinate; virtual;

  function  GetLength: double; virtual;
  procedure SetX(aIndex: longInt; aX: double);
  function  GetX(aIndex: longInt): double;
  procedure SetY(aIndex: longInt; aY: double);
  function  GetY(aIndex: longInt): double;
  function  GetArea: double; virtual;
    function GetLastLength: double;
    procedure SetRecalculateLabelProps(const Value: boolean);
  {
  procedure SetOrientedX(aIndex: longInt; aBegin: boolean; aCoord: double);
  procedure SetOrientedY(aIndex: longInt; aBegin: boolean; aCoord: double);
  }
public
  procedure GetOrientedCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
  function  GetOrientedCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
  procedure GivePointPropsForLength(Length: double; var pX, pY, pAngle: double);
  procedure GiveLabelProps(var aX, aY, Angle: double);
  procedure CalculateLabelProps;
  procedure RecalculateRect;
  function  GetSubAreaValue(FromBegin: boolean): double;
  procedure AddXY(aX, aY: double);
  procedure InsertXY(anIndex: longInt; aX, aY: double);
  procedure AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean); virtual;
  property  Xi[aIndex: longInt]: double read GetX write SetX;
  property  Yi[aIndex: longInt]: double read GetY write SetY;
published
  property CoordsCount: longInt read GetCoordsCount stored false;
  property Length      : double  read GetLength stored false;
  property LastLength  : double  read GetLastLength stored false;
  property Area: double read GetArea stored false;
  property RecalculateLabelProps: boolean write SetRecalculateLabelProps stored false;
end;



TPolyLineElement = class(TPolyElement)
public
   function IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
   function IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean; override;
   function GetAreaFromBegin(aBegin: Boolean): double;
end;


TMultiPolyLineElement = class(TPolyLineElement)
private
   FLinesOrigines: string;
   function  GetLength: double; override;
   function  GetOrigin(aIndex: longInt): boolean;
   procedure RemoveOrigin(aIndex: longInt);
   procedure InsertOrigin(aIndex: Longint; aVal: boolean);
   procedure SetOrigin(aIndex: longInt; Item: boolean);
   procedure InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);

   function  GetCoordsCount: longInt; override;
   function  GetCoordElement(aIndex: longInt): T2DCoordinate; override;
   function GetLinesOriginesHex: string;
   procedure SetLinesOriginesHex(const Value: string);
public
   procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); override;

   procedure  AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean); override;
   destructor Destroy; override;
   procedure  RemoveElement(aElement: TElement); override;
 {
   procedure  Load(aReader: TReader; aIndex: TEntity); override;
   procedure  Save(aWriter: TWriter); override;
 }
   function   HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
 {  function   CreateEnvelope: TElement; override; }
   function   IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
   function   GetSimpleGeometryByXY(aX,aY,Tolerance: double): TGeometry; override;

   property   LinesOrigines[aIndex: longInt]: boolean read GetOrigin write SetOrigin;
published
   property   LinesOriginesHex: string read GetLinesOriginesHex write SetLinesOriginesHex;
end;


TAreaElement = class(TPolyElement)
{ plocha obklopen souadnicemi }
private
  FSavedArea: double;
  FHoles: TRectEntity;
  FIsHole: boolean;
    FFixedAreaIndex: byte;
    FFixedArea: double;

  function GetArea: double; override;
  function GetWholeArea: double;
  function GetHoleOwnerArea: TPolyElement;
    procedure SetHoles(const Value: TRectEntity);
    procedure SetFixedArea(const Value: double);
    procedure SetFixedAreaIndex(const Value: byte);
    function GetFixedArea: double; virtual;
    function GetFixedAreaIndex: byte; virtual;
    function GetSameFAI: boolean; virtual;
  procedure SetSwapBorder(const Value: boolean); virtual;
    function GetRealArea: double;
    function GetRealWholeArea: double;
public
  destructor Destroy; override;
  procedure  ResetIDs; override;
  procedure AddHole(aHole: TAreaElement);
  function  GetHoleByXY(aX, aY: double): TAreaElement;
  function XYIsInside(aX, aY: double): boolean;
  function XYIsInsideNoHoles(aX, aY: double): boolean; virtual;
  function IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
published
  property Holes: TRectEntity read FHoles write SetHoles stored true;
  property NextHole: TAreaElement write AddHole stored false;
  property IsHole: boolean read FIsHole write FIsHole stored true;
  property HoleOwnerArea: TPolyElement read GetHoleOwnerArea stored false;
  property RealWholeArea: double read GetRealWholeArea stored false;
  property RealArea: double read GetRealArea stored false;
  property WholeArea: double read GetWholeArea stored false;
  property SavedArea: double read FSavedArea stored false;
  property SameFAI: boolean read GetSameFAI stored false;
  property FixedArea: double read GetFixedArea write SetFixedArea stored true;
  property FixedAreaIndex: byte read GetFixedAreaIndex write SetFixedAreaIndex stored true;
  property SwapBorder: boolean write SetSwapBorder stored false;
end;


THolesEntity = class(TRectEntity)
private
  FOwnerArea: TAreaElement;
public
  procedure SetOwnerArea(Value: TAreaElement);
published
  property OwnerArea: TAreaElement read FOwnerArea stored false;
end;

TMultiAreaElement = class(TAreaElement)
private
   function GetArea: double; override;
    function GetAreaElement(aIndex: integer): TAreaElement;
    function GetFixedArea: double; override;
    function GetFixedAreaIndex: byte; override;
    function GetSameFAI: boolean; override;
    procedure SetSwapBorder(const Value: boolean); override;
public
   function IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
   function GetSimpleGeometryByXY(aX,aY,Tolerance: double): TGeometry; override;
   property AreaElement[aIndex: integer]: TAreaElement read GetAreaElement;
end;

TPolyLineAreaElement = class(TAreaElement)
{ plocha obklopen liniemi }
private
  FLinesOrigines: string;
  function GetOrigin(aIndex: longInt): boolean;
  procedure RemoveOrigin(aIndex: longInt);
  procedure InsertOrigin(aIndex: Longint; aVal: boolean);
  procedure SetOrigin(aIndex: longInt; Item: boolean);
  procedure InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);

  function  GetCoordsCount: longInt; override;
  function  GetCoordElement(aIndex: longInt): T2DCoordinate; override;
  function GetLinesOriginesHex: string;
  procedure SetLinesOriginesHex(const Value: string);
  procedure SetSwapBorder(const Value: boolean); override;
public
  procedure  on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean); override;

  procedure AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean); override;
  destructor Destroy; override;
  procedure  RemoveElement(aElement: TElement); override;
{
  procedure  Load(aReader: TReader; aIndex: TEntity); override;
  procedure  Save(aWriter: TWriter); override;
}
  function   HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
{  function   CreateEnvelope: TElement; override; }
  function   CreateAreaElementNoHole: TAreaElement;
  function   CreateAreaElement: TAreaElement;
  function   IsHitByXY(aX, aY: double; Tolerance: double): boolean; override;
  function   XYIsInsideNoHoles(aX, aY: double): boolean; override;

  property   LinesOrigines[aIndex: longInt]: boolean read GetOrigin write SetOrigin;

published
  property   LinesOriginesHex: string read GetLinesOriginesHex write SetLinesOriginesHex;
end;


TPolyLineString = class(TGeometryString);
TPointString    = class(TGeometryString);
TTextString     = class(TGeometryString);
TAreaString     = class(TGeometryString);


implementation
uses
  SysUtils, Dialogs,
// KERNEL
  e_Class,
  M_Orbis,
  U_Props,
  U_String,
// GLIB
  AnalInt,
  MathLib;
// SPACE
  //E_SpaPrp,
  //E_Props;



{ TGeometry }

procedure TGeometry.CreateUUID;
begin
   UUID := GUIDToString(TGUID.NewGuid);
end;

function TGeometry.GetIsDone: boolean;
begin
   result:= not (ID = -1)
end;

procedure TGeometry.SetIsDone(const Value: boolean);
begin
   if Value then
      ID:= 0
   else
      ID:= -1
end;



procedure TGeometry.SetNewUUID(const Value: boolean);
begin
   if value then
     CreateUUID;
end;

procedure TGeometry.SetUUID(const Value: string);
begin
  FUUID := Value;
end;

function TGeometry.IsIsolatedFrom(aBiggerGeometry: TGeometry): boolean;
var
  J: longInt;
  aSE: TGeometry;
begin
  result:= true;

  for J:= 0 to LastIndex do
  begin
    aSE:= TGeometry(Elements[J]);
    if not (TElement(aSE) is TGeometry) then continue;
    if aSE.IsMyOwner(aBiggerGeometry) then
    begin
      result:= false;
      exit
    end
  end;
end;




function  TGeometry.IsHit(CoordEl: TCoordinate; Tolerance: double): boolean;
begin
   result:= false;
   if CoordEl = nil then exit;
   result:= IsHitByXY(CoordEl.X, CoordEl.Y, Tolerance);
end;

function TGeometry.IsHitByXY(aX, aY, Tolerance: double): boolean;
begin
   result:= Distance(aX, aY, X, Y) < abs(Tolerance)
end;



procedure TGeometry.HandleMessage(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   Handled := false;
   inherited HandleMessage(aMessage, Handled);
   if Handled then exit;
   case aMessage.Message of
      tm_DivideGeometry: on_tm_DivideGeometry(aMessage, Handled);
   end;
end;

procedure TGeometry.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
begin

end;

function TGeometry.GetSimpleGeometryByXY(aX,aY,Tolerance: double): TGeometry;
begin
   result := nil;
   if IsHitByXY(aX,aY,Tolerance) then
      result := Self;
end;


function TGeometry.GetTimeStampString: string;
begin
   result := DateTimeToStr(TimeStamp);
end;

procedure TGeometry.SetTimeStamp(const Value: TDateTime);
begin
  FTimeStamp := Value;
end;

procedure TGeometry.SetUID(const Value: string);
begin
  FUID := Value;
end;


procedure TGeometry.SetGIndex(const Value: integer);
begin
  FGIndex := Value;
end;

{ TTextElement }

procedure TTextElement.SetText(aVal: string);
begin
  FText:= aVal;
end;



function  TTextElement.GetText: string;
begin
  Result := FText
end;




function TTextElement.IsHitByXY(aX, aY, Tolerance: double): boolean;
begin
   result := PointIsInside(aX,aY);
end;

{ TPolyElement }

function  TPolyElement.GetArea: double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetCoordElement(0);
    if not (CrdP is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetCoordElement(I);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end;
end;



function TPolyElement.GetCoordsCount: longInt;
begin
  Result:= Count;
end;


function TPolyElement.GetLength: double;
var
  I: longInt;
{  X1, Y1, Z1, X2, Y2, Z2: double; }
  Crd1, Crd2: TCoordinate;
begin
  result:= 0;
{  GetCoord(0, true, X1, Y1);
  for I:= 0 to Count-2 do
  begin
    GetCoord(I+1, true, X2, Y2);
    result:= result + Distance(X1, Y1, X2, Y2);
    X1:= X2; Y1:= Y2; Z1:= Z2
  end;
}
  Crd1:= GetCoordElement(0);
  for I:= 0 to CoordsCount-1 do
  begin
    Crd2:= GetCoordElement(I);
    result:= result + Distance(Crd1.X, Crd1.Y, Crd2.X, Crd2.Y);
    Crd1:= Crd2
  end
end;



procedure TPolyElement.SetX(aIndex: longInt; aX: double);
var
  aCrd: TCoordinate;
begin
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  aCrd.X:= aX
end;


function  TPolyElement.GetX(aIndex: longInt): double;
var
  aCrd: TCoordinate;
begin
  result:= 0;
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  result:= aCrd.X
end;


procedure TPolyElement.SetY(aIndex: longInt; aY: double);
var
  aCrd: TCoordinate;
begin
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  aCrd.Y:= aY
end;


function  TPolyElement.GetY(aIndex: longInt): double;
var
  aCrd: TCoordinate;
begin
  result:= 0;
  if aIndex >= CoordsCount then exit;
  aCrd:= GetCoordElement(aIndex);
  result:= aCrd.Y
end;


procedure TPolyElement.GivePointPropsForLength(Length: double; var pX, pY, pAngle: double);
var
  AuxLen, AbscLen: double;
  X1, Y1, X2, Y2: double;
  Found: boolean;
  I: longInt;
begin
  pX:= 0;
  pY:= 0;
  pAngle:= 0;

  if Length > GetLength then
  begin
    GetOrientedCoord(Count-1, true, pX, pY);
    exit;
  end;

  if Count = 0 then exit;

  if Count = 1 then
  begin
    GetOrientedCoord(0, true, pX, pY);
    exit
  end;

  I:= 0;
  AuxLen:= 0;

  Found:= false;
  while (I < Count-1) and not Found do
  begin
    GetOrientedCoord(I, true, X1, Y1);
    GetOrientedCoord(I+1, true, X2, Y2);
    AbscLen:= Distance(X1, Y1, X2, Y2);

    if AuxLen + AbscLen > Length then
    begin
      GiveAbscPointByLength(X1, Y1, X2, Y2, Length-AuxLen, 0, pX, pY);
      pAngle:= Atan2M(X2-X1, Y2-Y1) + Pi/2;
      Found:= true;
    end
    else
    begin
      AuxLen:= AuxLen + AbscLen;
      inc(I)
    end
  end
end;



procedure TPolyElement.GiveLabelProps(var aX, aY, Angle: double);
var
  Half: double;
begin
  Half:= GetLength/2;

  GivePointPropsForLength(Half, aX, aY, Angle);
end;

procedure TPolyElement.CalculateLabelProps;
var
  aX, aY, aAngle: double;
begin
  GiveLabelProps(aX, aY, aAngle);
  SetAbsRefX(aX);
  SetAbsRefY(aY);
  Angle:= aAngle
end;


procedure TPolyElement.RecalculateRect;
var
  I: longInt;
  aX, aY: double;
begin
  MinX:= 1E30;
  MinY:= 1E30;
  MaxX:= -1E30;
  MaxY:= -1E30;
  for I:= 0 to Count-1 do
  begin
    GetOrientedCoord(I, true, aX, aY);
    FRect.UpdateByXY(aX, aY)
  end
end;


function  TPolyElement.GetCoordElement(aIndex: longInt): T2DCoordinate;
begin
  result:= nil;
  if aIndex >= CoordsCount then exit;
  result:= T2DCoordinate(Elements[aIndex]);
  if not (TElement(result) is T2DCoordinate) then
     result:= nil
end;



function TPolyElement.GetOrientedCoordElement(aIndex: longInt; aBegin: boolean): T2DCoordinate;
begin
  Result:= nil;
  if aIndex >= CoordsCount then exit;

  if not aBegin then aIndex:= CoordsCount-aIndex-1;
  result:= GetCoordElement(aIndex)
end;





procedure TPolyElement.GetOrientedCoord(aIndex: longInt; aBegin: boolean; var aX, aY: double);
var
  Crd: T2DCoordinate;
begin
  aX:= 0; aY:= 0;
  if aIndex >= CoordsCount then exit;
  Crd:= GetOrientedCoordElement(aIndex, aBegin);
  if Crd = nil then exit;
  aX:= Crd.X;
  aY:= Crd.Y
end;



function TPolyElement.GetSubAreaValue(FromBegin: boolean): double;
var
  I: longInt;
  X1, Y1, X2, Y2: double;
begin
  result:= 0;
  if Count > 1 then
  begin
    GetOrientedCoord(0, FromBegin, X1, Y1);
    for I:= 1 to Count-1 do
    begin
      GetOrientedCoord(I, FromBegin, X2, Y2);
      result:= result + (X2-X1)*(Y1+Y2);
      X1:= X2;
      Y1:= Y2
    end;
    result:= result/2
  end
end;



procedure TPolyElement.AddXY(aX, aY: double);
var
  aCoord: T2DCoordinate;
begin
  aCoord:= T2DCoordinate.Create;
  aCoord.X:= aX;
  aCoord.Y:= aY;
  AddElement(aCoord)
end;



procedure TPolyElement.InsertXY(anIndex: longInt; aX, aY: double);
var
  aCoord: T2DCoordinate;
begin
  aCoord:= T2DCoordinate.Create;
  aCoord.X:= aX;
  aCoord.Y:= aY;
  InsertElement(anIndex, aCoord);
end;



procedure TPolyElement.AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean);
var
  I: longInt;
begin
  if aBegin then
    for I:= 0 to aPoly.Count-1 do
    begin
      AddElement(TCoordinate(aPoly[I]))
    end
  else
    for I:= aPoly.Count-1 downTo 0 do
    begin
      AddElement(TCoordinate(aPoly[I]))
    end
end;





function TPolyElement.GetLastLength: double;
var
  I: longInt;
{  X1, Y1, Z1, X2, Y2, Z2: double; }
  Crd1, Crd2: TCoordinate;
begin
   result:= 0;
   i := CoordsCount - 1;
   if i < 1 then exit;
   Crd1:= GetCoordElement(i - 1);
   Crd2:= GetCoordElement(I);
   result:= Distance(Crd1.X, Crd1.Y, Crd2.X, Crd2.Y);
end;

procedure TPolyElement.SetRecalculateLabelProps(const Value: boolean);
begin
   CalculateLabelProps;
end;

{ TPolyLineElement }

function TPolyLineElement.IsHitByXY(aX, aY: double; Tolerance: double): boolean;
var
  I: longInt;
  Uvnitr: boolean;
  XP, YP, Vzd: double;

begin
  Result:= false;
  for i := 0 to LastIndex do
  begin
     if InTolerance(aX,aY,Xi[i],Yi[i],Tolerance) then
     begin
        result := true;
        exit;
     end;
  end;
  for I:= 0 to Count-2 do
  begin
    DejVztahyBod_Usecka(aX, aY, Xi[I], Yi[I], Xi[I+1], Yi[I+1],
                        Uvnitr, Xp, Yp, Vzd);

    if Uvnitr and (Vzd < Tolerance) then
    begin
      Result:= true;
      Break
    end
  end
end;


function  TPolyLineElement.GetAreaFromBegin(aBegin: boolean): double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetOrientedCoordElement(0, aBegin);
    if not (CrdP is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetOrientedCoordElement(I, aBegin);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    ShowMessage('GetCalculatedArea Error')
  end;
end;




function TPolyLineElement.IsOverRegionEx(Minx, MinY, MaxX, MaxY: double): Boolean;
var
   i: integer;
   X1,Y1,X2,Y2: double;
begin
   result := false;
   if Count < 2 then
      exit;
   for i := 0 to LastIndex - 1 do
   begin
      X1 := Xi[i];
      Y1 := Yi[i];
      X2 := Xi[i+1];
      Y2 := Yi[i+1];
      SwapInc(X1,X2);
      SwapInc(Y1,Y2);
      if (MinX > X2) or (MaxX < X1) or
         (MinY > Y2) or (MaxY < Y1) then continue;
      result := true;
      break;
   end;
end;

{ TAreaElement }


destructor TAreaElement.Destroy;
begin
  if FHoles <> nil then
  begin
    FHoles.Free
  end;
  inherited Destroy
end;


function TAreaElement.GetWholeArea: double;
begin
  result:= abs(GetRealWholeArea);
  FSavedArea:= result
end;


function TAreaElement.GetArea: double;
var
  I: longInt;
  aHole: TAreaElement;
begin
  result:= GetWholeArea;
  if Holes = nil then exit;

  if Holes.Count < 1 then exit;

  for I:= 0 to Holes.Count-1 do
  begin
    aHole:= TAreaElement(Holes.Elements[I]);
    if not (TElement(aHole) is TAreaElement) then continue;
    result:= result-abs(aHole.WholeArea)
  end;
end;


function TAreaElement.GetHoleOwnerArea: TPolyElement;
var
  aHoles: THolesEntity;
begin
  result:= nil;
  if not IsHole then exit;
  aHoles:= THolesEntity(findOwnerClass(THolesEntity));
  if aHoles = nil then exit;
  result:= aHoles.OwnerArea;
end;



function TAreaElement.GetRealArea: double;
var
  I: longInt;
  aHole: TAreaElement;
begin
  result:= GetRealWholeArea;
  if Holes = nil then exit;

  if Holes.Count < 1 then exit;

  for I:= 0 to Holes.Count-1 do
  begin
    aHole:= TAreaElement(Holes.Elements[I]);
    if not (TElement(aHole) is TAreaElement) then continue;
    result:= result-abs(aHole.RealWholeArea)
  end;
end;

function TAreaElement.GetRealWholeArea: double;
var
  CrdP, CrdI: T2DCoordinate;
  X1, Y1: double;
  I: longInt;
begin
  result:= 0;
  try
    CrdP:= GetCoordElement(0);
    if not (TElement(CrdP) is T2DCoordinate) then
    begin
      exit;
    end;
    X1:= CrdP.X; Y1:= CrdP.Y;
    result:= 0;
    for I:= 1 to CoordsCount - 1 do
    begin
      CrdI:= GetCoordElement(I);
      result:= result + (CrdI.X-X1) * (CrdI.Y+Y1)/2;
      X1:= CrdI.X; Y1:= CrdI.Y;
    end;
    result:= result + (CrdP.X-X1) * (CrdP.Y+Y1)/2
  except
    //Name := 'Ahoj';
    ShowMessage('GetCalculatedArea Error')
  end;
//  result := abs(result); // real
//  FSavedArea:= result
end;

procedure TAreaElement.AddHole(aHole: TAreaElement);
begin
  if FHoles = nil then
  begin
    FHoles:= THolesEntity.Create;
    THolesEntity(FHoles).FOwnerArea:= self; { pro zajitn vazby nahoru }
  end;
  if FHoles = nil then exit;

  FHoles.AddAndUpdateRect(aHole);
  aHole.FIsHole:= true
end;


function TAreaElement.XYIsInsideNoHoles(aX, aY: double): boolean;
var
  Sour: T2DCoordinate;
begin
  Sour:= T2DCoordinate.Create;
  Sour.X:= aX;
  Sour.Y:= aY;
  result:= InsideTest(Self, Sour);
  Sour.Free
end;




function  TAreaElement.GetHoleByXY(aX, aY: double): TAreaElement;
var
  I: longInt;
  aHole: TAreaElement;
begin
  result:= nil;
  if FHoles = nil then exit;
  if FHoles.Count = 0 then exit;
  I:= 0;

  while I < FHoles.Count do
  begin
    aHole:= TAreaElement(FHoles.Elements[I]);
    if (TElement(aHole) is TAreaElement) and
       not ((aX < aHole.MinX) or(aX > aHole.MaxX) or (aY < aHole.MinY) or (aY > aHole.MaxY))then
    begin
      if aHole.XYIsInsideNoHoles(aX, aY) then
      begin
         result:= aHole;
         exit;
      end
    end;
    inc(I);
  end;
end;



function TAreaElement.XYIsInside(aX, aY: double): boolean;
begin
  //result:= false;
  result:= XYIsInsideNoHoles(aX, aY);

  if result then
     result:= GetHoleByXY(aX, aY) = nil
end;





function TAreaElement.IsHitByXY(aX, aY: double; Tolerance: double): boolean;
begin
  Result:= false;
  if RegionIsInside(aX-Tolerance, aY-Tolerance, aX+Tolerance, aY+Tolerance) then
  begin
    result:= XYIsInside(aX, aY);
  end;
end;




procedure TAreaElement.ResetIDs;
begin
   inherited;
   if Holes <> nil then
   begin
      Holes.ResetIDs;
      Holes.ID := -2;
   end;
end;

procedure TAreaElement.SetHoles(const Value: TRectEntity);
begin
   FHoles := Value;
   if Fholes is THolesEntity then
      THolesEntity(FHoles).SetOwnerArea(Self);
end;

procedure TAreaElement.SetSwapBorder(const Value: boolean);
var
   QE: TQuickEntity;
   i: integer;
begin
   QE := TQuickEntity.Create;
   QE.AddAllFrom := Self;
   Clear;
   for i := QE.LastIndex downto 0 do
      AddElement(QE.Elements[i]);
   QE.Clear;
end;

procedure TAreaElement.SetFixedArea(const Value: double);
begin
  FFixedArea := Value;
end;

procedure TAreaElement.SetFixedAreaIndex(const Value: byte);
begin
  FFixedAreaIndex := Value;
end;

function TAreaElement.GetFixedArea: double;
begin
   result := FFixedArea;
end;

function TAreaElement.GetFixedAreaIndex: byte;
begin
   result := FFixedAreaIndex;
end;

function TAreaElement.GetSameFAI: boolean;
begin
   result := true;
end;

{ TPolyLineAreaElement }


procedure TPolyLineAreaElement.AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean);
begin
  AddAndUpdateRect(aPoly);
  LinesOrigines[Count-1]:= aBegin;
end;



procedure GivePosition(aIndex: longInt; var ByteNo: longInt; var ByteMask: byte);
{ ByteNo: posice v retezci, 1.Byt ma ByteNo=1, 1.bit v Bytu ma BytePos=0}
begin
  ByteNo:= aIndex div 8 + 1;
  if aIndex > 7 then
  begin
    ByteMask:= aIndex mod 8
  end
  else
    ByteMask:= aIndex;
{ prevod na masku}
  ByteMask:= 1 shl ByteMask
end;



procedure TPolyLineAreaElement.RemoveOrigin(aIndex: longInt);
var
  I: longInt;
begin
  if FLinesOrigines = '' then exit;
  for I:= aIndex + 1 to Count-1 do
  begin
    SetOrigin(I-1, GetOrigin(I))
  end;
end;


procedure TPolyLineAreaElement.InsertOrigin(aIndex: Longint; aVal: boolean);
var
  I: longInt;
begin
  if aIndex > Count then Exit;
  if aIndex < 0 then Exit;

  if (FLinesOrigines = '') or (aIndex >= Count-1) then
  begin
    SetOrigin(aIndex, aVal);
    exit;
  end;

  for I:= Count-1 downto aIndex do
  begin
    SetOrigin(I+1, GetOrigin(I))
  end;

  SetOrigin(aIndex, aVal)
end;



procedure  TPolyLineAreaElement.RemoveElement(aElement: TElement);
var
  I: longInt;
begin
  if (aElement is TPolyLineElement) and
     (FLinesOrigines <> '') then
  begin
    I:= IndexOf(aElement);
    if I > -1 then RemoveOrigin(I)
  end;
  inherited RemoveElement(aElement);
end;



procedure TPolyLineAreaElement.InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);
begin
  InsertElement(aIndex, aPoly);
  InsertOrigin(aIndex, aBegin);
end;



function TPolyLineAreaElement.GetOrigin(aIndex: longInt): boolean;
var
  ByteNo: longInt;
  ByteMask: byte;

begin
  Result:= false;
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  if (system.length(FlinesOrigines) >= ByteNo) then
  begin
    Result:= (ord(FLinesOrigines[ByteNo]) and ByteMask) <> 0
  end;
end;


procedure TPolyLineAreaElement.SetOrigin(aIndex: longInt; Item: boolean);
var
  ByteNo, ByteCount: longInt;
  ByteMask: byte;
begin
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  ByteCount:= system.length(FlinesOrigines);

  if ByteCount < ByteNo then
     SetLength(FLinesOrigines, ByteNo);

  if Item then
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask or ByteMask)
  else
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask)
end;





procedure TPolyLineAreaElement.SetSwapBorder(const Value: boolean);
var
   i: integer;
   LinCount: integer;
   e: TElement;
   L: TPolyLineElement;
begin
   LinCount := Count;
   for i := LinCount - 1 downto 0 do
   begin
      e := Elements[i];
      L := TPolyLineElement(e);
      AddPolyLine(L,not LinesOrigines[i]);
   end;
   for i := LinCount - 1 downto 0 do
   begin
      e := Elements[i];
      RemoveElement(e);
   end;
end;

function TPolyLineAreaElement.HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
var
  I: longInt;
  Nasel: boolean;
  Lin: TPolyLineElement;
begin
  I:= 0;
  Nasel:= false;
  while (I < Count) and not Nasel do
  begin
    if Elements[I] is TPolyLineElement then
    begin
      Lin:= TPolyLineElement(Elements[I]);
      Nasel:= (Lin = aPoly) and (LinesOrigines[I] = aBegin)
    end;
    if not Nasel then inc(I)
  end;
  result:= Nasel
end;


procedure TPolyLineAreaElement.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
var
  I: integer;
  OrigLine, NewLine: TPolyLineElement;
  Origin,NewOrigin: Boolean;
begin
  OrigLine:= TPolyLineElement(aMessage.Sender);
  NewLine:= TPolyLineElement(aMessage.Data);
  if not (TObject(OrigLine) is TPolyLineElement) then
     Exit;
  if not (TObject(NewLine) is TPolyLineElement) then
     Exit;
  I:= IndexOf(OrigLine);
  if I < 0 then
     Exit;
  Origin:= GetOrigin(I);
  if OrigLine.Angle = 0 then
     NewOrigin := Origin
  else
     NewOrigin := not Origin;
  if Origin then
  begin
     InsertPolyLine(I, NewLine, NewOrigin)
  end
  else
  begin
     InsertPolyLine(I+1, NewLine, NewOrigin)
  end;
end;




function TPolyLineAreaElement.CreateAreaElementNoHole: TAreaElement;
var
  I, J: longInt;
  Linie: TPolyLineElement;
  Crd: T2DCoordinate;
  Start: longInt;
begin
  Result:= TAreaElement.Create;

{  Result.Properties:= Properties; }

  Start:= 0;   { naten prvn souadnice prvn linie, u dalch lini a druh }
  { prirazeni souradnic }
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      if LinesOrigines[I] then
      for J:= Start to Linie.Count-1 do
      begin
        Crd:= T2DCoordinate(TCoordinate(Linie.Elements[J]).CreateDuplicate);
        Result.AddElement(Crd)
      end
      else
      for J:= Linie.Count-Start-1 downTo 0 do
      begin
        Crd:= T2DCoordinate(TCoordinate(Linie.Elements[J]).CreateDuplicate);
        Result.AddElement(Crd)
      end;
      Start:= 1
    end
  end;
  Result.Rect:= FRect.CreateDuplicate;
  // popisovy bod
  Result.RefCoords.X := RefCoords.X;
  Result.RefCoords.Y := RefCoords.Y;
end;



function TPolyLineAreaElement.CreateAreaElement: TAreaElement;
var
  I: longInt;
  Hole: TAreaElement;
begin
  //result:= nil;
  result:= CreateAreaElementNoHole;
  result.FixedArea := FixedArea;
  result.FixedAreaIndex := FixedAreaIndex;
  if Holes = nil then exit;
  for I:= 0 to Holes.Count-1 do
  begin
    Hole:= TAreaElement(Holes.Elements[I]);
    if Hole is TPolyLineAreaElement then
    begin
       Hole:= TPolyLineAreaElement(Hole).CreateAreaElementNoHole;
       result.AddHole(Hole)
    end
  end;
end;



function TPolyLineAreaElement.GetCoordsCount: longInt;
var
  I: longInt;
  Linie: TPolyLineElement;
begin
  Result:= 0;
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      Result:= Result + Linie.Count-1;
    end
  end
end;




function TPolyLineAreaElement.GetCoordElement(aIndex: longInt): T2DCoordinate;
var
  I, J, AuxI: longInt;
  Linie: TPolyLineElement;
  Nasel: boolean;
begin
  I:= 0;
  AuxI:= 0;
  Nasel:= false;
  result:= nil;
  while (I < Count) and not Nasel do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      if AuxI + Linie.Count-1 > aIndex then
      begin
        J:= aIndex - AuxI;
        Result:= T2DCoordinate(Linie.GetOrientedCoordElement(J, LinesOrigines[I]));
        Nasel:= true
      end
      else
        AuxI:= AuxI + Linie.Count-1
    end;
    if not Nasel then inc(I)
  end
end;




function TPolyLineAreaElement.IsHitByXY(aX, aY: double; Tolerance: double): boolean;
begin
  Result:= false;
  if RegionIsInside(aX-Tolerance, aY-Tolerance, aX+Tolerance, aY+Tolerance) then
  begin
    result:= XYIsInside(aX, aY);
  end;
end;



function TPolyLineAreaElement.XYIsInsideNoHoles(aX, aY: double): boolean;
var
  aArea: TAreaElement;
begin
  aArea:= CreateAreaElement;
  result:= aArea.XYIsInsideNoHoles(aX, aY);
  aArea.Free;
end;



destructor TPolyLineAreaElement.Destroy;
begin
  inherited Destroy
end;


function TPolyLineAreaElement.GetLinesOriginesHex: string;
var
  I: integer;
  aChar: char;
begin
  result:= '';
  for I:= 1 to system.length(FLinesOrigines) do
  begin
    aChar:= FLinesOrigines[i];
    result:= result + IntToHex(ord(aChar), 2);
  end;
end;

procedure TPolyLineAreaElement.SetLinesOriginesHex(const Value: string);
var
  I: integer;
  aChar: char;
  aLength: integer;

  function Char2ToInt(C1, C2: char): integer;
  var
    aRes: integer;
    aStr: string;
  begin
    aStr:= '$' + C1 + C2;
    val(aStr, Result, aRes);
  end;

  function HexToChr: char;
  begin
    Result:= chr(Char2ToInt(Value[2*I-1], Value[2*I]))
  end;

begin
  FLinesOrigines:= '';
  aLength:= system.length(Value);
  aLength:= aLength div 2;
  for I:= 1 to aLength do
  begin
    aChar:= HexToChr;
    FLinesOrigines:= FLinesOrigines + aChar
  end;
end;




{ TMultiPolyLineElement }


procedure TMultiPolyLineElement.AddPolyLine(aPoly: TPolyLineElement; aBegin: boolean);
begin
  AddAndUpdateRect(aPoly);
  LinesOrigines[Count-1]:= aBegin;
end;



procedure TMultiPolyLineElement.RemoveOrigin(aIndex: longInt);
var
  I: longInt;
begin
  if FLinesOrigines = '' then exit;
  for I:= aIndex + 1 to Count-1 do
  begin
    SetOrigin(I-1, GetOrigin(I))
  end;
end;


procedure TMultiPolyLineElement.InsertOrigin(aIndex: Longint; aVal: boolean);
var
  I: longInt;
begin
  if aIndex > Count then Exit;
  if aIndex < 0 then Exit;

  if (FLinesOrigines = '') or (aIndex >= Count-1) then
  begin
    SetOrigin(aIndex, aVal);
    exit;
  end;

  for I:= Count-1 downto aIndex do
  begin
    SetOrigin(I+1, GetOrigin(I))
  end;

  SetOrigin(aIndex, aVal)
end;



procedure  TMultiPolyLineElement.RemoveElement(aElement: TElement);
var
  I: longInt;
begin
  if (aElement is TPolyLineElement) and
     (FLinesOrigines <> '') then
  begin
    I:= IndexOf(aElement);
    if I > -1 then RemoveOrigin(I)
  end;
  inherited RemoveElement(aElement);
end;



procedure TMultiPolyLineElement.InsertPolyLine(aIndex: integer; aPoly: TPolyLineElement; aBegin: boolean);
begin
  InsertElement(aIndex, aPoly);
  InsertOrigin(aIndex, aBegin);
end;



function TMultiPolyLineElement.GetOrigin(aIndex: longInt): boolean;
var
  ByteNo: longInt;
  ByteMask: byte;

begin
  Result:= false;
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  if (system.length(FlinesOrigines) >= ByteNo) then
  begin
    Result:= (ord(FLinesOrigines[ByteNo]) and ByteMask) <> 0
  end;
end;


procedure TMultiPolyLineElement.SetOrigin(aIndex: longInt; Item: boolean);
var
  ByteNo, ByteCount: longInt;
  ByteMask: byte;
begin
  if aIndex < 0 then exit;
  GivePosition(aIndex, ByteNo, ByteMask);
  ByteCount:= system.length(FlinesOrigines);

  if ByteCount < ByteNo then
     SetLength(FLinesOrigines, ByteNo);

  if Item then
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask or ByteMask)
  else
     FLinesOrigines[ByteNo]:= chr(ord(FLinesOrigines[ByteNo])and not ByteMask)
end;





function TMultiPolyLineElement.HasLine(aPoly: TPolyLineElement; aBegin: boolean): boolean;
var
  I: longInt;
  Nasel: boolean;
  Lin: TPolyLineElement;
begin
  I:= 0;
  Nasel:= false;
  while (I < Count) and not Nasel do
  begin
    if Elements[I] is TPolyLineElement then
    begin
      Lin:= TPolyLineElement(Elements[I]);
      Nasel:= (Lin = aPoly) and (LinesOrigines[I] = aBegin)
    end;
    if not Nasel then inc(I)
  end;
  result:= Nasel
end;


procedure TMultiPolyLineElement.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
var
  I: integer;
  OrigLine, NewLine: TPolyLineElement;
  Origin,NewOrigin: Boolean;
begin
  OrigLine:= TPolyLineElement(aMessage.Sender);
  NewLine:= TPolyLineElement(aMessage.Data);
  if not (TObject(OrigLine) is TPolyLineElement) then
     Exit;
  if not (TObject(NewLine) is TPolyLineElement) then
     Exit;
  I:= IndexOf(OrigLine);
  if I < 0 then
     Exit;
  Origin:= GetOrigin(I);
  if OrigLine.Angle = 0 then
     NewOrigin := Origin
  else
     NewOrigin := not Origin;
  if Origin then
  begin
     InsertPolyLine(I, NewLine, NewOrigin)
  end
  else
  begin
     InsertPolyLine(I+1, NewLine, NewOrigin)
  end;
end;




function TMultiPolyLineElement.GetCoordsCount: longInt;
var
  I: longInt;
  Linie: TPolyLineElement;
begin
  Result:= 0;
  for I:= 0 to Count-1 do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      Result:= Result + Linie.Count-1;
    end
  end
end;




function TMultiPolyLineElement.GetCoordElement(aIndex: longInt): T2DCoordinate;
var
  I, J, AuxI: longInt;
  Linie: TPolyLineElement;
  Nasel: boolean;
begin
  I:= 0;
  AuxI:= 0;
  Nasel:= false;
  result:= nil;
  while (I < Count) and not Nasel do
  begin
    if TObject(Elements[I]) is TPolyLineElement then
    begin
      Linie:= TPolyLineElement(Elements[I]);
      if AuxI + Linie.Count-1 > aIndex then
      begin
        J:= aIndex - AuxI;
        Result:= T2DCoordinate(Linie.GetOrientedCoordElement(J, LinesOrigines[I]));
        Nasel:= true
      end
      else
        AuxI:= AuxI + Linie.Count-1
    end;
    if not Nasel then inc(I)
  end
end;




function TMultiPolyLineElement.IsHitByXY(aX, aY: double; Tolerance: double): boolean;
var
   i: integer;
   E: TElement;
begin
   result := false;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TGeometry) then Continue;
      result := TGeometry(E).IsHitByXY(aX,aY, Tolerance);
      if result then
         break;
   end;
end;




function TMultiPolyLineElement.GetLinesOriginesHex: string;
var
  I: integer;
  aChar: char;
begin
  result:= '';
  for I:= 1 to system.length(FLinesOrigines) do
  begin
    aChar:= FLinesOrigines[i];
    result:= result + IntToHex(ord(aChar), 2);
  end;
end;

procedure TMultiPolyLineElement.SetLinesOriginesHex(const Value: string);
var
  I: integer;
  aChar: char;
  aLength: integer;

  function Char2ToInt(C1, C2: char): integer;
  var
    aRes: integer;
    aStr: string;
  begin
    aStr:= '$' + C1 + C2;
    val(aStr, Result, aRes);
  end;

  function HexToChr: char;
  begin
    Result:= chr(Char2ToInt(Value[2*I-1], Value[2*I]))
  end;

begin
  FLinesOrigines:= '';
  aLength:= system.length(Value);
  aLength:= aLength div 2;
  for I:= 1 to aLength do
  begin
    aChar:= HexToChr;
    FLinesOrigines:= FLinesOrigines + aChar
  end;
end;





destructor TMultiPolyLineElement.Destroy;
begin
  inherited;
end;

function TMultiPolyLineElement.GetLength: double;
var
   i: integer;
   E: TElement;
begin
   result := 0;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TPolyElement) then Continue;
      result := result + TPolyElement(E).Length;
   end;
end;


function TMultiPolyLineElement.GetSimpleGeometryByXY(aX, aY, Tolerance: double): TGeometry;
var
   i: integer;
   E: TElement;
begin
   result := nil;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TPolyElement) then Continue;
      result := TPolyElement(E).GetSimpleGeometryByXY(aX,aY,Tolerance);
      if result <> nil then exit;
   end;
end;

{ TMultiAreaElement }

function TMultiAreaElement.GetArea: double;
var
   i: integer;
   E: TElement;
begin
   result := 0;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TPolyElement) then Continue;
      result := result + TPolyElement(E).Area;
   end;
end;


function TMultiAreaElement.GetAreaElement(aIndex: integer): TAreaElement;
begin
   result := TAreaElement(Elements[aIndex]);
end;

function TMultiAreaElement.GetFixedArea: double;
var
   i: integer;
   E: TElement;
begin
   result := 0;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TAreaElement) then Continue;
      result := result + TAreaElement(E).FixedArea;
   end;
end;

function TMultiAreaElement.GetFixedAreaIndex: byte;
begin
   result := 0;
   if NoElements then exit;
   result := AreaElement[0].FixedAreaIndex;
end;

function TMultiAreaElement.GetSameFAI: boolean;
var
   i: integer;
   E: TElement;
   FAI: byte;
begin
   result := true;
   FAI := FixedAreaIndex;
   for i := 0 to LastIndex do
   begin
      E := Elements[i];
      if not (E is TAreaElement) then Continue;
      if FAI <> TAreaElement(E).FixedAreaIndex then
      begin
         result := false;
         exit;
      end;
   end;
end;

function TMultiAreaElement.GetSimpleGeometryByXY(aX, aY, Tolerance: double): TGeometry;
var
   i: integer;
begin
   result := nil;
   for i := 0 to LastIndex do
   begin
      result := AreaElement[i].GetSimpleGeometryByXY(aX,aY,Tolerance);
      if result <> nil then exit;
   end;
end;


function TMultiAreaElement.IsHitByXY(aX, aY: double; Tolerance: double): boolean;
var
   i: integer;
begin
   result := false;
   if RegionIsInside(aX-Tolerance, aY-Tolerance, aX+Tolerance, aY+Tolerance) then
   for i := 0 to LastIndex do
   begin
      if AreaElement[i].IsHitByXY(aX,aY,Tolerance) then
      begin
         result := true;
         exit;
      end;
   end;
end;

procedure TMultiAreaElement.SetSwapBorder(const Value: boolean);
begin
   //
end;

{ TGeometryString }

procedure TGeometryString.on_tm_DivideGeometry(var aMessage: TOrbisMessage; var Handled: boolean);
begin
   AddElement(aMessage.Data)
end;





{ THolesEntity }

procedure THolesEntity.SetOwnerArea(Value: TAreaElement);
begin
   FOwnerArea := Value;
end;

{ TPointElement }

procedure TPointElement.SetMinX(aValue: double);
begin
   inherited;
   MaxX := aValue;
end;

procedure TPointElement.SetMinY(aValue: double);
begin
   inherited;
   MaxY := aValue;
end;

INITIALIZATION
   RegisterOrbisClass(TGeometry);
   RegisterOrbisClass(TPointElement);
   RegisterOrbisClass(TTextElement);
   RegisterOrbisClass(TPolyElement);
   RegisterOrbisClass(TPolyLineElement);
   RegisterOrbisClass(TMultiPolyLineElement);
   RegisterOrbisClass(TAreaElement);
   RegisterOrbisClass(TMultiAreaElement);
   RegisterOrbisClass(TPolyLineAreaElement);
   RegisterOrbisClass(THolesEntity);
   RegisterOrbisClass(TPointString);
   RegisterOrbisClass(TTextString);
   RegisterOrbisClass(TPolyLineString);
   RegisterOrbisClass(TAreaString);
END.
