unit E_TplPrp;

interface
uses
  DBTables, DB,
// KERNEL
  Elements,
// GLIB
  G_Elems,
// SPACE
  E_Space,
  E_SpaPrp,
// SYMBOL
  u_Symbol,
// TOPOL
  U_TopolT;




Type


  TTopolProps = class(TSpaceEntityProps)
  private
    FCislo   : longInt;
{    FPuvodniCislo: longInt; }
    FDruh    : smallInt;
    FBarva   : smallInt;
    FZnacka  : smallInt;
    FVyber   : boolean;
    FPresnost: byte;
    FSymbol  : smallInt;
    FDone    : boolean;
    function   GetPhysicalObjectType: char;
    function   GetObjectType: char; virtual;
{    function   GetObjectsOwner: TSpaceEntityString; virtual; abstract; }
    function   GetDBFName: string; virtual;
{ TODO -oO.S. -cZmeny pro ADO : 11.11.99 }
    function   GetDBFTable: TDataSet; virtual;
  protected
    function   GetSpaceID: longInt; override;
  public
    function   GetDBFFieldAsString(aItem: string): string; override;
    procedure  SetDBFFieldAsString(aItem, aValue: string); override;
    function  HasDBFItem(aItem: string): boolean; override;
    function  SetDBFRecord: boolean; override;
    procedure SetPropertyStringByName(AName: string; aValue: string); override;
{    property   ObjectOwner: TSpaceEntityString read GetObjectsOwner; }
  published
    property PhysicalObjectType: char read GetPhysicalObjectType stored false;
    property ObjectType: char read GetObjectType stored false;
    property Cislo:    longInt read FCislo    write FCislo stored true;
    property Druh:     smallInt read FDruh     write FDruh;
    property Barva:    smallInt read FBarva    write FBarva;
    property Znacka:   smallInt read FZnacka   write FZnacka;
    property Vyber:    boolean read FVyber    write FVyber;
    property Presnost: byte    read FPresnost write FPresnost;
    property Symbol     : smallInt read FSymbol  write FSymbol;
    property IsDone: boolean   read FDone     write FDone stored false default false;

    property DBFName:  string read GetDBFName stored false;
{ TODO -oO.S. -cZmeny pro ADO : 11.11.99 }
    property DBFTable: TDataSet read GetDBFTable stored false;
  end;


  TTopolPointProps = class(TTopolProps)
  private
    function     GetObjectType: char; override;
  end;

  TTopolTextProps = class(TTopolProps)
  private
    FCodePage: TCodePage;
    FStartPoint: TCoordinate;
    FTopolFont   : byte;
    FHeight      : double;
    FWidthCoef   : double;
    FDirection   : double; { Uhel v Radianech }
    FSlope       : double; { Uhel v Radianech }
    FDistance    : double; { Zvetseni default hodnoty v Bodech }
    FTextAtrib   : TTextAtrib;
    FTextMode    : byte;

    procedure SetCodePage(aCode: TCodePage);
    function GetCodePage: TCodePage;
    function  GetStartPoint: TCoordinate;
    procedure SetStartPoint(aStartPoint: TCoordinate);
    function  GetObjectType: char; override;
  public
    destructor  Destroy; override;
  published
    property TextAtrib: TTextAtrib read FTextAtrib write FTextAtrib;
    property  StartPoint: TCoordinate read GetStartPoint write SetStartPoint stored true;
    property CodePage: TCodePage read GetCodePage write SetCodePage;
    property TopolFont: byte read FTopolFont write FTopolFont;
    property Height: double read FHeight write FHeight;
    property WidthCoef: double read FWidthCoef write FWidthCoef;
    property Direction: double read FDirection write FDirection;
    property Slope: double read FSlope write FSlope;
    property Distance: double read FDistance write FDistance;
    property TextMode: byte read FTextMode write FTextMode;
  end;


  TTopolLineProps = class(TTopolProps)
  private
    FLeftArea  : longInt;
    FRightArea : longInt;
    FDrziSe    : TDrziSe;
    function  GetObjectType: char; override;
  public
    property DrziSe: TDrziSe read FDrziSe write FDrziSe;
    function GetRightAreaNoForBegin(aBegin: boolean): longInt;
  published
    property LeftArea  : longInt read FLeftArea write FLeftArea;
    property RightArea : longInt read FRightArea write FRightArea;
  end;


  TTopolAreaProps = class(TTopolProps)
  private
    FLineNo: longInt;
    FPaterNo: longInt;
    FHasChildren: boolean;
    function  GetObjectType: char; override;
    function GetHektars: double;
  published
    property Hektars: double read GetHektars stored false;
    property LineNo: longInt read FLineNo write FLineNo;
    property PaterNo: longInt read FPaterNo write FPaterNo;
    property HasChildren: boolean read FHasChildren write FHasChildren;
  end;


implementation
uses
// KERNEL
  e_Class,
  U_Props,
// SPACE
  E_Reg,
// TOPOL
  {!PubUnits,}
  C_TLDBF,
  E_TplReg;



                 { TTopolProps }



function TTopolProps.GetPhysicalObjectType: char;
begin
  result:= ' ';
  if SpaceEntity = nil then exit;

  if SpaceEntity is TPointElement then result:= 'B'
  else
  if SpaceEntity is TTextElement then result:= 'T'
  else
  if SpaceEntity is TPolyLineElement then result:= 'L'
  else
  if SpaceEntity is TAreaElement then result:= 'P'
  else
     result:= 'X';
end;


function TTopolProps.GetObjectType: char;
begin
  result:= GetPhysicalObjectType
end;


function TTopolProps.GetDBFName: string;
var
  aName: string;
  aProps: TSpaceEntityProps;
begin
  result:= '';
  aProps:= GetOrigProps;
  if not (aProps is TTopolProps) then exit;
  aName:= GetBlockName;
  if aName = '' then exit;
  result:= GetDBFNameForTopolObject(aName, ObjectType, TTopolProps(aProps).Druh);
end;





function TTopolProps.HasDBFItem(aItem: string): boolean;
var
  aDBFName: string;
begin
  result:= false;
  aDBFName:= GetDBFName;

  if TTopolDBFTable(TTopolObjectRegister(ObjectRegister).DBF).SetForName(aDBFName) then
     result:= TTopolDBFTable(TTopolObjectRegister(ObjectRegister).DBF).FieldExists(aItem);
end;




function TTopolProps.SetDBFRecord: boolean;
var
  aDBFName: string;
begin
  aDBFName:= GetDBFName;
  result:= TTopolDBFTable(ObjectRegister.DBF).SetForNameCislo(aDBFName, OrigNo);
end;



function TTopolProps.GetDBFFieldAsString(aItem: string): string;
begin
  result:= TTopolDBFTable(ObjectRegister.DBF).GetFieldAsString(aItem);
end;


procedure TTopolProps.SetDBFFieldAsString(aItem, aValue: string);
begin
  TTopolDBFTable(ObjectRegister.DBF).SetFieldAsString(aItem, aValue);
end;



procedure   TTopolProps.SetPropertyStringByName(AName: string; aValue: string);
var
  aDBFName: string;
  TopTable: TTopolDBFTable;
begin
  if GetPropIndex(Self, aName) > -1 then
  begin
     inherited SetPropertyStringByName(aName, aValue);
     exit;
  end;
//  try
    if ObjectRegister = nil then exit;
    if not HasDBFItem(aName) then
       exit;
    aDBFName:= GetDBFName;
    TopTable:= TTopolDBFTable(TTopolObjectRegister(ObjectRegister).DBF);
    TopTable.Active:= false;
    TopTable.ReadOnly:= False;
    TopTable.SetForNameCislo(aDBFName, Cislo);
    TopTable.Edit;
    TopTable.FieldByName(aName).AsString:= aValue;
    TopTable.Post;
    TopTable.Active:= false;
    TopTable.ReadOnly:= True;
//  except

//  end;
end;


function  TTopolProps.GetSpaceID: longInt;
begin
  result:= Cislo
end;





function  TTopolProps.GetDBFTable: TDataSet;
begin
  result:= nil;
  if not (ObjectRegister is TTopolObjectRegister) then exit;

  result:= TTopolObjectRegister(ObjectRegister).DBF;
end;



               { TTopolPointProps }

function  TTopolPointProps.GetObjectType: char;
begin
  result:= 'B'
end;

               { TTopolTextProps }

destructor TTopolTextProps.Destroy;
begin
  if FStartPoint <> nil then
     FStartPoint.Free;
  inherited destroy
end;


function  TTopolTextProps.GetObjectType: char;
begin
  result:= 'T'
end;

procedure TTopolTextProps.SetCodePage(aCode: TCodePage);
begin
  FCodePage:= aCode
end;

function TTopolTextProps.GetCodePage: TCodePage;
begin
  result:= FCodePage
end;

function TTopolTextProps.GetStartPoint: TCoordinate;
begin
   if not assigned(FStartPoint) then FStartPoint := T2DCoordinate.Create;
   result := FStartPoint;
end;

procedure   TTopolTextProps.SetStartPoint(aStartPoint: TCoordinate);
begin
   if assigned(FStartPoint) then FStartPoint.Free;
   FStartPoint := nil;
   FStartPoint := aStartPoint;
end;

               { TTopolLineProps }

function  TTopolLineProps.GetObjectType: char;
begin
  result:= 'L'
end;

function TTopolLineProps.GetRightAreaNoForBegin(aBegin: boolean): longInt;
begin
  result:= LeftArea;
  if not aBegin then
     result:= RightArea
end;

               { TTopolAreaProps }

function  TTopolAreaProps.GetObjectType: char;
begin
  result:= 'P'
end;


function TTopolAreaProps.GetHektars: double;
begin
  result:= 0;
  if not (SpaceEntity is TAreaElement) then exit;
  result:= TAreaElement(SpaceEntity).Area1/10000
end;

initialization
   RegisterOrbisClass(TTopolProps);
   RegisterOrbisClass(TTopolPointProps);
   RegisterOrbisClass(TTopolTextProps);
   RegisterOrbisClass(TTopolAreaProps);
   RegisterOrbisClass(TTopolLineProps);
end.
