Unit WWLink;

{$C FIXED PRELOAD PERMANENT}

interface
uses WinDos,WinProcs;

var Receiver:boolean;

Procedure _Init;     {inicializace promennych}
Procedure _Close;    {reset pouziteho hradwaroveho preruseni}

Function _Open(ComPort       : Integer;
                    BaudRate      : Integer;
                    Parity        : Char;
                    WordSize      : Integer;
                    StopBits      : Integer) : Boolean;
                                                {vraci True v pripade, ze port se podarilo otevrit}

Function _Buffer_Check(var C : Byte) : Boolean; {znak z bufferu, jestlize _Buffer_Check je true}
Procedure _StopReceive;                         {zastavuje ukladani prijatych byte do bufferu}
Procedure _StartReceive;                        {povoluje ukladani prijatych byte do bufferu}
Function _Send(C : Byte):boolean;               {vyslani byte, vraci True v pripade, ze se vysilani povedlo}
Procedure _FreeBuffer;                          {vyprazdnuje buffer}
Function _IsOverflow:boolean;                   {vraci True v pripade ze buffer "pretekl"}

implementation

const
  UART_THR = $00;    {offset UART registru v IBM PC }
  UART_RBR = $00;
  UART_IER = $01;
  UART_IIR = $02;
  UART_LCR = $03;
  UART_MCR = $04;
  UART_LSR = $05;
  UART_MSR = $06;

  I8088_IMR = $21;   {adresa portu Interrupt Mask Register }

  _Num_Bauds = 8;
  _Baud_Table : array [1.._Num_Bauds] of record
                                                     Baud, Bits : integer
                                                   end
                   = ((Baud:110;  Bits:$00),
                      (Baud:150;  Bits:$20),
                      (Baud:300;  Bits:$40),
                      (Baud:600;  Bits:$60),
                      (Baud:1200; Bits:$80),
                      (Baud:2400; Bits:$A0),
                      (Baud:4800; Bits:$C0),
                      (Baud:9600; Bits:$E0));

  _Buffer_Max    = 512;
  _Delay:longint = 36;

type _TBuffer = Array[0.._Buffer_Max] of byte;
     _PBuffer = ^_TBuffer;

var
   _OriginalVector:pointer; {puvodni vektor preruseni}
  _Buffer       :_PBuffer;
  _Base         :Integer;   {bazova adresa aktualniho portu}
  _Open_Flag    :Boolean;   {true kdyz je port otevren}
  _Irq          :Integer;   {irq aktualniho otevreneho portu}
  _Port         :Integer;   {otevreny port (1 nebo 2) }

  _Buffer_Overflow :Boolean;  {true pri preteceni bufferu}
  _Buffer_Used     :Integer;
  _MaxBufferUsed   :Integer;

    {_Buffer je prazdny kdyz Head = Tail }
  _Buffer_Head  : Integer;   {Lokace v _Buffer pro vlozeni dalsiho znaku}
  _Buffer_Tail  : Integer;   {Lokace v _Buffer pro cteni dalsiho znaku}
  _Buffer_NewTail : Integer;

Function _IsOverFlow:boolean;
begin
     _IsOverflow:=_Buffer_Overflow;
end;

Procedure _StopReceive;
begin
     Receiver:=False;
end;

Procedure _StartReceive;
begin
     Receiver:=True;
end;

Procedure BIOS_RS232_Init(ComPort, ComParm:Integer);
{volani sluzby BIOSu, ktera inicializuje UART}
begin
     asm
         mov ax,ComParm
         and ax,00FFh
         mov dx,ComPort
         int 14h
     end;
end;

Procedure _Isr;  Interrupt;
{interrupt - vola ji UART pri prijmu jednoho znaku}
var _Buffer_NewHead:word;
    _b:byte;
begin
     Inline($FB);
     _b:=Port[UART_RBR + _Base];
     If Receiver then
     begin
          _Buffer^[_Buffer_Head]:=_b;
          _Buffer_NewHead := _Buffer_Head + 1;
          if _Buffer_NewHead>_Buffer_Max then _Buffer_NewHead:=0;
          if _Buffer_NewHead = _Buffer_Tail then _Buffer_Overflow:=TRUE else
          _Buffer_Head := _Buffer_NewHead;
          Inc(_Buffer_Used);
          if _Buffer_Used > _MaxBufferUsed then _MaxBufferUsed := _Buffer_Used;
     end;
     InLine($FA);
     Port[$20] := $20;  {konec preruseni pro 8259}
end;

Procedure _Init;
{inicializace promennych}
begin
     _Open_Flag:=False;
     _Buffer_Overflow:=False;
     _Buffer_Used:=0;
     _MaxBufferUsed:=0;
     Receiver:=False;
     New(_Buffer);
end;

Procedure _FreeBuffer;
begin
     _Buffer_Overflow:=False;
     _Buffer_Used:=0;
     _MaxBufferUsed:=0;
     _Buffer_Head:=0;
     _Buffer_Tail:=0;
end;

Procedure _Close;
{reset prerusovaciho sytemu UART}
var
  i, m : Integer;
begin
  if _Open_Flag then
  begin
      {zakazat preruseni IRQ pro 8259}
      inline($FA {cli} );
      i := Port[I8088_IMR];        {vzit masku preruseni}
      m := 1 shl _Irq;             {nastavit puvodni masku}
      Port[I8088_IMR]:= i or m;

      {zakaz 8250 preruseni pri cteni dat}
      Port[UART_IER + _Base]:=0;

      {nulovat OUT2 na 8250}
      Port[UART_MCR + _Base]:=0;
      inline($FB {sti} );

      SetIntVec(_Irq+8,_OriginalVector);

      {reinicializace dat}
      _Open_Flag:=False;
      _StopReceive;
      Dispose(_Buffer);
    end
end;

Function _Open(ComPort       : Integer;
                    BaudRate      : Integer;
                    Parity        : Char;
                    WordSize      : Integer;
                    StopBits      : Integer) : Boolean;
{pokus o otevreni komunikacniho portu}
var
  ComParm : Integer;
  i, m : Integer;
  s:string;
begin
     if _Open_Flag then _Close;
     if ComPort=2  then
     _Port:=2 else _Port:=1;  {standardne COM1}

     str(_port,s); s:='COM'+s+#0;
     i:=OpenComm(@s[1],1000,1000);
     If i<0 then
     begin
          _Open:=False;
          Exit;
     end;
     CloseComm(i);
     _StartReceive;
     If _Port=2 then _Base:=$2F8 else _Base:=$3F8;
     _Irq:=Hi(_Base)+1;

     if (Port[UART_IIR+_Base] and $00F8)<>0 then _Open := FALSE
     else
     begin
          _Buffer_Head:=0;
          _Buffer_Tail:=0;
          _Buffer_Overflow:=False;

          ComParm:=$0000;

          {nastaveni prenosove rychlosti}
          i:=0;
          repeat
                Inc(i);
          until (_Baud_Table[i].Baud=BaudRate) or (i=_Num_Bauds);
          ComParm:=ComParm or _Baud_Table[i].Bits;

          {nastaveni parity}
          if Parity in ['E','e'] then ComParm:=ComParm or $0018
             else if Parity in ['O','o'] then ComParm:=ComParm or $0008
               else ComParm:=ComParm or $0000; {standardne bez parity}

          {nastaveni delky slova}
          if WordSize=7 then ComParm:=ComParm or $0002
          else ComParm:=ComParm or $0003; {standardne 8 bitu}

          {nastaveni poctu stop bitu}
          if StopBits = 2 then ComParm:=ComParm or $0004
          else ComParm:=ComParm or $0000; {standardne 1 stop bit}

          {funkce BIOSu pro nastaveni COMu}
          BIOS_RS232_Init(_Port-1,ComParm);

          {uschovani puvodniho vektoru a nastaveni vlastniho}
          GetIntVec(_Irq+8, _OriginalVector);
          SetIntVec(_Irq+8, @_Isr);

          inline($FA {cli});
          {Nulovat DLAB}
          Port[UART_LCR+_Base]:=Port[UART_LCR+_Base] and $7F;

          {vyprazdneni vstupniho bufferu ???? !!}
          i:=Port[UART_LSR+_Base];
          i:=Port[UART_RBR+_Base];

          {povolit zpracovani preruseni pro 8259}
          i:=Port[I8088_IMR];
          m:=(1 shl _Irq) xor $00FF;
          Port[I8088_IMR] := i and m;

          {povoleni preruseni od prijateho znaku}
          Port[UART_IER+_Base]:=$01;

          {aktivace OUT2}
          i:=Port[UART_MCR+_Base];
          Port[UART_MCR+_Base]:= i or $08; {????????????????????????????? !!}

          inline($FB {sti});
          _Open_Flag:=True;
          _Open:=True;

          {aktivace RTS,OUT2 a DTR}
          Port[UART_MCR+_Base]:=$0B;
          {Vyprazdneni vstupu}
          i:=Port[UART_LSR + _Base];
     end;
end;


Function _Buffer_Check(var C:Byte):Boolean;
{je-li neco v bufferu vraci True a prislusny byte}
begin
     if _Buffer_Head=_Buffer_Tail then _Buffer_Check:=False
     else
     begin
          C:=Ord(_Buffer^[_Buffer_Tail]);
          Inc(_Buffer_Tail);
          if _Buffer_Tail>_Buffer_Max then _Buffer_Tail:=0;
          Dec(_Buffer_Used);
          _Buffer_Check:=True;
     end;
end;

Function _Send(C : Byte):boolean;
{vyslani jednoho byte}
var
  i, m: Integer;
  counter:longint;
begin
     Port[UART_MCR+_Base]:=$0B; {nastavit OUT2,DTR a RTS}

     {cekani na nastaveni CTS}
     counter:=GetTickCount+_Delay*55; {*55  kvuli shodnosti s DOSem}
     While (counter >=GetTickCount) and ((Port[UART_MSR + _Base] and $10) = 0) do;
     If counter <GetTickCount then begin _Send:=False; Exit; end;
     {CTS nebylo nastaveno v pozadovanem case}

     {cekani na vyprazdneni vysilaciho bufferu}
     counter:=GetTickCount+_Delay*55; {*55  kvuli shodnosti s DOSem}
     While (Counter>=GetTickCount) and ((Port[UART_LSR + _Base] and $20) = 0) do;

     If counter >=GetTickCount then
     begin  {cekani dopadlo dobre, je vysilani}
          inline($FA {cli});
          Port[UART_THR +_Base]:=C;
          inline($FB {sti});
          _Send:=True;
     end else _Send:=False; {vysilaci buffer se nepodarilo vyprazdnit}
end;

end.